@@ -22,11 +22,9 @@ Python Type                              BSON Type      Supported Direction
 None                                     null           both
 bool                                     boolean        both
 int [#int]_                              int32 / int64  py -> bson
-long                                     int64          py -> bson
 `bson.int64.Int64`                       int64          both
 float                                    number (real)  both
-string                                   string         py -> bson
-unicode                                  string         both
+str                                      string         both
 list                                     array          both
 dict / `SON`                             object         both
 datetime.datetime [#dt]_ [#dt2]_         date           both
@@ -36,17 +34,11 @@ compiled re [#re]_                       regex          py -> bson
 `bson.objectid.ObjectId`                 oid            both
 `bson.dbref.DBRef`                       dbref          both
 None                                     undefined      bson -> py
-unicode                                  code           bson -> py
-`bson.code.Code`                         code           py -> bson
-unicode                                  symbol         bson -> py
-bytes (Python 3) [#bytes]_               binary         both
+`bson.code.Code`                         code           both
+str                                      symbol         bson -> py
+bytes [#bytes]_                          binary         both
 =======================================  =============  ===================
 
-Note that, when using Python 2.x, to save binary data it must be wrapped as
-an instance of `bson.binary.Binary`. Otherwise it will be saved as a BSON
-string and retrieved as unicode. Users of Python 3.x can use the Python bytes
-type.
-
 .. [#int] A Python int will be saved as a BSON int32 or BSON int64 depending
    on its size. A BSON int32 will always decode to a Python int. A BSON
    int64 will always decode to a :class:`~bson.int64.Int64`.
@@ -58,10 +50,8 @@ type.
    objects from ``re.compile()`` are both saved as BSON regular expressions.
    BSON regular expressions are decoded as :class:`~bson.regex.Regex`
    instances.
-.. [#bytes] The bytes type from Python 3.x is encoded as BSON binary with
-   subtype 0. In Python 3.x it will be decoded back to bytes. In Python 2.x
-   it will be decoded to an instance of :class:`~bson.binary.Binary` with
-   subtype 0.
+.. [#bytes] The bytes type is encoded as BSON binary with
+   subtype 0. It will be decoded back to bytes.
 """
 
 import calendar
@@ -74,15 +64,16 @@ import sys
 import uuid
 from codecs import utf_8_decode as _utf_8_decode
 from codecs import utf_8_encode as _utf_8_encode
+from collections import abc as _abc
 
 from bson.binary import (
     ALL_UUID_SUBTYPES,
     CSHARP_LEGACY,
     JAVA_LEGACY,
     OLD_UUID_SUBTYPE,
+    STANDARD,
     UUID_SUBTYPE,
     Binary,
-    UUIDLegacy,
     UuidRepresentation,
 )
 from bson.code import Code
@@ -94,7 +85,6 @@ from bson.int64 import Int64
 from bson.max_key import MaxKey
 from bson.min_key import MinKey
 from bson.objectid import ObjectId
-from bson.py3compat import PY3, abc, b, iteritems, reraise, string_type, text_type
 from bson.regex import Regex
 from bson.son import RE_TYPE, SON
 from bson.timestamp import Timestamp
@@ -143,53 +133,18 @@ _UNPACK_LONG_FROM = struct.Struct("<q").unpack_from
 _UNPACK_TIMESTAMP_FROM = struct.Struct("<II").unpack_from
 
 
-if PY3:
-    _OBJEND = 0
-    # Only used to generate the _ELEMENT_GETTER dict
-    def _maybe_ord(element_type):
-        return ord(element_type)
-
-    # Only used in _raise_unkown_type below
-    def _elt_to_hex(element_type):
-        return chr(element_type).encode()
-
-    _supported_buffer_types = (bytes, bytearray)
-else:
-    _OBJEND = b"\x00"
-
-    def _maybe_ord(element_type):
-        return element_type
-
-    def _elt_to_hex(element_type):
-        return element_type
-
-    _supported_buffer_types = (bytes,)
-
-
-if platform.python_implementation() == "Jython":
-    # This is why we can't have nice things.
-    # https://bugs.jython.org/issue2788
-    def get_data_and_view(data):
-        if isinstance(data, _supported_buffer_types):
-            return data, data
-        data = memoryview(data).tobytes()
-        return data, data
-
-
-else:
-
-    def get_data_and_view(data):
-        if isinstance(data, _supported_buffer_types):
-            return data, memoryview(data)
-        view = memoryview(data)
-        return view.tobytes(), view
+def get_data_and_view(data):
+    if isinstance(data, (bytes, bytearray)):
+        return data, memoryview(data)
+    view = memoryview(data)
+    return view.tobytes(), view
 
 
 def _raise_unknown_type(element_type, element_name):
     """Unknown type helper."""
     raise InvalidBSON(
         "Detected unknown BSON type %r for fieldname '%s'. Are "
-        "you using the latest driver version?" % (_elt_to_hex(element_type), element_name)
+        "you using the latest driver version?" % (chr(element_type).encode(), element_name)
     )
 
 
@@ -199,7 +154,7 @@ def _get_int(data, view, position, dummy0, dummy1, dummy2):
 
 
 def _get_c_string(data, view, position, opts):
-    """Decode a BSON 'C' string to python unicode string."""
+    """Decode a BSON 'C' string to python str."""
     end = data.index(b"\x00", position)
     return _utf_8_decode(view[position:end], opts.unicode_decode_error_handler, True)[0], end + 1
 
@@ -210,13 +165,13 @@ def _get_float(data, view, position, dummy0, dummy1, dummy2):
 
 
 def _get_string(data, view, position, obj_end, opts, dummy):
-    """Decode a BSON string to python unicode string."""
+    """Decode a BSON string to python str."""
     length = _UNPACK_INT_FROM(data, position)[0]
     position += 4
     if length < 1 or obj_end - position < length:
         raise InvalidBSON("invalid string length")
     end = position + length - 1
-    if data[end] != _OBJEND:
+    if data[end] != 0:
         raise InvalidBSON("invalid end of string")
     return _utf_8_decode(view[position:end], opts.unicode_decode_error_handler, True)[0], end + 1
 
@@ -228,7 +183,7 @@ def _get_object_size(data, position, obj_end):
     except struct.error as exc:
         raise InvalidBSON(str(exc))
     end = position + obj_size - 1
-    if data[end] != _OBJEND:
+    if data[end] != 0:
         raise InvalidBSON("bad eoo")
     if end >= obj_end:
         raise InvalidBSON("invalid object length")
@@ -247,7 +202,12 @@ def _get_object(data, view, position, obj_end, opts, dummy):
     obj = _elements_to_dict(data, view, position + 4, end, opts)
 
     position += obj_size
-    if "$ref" in obj:
+    # If DBRef validation fails, return a normal doc.
+    if (
+        isinstance(obj.get("$ref"), str)
+        and "$id" in obj
+        and isinstance(obj.get("$db"), (str, type(None)))
+    ):
         return (DBRef(obj.pop("$ref"), obj.pop("$id", None), obj.pop("$db", None), obj), position)
     return obj, position
 
@@ -256,7 +216,7 @@ def _get_array(data, view, position, obj_end, opts, element_name):
     """Decode a BSON array to python list."""
     size = _UNPACK_INT_FROM(data, position)[0]
     end = position + size - 1
-    if data[end] != _OBJEND:
+    if data[end] != 0:
         raise InvalidBSON("bad eoo")
 
     position += 4
@@ -307,23 +267,19 @@ def _get_binary(data, view, position, obj_end, opts, dummy1):
         raise InvalidBSON("bad binary object length")
 
     # Convert UUID subtypes to native UUIDs.
-    # TODO: PYTHON-2245 Decoding should follow UUID spec in PyMongo 4.0+
     if subtype in ALL_UUID_SUBTYPES:
-        uuid_representation = opts.uuid_representation
+        uuid_rep = opts.uuid_representation
         binary_value = Binary(data[position:end], subtype)
-        if uuid_representation == UuidRepresentation.UNSPECIFIED:
+        if (
+            (uuid_rep == UuidRepresentation.UNSPECIFIED)
+            or (subtype == UUID_SUBTYPE and uuid_rep != STANDARD)
+            or (subtype == OLD_UUID_SUBTYPE and uuid_rep == STANDARD)
+        ):
             return binary_value, end
-        if subtype == UUID_SUBTYPE:
-            # Legacy behavior: use STANDARD with binary subtype 4.
-            uuid_representation = UuidRepresentation.STANDARD
-        elif uuid_representation == UuidRepresentation.STANDARD:
-            # subtype == OLD_UUID_SUBTYPE
-            # Legacy behavior: STANDARD is the same as PYTHON_LEGACY.
-            uuid_representation = UuidRepresentation.PYTHON_LEGACY
-        return binary_value.as_uuid(uuid_representation), end
+        return binary_value.as_uuid(uuid_rep), end
 
-    # Python3 special case. Decode subtype 0 to 'bytes'.
-    if PY3 and subtype == 0:
+    # Decode subtype 0 to 'bytes'.
+    if subtype == 0:
         value = data[position:end]
     else:
         value = Binary(data[position:end], subtype)
@@ -408,27 +364,27 @@ def _get_decimal128(data, view, position, dummy0, dummy1, dummy2):
 #   - obj_end: int, end of object to decode in 'data' if variable-length type
 #   - opts: a CodecOptions
 _ELEMENT_GETTER = {
-    _maybe_ord(BSONNUM): _get_float,
-    _maybe_ord(BSONSTR): _get_string,
-    _maybe_ord(BSONOBJ): _get_object,
-    _maybe_ord(BSONARR): _get_array,
-    _maybe_ord(BSONBIN): _get_binary,
-    _maybe_ord(BSONUND): lambda u, v, w, x, y, z: (None, w),  # Deprecated undefined
-    _maybe_ord(BSONOID): _get_oid,
-    _maybe_ord(BSONBOO): _get_boolean,
-    _maybe_ord(BSONDAT): _get_date,
-    _maybe_ord(BSONNUL): lambda u, v, w, x, y, z: (None, w),
-    _maybe_ord(BSONRGX): _get_regex,
-    _maybe_ord(BSONREF): _get_ref,  # Deprecated DBPointer
-    _maybe_ord(BSONCOD): _get_code,
-    _maybe_ord(BSONSYM): _get_string,  # Deprecated symbol
-    _maybe_ord(BSONCWS): _get_code_w_scope,
-    _maybe_ord(BSONINT): _get_int,
-    _maybe_ord(BSONTIM): _get_timestamp,
-    _maybe_ord(BSONLON): _get_int64,
-    _maybe_ord(BSONDEC): _get_decimal128,
-    _maybe_ord(BSONMIN): lambda u, v, w, x, y, z: (MinKey(), w),
-    _maybe_ord(BSONMAX): lambda u, v, w, x, y, z: (MaxKey(), w),
+    ord(BSONNUM): _get_float,
+    ord(BSONSTR): _get_string,
+    ord(BSONOBJ): _get_object,
+    ord(BSONARR): _get_array,
+    ord(BSONBIN): _get_binary,
+    ord(BSONUND): lambda u, v, w, x, y, z: (None, w),  # Deprecated undefined
+    ord(BSONOID): _get_oid,
+    ord(BSONBOO): _get_boolean,
+    ord(BSONDAT): _get_date,
+    ord(BSONNUL): lambda u, v, w, x, y, z: (None, w),
+    ord(BSONRGX): _get_regex,
+    ord(BSONREF): _get_ref,  # Deprecated DBPointer
+    ord(BSONCOD): _get_code,
+    ord(BSONSYM): _get_string,  # Deprecated symbol
+    ord(BSONCWS): _get_code_w_scope,
+    ord(BSONINT): _get_int,
+    ord(BSONTIM): _get_timestamp,
+    ord(BSONLON): _get_int64,
+    ord(BSONDEC): _get_decimal128,
+    ord(BSONMIN): lambda u, v, w, x, y, z: (MinKey(), w),
+    ord(BSONMAX): lambda u, v, w, x, y, z: (MaxKey(), w),
 }
 
 
@@ -437,7 +393,6 @@ if _USE_C:
     def _element_to_dict(data, view, position, obj_end, opts):
         return _cbson._element_to_dict(data, position, obj_end, opts)
 
-
 else:
 
     def _element_to_dict(data, view, position, obj_end, opts):
@@ -491,7 +446,7 @@ def _bson_to_dict(data, opts):
     except Exception:
         # Change exception type to InvalidBSON but preserve traceback.
         _, exc_value, exc_tb = sys.exc_info()
-        reraise(InvalidBSON, exc_value, exc_tb)
+        raise InvalidBSON(str(exc_value)).with_traceback(exc_tb)
 
 
 if _USE_C:
@@ -503,7 +458,7 @@ _PACK_INT = struct.Struct("<i").pack
 _PACK_LENGTH_SUBTYPE = struct.Struct("<iB").pack
 _PACK_LONG = struct.Struct("<q").pack
 _PACK_TIMESTAMP = struct.Struct("<II").pack
-_LIST_NAMES = tuple(b(str(i)) + b"\x00" for i in range(1000))
+_LIST_NAMES = tuple((str(i) + "\x00").encode("utf8") for i in range(1000))
 
 
 def gen_list_name():
@@ -518,7 +473,7 @@ def gen_list_name():
 
     counter = itertools.count(1000)
     while True:
-        yield b(str(next(counter))) + b"\x00"
+        yield (str(next(counter)) + "\x00").encode("utf8")
 
 
 def _make_c_string_check(string):
@@ -549,19 +504,12 @@ def _make_c_string(string):
         return _utf_8_encode(string)[0] + b"\x00"
 
 
-if PY3:
-
-    def _make_name(string):
-        """Make a 'C' string suitable for a BSON key."""
-        # Keys can only be text in python 3.
-        if "\x00" in string:
-            raise InvalidDocument("BSON keys / regex patterns must not " "contain a NUL character")
-        return _utf_8_encode(string)[0] + b"\x00"
-
-
-else:
-    # Keys can be unicode or bytes in python 2.
-    _make_name = _make_c_string_check
+def _make_name(string):
+    """Make a 'C' string suitable for a BSON key."""
+    # Keys can only be text in python 3.
+    if "\x00" in string:
+        raise InvalidDocument("BSON keys / regex patterns must not " "contain a NUL character")
+    return _utf_8_encode(string)[0] + b"\x00"
 
 
 def _encode_float(name, value, dummy0, dummy1):
@@ -569,30 +517,17 @@ def _encode_float(name, value, dummy0, dummy1):
     return b"\x01" + name + _PACK_FLOAT(value)
 
 
-if PY3:
-
-    def _encode_bytes(name, value, dummy0, dummy1):
-        """Encode a python bytes."""
-        # Python3 special case. Store 'bytes' as BSON binary subtype 0.
-        return b"\x05" + name + _PACK_INT(len(value)) + b"\x00" + value
-
-
-else:
-
-    def _encode_bytes(name, value, dummy0, dummy1):
-        """Encode a python str (python 2.x)."""
-        try:
-            _utf_8_decode(value, None, True)
-        except UnicodeError:
-            raise InvalidStringData("strings in documents must be valid " "UTF-8: %r" % (value,))
-        return b"\x02" + name + _PACK_INT(len(value) + 1) + value + b"\x00"
+def _encode_bytes(name, value, dummy0, dummy1):
+    """Encode a python bytes."""
+    # Python3 special case. Store 'bytes' as BSON binary subtype 0.
+    return b"\x05" + name + _PACK_INT(len(value)) + b"\x00" + value
 
 
 def _encode_mapping(name, value, check_keys, opts):
     """Encode a mapping type."""
     if _raw_document_class(value):
         return b"\x03" + name + value.raw
-    data = b"".join([_element_to_bson(key, val, check_keys, opts) for key, val in iteritems(value)])
+    data = b"".join([_element_to_bson(key, val, check_keys, opts) for key, val in value.items()])
     return b"\x03" + name + _PACK_INT(len(data) + 5) + data + b"\x00"
 
 
@@ -605,7 +540,7 @@ def _encode_dbref(name, value, check_keys, opts):
     buf += _name_value_to_bson(b"$id\x00", value.id, check_keys, opts)
     if value.database is not None:
         buf += _name_value_to_bson(b"$db\x00", value.database, check_keys, opts)
-    for key, val in iteritems(value._DBRef__kwargs):
+    for key, val in value._DBRef__kwargs.items():
         buf += _element_to_bson(key, val, check_keys, opts)
 
     buf += b"\x00"
@@ -621,7 +556,7 @@ def _encode_list(name, value, check_keys, opts):
 
 
 def _encode_text(name, value, dummy0, dummy1):
-    """Encode a python unicode (python 2.x) / str (python 3.x)."""
+    """Encode a python str."""
     value = _utf_8_encode(value)[0]
     return b"\x02" + name + _PACK_INT(len(value) + 1) + value + b"\x00"
 
@@ -665,12 +600,11 @@ def _encode_none(name, dummy0, dummy1, dummy2):
 def _encode_regex(name, value, dummy0, dummy1):
     """Encode a python regex or bson.regex.Regex."""
     flags = value.flags
-    # Python 2 common case
-    if flags == 0:
-        return b"\x0B" + name + _make_c_string_check(value.pattern) + b"\x00"
     # Python 3 common case
-    elif flags == re.UNICODE:
+    if flags == re.UNICODE:
         return b"\x0B" + name + _make_c_string_check(value.pattern) + b"u\x00"
+    elif flags == 0:
+        return b"\x0B" + name + _make_c_string_check(value.pattern) + b"\x00"
     else:
         sflags = b""
         if flags & re.IGNORECASE:
@@ -752,8 +686,7 @@ _ENCODERS = {
     float: _encode_float,
     int: _encode_int,
     list: _encode_list,
-    # unicode in py2, str in py3
-    text_type: _encode_text,
+    str: _encode_text,
     tuple: _encode_list,
     type(None): _encode_none,
     uuid.UUID: _encode_uuid,
@@ -768,10 +701,9 @@ _ENCODERS = {
     RE_TYPE: _encode_regex,
     SON: _encode_mapping,
     Timestamp: _encode_timestamp,
-    UUIDLegacy: _encode_binary,
     Decimal128: _encode_decimal128,
     # Special case. This will never be looked up directly.
-    abc.Mapping: _encode_mapping,
+    _abc.Mapping: _encode_mapping,
 }
 
 
@@ -787,9 +719,6 @@ _MARKERS = {
     255: _encode_minkey,
 }
 
-if not PY3:
-    _ENCODERS[long] = _encode_long
-
 
 _BUILT_IN_TYPES = tuple(t for t in _ENCODERS)
 
@@ -848,7 +777,7 @@ def _name_value_to_bson(
 
 def _element_to_bson(key, value, check_keys, opts):
     """Encode a single key, value pair."""
-    if not isinstance(key, string_type):
+    if not isinstance(key, str):
         raise InvalidDocument("documents must have only string keys, " "key was %r" % (key,))
     if check_keys:
         if key.startswith("$"):
@@ -868,7 +797,7 @@ def _dict_to_bson(doc, check_keys, opts, top_level=True):
         elements = []
         if top_level and "_id" in doc:
             elements.append(_name_value_to_bson(b"_id\x00", doc["_id"], check_keys, opts))
-        for (key, value) in iteritems(doc):
+        for key, value in doc.items():
             if not top_level or key != "_id":
                 elements.append(_element_to_bson(key, value, check_keys, opts))
     except AttributeError:
@@ -965,45 +894,6 @@ def decode(data, codec_options=DEFAULT_CODEC_OPTIONS):
     return _bson_to_dict(data, codec_options)
 
 
-def _decode_all(data, codec_options):
-    """Decode BSON data to multiple documents."""
-    data, view = get_data_and_view(data)
-    if not isinstance(codec_options, CodecOptions):
-        raise _CODEC_OPTIONS_TYPE_ERROR
-
-    data_len = len(data)
-    docs = []
-    position = 0
-    end = data_len - 1
-    use_raw = _raw_document_class(codec_options.document_class)
-    try:
-        while position < end:
-            obj_size = _UNPACK_INT_FROM(data, position)[0]
-            if data_len - position < obj_size:
-                raise InvalidBSON("invalid object size")
-            obj_end = position + obj_size - 1
-            if data[obj_end] != _OBJEND:
-                raise InvalidBSON("bad eoo")
-            if use_raw:
-                docs.append(
-                    codec_options.document_class(data[position : obj_end + 1], codec_options)
-                )
-            else:
-                docs.append(_elements_to_dict(data, view, position + 4, obj_end, codec_options))
-            position += obj_size
-        return docs
-    except InvalidBSON:
-        raise
-    except Exception:
-        # Change exception type to InvalidBSON but preserve traceback.
-        _, exc_value, exc_tb = sys.exc_info()
-        reraise(InvalidBSON, exc_value, exc_tb)
-
-
-if _USE_C:
-    _decode_all = _cbson._decode_all
-
-
 def decode_all(data, codec_options=DEFAULT_CODEC_OPTIONS):
     """Decode BSON data to multiple documents.
 
@@ -1026,19 +916,42 @@ def decode_all(data, codec_options=DEFAULT_CODEC_OPTIONS):
 
        Replaced `as_class`, `tz_aware`, and `uuid_subtype` options with
        `codec_options`.
-
-    .. versionchanged:: 2.7
-       Added `compile_re` option. If set to False, PyMongo represented BSON
-       regular expressions as :class:`~bson.regex.Regex` objects instead of
-       attempting to compile BSON regular expressions as Python native
-       regular expressions, thus preventing errors for some incompatible
-       patterns, see `PYTHON-500`_.
-
-    .. _PYTHON-500: https://jira.mongodb.org/browse/PYTHON-500
     """
+    data, view = get_data_and_view(data)
     if not isinstance(codec_options, CodecOptions):
         raise _CODEC_OPTIONS_TYPE_ERROR
-    return _decode_all(data, codec_options)
+
+    data_len = len(data)
+    docs = []
+    position = 0
+    end = data_len - 1
+    use_raw = _raw_document_class(codec_options.document_class)
+    try:
+        while position < end:
+            obj_size = _UNPACK_INT_FROM(data, position)[0]
+            if data_len - position < obj_size:
+                raise InvalidBSON("invalid object size")
+            obj_end = position + obj_size - 1
+            if data[obj_end] != 0:
+                raise InvalidBSON("bad eoo")
+            if use_raw:
+                docs.append(
+                    codec_options.document_class(data[position : obj_end + 1], codec_options)
+                )
+            else:
+                docs.append(_elements_to_dict(data, view, position + 4, obj_end, codec_options))
+            position += obj_size
+        return docs
+    except InvalidBSON:
+        raise
+    except Exception:
+        # Change exception type to InvalidBSON but preserve traceback.
+        _, exc_value, exc_tb = sys.exc_info()
+        raise InvalidBSON(str(exc_value)).with_traceback(exc_tb)
+
+
+if _USE_C:
+    decode_all = _cbson.decode_all
 
 
 def _decode_selective(rawdoc, fields, codec_options):
@@ -1049,7 +962,7 @@ def _decode_selective(rawdoc, fields, codec_options):
     else:
         # Else, use the specified document_class.
         doc = codec_options.document_class()
-    for key, value in iteritems(rawdoc):
+    for key, value in rawdoc.items():
         if key in fields:
             if fields[key] == 1:
                 doc[key] = _bson_to_dict(rawdoc.raw, codec_options)[key]
@@ -1260,15 +1173,6 @@ class BSON(bytes):
 
            Replaced `as_class`, `tz_aware`, and `uuid_subtype` options with
            `codec_options`.
-
-        .. versionchanged:: 2.7
-           Added `compile_re` option. If set to False, PyMongo represented BSON
-           regular expressions as :class:`~bson.regex.Regex` objects instead of
-           attempting to compile BSON regular expressions as Python native
-           regular expressions, thus preventing errors for some incompatible
-           patterns, see `PYTHON-500`_.
-
-        .. _PYTHON-500: https://jira.mongodb.org/browse/PYTHON-500
         """
         return decode(self, codec_options)
 
