@@ -16,13 +16,16 @@
 
 from copy import deepcopy
 
-from bson.py3compat import iteritems, string_type
+from bson._helpers import _getstate_slots, _setstate_slots
 from bson.son import SON
 
 
 class DBRef(object):
     """A reference to a document stored in MongoDB."""
 
+    __slots__ = "__collection", "__id", "__database", "__kwargs"
+    __getstate__ = _getstate_slots
+    __setstate__ = _setstate_slots
     # DBRef isn't actually a BSON "type" so this number was arbitrarily chosen.
     _type_marker = 100
 
@@ -44,10 +47,10 @@ class DBRef(object):
 
         .. seealso:: The MongoDB documentation on `dbrefs <https://dochub.mongodb.org/core/dbrefs>`_.
         """
-        if not isinstance(collection, string_type):
-            raise TypeError("collection must be an " "instance of %s" % string_type.__name__)
-        if database is not None and not isinstance(database, string_type):
-            raise TypeError("database must be an " "instance of %s" % string_type.__name__)
+        if not isinstance(collection, str):
+            raise TypeError("collection must be an instance of str")
+        if database is not None and not isinstance(database, str):
+            raise TypeError("database must be an instance of str")
 
         self.__collection = collection
         self.__id = id
@@ -57,7 +60,7 @@ class DBRef(object):
 
     @property
     def collection(self):
-        """Get the name of this DBRef's collection as unicode."""
+        """Get the name of this DBRef's collection."""
         return self.__collection
 
     @property
@@ -79,12 +82,6 @@ class DBRef(object):
         except KeyError:
             raise AttributeError(key)
 
-    # Have to provide __setstate__ to avoid
-    # infinite recursion since we override
-    # __getattr__.
-    def __setstate__(self, state):
-        self.__dict__.update(state)
-
     def as_doc(self):
         """Get the SON document representation of this DBRef.
 
@@ -97,7 +94,7 @@ class DBRef(object):
         return doc
 
     def __repr__(self):
-        extra = "".join([", %s=%r" % (k, v) for k, v in iteritems(self.__kwargs)])
+        extra = "".join([", %s=%r" % (k, v) for k, v in self.__kwargs.items()])
         if self.database is None:
             return "DBRef(%r, %r%s)" % (self.collection, self.id, extra)
         return "DBRef(%r, %r, %r%s)" % (self.collection, self.id, self.database, extra)
