@@ -1,4 +1,4 @@
-# Copyright 2010-2015 MongoDB, Inc.
+# Copyright 2010-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,29 +17,36 @@
 
 
 class MaxKey(object):
-    """MongoDB internal MaxKey type.
+    """MongoDB internal MaxKey type."""
 
-    .. versionchanged:: 2.7
-       ``MaxKey`` now implements comparison operators.
-    """
+    __slots__ = ()
 
     _type_marker = 127
 
+    def __getstate__(self):
+        return {}
+
+    def __setstate__(self, state):
+        pass
+
     def __eq__(self, other):
         return isinstance(other, MaxKey)
 
+    def __hash__(self):
+        return hash(self._type_marker)
+
     def __ne__(self, other):
         return not self == other
 
     def __le__(self, other):
         return isinstance(other, MaxKey)
-    
+
     def __lt__(self, dummy):
         return False
 
     def __ge__(self, dummy):
         return True
-    
+
     def __gt__(self, other):
         return not isinstance(other, MaxKey)
 
