@@ -17,14 +17,18 @@
 
 
 class MinKey(object):
-    """MongoDB internal MinKey type.
+    """MongoDB internal MinKey type."""
 
-    .. versionchanged:: 2.7
-       ``MinKey`` now implements comparison operators.
-    """
+    __slots__ = ()
 
     _type_marker = 255
 
+    def __getstate__(self):
+        return {}
+
+    def __setstate__(self, state):
+        pass
+
     def __eq__(self, other):
         return isinstance(other, MinKey)
 
