@@ -26,7 +26,6 @@ import time
 from random import SystemRandom
 
 from bson.errors import InvalidId
-from bson.py3compat import PY3, bytes_from_hex, string_type, text_type
 from bson.tz_util import utc
 
 _MAX_COUNTER_VALUE = 0xFFFFFF
@@ -69,7 +68,7 @@ class ObjectId(object):
 
         By default, ``ObjectId()`` creates a new unique identifier. The
         optional parameter `oid` can be an :class:`ObjectId`, or any 12
-        :class:`bytes` or, in Python 2, any 12-character :class:`str`.
+        :class:`bytes`.
 
         For example, the 12 bytes b'foo-bar-quux' do not follow the ObjectId
         specification but they are acceptable input::
@@ -77,14 +76,10 @@ class ObjectId(object):
           >>> ObjectId(b'foo-bar-quux')
           ObjectId('666f6f2d6261722d71757578')
 
-        `oid` can also be a :class:`unicode` or :class:`str` of 24 hex digits::
+        `oid` can also be a :class:`str` of 24 hex digits::
 
           >>> ObjectId('0123456789ab0123456789ab')
           ObjectId('0123456789ab0123456789ab')
-          >>>
-          >>> # A u-prefixed unicode literal:
-          >>> ObjectId(u'0123456789ab0123456789ab')
-          ObjectId('0123456789ab0123456789ab')
 
         Raises :class:`~bson.errors.InvalidId` if `oid` is not 12 bytes nor
         24 hex digits, or :class:`TypeError` if `oid` is not an accepted type.
@@ -197,19 +192,17 @@ class ObjectId(object):
         """
         if isinstance(oid, ObjectId):
             self.__id = oid.binary
-        # bytes or unicode in python 2, str in python 3
-        elif isinstance(oid, string_type):
+        elif isinstance(oid, str):
             if len(oid) == 24:
                 try:
-                    self.__id = bytes_from_hex(oid)
+                    self.__id = bytes.fromhex(oid)
                 except (TypeError, ValueError):
                     _raise_invalid_id(oid)
             else:
                 _raise_invalid_id(oid)
         else:
             raise TypeError(
-                "id must be an instance of (bytes, %s, ObjectId), "
-                "not %s" % (text_type.__name__, type(oid))
+                "id must be an instance of (bytes, str, ObjectId), " "not %s" % (type(oid),)
             )
 
     @property
@@ -246,15 +239,13 @@ class ObjectId(object):
         # ObjectIds pickled in python 2.x used `str` for __id.
         # In python 3.x this has to be converted to `bytes`
         # by encoding latin-1.
-        if PY3 and isinstance(oid, text_type):
+        if isinstance(oid, str):
             self.__id = oid.encode("latin-1")
         else:
             self.__id = oid
 
     def __str__(self):
-        if PY3:
-            return binascii.hexlify(self.__id).decode()
-        return binascii.hexlify(self.__id)
+        return binascii.hexlify(self.__id).decode()
 
     def __repr__(self):
         return "ObjectId('%s')" % (str(self),)
