@@ -17,7 +17,7 @@
 
 import re
 
-from bson.py3compat import string_type, text_type
+from bson._helpers import _getstate_slots, _setstate_slots
 from bson.son import RE_TYPE
 
 
@@ -42,6 +42,11 @@ def str_flags_to_int(str_flags):
 class Regex(object):
     """BSON regular expression data."""
 
+    __slots__ = ("pattern", "flags")
+
+    __getstate__ = _getstate_slots
+    __setstate__ = _setstate_slots
+
     _type_marker = 11
 
     @classmethod
@@ -55,7 +60,7 @@ class Regex(object):
           >>> pattern = re.compile('.*')
           >>> regex = Regex.from_native(pattern)
           >>> regex.flags ^= re.UNICODE
-          >>> db.collection.insert({'pattern': regex})
+          >>> db.collection.insert_one({'pattern': regex})
 
         :Parameters:
           - `regex`: A regular expression object from ``re.compile()``.
@@ -85,11 +90,11 @@ class Regex(object):
           - `flags`: (optional) an integer bitmask, or a string of flag
             characters like "im" for IGNORECASE and MULTILINE
         """
-        if not isinstance(pattern, (text_type, bytes)):
+        if not isinstance(pattern, (str, bytes)):
             raise TypeError("pattern must be a string, not %s" % type(pattern))
         self.pattern = pattern
 
-        if isinstance(flags, string_type):
+        if isinstance(flags, str):
             self.flags = str_flags_to_int(flags)
         elif isinstance(flags, int):
             self.flags = flags
