@@ -20,8 +20,7 @@ dictionary."""
 
 import copy
 import re
-
-from bson.py3compat import abc, iteritems
+from collections.abc import Mapping as _Mapping
 
 # This sort of sucks, but seems to be as good as it gets...
 # This is essentially the same as re._pattern_type
@@ -33,7 +32,7 @@ class SON(dict):
 
     A subclass of dict that maintains ordering of keys and provides a
     few extra niceties for dealing with SON. SON provides an API
-    similar to collections.OrderedDict from Python 2.7+.
+    similar to collections.OrderedDict.
     """
 
     def __init__(self, data=None, **kwargs):
@@ -62,9 +61,6 @@ class SON(dict):
         self.__keys.remove(key)
         dict.__delitem__(self, key)
 
-    def keys(self):
-        return list(self.__keys)
-
     def copy(self):
         other = SON()
         other.update(self)
@@ -80,24 +76,16 @@ class SON(dict):
     def has_key(self, key):
         return key in self.__keys
 
-    # third level takes advantage of second level definitions
-    def iteritems(self):
-        for k in self:
-            yield (k, self[k])
-
     def iterkeys(self):
         return self.__iter__()
 
     # fourth level uses definitions from lower levels
     def itervalues(self):
-        for _, v in self.iteritems():
+        for _, v in self.items():
             yield v
 
     def values(self):
-        return [v for _, v in self.iteritems()]
-
-    def items(self):
-        return [(key, self[key]) for key in self]
+        return [v for _, v in self.items()]
 
     def clear(self):
         self.__keys = []
@@ -124,7 +112,7 @@ class SON(dict):
 
     def popitem(self):
         try:
-            k, v = next(self.iteritems())
+            k, v = next(iter(self.items()))
         except StopIteration:
             raise KeyError("container is empty")
         del self[k]
@@ -134,8 +122,8 @@ class SON(dict):
         # Make progressively weaker assumptions about "other"
         if other is None:
             pass
-        elif hasattr(other, "iteritems"):  # iteritems saves memory and lookups
-            for k, v in other.iteritems():
+        elif hasattr(other, "items"):
+            for k, v in other.items():
                 self[k] = v
         elif hasattr(other, "keys"):
             for k in other.keys():
@@ -157,7 +145,7 @@ class SON(dict):
         regular dictionary is order-insensitive.
         """
         if isinstance(other, SON):
-            return len(self) == len(other) and self.items() == other.items()
+            return len(self) == len(other) and list(self.items()) == list(other.items())
         return self.to_dict() == other
 
     def __ne__(self, other):
@@ -176,8 +164,8 @@ class SON(dict):
         def transform_value(value):
             if isinstance(value, list):
                 return [transform_value(v) for v in value]
-            elif isinstance(value, abc.Mapping):
-                return dict([(k, transform_value(v)) for k, v in iteritems(value)])
+            elif isinstance(value, _Mapping):
+                return dict([(k, transform_value(v)) for k, v in value.items()])
             else:
                 return value
 
@@ -189,7 +177,7 @@ class SON(dict):
         if val_id in memo:
             return memo.get(val_id)
         memo[val_id] = out
-        for k, v in self.iteritems():
+        for k, v in self.items():
             if not isinstance(v, RE_TYPE):
                 v = copy.deepcopy(v, memo)
             out[k] = v
