@@ -18,7 +18,7 @@
 import calendar
 import datetime
 
-from bson.py3compat import integer_types
+from bson._helpers import _getstate_slots, _setstate_slots
 from bson.tz_util import utc
 
 UPPERBOUND = 4294967296
@@ -27,6 +27,11 @@ UPPERBOUND = 4294967296
 class Timestamp(object):
     """MongoDB internal timestamps used in the opLog."""
 
+    __slots__ = ("__time", "__inc")
+
+    __getstate__ = _getstate_slots
+    __setstate__ = _setstate_slots
+
     _type_marker = 17
 
     def __init__(self, time, inc):
@@ -51,9 +56,9 @@ class Timestamp(object):
             if time.utcoffset() is not None:
                 time = time - time.utcoffset()
             time = int(calendar.timegm(time.timetuple()))
-        if not isinstance(time, integer_types):
+        if not isinstance(time, int):
             raise TypeError("time must be an instance of int")
-        if not isinstance(inc, integer_types):
+        if not isinstance(inc, int):
             raise TypeError("inc must be an instance of int")
         if not 0 <= time < UPPERBOUND:
             raise ValueError("time must be contained in [0, 2**32)")
