@@ -20,7 +20,8 @@ The :mod:`gridfs` package is an implementation of GridFS on top of
 .. seealso:: The MongoDB documentation on `gridfs <https://dochub.mongodb.org/core/gridfs>`_.
 """
 
-from bson.py3compat import abc
+from collections import abc
+
 from gridfs.errors import NoFile
 from gridfs.grid_file import (
     DEFAULT_CHUNK_SIZE,
@@ -39,7 +40,7 @@ from pymongo.errors import ConfigurationError, OperationFailure
 class GridFS(object):
     """An instance of GridFS on top of a single Database."""
 
-    def __init__(self, database, collection="fs", disable_md5=False):
+    def __init__(self, database, collection="fs"):
         """Create a new instance of :class:`GridFS`.
 
         Raises :class:`TypeError` if `database` is not an instance of
@@ -48,14 +49,18 @@ class GridFS(object):
         :Parameters:
           - `database`: database to use
           - `collection` (optional): root collection to use
-          - `disable_md5` (optional): When True, MD5 checksums will not be
-            computed for uploaded files. Useful in environments where MD5
-            cannot be used for regulatory or other reasons. Defaults to False.
+
+        .. versionchanged:: 4.0
+           Removed the `disable_md5` parameter. See
+           :ref:`removed-gridfs-checksum` for details.
 
         .. versionchanged:: 3.11
            Running a GridFS operation in a transaction now always raises an
            error. GridFS does not support multi-document transactions.
 
+        .. versionchanged:: 3.7
+           Added the `disable_md5` parameter.
+
         .. versionchanged:: 3.1
            Indexes are only ensured on the first write to the DB.
 
@@ -76,7 +81,6 @@ class GridFS(object):
         self.__collection = database[collection]
         self.__files = self.__collection.files
         self.__chunks = self.__collection.chunks
-        self.__disable_md5 = disable_md5
 
     def new_file(self, **kwargs):
         """Create a new file in GridFS.
@@ -92,7 +96,7 @@ class GridFS(object):
         :Parameters:
           - `**kwargs` (optional): keyword arguments for file creation
         """
-        return GridIn(self.__collection, disable_md5=self.__disable_md5, **kwargs)
+        return GridIn(self.__collection, **kwargs)
 
     def put(self, data, **kwargs):
         """Put data in GridFS as a new file.
@@ -105,12 +109,11 @@ class GridFS(object):
           finally:
               f.close()
 
-        `data` can be either an instance of :class:`str` (:class:`bytes`
-        in python 3) or a file-like object providing a :meth:`read` method.
-        If an `encoding` keyword argument is passed, `data` can also be a
-        :class:`unicode` (:class:`str` in python 3) instance, which will
-        be encoded as `encoding` before being written. Any keyword arguments
-        will be passed through to the created file - see
+        `data` can be either an instance of :class:`bytes` or a file-like
+        object providing a :meth:`read` method. If an `encoding` keyword
+        argument is passed, `data` can also be a :class:`str` instance, which
+        will be encoded as `encoding` before being written. Any keyword
+        arguments will be passed through to the created file - see
         :meth:`~gridfs.grid_file.GridIn` for possible arguments. Returns the
         ``"_id"`` of the created file.
 
@@ -125,7 +128,7 @@ class GridFS(object):
         .. versionchanged:: 3.0
            w=0 writes to GridFS are now prohibited.
         """
-        grid_file = GridIn(self.__collection, disable_md5=self.__disable_md5, **kwargs)
+        grid_file = GridIn(self.__collection, **kwargs)
         try:
             grid_file.write(data)
         finally:
@@ -425,7 +428,6 @@ class GridFSBucket(object):
         chunk_size_bytes=DEFAULT_CHUNK_SIZE,
         write_concern=None,
         read_preference=None,
-        disable_md5=False,
     ):
         """Create a new instance of :class:`GridFSBucket`.
 
@@ -445,13 +447,17 @@ class GridFSBucket(object):
             (the default) db.write_concern is used.
           - `read_preference` (optional): The read preference to use. If
             ``None`` (the default) db.read_preference is used.
-          - `disable_md5` (optional): When True, MD5 checksums will not be
-            computed for uploaded files. Useful in environments where MD5
-            cannot be used for regulatory or other reasons. Defaults to False.
+
+        .. versionchanged:: 4.0
+           Removed the `disable_md5` parameter. See
+           :ref:`removed-gridfs-checksum` for details.
 
         .. versionchanged:: 3.11
-           Running a GridFS operation in a transaction now always raises an
-           error. GridFSBucket does not support multi-document transactions.
+           Running a GridFSBucket operation in a transaction now always raises
+           an error. GridFSBucket does not support multi-document transactions.
+
+        .. versionchanged:: 3.7
+           Added the `disable_md5` parameter.
 
         .. versionadded:: 3.1
 
@@ -468,8 +474,6 @@ class GridFSBucket(object):
 
         self._bucket_name = bucket_name
         self._collection = db[bucket_name]
-        self._disable_md5 = disable_md5
-
         self._chunks = self._collection.chunks.with_options(
             write_concern=write_concern, read_preference=read_preference
         )
@@ -527,7 +531,7 @@ class GridFSBucket(object):
         if metadata is not None:
             opts["metadata"] = metadata
 
-        return GridIn(self._collection, session=session, disable_md5=self._disable_md5, **opts)
+        return GridIn(self._collection, session=session, **opts)
 
     def open_upload_stream_with_id(
         self, file_id, filename, chunk_size_bytes=None, metadata=None, session=None
@@ -583,7 +587,7 @@ class GridFSBucket(object):
         if metadata is not None:
             opts["metadata"] = metadata
 
-        return GridIn(self._collection, session=session, disable_md5=self._disable_md5, **opts)
+        return GridIn(self._collection, session=session, **opts)
 
     def upload_from_stream(
         self, filename, source, chunk_size_bytes=None, metadata=None, session=None
