@@ -19,11 +19,15 @@ import hashlib
 import hmac
 import os
 import socket
+from base64 import standard_b64decode, standard_b64encode
+from collections import namedtuple
+from urllib.parse import quote
 
-try:
-    from urllib import quote
-except ImportError:
-    from urllib.parse import quote
+from bson.binary import Binary
+from bson.son import SON
+from pymongo.auth_aws import _authenticate_aws
+from pymongo.errors import ConfigurationError, OperationFailure
+from pymongo.saslprep import saslprep
 
 HAVE_KERBEROS = True
 _USE_PRINCIPAL = False
@@ -38,15 +42,6 @@ except ImportError:
     except ImportError:
         HAVE_KERBEROS = False
 
-from base64 import standard_b64decode, standard_b64encode
-from collections import namedtuple
-
-from bson.binary import Binary
-from bson.py3compat import PY3, _unicode, string_type
-from bson.son import SON
-from pymongo.auth_aws import _authenticate_aws
-from pymongo.errors import ConfigurationError, OperationFailure
-from pymongo.saslprep import saslprep
 
 MECHANISMS = frozenset(
     [
@@ -151,95 +146,9 @@ def _build_credentials_tuple(mech, source, user, passwd, extra, database):
         return MongoCredential(mech, source_database, user, passwd, None, _Cache())
 
 
-if PY3:
-
-    def _xor(fir, sec):
-        """XOR two byte strings together (python 3.x)."""
-        return b"".join([bytes([x ^ y]) for x, y in zip(fir, sec)])
-
-    _from_bytes = int.from_bytes
-    _to_bytes = int.to_bytes
-else:
-    from binascii import hexlify as _hexlify
-    from binascii import unhexlify as _unhexlify
-
-    def _xor(fir, sec):
-        """XOR two byte strings together (python 2.x)."""
-        return b"".join([chr(ord(x) ^ ord(y)) for x, y in zip(fir, sec)])
-
-    def _from_bytes(value, dummy, _int=int, _hexlify=_hexlify):
-        """An implementation of int.from_bytes for python 2.x."""
-        return _int(_hexlify(value), 16)
-
-    def _to_bytes(value, length, dummy, _unhexlify=_unhexlify):
-        """An implementation of int.to_bytes for python 2.x."""
-        fmt = "%%0%dx" % (2 * length,)
-        return _unhexlify(fmt % value)
-
-
-try:
-    # The fastest option, if it's been compiled to use OpenSSL's HMAC.
-    from backports.pbkdf2 import pbkdf2_hmac as _hi
-except ImportError:
-    try:
-        # Python 2.7.8+, or Python 3.4+.
-        from hashlib import pbkdf2_hmac as _hi
-    except ImportError:
-
-        def _hi(hash_name, data, salt, iterations):
-            """A simple implementation of PBKDF2-HMAC."""
-            mac = hmac.HMAC(data, None, getattr(hashlib, hash_name))
-
-            def _digest(msg, mac=mac):
-                """Get a digest for msg."""
-                _mac = mac.copy()
-                _mac.update(msg)
-                return _mac.digest()
-
-            from_bytes = _from_bytes
-            to_bytes = _to_bytes
-
-            _u1 = _digest(salt + b"\x00\x00\x00\x01")
-            _ui = from_bytes(_u1, "big")
-            for _ in range(iterations - 1):
-                _u1 = _digest(_u1)
-                _ui ^= from_bytes(_u1, "big")
-            return to_bytes(_ui, mac.digest_size, "big")
-
-
-try:
-    from hmac import compare_digest
-except ImportError:
-    if PY3:
-
-        def _xor_bytes(a, b):
-            return a ^ b
-
-    else:
-
-        def _xor_bytes(a, b, _ord=ord):
-            return _ord(a) ^ _ord(b)
-
-    # Python 2.x < 2.7.7
-    # Note: This method is intentionally obtuse to prevent timing attacks. Do
-    # not refactor it!
-    # References:
-    #  - http://bugs.python.org/issue14532
-    #  - http://bugs.python.org/issue14955
-    #  - http://bugs.python.org/issue15061
-    def compare_digest(a, b, _xor_bytes=_xor_bytes):
-        left = None
-        right = b
-        if len(a) == len(b):
-            left = a
-            result = 0
-        if len(a) != len(b):
-            left = b
-            result = 1
-
-        for x, y in zip(left, right):
-            result |= _xor_bytes(x, y)
-        return result == 0
+def _xor(fir, sec):
+    """XOR two byte strings together (python 3.x)."""
+    return b"".join([bytes([x ^ y]) for x, y in zip(fir, sec)])
 
 
 def _parse_scram_response(response):
@@ -309,7 +218,7 @@ def _authenticate_scram(credentials, sock_info, mechanism):
     # Salt and / or iterations could change for a number of different
     # reasons. Either changing invalidates the cache.
     if not client_key or salt != csalt or iterations != citerations:
-        salted_pass = _hi(digest, data, standard_b64decode(salt), iterations)
+        salted_pass = hashlib.pbkdf2_hmac(digest, data, standard_b64decode(salt), iterations)
         client_key = _hmac(salted_pass, b"Client Key", digestmod).digest()
         server_key = _hmac(salted_pass, b"Server Key", digestmod).digest()
         cache.data = (client_key, server_key, salt, iterations)
@@ -331,7 +240,7 @@ def _authenticate_scram(credentials, sock_info, mechanism):
     res = sock_info.command(source, cmd)
 
     parsed = _parse_scram_response(res["payload"])
-    if not compare_digest(parsed[b"v"], server_sig):
+    if not hmac.compare_digest(parsed[b"v"], server_sig):
         raise OperationFailure("Server returned an invalid signature.")
 
     # A third empty challenge may be required if the server does not support
@@ -351,17 +260,17 @@ def _authenticate_scram(credentials, sock_info, mechanism):
 
 def _password_digest(username, password):
     """Get a password digest to use for authentication."""
-    if not isinstance(password, string_type):
-        raise TypeError("password must be an " "instance of %s" % (string_type.__name__,))
+    if not isinstance(password, str):
+        raise TypeError("password must be an instance of str")
     if len(password) == 0:
         raise ValueError("password can't be empty")
-    if not isinstance(username, string_type):
-        raise TypeError("password must be an " "instance of  %s" % (string_type.__name__,))
+    if not isinstance(username, str):
+        raise TypeError("username must be an instance of str")
 
     md5hash = hashlib.md5()
     data = "%s:mongo:%s" % (username, password)
     md5hash.update(data.encode("utf-8"))
-    return _unicode(md5hash.hexdigest())
+    return md5hash.hexdigest()
 
 
 def _auth_key(nonce, username, password):
@@ -370,7 +279,7 @@ def _auth_key(nonce, username, password):
     md5hash = hashlib.md5()
     data = "%s%s%s" % (nonce, username, digest)
     md5hash.update(data.encode("utf-8"))
-    return _unicode(md5hash.hexdigest())
+    return md5hash.hexdigest()
 
 
 def _canonicalize_hostname(hostname):
@@ -522,38 +431,6 @@ def _authenticate_plain(credentials, sock_info):
     sock_info.command(source, cmd)
 
 
-def _authenticate_cram_md5(credentials, sock_info):
-    """Authenticate using CRAM-MD5 (RFC 2195)"""
-    source = credentials.source
-    username = credentials.username
-    password = credentials.password
-    # The password used as the mac key is the
-    # same as what we use for MONGODB-CR
-    passwd = _password_digest(username, password)
-    cmd = SON(
-        [
-            ("saslStart", 1),
-            ("mechanism", "CRAM-MD5"),
-            ("payload", Binary(b"")),
-            ("autoAuthorize", 1),
-        ]
-    )
-    response = sock_info.command(source, cmd)
-    # MD5 as implicit default digest for digestmod is deprecated
-    # in python 3.4
-    mac = hmac.HMAC(key=passwd.encode("utf-8"), digestmod=hashlib.md5)
-    mac.update(response["payload"])
-    challenge = username.encode("utf-8") + b" " + mac.hexdigest().encode("utf-8")
-    cmd = SON(
-        [
-            ("saslContinue", 1),
-            ("conversationId", response["conversationId"]),
-            ("payload", Binary(challenge)),
-        ]
-    )
-    sock_info.command(source, cmd)
-
-
 def _authenticate_x509(credentials, sock_info):
     """Authenticate using MONGODB-X509."""
     ctx = sock_info.auth_ctx.get(credentials)
@@ -562,11 +439,6 @@ def _authenticate_x509(credentials, sock_info):
         return
 
     cmd = _X509Context(credentials).speculate_command()
-    if credentials.username is None and sock_info.max_wire_version < 5:
-        raise ConfigurationError(
-            "A username is required for MONGODB-X509 authentication "
-            "when connected to MongoDB versions older than 3.4."
-        )
     sock_info.command("$external", cmd)
 
 
@@ -600,14 +472,11 @@ def _authenticate_default(credentials, sock_info):
             return _authenticate_scram(credentials, sock_info, "SCRAM-SHA-256")
         else:
             return _authenticate_scram(credentials, sock_info, "SCRAM-SHA-1")
-    elif sock_info.max_wire_version >= 3:
-        return _authenticate_scram(credentials, sock_info, "SCRAM-SHA-1")
     else:
-        return _authenticate_mongo_cr(credentials, sock_info)
+        return _authenticate_scram(credentials, sock_info, "SCRAM-SHA-1")
 
 
 _AUTH_MAP = {
-    "CRAM-MD5": _authenticate_cram_md5,
     "GSSAPI": _authenticate_gssapi,
     "MONGODB-CR": _authenticate_mongo_cr,
     "MONGODB-X509": _authenticate_x509,
@@ -677,8 +546,3 @@ def authenticate(credentials, sock_info):
     mechanism = credentials.mechanism
     auth_func = _AUTH_MAP.get(mechanism)
     auth_func(credentials, sock_info)
-
-
-def logout(source, sock_info):
-    """Log out from a database."""
-    sock_info.command(source, {"logout": 1})
