@@ -42,7 +42,6 @@ from pymongo.message import (
     _INSERT,
     _UPDATE,
     _BulkWriteContext,
-    _do_batched_insert,
     _EncryptedBulkWriteContext,
     _randint,
 )
@@ -60,12 +59,6 @@ _WRITE_CONCERN_ERROR = 64
 _COMMANDS = ("insert", "update", "delete")
 
 
-# These string literals are used when we create fake server return
-# documents client side. We use unicode literals in python 2.x to
-# match the actual return values from the server.
-_UOP = u"op"
-
-
 class _Run(object):
     """Represents a batch of write operations."""
 
@@ -127,7 +120,7 @@ def _merge_command(run, full_result, offset, result):
             idx = doc["index"] + offset
             replacement["index"] = run.index(idx)
             # Add the failed operation to the error document.
-            replacement[_UOP] = run.ops[idx]
+            replacement["op"] = run.ops[idx]
             full_result["writeErrors"].append(replacement)
 
     wce = _get_wce_doc(result)
@@ -266,14 +259,6 @@ class _Bulk(object):
     def _execute_command(
         self, generator, write_concern, session, sock_info, op_id, retryable, full_result
     ):
-        if sock_info.max_wire_version < 5:
-            if self.uses_collation:
-                raise ConfigurationError("Must be connected to MongoDB 3.4+ to use a collation.")
-            if self.uses_hint:
-                raise ConfigurationError("Must be connected to MongoDB 3.4+ to use hint.")
-        if sock_info.max_wire_version < 6 and self.uses_array_filters:
-            raise ConfigurationError("Must be connected to MongoDB 3.6+ to use arrayFilters.")
-
         db_name = self.collection.database.name
         client = self.collection.database.client
         listeners = client._event_listeners
@@ -302,7 +287,7 @@ class _Bulk(object):
                 cmd = SON([(cmd_name, self.collection.name), ("ordered", self.ordered)])
                 if not write_concern.is_server_default:
                     cmd["writeConcern"] = write_concern.document
-                if self.bypass_doc_val and sock_info.max_wire_version >= 4:
+                if self.bypass_doc_val:
                     cmd["bypassDocumentValidation"] = True
                 if session:
                     # Start a new retryable write unless one was already
@@ -370,38 +355,6 @@ class _Bulk(object):
             _raise_bulk_write_error(full_result)
         return full_result
 
-    def execute_insert_no_results(self, sock_info, run, op_id, acknowledged):
-        """Execute insert, returning no results."""
-        db = self.collection.database
-        concern = {"w": int(self.ordered)}
-        cmd = SON(
-            [("insert", self.collection.name), ("ordered", self.ordered), ("writeConcern", concern)]
-        )
-        if self.bypass_doc_val and sock_info.max_wire_version >= 4:
-            cmd["bypassDocumentValidation"] = True
-        bwc = _BulkWriteContext(
-            db.name,
-            "insert",
-            sock_info,
-            op_id,
-            db.client._event_listeners,
-            None,
-            _INSERT,
-            self.collection.codec_options,
-            cmd_legacy=cmd,
-        )
-        # Legacy batched OP_INSERT.
-        _do_batched_insert(
-            self.collection.full_name,
-            run.ops,
-            True,
-            acknowledged,
-            concern,
-            not self.ordered,
-            self.collection.codec_options,
-            bwc,
-        )
-
     def execute_op_msg_no_results(self, sock_info, generator):
         """Execute write commands with OP_MSG and w=0 writeConcern, unordered."""
         db_name = self.collection.database.name
@@ -474,65 +427,14 @@ class _Bulk(object):
         if self.uses_hint:
             raise ConfigurationError("hint is unsupported for unacknowledged writes.")
         # Cannot have both unacknowledged writes and bypass document validation.
-        if self.bypass_doc_val and sock_info.max_wire_version >= 4:
+        if self.bypass_doc_val:
             raise OperationFailure(
                 "Cannot set bypass_document_validation with" " unacknowledged write concern"
             )
 
-        # OP_MSG
-        if sock_info.max_wire_version > 5:
-            if self.ordered:
-                return self.execute_command_no_results(sock_info, generator)
-            return self.execute_op_msg_no_results(sock_info, generator)
-
-        coll = self.collection
-        # If ordered is True we have to send GLE or use write
-        # commands so we can abort on the first error.
-        write_concern = WriteConcern(w=int(self.ordered))
-        op_id = _randint()
-
-        next_run = next(generator)
-        while next_run:
-            # An ordered bulk write needs to send acknowledged writes to short
-            # circuit the next run. However, the final message on the final
-            # run can be unacknowledged.
-            run = next_run
-            next_run = next(generator, None)
-            needs_ack = self.ordered and next_run is not None
-            try:
-                if run.op_type == _INSERT:
-                    self.execute_insert_no_results(sock_info, run, op_id, needs_ack)
-                elif run.op_type == _UPDATE:
-                    for operation in run.ops:
-                        doc = operation["u"]
-                        check_keys = True
-                        if doc and next(iter(doc)).startswith("$"):
-                            check_keys = False
-                        coll._update(
-                            sock_info,
-                            operation["q"],
-                            doc,
-                            operation["upsert"],
-                            check_keys,
-                            operation["multi"],
-                            write_concern=write_concern,
-                            op_id=op_id,
-                            ordered=self.ordered,
-                            bypass_doc_val=self.bypass_doc_val,
-                        )
-                else:
-                    for operation in run.ops:
-                        coll._delete(
-                            sock_info,
-                            operation["q"],
-                            not operation["limit"],
-                            write_concern,
-                            op_id,
-                            self.ordered,
-                        )
-            except OperationFailure:
-                if self.ordered:
-                    break
+        if self.ordered:
+            return self.execute_command_no_results(sock_info, generator)
+        return self.execute_op_msg_no_results(sock_info, generator)
 
     def execute(self, write_concern, session):
         """Execute operations."""
@@ -555,172 +457,3 @@ class _Bulk(object):
                 self.execute_no_results(sock_info, generator)
         else:
             return self.execute_command(generator, write_concern, session)
-
-
-class BulkUpsertOperation(object):
-    """An interface for adding upsert operations."""
-
-    __slots__ = ("__selector", "__bulk", "__collation")
-
-    def __init__(self, selector, bulk, collation):
-        self.__selector = selector
-        self.__bulk = bulk
-        self.__collation = collation
-
-    def update_one(self, update):
-        """Update one document matching the selector.
-
-        :Parameters:
-          - `update` (dict): the update operations to apply
-        """
-        self.__bulk.add_update(
-            self.__selector, update, multi=False, upsert=True, collation=self.__collation
-        )
-
-    def update(self, update):
-        """Update all documents matching the selector.
-
-        :Parameters:
-          - `update` (dict): the update operations to apply
-        """
-        self.__bulk.add_update(
-            self.__selector, update, multi=True, upsert=True, collation=self.__collation
-        )
-
-    def replace_one(self, replacement):
-        """Replace one entire document matching the selector criteria.
-
-        :Parameters:
-          - `replacement` (dict): the replacement document
-        """
-        self.__bulk.add_replace(
-            self.__selector, replacement, upsert=True, collation=self.__collation
-        )
-
-
-class BulkWriteOperation(object):
-    """An interface for adding update or remove operations."""
-
-    __slots__ = ("__selector", "__bulk", "__collation")
-
-    def __init__(self, selector, bulk, collation):
-        self.__selector = selector
-        self.__bulk = bulk
-        self.__collation = collation
-
-    def update_one(self, update):
-        """Update one document matching the selector criteria.
-
-        :Parameters:
-          - `update` (dict): the update operations to apply
-        """
-        self.__bulk.add_update(self.__selector, update, multi=False, collation=self.__collation)
-
-    def update(self, update):
-        """Update all documents matching the selector criteria.
-
-        :Parameters:
-          - `update` (dict): the update operations to apply
-        """
-        self.__bulk.add_update(self.__selector, update, multi=True, collation=self.__collation)
-
-    def replace_one(self, replacement):
-        """Replace one entire document matching the selector criteria.
-
-        :Parameters:
-          - `replacement` (dict): the replacement document
-        """
-        self.__bulk.add_replace(self.__selector, replacement, collation=self.__collation)
-
-    def remove_one(self):
-        """Remove a single document matching the selector criteria."""
-        self.__bulk.add_delete(self.__selector, _DELETE_ONE, collation=self.__collation)
-
-    def remove(self):
-        """Remove all documents matching the selector criteria."""
-        self.__bulk.add_delete(self.__selector, _DELETE_ALL, collation=self.__collation)
-
-    def upsert(self):
-        """Specify that all chained update operations should be
-        upserts.
-
-        :Returns:
-          - A :class:`BulkUpsertOperation` instance, used to add
-            update operations to this bulk operation.
-        """
-        return BulkUpsertOperation(self.__selector, self.__bulk, self.__collation)
-
-
-class BulkOperationBuilder(object):
-    """**DEPRECATED**: An interface for executing a batch of write operations."""
-
-    __slots__ = "__bulk"
-
-    def __init__(self, collection, ordered=True, bypass_document_validation=False):
-        """**DEPRECATED**: Initialize a new BulkOperationBuilder instance.
-
-        :Parameters:
-          - `collection`: A :class:`~pymongo.collection.Collection` instance.
-          - `ordered` (optional): If ``True`` all operations will be executed
-            serially, in the order provided, and the entire execution will
-            abort on the first error. If ``False`` operations will be executed
-            in arbitrary order (possibly in parallel on the server), reporting
-            any errors that occurred after attempting all operations. Defaults
-            to ``True``.
-          - `bypass_document_validation`: (optional) If ``True``, allows the
-            write to opt-out of document level validation. Default is
-            ``False``.
-
-        .. note:: `bypass_document_validation` requires server version
-          **>= 3.2**
-
-        .. versionchanged:: 3.5
-           Deprecated. Use :meth:`~pymongo.collection.Collection.bulk_write`
-           instead.
-
-        .. versionchanged:: 3.2
-          Added bypass_document_validation support
-        """
-        self.__bulk = _Bulk(collection, ordered, bypass_document_validation)
-
-    def find(self, selector, collation=None):
-        """Specify selection criteria for bulk operations.
-
-        :Parameters:
-          - `selector` (dict): the selection criteria for update
-            and remove operations.
-          - `collation` (optional): An instance of
-            :class:`~pymongo.collation.Collation`. This option is only
-            supported on MongoDB 3.4 and above.
-
-        :Returns:
-          - A :class:`BulkWriteOperation` instance, used to add
-            update and remove operations to this bulk operation.
-
-        .. versionchanged:: 3.4
-           Added the `collation` option.
-
-        """
-        validate_is_mapping("selector", selector)
-        return BulkWriteOperation(selector, self.__bulk, collation)
-
-    def insert(self, document):
-        """Insert a single document.
-
-        :Parameters:
-          - `document` (dict): the document to insert
-
-        .. seealso:: :ref:`writes-and-ids`
-        """
-        self.__bulk.add_insert(document)
-
-    def execute(self, write_concern=None):
-        """Execute all provided operations.
-
-        :Parameters:
-          - write_concern (optional): the write concern for this bulk
-            execution.
-        """
-        if write_concern is not None:
-            write_concern = WriteConcern(**write_concern)
-        return self.__bulk.execute(write_concern, session=None)
