@@ -134,14 +134,14 @@ Classes
 """
 
 import collections
+import time
 import uuid
+from collections.abc import Mapping as _Mapping
 
 from bson.binary import Binary
 from bson.int64 import Int64
-from bson.py3compat import abc, integer_types
 from bson.son import SON
 from bson.timestamp import Timestamp
-from pymongo import monotonic
 from pymongo.cursor import _SocketManager
 from pymongo.errors import (
     ConfigurationError,
@@ -276,7 +276,7 @@ class TransactionOptions(object):
                     "options." % (read_preference,)
                 )
         if max_commit_time_ms is not None:
-            if not isinstance(max_commit_time_ms, integer_types):
+            if not isinstance(max_commit_time_ms, int):
                 raise TypeError("max_commit_time_ms must be an integer or None")
 
     @property
@@ -433,7 +433,7 @@ _WITH_TRANSACTION_RETRY_TIME_LIMIT = 120
 
 def _within_time_limit(start_time):
     """Are we within the with_transaction retry limit?"""
-    return monotonic.time() - start_time < _WITH_TRANSACTION_RETRY_TIME_LIMIT
+    return time.monotonic() - start_time < _WITH_TRANSACTION_RETRY_TIME_LIMIT
 
 
 class ClientSession(object):
@@ -449,12 +449,11 @@ class ClientSession(object):
     :meth:`~pymongo.mongo_client.MongoClient.start_session`.
     """
 
-    def __init__(self, client, server_session, options, authset, implicit):
+    def __init__(self, client, server_session, options, implicit):
         # A MongoClient, a _ServerSession, a SessionOptions, and a set.
         self._client = client
         self._server_session = server_session
         self._options = options
-        self._authset = authset
         self._cluster_time = None
         self._operation_time = None
         self._snapshot_time = None
@@ -626,7 +625,7 @@ class ClientSession(object):
 
         .. versionadded:: 3.9
         """
-        start_time = monotonic.time()
+        start_time = time.monotonic()
         while True:
             self.start_transaction(read_concern, write_concern, read_preference, max_commit_time_ms)
             try:
@@ -829,7 +828,7 @@ class ClientSession(object):
             :data:`~pymongo.client_session.ClientSession.cluster_time` from
             another `ClientSession` instance.
         """
-        if not isinstance(cluster_time, abc.Mapping):
+        if not isinstance(cluster_time, _Mapping):
             raise TypeError("cluster_time must be a subclass of collections.Mapping")
         if not isinstance(cluster_time.get("clusterTime"), Timestamp):
             raise ValueError("Invalid cluster_time")
@@ -920,7 +919,7 @@ class ClientSession(object):
         if self.options.snapshot:
             self._update_read_concern(command, sock_info)
 
-        self._server_session.last_use = monotonic.time()
+        self._server_session.last_use = time.monotonic()
         command["lsid"] = self._server_session.session_id
 
         if is_retryable:
@@ -963,12 +962,15 @@ class ClientSession(object):
             if self._snapshot_time is not None:
                 rc["atClusterTime"] = self._snapshot_time
 
+    def __copy__(self):
+        raise TypeError("A ClientSession cannot be copied, create a new session instead")
+
 
 class _ServerSession(object):
     def __init__(self, generation):
         # Ensure id is type 4, regardless of CodecOptions.uuid_representation.
         self.session_id = {"id": Binary(uuid.uuid4().bytes, 4)}
-        self.last_use = monotonic.time()
+        self.last_use = time.monotonic()
         self._transaction_id = 0
         self.dirty = False
         self.generation = generation
@@ -982,7 +984,7 @@ class _ServerSession(object):
         self.dirty = True
 
     def timed_out(self, session_timeout_minutes):
-        idle_seconds = monotonic.time() - self.last_use
+        idle_seconds = time.monotonic() - self.last_use
 
         # Timed out if we have less than a minute to live.
         return idle_seconds > (session_timeout_minutes - 1) * 60
