@@ -17,7 +17,6 @@
 from collections import deque
 
 from bson import _convert_raw_document_lists_to_streams
-from bson.py3compat import integer_types
 from pymongo.cursor import _CURSOR_CLOSED_ERRORS, _SocketManager
 from pymongo.errors import ConnectionFailure, InvalidOperation, OperationFailure
 from pymongo.message import _CursorAddress, _GetMore, _RawBatchGetMore
@@ -34,16 +33,12 @@ class CommandCursor(object):
         collection,
         cursor_info,
         address,
-        retrieved=0,
         batch_size=0,
         max_await_time_ms=None,
         session=None,
         explicit_session=False,
     ):
-        """Create a new command cursor.
-
-        The parameter 'retrieved' is unused.
-        """
+        """Create a new command cursor."""
         self.__sock_mgr = None
         self.__collection = collection
         self.__id = cursor_info["id"]
@@ -65,7 +60,7 @@ class CommandCursor(object):
 
         self.batch_size(batch_size)
 
-        if not isinstance(max_await_time_ms, integer_types) and max_await_time_ms is not None:
+        if not isinstance(max_await_time_ms, int) and max_await_time_ms is not None:
             raise TypeError("max_await_time_ms must be an integer or None")
 
     def __del__(self):
@@ -119,7 +114,7 @@ class CommandCursor(object):
         :Parameters:
           - `batch_size`: The size of each batch of results requested.
         """
-        if not isinstance(batch_size, integer_types):
+        if not isinstance(batch_size, int):
             raise TypeError("batch_size must be an integer")
         if batch_size < 0:
             raise ValueError("batch_size must be >= 0")
@@ -289,8 +284,7 @@ class CommandCursor(object):
         if not len(self.__data) and not self.__killed and get_more_allowed:
             self._refresh()
         if len(self.__data):
-            coll = self.__collection
-            return coll.database._fix_outgoing(self.__data.popleft(), coll)
+            return self.__data.popleft()
         else:
             return None
 
@@ -309,7 +303,6 @@ class RawBatchCommandCursor(CommandCursor):
         collection,
         cursor_info,
         address,
-        retrieved=0,
         batch_size=0,
         max_await_time_ms=None,
         session=None,
@@ -328,7 +321,6 @@ class RawBatchCommandCursor(CommandCursor):
             collection,
             cursor_info,
             address,
-            retrieved,
             batch_size,
             max_await_time_ms,
             session,
