@@ -17,11 +17,12 @@
 
 import datetime
 import warnings
+from collections import OrderedDict, abc
+from urllib.parse import unquote_plus
 
 from bson import SON
 from bson.binary import UuidRepresentation
 from bson.codec_options import CodecOptions, TypeRegistry
-from bson.py3compat import PY3, abc, integer_types, iteritems, string_type
 from bson.raw_bson import RawBSONDocument
 from pymongo.auth import MECHANISMS
 from pymongo.compression_support import (
@@ -29,43 +30,32 @@ from pymongo.compression_support import (
     validate_zlib_compression_level,
 )
 from pymongo.driver_info import DriverInfo
-from pymongo.encryption_options import validate_auto_encryption_opts_or_none
 from pymongo.errors import ConfigurationError
 from pymongo.monitoring import _validate_event_listeners
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import _MONGOS_MODES, _ServerMode
 from pymongo.server_api import ServerApi
-from pymongo.ssl_support import validate_allow_invalid_certs, validate_cert_reqs
 from pymongo.write_concern import DEFAULT_WRITE_CONCERN, WriteConcern
 
-try:
-    from collections import OrderedDict
-
-    ORDERED_TYPES = (SON, OrderedDict)
-except ImportError:
-    ORDERED_TYPES = (SON,)
-
-if PY3:
-    from urllib.parse import unquote_plus
-else:
-    from urllib import unquote_plus
+ORDERED_TYPES = (SON, OrderedDict)
 
 # Defaults until we connect to a server and get updated limits.
-MAX_BSON_SIZE = 16 * (1024 ** 2)
+MAX_BSON_SIZE = 16 * (1024**2)
 MAX_MESSAGE_SIZE = 2 * MAX_BSON_SIZE
 MIN_WIRE_VERSION = 0
 MAX_WIRE_VERSION = 0
 MAX_WRITE_BATCH_SIZE = 1000
 
 # What this version of PyMongo supports.
-MIN_SUPPORTED_SERVER_VERSION = "2.6"
-MIN_SUPPORTED_WIRE_VERSION = 2
-MAX_SUPPORTED_WIRE_VERSION = 13
+MIN_SUPPORTED_SERVER_VERSION = "3.6"
+MIN_SUPPORTED_WIRE_VERSION = 6
+MAX_SUPPORTED_WIRE_VERSION = 14
 
 # Frequency to call hello on servers, in seconds.
 HEARTBEAT_FREQUENCY = 10
 
-# Frequency to process kill-cursors, in seconds. See MongoClient.close_cursor.
+# Frequency to clean up unclosed cursors, in seconds.
+# See MongoClient._process_kill_cursors.
 KILL_CURSOR_FREQUENCY = 1
 
 # Frequency to process events queue, in seconds.
@@ -92,6 +82,9 @@ MAX_POOL_SIZE = 100
 # Default value for minPoolSize.
 MIN_POOL_SIZE = 0
 
+# The maximum number of concurrent connection creation attempts per pool.
+MAX_CONNECTING = 2
+
 # Default value for maxIdleTimeMS.
 MAX_IDLE_TIME_MS = None
 
@@ -120,6 +113,9 @@ UNAUTHORIZED_CODES = (13, 16547, 16548)
 # From the driver sessions spec.
 _MAX_END_SESSIONS = 10000
 
+# Default value for srvServiceName
+SRV_SERVICE_NAME = "mongodb"
+
 
 def partition_node(node):
     """Split a host:port string into (host, int(port)) pair."""
@@ -168,7 +164,7 @@ def validate_boolean(option, value):
 
 def validate_boolean_or_string(option, value):
     """Validates that value is True, False, 'true', or 'false'."""
-    if isinstance(value, string_type):
+    if isinstance(value, str):
         if value not in ("true", "false"):
             raise ValueError("The value of %s must be " "'true' or 'false'" % (option,))
         return value == "true"
@@ -177,9 +173,9 @@ def validate_boolean_or_string(option, value):
 
 def validate_integer(option, value):
     """Validates that 'value' is an integer (or basestring representation)."""
-    if isinstance(value, integer_types):
+    if isinstance(value, int):
         return value
-    elif isinstance(value, string_type):
+    elif isinstance(value, str):
         try:
             return int(value)
         except ValueError:
@@ -229,14 +225,10 @@ def validate_non_negative_integer_or_none(option, value):
 
 
 def validate_string(option, value):
-    """Validates that 'value' is an instance of `basestring` for Python 2
-    or `str` for Python 3.
-    """
-    if isinstance(value, string_type):
+    """Validates that 'value' is an instance of `str`."""
+    if isinstance(value, str):
         return value
-    raise TypeError(
-        "Wrong type for %s, value must be " "an instance of %s" % (option, string_type.__name__)
-    )
+    raise TypeError("Wrong type for %s, value must be an instance of " "str" % (option,))
 
 
 def validate_string_or_none(option, value):
@@ -248,9 +240,9 @@ def validate_string_or_none(option, value):
 
 def validate_int_or_basestring(option, value):
     """Validates that 'value' is an integer or string."""
-    if isinstance(value, integer_types):
+    if isinstance(value, int):
         return value
-    elif isinstance(value, string_type):
+    elif isinstance(value, str):
         try:
             return int(value)
         except ValueError:
@@ -260,9 +252,9 @@ def validate_int_or_basestring(option, value):
 
 def validate_non_negative_int_or_basestring(option, value):
     """Validates that 'value' is an integer or string."""
-    if isinstance(value, integer_types):
+    if isinstance(value, int):
         return value
-    elif isinstance(value, string_type):
+    elif isinstance(value, str):
         try:
             val = int(value)
         except ValueError:
@@ -349,7 +341,7 @@ def validate_read_preference(dummy, value):
 
 
 def validate_read_preference_mode(dummy, value):
-    """Validate read preference mode for a MongoReplicaSetClient.
+    """Validate read preference mode for a MongoClient.
 
     .. versionchanged:: 3.5
        Returns the original ``value`` instead of the validated read preference
@@ -362,10 +354,7 @@ def validate_read_preference_mode(dummy, value):
 
 def validate_auth_mechanism(option, value):
     """Validate the authMechanism URI option."""
-    # CRAM-MD5 is for server testing only. Undocumented,
-    # unsupported, may be removed at any time. You have
-    # been warned.
-    if value not in MECHANISMS and value != "CRAM-MD5":
+    if value not in MECHANISMS:
         raise ValueError("%s must be in %s" % (option, tuple(MECHANISMS)))
     return value
 
@@ -586,19 +575,22 @@ def validate_tzinfo(dummy, value):
     return value
 
 
+def validate_auto_encryption_opts_or_none(option, value):
+    """Validate the driver keyword arg."""
+    if value is None:
+        return value
+    from pymongo.encryption_options import AutoEncryptionOpts
+
+    if not isinstance(value, AutoEncryptionOpts):
+        raise TypeError("%s must be an instance of AutoEncryptionOpts" % (option,))
+
+    return value
+
+
 # Dictionary where keys are the names of public URI options, and values
-# are lists of aliases for that option. Aliases of option names are assumed
-# to have been deprecated.
+# are lists of aliases for that option.
 URI_OPTIONS_ALIAS_MAP = {
-    "journal": ["j"],
-    "wtimeoutms": ["wtimeout"],
     "tls": ["ssl"],
-    "tlsallowinvalidcertificates": ["ssl_cert_reqs"],
-    "tlsallowinvalidhostnames": ["ssl_match_hostname"],
-    "tlscrlfile": ["ssl_crlfile"],
-    "tlscafile": ["ssl_ca_certs"],
-    "tlscertificatekeyfile": ["ssl_certfile"],
-    "tlscertificatekeyfilepassword": ["ssl_pem_passphrase"],
 }
 
 # Dictionary where keys are the names of URI options, and values
@@ -617,7 +609,8 @@ URI_OPTIONS_VALIDATOR_MAP = {
     "journal": validate_boolean_or_string,
     "localthresholdms": validate_positive_float_or_zero,
     "maxidletimems": validate_timeout_or_none,
-    "maxpoolsize": validate_positive_integer_or_none,
+    "maxconnecting": validate_positive_integer,
+    "maxpoolsize": validate_non_negative_integer_or_none,
     "maxstalenessseconds": validate_max_staleness,
     "readconcernlevel": validate_string_or_none,
     "readpreference": validate_read_preference_mode,
@@ -628,22 +621,19 @@ URI_OPTIONS_VALIDATOR_MAP = {
     "loadbalanced": validate_boolean_or_string,
     "serverselectiontimeoutms": validate_timeout_or_zero,
     "sockettimeoutms": validate_timeout_or_none_or_zero,
-    "ssl_keyfile": validate_readable,
     "tls": validate_boolean_or_string,
-    "tlsallowinvalidcertificates": validate_allow_invalid_certs,
-    "ssl_cert_reqs": validate_cert_reqs,
-    # Normalized to ssl_match_hostname which is the logical inverse of tlsallowinvalidhostnames
-    "tlsallowinvalidhostnames": lambda *x: not validate_boolean_or_string(*x),
-    "ssl_match_hostname": validate_boolean_or_string,
+    "tlsallowinvalidcertificates": validate_boolean_or_string,
+    "tlsallowinvalidhostnames": validate_boolean_or_string,
     "tlscafile": validate_readable,
     "tlscertificatekeyfile": validate_readable,
     "tlscertificatekeyfilepassword": validate_string_or_none,
-    # Normalized to ssl_check_ocsp_endpoint which is the logical inverse of tlsdisableocspendpointcheck
-    "tlsdisableocspendpointcheck": lambda *x: not validate_boolean_or_string(*x),
+    "tlsdisableocspendpointcheck": validate_boolean_or_string,
     "tlsinsecure": validate_boolean_or_string,
     "w": validate_non_negative_int_or_basestring,
     "wtimeoutms": validate_non_negative_integer,
     "zlibcompressionlevel": validate_zlib_compression_level,
+    "srvservicename": validate_string,
+    "srvmaxhosts": validate_non_negative_integer,
 }
 
 # Dictionary where keys are the names of URI options specific to pymongo,
@@ -654,7 +644,6 @@ NONSPEC_OPTIONS_VALIDATOR_MAP = {
     "server_api": validate_server_api_or_none,
     "fsync": validate_boolean_or_string,
     "minpoolsize": validate_non_negative_integer,
-    "socketkeepalive": validate_boolean_or_string,
     "tlscrlfile": validate_readable,
     "tz_aware": validate_boolean_or_string,
     "unicode_decode_error_handler": validate_unicode_decode_error_handler,
@@ -683,16 +672,7 @@ KW_VALIDATORS = {
 # variant need not be included here. Options whose public and internal
 # names are the same need not be included here.
 INTERNAL_URI_OPTION_NAME_MAP = {
-    "j": "journal",
-    "wtimeout": "wtimeoutms",
-    "tls": "ssl",
-    "tlsallowinvalidcertificates": "ssl_cert_reqs",
-    "tlsallowinvalidhostnames": "ssl_match_hostname",
-    "tlscrlfile": "ssl_crlfile",
-    "tlscafile": "ssl_ca_certs",
-    "tlscertificatekeyfile": "ssl_certfile",
-    "tlscertificatekeyfilepassword": "ssl_pem_passphrase",
-    "tlsdisableocspendpointcheck": "ssl_check_ocsp_endpoint",
+    "ssl": "tls",
 }
 
 # Map from deprecated URI option names to a tuple indicating the method of
@@ -705,41 +685,13 @@ URI_OPTIONS_DEPRECATION_MAP = {
     #   preserved for renamed options as they are part of user warnings.
     # - 'removed': <message> may suggest the rationale for deprecating the
     #   option and/or recommend remedial action.
-    "j": ("renamed", "journal"),
-    "wtimeout": ("renamed", "wTimeoutMS"),
-    "ssl_cert_reqs": ("renamed", "tlsAllowInvalidCertificates"),
-    "ssl_match_hostname": ("renamed", "tlsAllowInvalidHostnames"),
-    "ssl_crlfile": ("renamed", "tlsCRLFile"),
-    "ssl_ca_certs": ("renamed", "tlsCAFile"),
-    "ssl_certfile": (
-        "removed",
-        (
-            "Instead of using ssl_certfile to specify the certificate file, "
-            "use tlsCertificateKeyFile to pass a single file containing both "
-            "the client certificate and the private key"
-        ),
-    ),
-    "ssl_keyfile": (
-        "removed",
-        (
-            "Instead of using ssl_keyfile to specify the private keyfile, "
-            "use tlsCertificateKeyFile to pass a single file containing both "
-            "the client certificate and the private key"
-        ),
-    ),
-    "ssl_pem_passphrase": ("renamed", "tlsCertificateKeyFilePassword"),
-    "waitqueuemultiple": (
-        "removed",
-        (
-            "Instead of using waitQueueMultiple to bound queuing, limit the size "
-            "of the thread pool in your application server"
-        ),
-    ),
+    # For example:
+    # 'wtimeout': ('renamed', 'wTimeoutMS'),
 }
 
 # Augment the option validator map with pymongo-specific option information.
 URI_OPTIONS_VALIDATOR_MAP.update(NONSPEC_OPTIONS_VALIDATOR_MAP)
-for optname, aliases in iteritems(URI_OPTIONS_ALIAS_MAP):
+for optname, aliases in URI_OPTIONS_ALIAS_MAP.items():
     for alias in aliases:
         if alias not in URI_OPTIONS_VALIDATOR_MAP:
             URI_OPTIONS_VALIDATOR_MAP[alias] = URI_OPTIONS_VALIDATOR_MAP[optname]
@@ -798,7 +750,7 @@ def get_validated_options(options, warn=True):
         get_normed_key = lambda x: x.lower()
         get_setter_key = lambda x: x
 
-    for opt, value in iteritems(options):
+    for opt, value in options.items():
         normed_key = get_normed_key(opt)
         try:
             validator = URI_OPTIONS_VALIDATOR_MAP.get(normed_key, raise_config_error)
