@@ -21,7 +21,6 @@ from collections import deque
 
 from bson import RE_TYPE, _convert_raw_document_lists_to_streams
 from bson.code import Code
-from bson.py3compat import integer_types, iteritems, string_type
 from bson.son import SON
 from pymongo import helpers
 from pymongo.collation import validate_collation_or_none
@@ -145,9 +144,7 @@ class Cursor(object):
         sort=None,
         allow_partial_results=False,
         oplog_replay=False,
-        modifiers=None,
         batch_size=0,
-        manipulate=True,
         collation=None,
         hint=None,
         max_scan=None,
@@ -213,12 +210,7 @@ class Cursor(object):
             raise ValueError("not a valid value for cursor_type")
         validate_boolean("allow_partial_results", allow_partial_results)
         validate_boolean("oplog_replay", oplog_replay)
-        if modifiers is not None:
-            warnings.warn(
-                "the 'modifiers' parameter is deprecated", DeprecationWarning, stacklevel=2
-            )
-            validate_is_mapping("modifiers", modifiers)
-        if not isinstance(batch_size, integer_types):
+        if not isinstance(batch_size, int):
             raise TypeError("batch_size must be an integer")
         if batch_size < 0:
             raise ValueError("batch_size must be >= 0")
@@ -227,8 +219,6 @@ class Cursor(object):
             allow_disk_use = validate_boolean("allow_disk_use", allow_disk_use)
 
         if projection is not None:
-            if not projection:
-                projection = {"_id": 1}
             projection = helpers._fields_list_to_dict(projection, "projection")
 
         self.__spec = spec
@@ -236,7 +226,6 @@ class Cursor(object):
         self.__skip = skip
         self.__limit = limit
         self.__batch_size = batch_size
-        self.__modifiers = modifiers and modifiers.copy() or {}
         self.__ordering = sort and helpers._index_document(sort) or None
         self.__max_scan = max_scan
         self.__explain = False
@@ -245,7 +234,6 @@ class Cursor(object):
         self.__max_await_time_ms = None
         self.__max = max
         self.__min = min
-        self.__manipulate = manipulate
         self.__collation = validate_collation_or_none(collation)
         self.__return_key = return_key
         self.__show_record_id = show_record_id
@@ -356,9 +344,7 @@ class Cursor(object):
             "hint",
             "batch_size",
             "max_scan",
-            "manipulate",
             "query_flags",
-            "modifiers",
             "collation",
             "empty",
             "show_record_id",
@@ -369,7 +355,7 @@ class Cursor(object):
         )
         data = dict(
             (k, v)
-            for k, v in iteritems(self.__dict__)
+            for k, v in self.__dict__.items()
             if k.startswith("_Cursor__") and k[9:] in values_to_clone
         )
         if deepcopy:
@@ -415,7 +401,7 @@ class Cursor(object):
 
     def __query_spec(self):
         """Get the spec to use for a query."""
-        operators = self.__modifiers.copy()
+        operators = {}
         if self.__ordering:
             operators["$orderby"] = self.__ordering
         if self.__explain:
@@ -544,7 +530,7 @@ class Cursor(object):
 
         .. seealso:: The MongoDB documentation on `limit <https://dochub.mongodb.org/core/limit>`_.
         """
-        if not isinstance(limit, integer_types):
+        if not isinstance(limit, int):
             raise TypeError("limit must be an integer")
         if self.__exhaust:
             raise InvalidOperation("Can't use limit and exhaust together.")
@@ -573,7 +559,7 @@ class Cursor(object):
         :Parameters:
           - `batch_size`: The size of each batch of results requested.
         """
-        if not isinstance(batch_size, integer_types):
+        if not isinstance(batch_size, int):
             raise TypeError("batch_size must be an integer")
         if batch_size < 0:
             raise ValueError("batch_size must be >= 0")
@@ -594,7 +580,7 @@ class Cursor(object):
         :Parameters:
           - `skip`: the number of results to skip
         """
-        if not isinstance(skip, integer_types):
+        if not isinstance(skip, int):
             raise TypeError("skip must be an integer")
         if skip < 0:
             raise ValueError("skip must be >= 0")
@@ -616,7 +602,7 @@ class Cursor(object):
         :Parameters:
           - `max_time_ms`: the time limit after which the operation is aborted
         """
-        if not isinstance(max_time_ms, integer_types) and max_time_ms is not None:
+        if not isinstance(max_time_ms, int) and max_time_ms is not None:
             raise TypeError("max_time_ms must be an integer or None")
         self.__check_okay_to_chain()
 
@@ -640,7 +626,7 @@ class Cursor(object):
 
         .. versionadded:: 3.2
         """
-        if not isinstance(max_await_time_ms, integer_types) and max_await_time_ms is not None:
+        if not isinstance(max_await_time_ms, int) and max_await_time_ms is not None:
             raise TypeError("max_await_time_ms must be an integer or None")
         self.__check_okay_to_chain()
 
@@ -718,7 +704,7 @@ class Cursor(object):
             self.__limit = limit
             return self
 
-        if isinstance(index, integer_types):
+        if isinstance(index, int):
             if index < 0:
                 raise IndexError("Cursor instances do not support negative " "indices")
             clone = self.clone()
@@ -843,66 +829,6 @@ class Cursor(object):
         self.__ordering = helpers._index_document(keys)
         return self
 
-    def count(self, with_limit_and_skip=False):
-        """**DEPRECATED** - Get the size of the results set for this query.
-
-        The :meth:`count` method is deprecated and **not** supported in a
-        transaction. Please use
-        :meth:`~pymongo.collection.Collection.count_documents` instead.
-
-        Returns the number of documents in the results set for this query. Does
-        not take :meth:`limit` and :meth:`skip` into account by default - set
-        `with_limit_and_skip` to ``True`` if that is the desired behavior.
-        Raises :class:`~pymongo.errors.OperationFailure` on a database error.
-
-        When used with MongoDB >= 2.6, :meth:`~count` uses any :meth:`~hint`
-        applied to the query. In the following example the hint is passed to
-        the count command:
-
-          collection.find({'field': 'value'}).hint('field_1').count()
-
-        The :meth:`count` method obeys the
-        :attr:`~pymongo.collection.Collection.read_preference` of the
-        :class:`~pymongo.collection.Collection` instance on which
-        :meth:`~pymongo.collection.Collection.find` was called.
-
-        :Parameters:
-          - `with_limit_and_skip` (optional): take any :meth:`limit` or
-            :meth:`skip` that has been applied to this cursor into account when
-            getting the count
-
-        .. note:: The `with_limit_and_skip` parameter requires server
-           version **>= 1.1.4-**
-
-        .. versionchanged:: 3.7
-           Deprecated.
-
-        .. versionchanged:: 2.8
-           The :meth:`~count` method now supports :meth:`~hint`.
-        """
-        warnings.warn(
-            "count is deprecated. Use Collection.count_documents " "instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        validate_boolean("with_limit_and_skip", with_limit_and_skip)
-        cmd = SON([("count", self.__collection.name), ("query", self.__spec)])
-        if self.__max_time_ms is not None:
-            cmd["maxTimeMS"] = self.__max_time_ms
-        if self.__comment:
-            cmd["comment"] = self.__comment
-
-        if self.__hint is not None:
-            cmd["hint"] = self.__hint
-
-        if with_limit_and_skip:
-            if self.__limit:
-                cmd["limit"] = self.__limit
-            if self.__skip:
-                cmd["skip"] = self.__skip
-
-        return self.__collection._count(cmd, self.__collation, session=self.__session)
-
     def distinct(self, key):
         """Get a list of distinct values for `key` among all documents
         in the result set of this query.
@@ -957,7 +883,7 @@ class Cursor(object):
             self.__hint = None
             return
 
-        if isinstance(index, string_type):
+        if isinstance(index, str):
             self.__hint = index
         else:
             self.__hint = helpers._index_document(index)
@@ -982,9 +908,6 @@ class Cursor(object):
 
         :Parameters:
           - `index`: index to hint on (as an index specifier)
-
-        .. versionchanged:: 2.8
-           The :meth:`~hint` method accepts the name of the index.
         """
         self.__check_okay_to_chain()
         self.__set_hint(index)
@@ -1166,13 +1089,9 @@ class Cursor(object):
 
         if self.__id is None:  # Query
             if (self.__min or self.__max) and not self.__hint:
-                warnings.warn(
-                    "using a min/max query operator without "
-                    "specifying a Cursor.hint is deprecated. A "
-                    "hint will be required when using min/max in "
-                    "PyMongo 4.0",
-                    DeprecationWarning,
-                    stacklevel=3,
+                raise InvalidOperation(
+                    "Passing a 'hint' is required when using the min/max query"
+                    " option to ensure the query utilizes the correct index"
                 )
             q = self._query_class(
                 self.__query_flags,
@@ -1244,10 +1163,6 @@ class Cursor(object):
     def cursor_id(self):
         """Returns the id of the cursor
 
-        Useful if you need to manage cursor ids and want to handle killing
-        cursors manually using
-        :meth:`~pymongo.mongo_client.MongoClient.kill_cursors`
-
         .. versionadded:: 2.2
         """
         return self.__id
@@ -1278,11 +1193,7 @@ class Cursor(object):
         if self.__empty:
             raise StopIteration
         if len(self.__data) or self._refresh():
-            if self.__manipulate:
-                _db = self.__collection.database
-                return _db._fix_outgoing(self.__data.popleft(), self.__collection)
-            else:
-                return self.__data.popleft()
+            return self.__data.popleft()
         else:
             raise StopIteration
 
@@ -1317,7 +1228,7 @@ class Cursor(object):
         if not hasattr(x, "items"):
             y, is_list, iterator = [], True, enumerate(x)
         else:
-            y, is_list, iterator = {}, False, iteritems(x)
+            y, is_list, iterator = {}, False, x.items()
 
         if memo is None:
             memo = {}
@@ -1356,14 +1267,8 @@ class RawBatchCursor(Cursor):
 
         .. seealso:: The MongoDB documentation on `cursors <https://dochub.mongodb.org/core/cursors>`_.
         """
-        manipulate = kwargs.get("manipulate")
-        kwargs["manipulate"] = False
         super(RawBatchCursor, self).__init__(*args, **kwargs)
 
-        # Throw only after cursor's initialized, to prevent errors in __del__.
-        if manipulate:
-            raise InvalidOperation("Cannot use RawBatchCursor with manipulate=True")
-
     def _unpack_response(
         self, response, cursor_id, codec_options, user_fields=None, legacy_response=False
     ):
