@@ -24,6 +24,7 @@ except ImportError:
     _HAVE_PYMONGOCRYPT = False
 
 from pymongo.errors import ConfigurationError
+from pymongo.uri_parser import _parse_kms_tls_options
 
 
 class AutoEncryptionOpts(object):
@@ -40,6 +41,7 @@ class AutoEncryptionOpts(object):
         mongocryptd_bypass_spawn=False,
         mongocryptd_spawn_path="mongocryptd",
         mongocryptd_spawn_args=None,
+        kms_tls_options=None,
     ):
         """Options to configure automatic client-side field level encryption.
 
@@ -58,9 +60,8 @@ class AutoEncryptionOpts(object):
         See :ref:`automatic-client-side-encryption` for an example.
 
         :Parameters:
-          - `kms_providers`: Map of KMS provider options. Two KMS providers
-            are supported: "aws" and "local". The kmsProviders map values
-            differ by provider:
+          - `kms_providers`: Map of KMS provider options. The `kms_providers`
+            map values differ by provider:
 
               - `aws`: Map with "accessKeyId" and "secretAccessKey" as strings.
                 These are the AWS access key ID and AWS secret access key used
@@ -72,12 +73,14 @@ class AutoEncryptionOpts(object):
                 These are the Azure Active Directory credentials used to
                 generate Azure Key Vault messages.
               - `gcp`: Map with "email" as a string and "privateKey"
-                as `bytes` or a base64 encoded string (unicode on Python 2).
+                as `bytes` or a base64 encoded string.
                 Additionally, "endpoint" may also be specified as a string
                 (defaults to 'oauth2.googleapis.com'). These are the
                 credentials used to generate Google Cloud KMS messages.
+              - `kmip`: Map with "endpoint" as a host with required port.
+                For example: ``{"endpoint": "example.com:443"}``.
               - `local`: Map with "key" as `bytes` (96 bytes in length) or
-                a base64 encoded string (unicode on Python 2) which decodes
+                a base64 encoded string which decodes
                 to 96 bytes. "key" is the master key used to encrypt/decrypt
                 data keys. This key should be generated and stored as securely
                 as possible.
@@ -123,6 +126,20 @@ class AutoEncryptionOpts(object):
             ``['--idleShutdownTimeoutSecs=60']``. If the list does not include
             the ``idleShutdownTimeoutSecs`` option then
             ``'--idleShutdownTimeoutSecs=60'`` will be added.
+          - `kms_tls_options` (optional):  A map of KMS provider names to TLS
+            options to use when creating secure connections to KMS providers.
+            Accepts the same TLS options as
+            :class:`pymongo.mongo_client.MongoClient`. For example, to
+            override the system default CA file::
+
+              kms_tls_options={'kmip': {'tlsCAFile': certifi.where()}}
+
+            Or to supply a client certificate::
+
+              kms_tls_options={'kmip': {'tlsCertificateKeyFile': 'client.pem'}}
+
+        .. versionchanged:: 4.0
+           Added the `kms_tls_options` parameter and the "kmip" KMS provider.
 
         .. versionadded:: 3.9
         """
@@ -148,13 +165,5 @@ class AutoEncryptionOpts(object):
             raise TypeError("mongocryptd_spawn_args must be a list")
         if not any("idleShutdownTimeoutSecs" in s for s in self._mongocryptd_spawn_args):
             self._mongocryptd_spawn_args.append("--idleShutdownTimeoutSecs=60")
-
-
-def validate_auto_encryption_opts_or_none(option, value):
-    """Validate the driver keyword arg."""
-    if value is None:
-        return value
-    if not isinstance(value, AutoEncryptionOpts):
-        raise TypeError("%s must be an instance of AutoEncryptionOpts" % (option,))
-
-    return value
+        # Maps KMS provider name to a SSLContext.
+        self._kms_ssl_contexts = _parse_kms_tls_options(kms_tls_options)
