@@ -16,8 +16,8 @@
 
 import sys
 import traceback
+from collections import abc
 
-from bson.py3compat import abc, iteritems, itervalues, string_type
 from bson.son import SON
 from pymongo import ASCENDING
 from pymongo.errors import (
@@ -30,7 +30,7 @@ from pymongo.errors import (
     WriteError,
     WTimeoutError,
 )
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import HelloCompat
 
 # From the SDAM spec, the "node is shutting down" codes.
 _SHUTDOWN_CODES = frozenset(
@@ -42,7 +42,7 @@ _SHUTDOWN_CODES = frozenset(
 # From the SDAM spec, the "not primary" error codes are combined with the
 # "node is recovering" error codes (of which the "node is shutting down"
 # errors are a subset).
-_NOT_MASTER_CODES = (
+_NOT_PRIMARY_CODES = (
     frozenset(
         [
             10058,  # LegacyNotPrimary <=3.2 "not primary" error code
@@ -56,7 +56,7 @@ _NOT_MASTER_CODES = (
     | _SHUTDOWN_CODES
 )
 # From the retryable writes spec.
-_RETRYABLE_ERROR_CODES = _NOT_MASTER_CODES | frozenset(
+_RETRYABLE_ERROR_CODES = _NOT_PRIMARY_CODES | frozenset(
     [
         7,  # HostNotFound
         6,  # HostUnreachable
@@ -65,12 +65,11 @@ _RETRYABLE_ERROR_CODES = _NOT_MASTER_CODES | frozenset(
         262,  # ExceededTimeLimit
     ]
 )
-_UUNDER = u"_"
 
 
 def _gen_index_name(keys):
     """Generate an index name from the set of fields it is over."""
-    return _UUNDER.join(["%s_%s" % item for item in keys])
+    return "_".join(["%s_%s" % item for item in keys])
 
 
 def _index_list(key_or_list, direction=None):
@@ -81,8 +80,10 @@ def _index_list(key_or_list, direction=None):
     if direction is not None:
         return [(key_or_list, direction)]
     else:
-        if isinstance(key_or_list, string_type):
+        if isinstance(key_or_list, str):
             return [(key_or_list, ASCENDING)]
+        if isinstance(key_or_list, abc.ItemsView):
+            return list(key_or_list)
         elif not isinstance(key_or_list, (list, tuple)):
             raise TypeError(
                 "if no direction is specified, " "key_or_list must be an instance of list"
@@ -99,7 +100,7 @@ def _index_document(index_list):
         raise TypeError(
             "passing a dict to sort/create_index/hint is not "
             "allowed - use a list of tuples instead. did you "
-            "mean %r?" % list(iteritems(index_list))
+            "mean %r?" % list(index_list.items())
         )
     elif not isinstance(index_list, (list, tuple)):
         raise TypeError("must use a list of (key, direction) pairs, " "not: " + repr(index_list))
@@ -108,9 +109,9 @@ def _index_document(index_list):
 
     index = SON()
     for (key, value) in index_list:
-        if not isinstance(key, string_type):
-            raise TypeError("first item in each key pair must be a string")
-        if not isinstance(value, (string_type, int, abc.Mapping)):
+        if not isinstance(key, str):
+            raise TypeError("first item in each key pair must be an instance of str")
+        if not isinstance(value, (str, int, abc.Mapping)):
             raise TypeError(
                 "second item in each key pair must be 1, -1, "
                 "'2d', or another valid MongoDB index specifier."
@@ -143,7 +144,7 @@ def _check_command_response(
     # Mongos returns the error details in a 'raw' object
     # for some errors.
     if "raw" in response:
-        for shard in itervalues(response["raw"]):
+        for shard in response["raw"].values():
             # Grab the first non-empty raw error from a shard.
             if shard.get("errmsg") and not shard.get("ok"):
                 details = shard
@@ -163,7 +164,7 @@ def _check_command_response(
 
     # Server is "not primary" or "recovering"
     if code is not None:
-        if code in _NOT_MASTER_CODES:
+        if code in _NOT_PRIMARY_CODES:
             raise NotPrimaryError(errmsg, response)
     elif HelloCompat.LEGACY_ERROR in errmsg or "node is recovering" in errmsg:
         raise NotPrimaryError(errmsg, response)
@@ -180,39 +181,6 @@ def _check_command_response(
     raise OperationFailure(errmsg, code, response, max_wire_version)
 
 
-def _check_gle_response(result, max_wire_version):
-    """Return getlasterror response as a dict, or raise OperationFailure."""
-    # Did getlasterror itself fail?
-    _check_command_response(result, max_wire_version)
-
-    if result.get("wtimeout", False):
-        # MongoDB versions before 1.8.0 return the error message in an "errmsg"
-        # field. If "errmsg" exists "err" will also exist set to None, so we
-        # have to check for "errmsg" first.
-        raise WTimeoutError(result.get("errmsg", result.get("err")), result.get("code"), result)
-
-    error_msg = result.get("err", "")
-    if error_msg is None:
-        return result
-
-    if error_msg.startswith(HelloCompat.LEGACY_ERROR):
-        raise NotPrimaryError(error_msg, result)
-
-    details = result
-
-    # mongos returns the error code in an error object for some errors.
-    if "errObjects" in result:
-        for errobj in result["errObjects"]:
-            if errobj.get("err") == error_msg:
-                details = errobj
-                break
-
-    code = details.get("code")
-    if code in (11000, 11001, 12582):
-        raise DuplicateKeyError(details["err"], code, result)
-    raise OperationFailure(details["err"], code, result)
-
-
 def _raise_last_write_error(write_errors):
     # If the last batch had multiple errors only report
     # the last error to emulate continue_on_error.
@@ -253,16 +221,6 @@ def _check_write_command_response(result):
         _raise_write_concern_error(wce)
 
 
-def _raise_last_error(bulk_write_result):
-    """Backward compatibility helper for insert error handling."""
-    # Prefer write errors over write concern errors
-    write_errors = bulk_write_result.get("writeErrors")
-    if write_errors:
-        _raise_last_write_error(write_errors)
-
-    _raise_write_concern_error(bulk_write_result["writeConcernErrors"][-1])
-
-
 def _fields_list_to_dict(fields, option_name):
     """Takes a sequence of field names and returns a matching dictionary.
 
@@ -276,10 +234,9 @@ def _fields_list_to_dict(fields, option_name):
         return fields
 
     if isinstance(fields, (abc.Sequence, abc.Set)):
-        if not all(isinstance(field, string_type) for field in fields):
+        if not all(isinstance(field, str) for field in fields):
             raise TypeError(
-                "%s must be a list of key names, each an "
-                "instance of %s" % (option_name, string_type.__name__)
+                "%s must be a list of key names, each an " "instance of str" % (option_name,)
             )
         return dict.fromkeys(fields, 1)
 
