@@ -180,10 +180,9 @@ will not add that listener to existing client instances.
   handler first.
 """
 
-from collections import namedtuple
+from collections import abc, namedtuple
 
-from bson.py3compat import abc
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import HelloCompat
 from pymongo.helpers import _handle_exception
 
 _Listeners = namedtuple(
@@ -262,6 +261,18 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
+    def pool_ready(self, event):
+        """Abstract method to handle a :class:`PoolReadyEvent`.
+
+        Emitted when a Connection Pool is marked ready.
+
+        :Parameters:
+          - `event`: An instance of :class:`PoolReadyEvent`.
+
+        .. versionadded:: 4.0
+        """
+        raise NotImplementedError
+
     def pool_cleared(self, event):
         """Abstract method to handle a `PoolClearedEvent`.
 
@@ -587,16 +598,12 @@ class CommandStartedEvent(_CommandEvent):
 
     __slots__ = ("__cmd", "__db")
 
-    def __init__(
-        self, command, database_name, request_id, connection_id, operation_id, service_id=None
-    ):
+    def __init__(self, command, database_name, *args, service_id=None):
         if not command:
             raise ValueError("%r is not a valid command" % (command,))
         # Command name must be first key.
         command_name = next(iter(command))
-        super(CommandStartedEvent, self).__init__(
-            command_name, request_id, connection_id, operation_id, service_id=service_id
-        )
+        super(CommandStartedEvent, self).__init__(command_name, *args, service_id=service_id)
         cmd_name, cmd_doc = command_name.lower(), command[command_name]
         if cmd_name in _SENSITIVE_COMMANDS or _is_speculative_authenticate(cmd_name, command):
             self.__cmd = {}
@@ -698,19 +705,8 @@ class CommandFailedEvent(_CommandEvent):
 
     __slots__ = ("__duration_micros", "__failure")
 
-    def __init__(
-        self,
-        duration,
-        failure,
-        command_name,
-        request_id,
-        connection_id,
-        operation_id,
-        service_id=None,
-    ):
-        super(CommandFailedEvent, self).__init__(
-            command_name, request_id, connection_id, operation_id, service_id=service_id
-        )
+    def __init__(self, duration, failure, *args, service_id=None):
+        super(CommandFailedEvent, self).__init__(*args, service_id=service_id)
         self.__duration_micros = _to_micros(duration)
         self.__failure = failure
 
@@ -783,6 +779,19 @@ class PoolCreatedEvent(_PoolEvent):
         return "%s(%r, %r)" % (self.__class__.__name__, self.address, self.__options)
 
 
+class PoolReadyEvent(_PoolEvent):
+    """Published when a Connection Pool is marked ready.
+
+    :Parameters:
+     - `address`: The address (host, port) pair of the server this Pool is
+       attempting to connect to.
+
+    .. versionadded:: 4.0
+    """
+
+    __slots__ = ()
+
+
 class PoolClearedEvent(_PoolEvent):
     """Published when a Connection Pool is cleared.
 
@@ -1246,12 +1255,7 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
 
     @property
     def reply(self):
-        """An instance of :class:`~pymongo.ismaster.IsMaster`.
-
-        .. warning:: :class:`~pymongo.ismaster.IsMaster` is deprecated.
-          Starting with PyMongo 4.0 this attribute will return an instance
-          of :class:`~pymongo.hello.Hello`, which provides the same API.
-        """
+        """An instance of :class:`~pymongo.hello.Hello`."""
         return self.__reply
 
     @property
@@ -1381,10 +1385,11 @@ class _EventListeners(object):
     def event_listeners(self):
         """List of registered event listeners."""
         return (
-            self.__command_listeners +
-            self.__server_heartbeat_listeners +
-            self.__server_listeners +
-            self.__topology_listeners
+            self.__command_listeners
+            + self.__server_heartbeat_listeners
+            + self.__server_listeners
+            + self.__topology_listeners
+            + self.__cmap_listeners
         )
 
     def publish_command_start(
@@ -1642,6 +1647,15 @@ class _EventListeners(object):
             except Exception:
                 _handle_exception()
 
+    def publish_pool_ready(self, address):
+        """Publish a :class:`PoolReadyEvent` to all pool listeners."""
+        event = PoolReadyEvent(address)
+        for subscriber in self.__cmap_listeners:
+            try:
+                subscriber.pool_ready(event)
+            except Exception:
+                _handle_exception()
+
     def publish_pool_cleared(self, address, service_id):
         """Publish a :class:`PoolClearedEvent` to all pool listeners."""
         event = PoolClearedEvent(address, service_id)
