@@ -14,7 +14,6 @@
 
 """Operation class definitions."""
 
-from bson.py3compat import string_type
 from pymongo import helpers
 from pymongo.collation import validate_collation_or_none
 from pymongo.common import validate_boolean, validate_is_mapping, validate_list
@@ -83,7 +82,7 @@ class DeleteOne(object):
         if filter is not None:
             validate_is_mapping("filter", filter)
         if hint is not None:
-            if not isinstance(hint, string_type):
+            if not isinstance(hint, str):
                 hint = helpers._index_document(hint)
         self._filter = filter
         self._collation = collation
@@ -135,7 +134,7 @@ class DeleteMany(object):
         if filter is not None:
             validate_is_mapping("filter", filter)
         if hint is not None:
-            if not isinstance(hint, string_type):
+            if not isinstance(hint, str):
                 hint = helpers._index_document(hint)
         self._filter = filter
         self._collation = collation
@@ -192,7 +191,7 @@ class ReplaceOne(object):
         if upsert is not None:
             validate_boolean("upsert", upsert)
         if hint is not None:
-            if not isinstance(hint, string_type):
+            if not isinstance(hint, str):
                 hint = helpers._index_document(hint)
 
         self._filter = filter
@@ -245,7 +244,7 @@ class _UpdateOp(object):
         if array_filters is not None:
             validate_list("array_filters", array_filters)
         if hint is not None:
-            if not isinstance(hint, string_type):
+            if not isinstance(hint, str):
                 hint = helpers._index_document(hint)
 
         self._filter = filter
@@ -405,9 +404,8 @@ class IndexModel(object):
         The key(s) must be an instance of :class:`basestring`
         (:class:`str` in python 3), and the direction(s) must be one of
         (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`,
-        :data:`~pymongo.GEO2D`, :data:`~pymongo.GEOHAYSTACK`,
-        :data:`~pymongo.GEOSPHERE`, :data:`~pymongo.HASHED`,
-        :data:`~pymongo.TEXT`).
+        :data:`~pymongo.GEO2D`, :data:`~pymongo.GEOSPHERE`,
+        :data:`~pymongo.HASHED`, :data:`~pymongo.TEXT`).
 
         Valid options include, but are not limited to:
 
@@ -456,7 +454,7 @@ class IndexModel(object):
            Added the ``partialFilterExpression`` option to support partial
            indexes.
 
-        .. _wildcard index: https://docs.mongodb.com/master/core/index-wildcard/
+        .. _wildcard index: https://docs.mongodb.com/master/core/index-wildcard/#wildcard-index-core
         """
         keys = _index_list(keys)
         if "name" not in kwargs:
