@@ -14,7 +14,8 @@
 
 """Utilities for choosing which member of a replica set to read from."""
 
-from bson.py3compat import abc, integer_types
+from collections import abc
+
 from pymongo import max_staleness_selectors
 from pymongo.errors import ConfigurationError
 from pymongo.server_selectors import (
@@ -39,7 +40,7 @@ _MONGOS_MODES = (
 
 
 def _validate_tag_sets(tag_sets):
-    """Validate tag sets for a MongoReplicaSetClient."""
+    """Validate tag sets for a MongoClient."""
     if tag_sets is None:
         return tag_sets
 
@@ -72,7 +73,7 @@ def _validate_max_staleness(max_staleness):
     if max_staleness == -1:
         return -1
 
-    if not isinstance(max_staleness, integer_types):
+    if not isinstance(max_staleness, int):
         raise TypeError(_invalid_max_staleness_msg(max_staleness))
 
     if max_staleness <= 0:
@@ -138,7 +139,7 @@ class _ServerMode(object):
         To specify a priority-order for tag sets, provide a list of
         tag sets: ``[{'dc': 'ny'}, {'dc': 'la'}, {}]``. A final, empty tag
         set, ``{}``, means "read from any member that matches the mode,
-        ignoring tags." MongoReplicaSetClient tries each set of tags in turn
+        ignoring tags." MongoClient tries each set of tags in turn
         until it finds a set of tags with at least one matching member.
 
            .. seealso:: `Data-Center Awareness
