@@ -66,7 +66,7 @@ class Server(object):
                 )
             )
         self._monitor.close()
-        self._pool.reset()
+        self._pool.reset_without_pause()
 
     def request_check(self):
         """Check the server's state soon."""
