@@ -14,9 +14,10 @@
 
 """Represent one server the driver is connected to."""
 
+import time
+
 from bson import EPOCH_NAIVE
-from pymongo.ismaster import IsMaster
-from pymongo.monotonic import time as _time
+from pymongo.hello import Hello
 from pymongo.server_type import SERVER_TYPE
 
 
@@ -25,12 +26,9 @@ class ServerDescription(object):
 
     :Parameters:
       - `address`: A (host, port) pair
-      - `ismaster`: Optional Hello instance
+      - `hello`: Optional Hello instance
       - `round_trip_time`: Optional float
       - `error`: Optional, the last error attempting to connect to the server
-
-    .. warning:: The `ismaster` parameter will be renamed to `hello` in PyMongo
-      4.0.
     """
 
     __slots__ = (
@@ -59,9 +57,10 @@ class ServerDescription(object):
         "_topology_version",
     )
 
-    def __init__(self, address, ismaster=None, round_trip_time=None, error=None):
+    def __init__(self, address, hello=None, round_trip_time=None, error=None):
         self._address = address
-        hello = ismaster or IsMaster({})
+        if not hello:
+            hello = Hello({})
 
         self._server_type = hello.server_type
         self._all_hosts = hello.all_hosts
@@ -81,7 +80,7 @@ class ServerDescription(object):
         self._ls_timeout_minutes = hello.logical_session_timeout_minutes
         self._round_trip_time = round_trip_time
         self._me = hello.me
-        self._last_update_time = _time()
+        self._last_update_time = time.monotonic()
         self._error = error
         self._topology_version = hello.topology_version
         if error:
