@@ -15,32 +15,28 @@
 """Internal class to monitor a topology of one or more servers."""
 
 import os
+import queue
 import random
 import threading
+import time
 import warnings
 import weakref
 
-from bson.py3compat import PY3, itervalues
-
-if PY3:
-    import queue as Queue
-else:
-    import Queue
-
 from pymongo import common, helpers, periodic_executor
 from pymongo.client_session import _ServerSessionPool
 from pymongo.errors import (
     ConfigurationError,
     ConnectionFailure,
+    InvalidOperation,
     NetworkTimeout,
     NotPrimaryError,
     OperationFailure,
+    PyMongoError,
     ServerSelectionTimeoutError,
     WriteError,
 )
-from pymongo.ismaster import IsMaster
+from pymongo.hello import Hello
 from pymongo.monitor import SrvMonitor
-from pymongo.monotonic import time as _time
 from pymongo.pool import PoolOptions
 from pymongo.server import Server
 from pymongo.server_description import ServerDescription
@@ -69,7 +65,7 @@ def process_events_queue(queue_ref):
     while True:
         try:
             event = q.get_nowait()
-        except Queue.Empty:
+        except queue.Empty:
             break
         else:
             fn, args = event
@@ -93,7 +89,7 @@ class Topology(object):
         self.__events_executor = None
 
         if self._publish_server or self._publish_tp:
-            self._events = Queue.Queue(maxsize=100)
+            self._events = queue.Queue(maxsize=100)
 
         if self._publish_tp:
             self._events.put((self._listeners.publish_topology_opened, (self._topology_id,)))
@@ -126,6 +122,7 @@ class Topology(object):
         # Store the seed list to help diagnose errors in _error_message().
         self._seed_addresses = list(topology_description.server_descriptions())
         self._opened = False
+        self._closed = False
         self._lock = threading.Lock()
         self._condition = self._settings.condition_class(self._lock)
         self._servers = {}
@@ -140,7 +137,7 @@ class Topology(object):
 
             executor = periodic_executor.PeriodicExecutor(
                 interval=common.EVENTS_QUEUE_FREQUENCY,
-                min_interval=0.5,
+                min_interval=common.MIN_HEARTBEAT_INTERVAL,
                 target=target,
                 name="pymongo_events_thread",
             )
@@ -214,7 +211,7 @@ class Topology(object):
 
     def _select_servers_loop(self, selector, timeout, address):
         """select_servers() guts. Hold the lock when calling this."""
-        now = _time()
+        now = time.monotonic()
         end_time = now + timeout
         server_descriptions = self._description.apply_selector(
             selector, address, custom_selector=self._settings.server_selector
@@ -237,7 +234,7 @@ class Topology(object):
             # held the lock until now.
             self._condition.wait(common.MIN_HEARTBEAT_INTERVAL)
             self._description.check_compatible()
-            now = _time()
+            now = time.monotonic()
             server_descriptions = self._description.apply_selector(
                 selector, address, custom_selector=self._settings.server_selector
             )
@@ -247,7 +244,14 @@ class Topology(object):
 
     def select_server(self, selector, server_selection_timeout=None, address=None):
         """Like select_servers, but choose a random server if several match."""
-        return random.choice(self.select_servers(selector, server_selection_timeout, address))
+        servers = self.select_servers(selector, server_selection_timeout, address)
+        if len(servers) == 1:
+            return servers[0]
+        server1, server2 = random.sample(servers, 2)
+        if server1.pool.operation_count <= server2.pool.operation_count:
+            return server1
+        else:
+            return server2
 
     def select_server_by_address(self, address, server_selection_timeout=None):
         """Return a Server for "address", reconnecting if necessary.
@@ -280,6 +284,15 @@ class Topology(object):
             # This is a stale hello response. Ignore it.
             return
 
+        new_td = updated_topology_description(self._description, server_description)
+        # CMAP: Ensure the pool is "ready" when the server is selectable.
+        if server_description.is_readable or (
+            server_description.is_server_type_known and new_td.topology_type == TOPOLOGY_TYPE.Single
+        ):
+            server = self._servers.get(server_description.address)
+            if server:
+                server.pool.ready()
+
         suppress_event = (self._publish_server or self._publish_tp) and sd_old == server_description
         if self._publish_server and not suppress_event:
             self._events.put(
@@ -289,8 +302,7 @@ class Topology(object):
                 )
             )
 
-        self._description = updated_topology_description(self._description, server_description)
-
+        self._description = new_td
         self._update_servers()
         self._receive_cluster_time_no_lock(server_description.cluster_time)
 
@@ -321,7 +333,7 @@ class Topology(object):
         self._condition.notify_all()
 
     def on_change(self, server_description, reset_pool=False):
-        """Process a new ServerDescription after a hello call completes."""
+        """Process a new ServerDescription after an hello call completes."""
         # We do no I/O holding the lock.
         with self._lock:
             # Monitors may continue working on hello calls for some time
@@ -340,8 +352,6 @@ class Topology(object):
         Hold the lock when calling this.
         """
         td_old = self._description
-        if td_old.topology_type not in SRV_POLLING_TOPOLOGIES:
-            return
         self._description = _updated_topology_description_srv_polling(self._description, seedlist)
 
         self._update_servers()
@@ -434,15 +444,6 @@ class Topology(object):
             self._request_check_all()
             self._condition.wait(wait_time)
 
-    def handle_getlasterror(self, address, error_msg):
-        """Clear our pool for a server, mark it Unknown, and check it soon."""
-        error = NotPrimaryError(error_msg, {"code": 10107, "errmsg": error_msg})
-        with self._lock:
-            server = self._servers.get(address)
-            if server:
-                self._process_change(ServerDescription(address, error=error), True)
-                server.request_check()
-
     def data_bearing_servers(self):
         """Return a list of all data-bearing servers.
 
@@ -462,10 +463,17 @@ class Topology(object):
                 servers.append((server, server.pool.gen.get_overall()))
 
         for server, generation in servers:
-            server.pool.remove_stale_sockets(generation, all_credentials)
+            try:
+                server.pool.remove_stale_sockets(generation, all_credentials)
+            except PyMongoError as exc:
+                ctx = _ErrorContext(exc, 0, generation, False, None)
+                self.handle_error(server.description.address, ctx)
+                raise
 
     def close(self):
-        """Clear pools and terminate monitors. Topology reopens on demand."""
+        """Clear pools and terminate monitors. Topology does not reopen on
+        demand. Any further operations will raise
+        :exc:`~.errors.InvalidOperation`."""
         with self._lock:
             for server in self._servers.values():
                 server.close()
@@ -481,6 +489,7 @@ class Topology(object):
                 self._srv_monitor.close()
 
             self._opened = False
+            self._closed = True
 
         # Publish only after releasing the lock.
         if self._publish_tp:
@@ -550,6 +559,9 @@ class Topology(object):
 
         Hold the lock when calling this.
         """
+        if self._closed:
+            raise InvalidOperation("Cannot use MongoClient after close")
+
         if not self._opened:
             self._opened = True
             self._update_servers()
@@ -567,12 +579,12 @@ class Topology(object):
                 self._process_change(
                     ServerDescription(
                         self._seed_addresses[0],
-                        IsMaster({"ok": 1, "serviceId": self._topology_id, "maxWireVersion": 13}),
+                        Hello({"ok": 1, "serviceId": self._topology_id, "maxWireVersion": 13}),
                     )
                 )
 
         # Ensure that the monitors are open.
-        for server in itervalues(self._servers):
+        for server in self._servers.values():
             server.open()
 
     def _is_stale_error(self, address, err_ctx):
@@ -612,7 +624,7 @@ class Topology(object):
         elif issubclass(exc_type, WriteError):
             # Ignore writeErrors.
             return
-        elif issubclass(exc_type, NotPrimaryError):
+        elif issubclass(exc_type, (NotPrimaryError, OperationFailure)):
             # As per the SDAM spec if:
             #   - the server sees a "not primary" error, and
             #   - the server is not shutting down, and
@@ -621,15 +633,27 @@ class Topology(object):
             # as Unknown and request an immediate check of the server.
             # Otherwise, we clear the connection pool, mark the server as
             # Unknown and request an immediate check of the server.
-            err_code = error.details.get("code", -1)
-            is_shutting_down = err_code in helpers._SHUTDOWN_CODES
-            # Mark server Unknown, clear the pool, and request check.
-            if not self._settings.load_balanced:
-                self._process_change(ServerDescription(address, error=error))
-            if is_shutting_down or (err_ctx.max_wire_version <= 7):
+            if hasattr(error, "code"):
+                err_code = error.code
+            else:
+                # Default error code if one does not exist.
+                default = 10107 if isinstance(error, NotPrimaryError) else None
+                err_code = error.details.get("code", default)
+            if err_code in helpers._NOT_PRIMARY_CODES:
+                is_shutting_down = err_code in helpers._SHUTDOWN_CODES
+                # Mark server Unknown, clear the pool, and request check.
+                if not self._settings.load_balanced:
+                    self._process_change(ServerDescription(address, error=error))
+                if is_shutting_down or (err_ctx.max_wire_version <= 7):
+                    # Clear the pool.
+                    server.reset(service_id)
+                server.request_check()
+            elif not err_ctx.completed_handshake:
+                # Unknown command error during the connection handshake.
+                if not self._settings.load_balanced:
+                    self._process_change(ServerDescription(address, error=error))
                 # Clear the pool.
                 server.reset(service_id)
-            server.request_check()
         elif issubclass(exc_type, ConnectionFailure):
             # "Client MUST replace the server's description with type Unknown
             # ... MUST NOT request an immediate check of the server."
@@ -641,14 +665,6 @@ class Topology(object):
             # reading or writing`_, clients MUST cancel the hello check on
             # that server and close the current monitoring connection."
             server._monitor.cancel_check()
-        elif issubclass(exc_type, OperationFailure):
-            # Do not request an immediate check since the server is likely
-            # shutting down.
-            if error.code in helpers._NOT_MASTER_CODES:
-                if not self._settings.load_balanced:
-                    self._process_change(ServerDescription(address, error=error))
-                # Clear the pool.
-                server.reset(service_id)
 
     def handle_error(self, address, err_ctx):
         """Handle an application error.
@@ -719,10 +735,11 @@ class Topology(object):
             connect_timeout=options.connect_timeout,
             socket_timeout=options.connect_timeout,
             ssl_context=options._ssl_context,
-            ssl_match_hostname=options.ssl_match_hostname,
+            tls_allow_invalid_hostnames=options.tls_allow_invalid_hostnames,
             event_listeners=options._event_listeners,
             appname=options.appname,
             driver=options.driver,
+            pause_enabled=False,
             server_api=options.server_api,
         )
 
@@ -792,6 +809,19 @@ class Topology(object):
             msg = "CLOSED "
         return "<%s %s%r>" % (self.__class__.__name__, msg, self._description)
 
+    def eq_props(self):
+        """The properties to use for MongoClient/Topology equality checks."""
+        ts = self._settings
+        return (tuple(sorted(ts.seeds)), ts.replica_set_name, ts.fqdn, ts.srv_service_name)
+
+    def __eq__(self, other):
+        if isinstance(other, self.__class__):
+            return self.eq_props() == other.eq_props()
+        return NotImplemented
+
+    def __hash__(self):
+        return hash(self.eq_props())
+
 
 class _ErrorContext(object):
     """An error with context for SDAM error handling."""
