@@ -15,6 +15,7 @@
 """Represent a deployment of MongoDB servers."""
 
 from collections import namedtuple
+from random import sample
 
 from pymongo import common
 from pymongo.errors import ConfigurationError
@@ -76,7 +77,7 @@ class TopologyDescription(object):
             self._init_incompatible_err()
 
         # Server Discovery And Monitoring Spec: Whenever a client updates the
-        # TopologyDescription from a hello response, it MUST set
+        # TopologyDescription from an hello response, it MUST set
         # TopologyDescription.logicalSessionTimeoutMinutes to the smallest
         # logicalSessionTimeoutMinutes value among ServerDescriptions of all
         # data-bearing server types. If any have a null
@@ -241,22 +242,36 @@ class TopologyDescription(object):
     def heartbeat_frequency(self):
         return self._topology_settings.heartbeat_frequency
 
-    def apply_selector(self, selector, address, custom_selector=None):
-        def apply_local_threshold(selection):
-            if not selection:
-                return []
+    @property
+    def srv_max_hosts(self):
+        return self._topology_settings._srv_max_hosts
 
-            settings = self._topology_settings
+    def _apply_local_threshold(self, selection):
+        if not selection:
+            return []
+        # Round trip time in seconds.
+        fastest = min(s.round_trip_time for s in selection.server_descriptions)
+        threshold = self._topology_settings.local_threshold_ms / 1000.0
+        return [
+            s for s in selection.server_descriptions if (s.round_trip_time - fastest) <= threshold
+        ]
 
-            # Round trip time in seconds.
-            fastest = min(s.round_trip_time for s in selection.server_descriptions)
-            threshold = settings.local_threshold_ms / 1000.0
-            return [
-                s
-                for s in selection.server_descriptions
-                if (s.round_trip_time - fastest) <= threshold
-            ]
+    def apply_selector(self, selector, address=None, custom_selector=None):
+        """List of servers matching the provided selector(s).
 
+        :Parameters:
+          - `selector`: a callable that takes a Selection as input and returns
+            a Selection as output. For example, an instance of a read
+            preference from :mod:`~pymongo.read_preferences`.
+          - `address` (optional): A server address to select.
+          - `custom_selector` (optional): A callable that augments server
+            selection rules. Accepts a list of
+            :class:`~pymongo.server_description.ServerDescription` objects and
+            return a list of server descriptions that should be considered
+            suitable for the desired operation.
+
+        .. versionadded:: 3.4
+        """
         if getattr(selector, "min_wire_version", 0):
             common_wv = self.common_wire_version
             if common_wv and common_wv < selector.min_wire_version:
@@ -265,7 +280,9 @@ class TopologyDescription(object):
                     " wire version is %d" % (selector, selector.min_wire_version, common_wv)
                 )
 
-        if self.topology_type in (TOPOLOGY_TYPE.Single, TOPOLOGY_TYPE.LoadBalanced):
+        if self.topology_type == TOPOLOGY_TYPE.Unknown:
+            return []
+        elif self.topology_type in (TOPOLOGY_TYPE.Single, TOPOLOGY_TYPE.LoadBalanced):
             # Ignore selectors for standalone and load balancer mode.
             return self.known_servers
         elif address:
@@ -283,7 +300,7 @@ class TopologyDescription(object):
             selection = selection.with_server_descriptions(
                 custom_selector(selection.server_descriptions)
             )
-        return apply_local_threshold(selection)
+        return self._apply_local_threshold(selection)
 
     def has_readable_server(self, read_preference=ReadPreference.PRIMARY):
         """Does this topology have any readable servers available matching the
@@ -300,7 +317,7 @@ class TopologyDescription(object):
         .. versionadded:: 3.4
         """
         common.validate_read_preference("read_preference", read_preference)
-        return any(self.apply_selector(read_preference, None))
+        return any(self.apply_selector(read_preference))
 
     def has_writable_server(self):
         """Does this topology have a writable server available?
@@ -445,7 +462,6 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
       - `seedlist`: a list of new seeds new ServerDescription that resulted from
         a hello call
     """
-    assert topology_description.topology_type in SRV_POLLING_TOPOLOGIES
     # Create a copy of the server descriptions.
     sds = topology_description.server_descriptions()
 
@@ -453,16 +469,22 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
     if set(sds.keys()) == set(seedlist):
         return topology_description
 
-    # Add SDs corresponding to servers recently added to the SRV record.
-    for address in seedlist:
-        if address not in sds:
-            sds[address] = ServerDescription(address)
-
     # Remove SDs corresponding to servers no longer part of the SRV record.
     for address in list(sds.keys()):
         if address not in seedlist:
             sds.pop(address)
 
+    if topology_description.srv_max_hosts != 0:
+        new_hosts = set(seedlist) - set(sds.keys())
+        n_to_add = topology_description.srv_max_hosts - len(sds)
+        if n_to_add > 0:
+            seedlist = sample(new_hosts, min(n_to_add, len(new_hosts)))
+        else:
+            seedlist = []
+    # Add SDs corresponding to servers recently added to the SRV record.
+    for address in seedlist:
+        if address not in sds:
+            sds[address] = ServerDescription(address)
     return TopologyDescription(
         topology_description.topology_type,
         sds,
