@@ -17,16 +17,12 @@
 import re
 import sys
 import warnings
+from urllib.parse import unquote_plus
 
-from bson.py3compat import PY3, string_type
-
-if PY3:
-    from urllib.parse import unquote_plus
-else:
-    from urllib import unquote_plus
-
+from pymongo.client_options import _parse_ssl_options
 from pymongo.common import (
     INTERNAL_URI_OPTION_NAME_MAP,
+    SRV_SERVICE_NAME,
     URI_OPTIONS_DEPRECATION_MAP,
     _CaseInsensitiveDictionary,
     get_validated_options,
@@ -41,32 +37,45 @@ SRV_SCHEME_LEN = len(SRV_SCHEME)
 DEFAULT_PORT = 27017
 
 
+def _unquoted_percent(s):
+    """Check for unescaped percent signs.
+
+    :Paramaters:
+        - `s`: A string. `s` can have things like '%25', '%2525',
+           and '%E2%85%A8' but cannot have unquoted percent like '%foo'.
+    """
+    for i in range(len(s)):
+        if s[i] == "%":
+            sub = s[i : i + 3]
+            # If unquoting yields the same string this means there was an
+            # unquoted %.
+            if unquote_plus(sub) == sub:
+                return True
+    return False
+
+
 def parse_userinfo(userinfo):
     """Validates the format of user information in a MongoDB URI.
-    Reserved characters like ':', '/', '+' and '@' must be escaped
-    following RFC 3986.
+    Reserved characters that are gen-delimiters (":", "/", "?", "#", "[",
+    "]", "@") as per RFC 3986 must be escaped.
 
     Returns a 2-tuple containing the unescaped username followed
     by the unescaped password.
 
     :Paramaters:
         - `userinfo`: A string of the form <username>:<password>
-
-    .. versionchanged:: 2.2
-       Now uses `urllib.unquote_plus` so `+` characters must be escaped.
     """
-    if "@" in userinfo or userinfo.count(":") > 1:
-        if PY3:
-            quote_fn = "urllib.parse.quote_plus"
-        else:
-            quote_fn = "urllib.quote_plus"
+    if "@" in userinfo or userinfo.count(":") > 1 or _unquoted_percent(userinfo):
         raise InvalidURI(
-            "Username and password must be escaped according to " "RFC 3986, use %s()." % quote_fn
+            "Username and password must be escaped according to "
+            "RFC 3986, use urllib.parse.quote_plus"
         )
+
     user, _, passwd = userinfo.partition(":")
     # No password is expected with GSSAPI authentication.
     if not user:
         raise InvalidURI("The empty string is not valid username.")
+
     return unquote_plus(user), unquote_plus(passwd)
 
 
@@ -119,7 +128,7 @@ def parse_host(entity, default_port=DEFAULT_PORT):
                 "and ']' according to RFC 2732."
             )
         host, port = host.split(":", 1)
-    if isinstance(port, string_type):
+    if isinstance(port, str):
         if not port.isdigit() or int(port) > 65535 or int(port) <= 0:
             raise ValueError("Port must be an integer between 0 and 65535: %s" % (port,))
         port = int(port)
@@ -138,11 +147,6 @@ _IMPLICIT_TLSINSECURE_OPTS = {
     "tlsdisableocspendpointcheck",
 }
 
-# Options that cannot be specified when tlsInsecure is also specified.
-_TLSINSECURE_EXCLUDE_OPTS = {k for k in _IMPLICIT_TLSINSECURE_OPTS} | {
-    INTERNAL_URI_OPTION_NAME_MAP[k] for k in _IMPLICIT_TLSINSECURE_OPTS
-}
-
 
 def _parse_options(opts, delim):
     """Helper method for split_options which creates the options dict.
@@ -173,21 +177,18 @@ def _handle_security_options(options):
         - `options`: Instance of _CaseInsensitiveDictionary containing
           MongoDB URI options.
     """
+    # Implicitly defined options must not be explicitly specified.
     tlsinsecure = options.get("tlsinsecure")
     if tlsinsecure is not None:
-        for opt in _TLSINSECURE_EXCLUDE_OPTS:
+        for opt in _IMPLICIT_TLSINSECURE_OPTS:
             if opt in options:
                 err_msg = "URI options %s and %s cannot be specified " "simultaneously."
                 raise InvalidURI(
                     err_msg % (options.cased_key("tlsinsecure"), options.cased_key(opt))
                 )
 
-    # Convenience function to retrieve option values based on public or private names.
-    def _getopt(opt):
-        return options.get(opt) or options.get(INTERNAL_URI_OPTION_NAME_MAP[opt])
-
     # Handle co-occurence of OCSP & tlsAllowInvalidCertificates options.
-    tlsallowinvalidcerts = _getopt("tlsallowinvalidcertificates")
+    tlsallowinvalidcerts = options.get("tlsallowinvalidcertificates")
     if tlsallowinvalidcerts is not None:
         if "tlsdisableocspendpointcheck" in options:
             err_msg = "URI options %s and %s cannot be specified " "simultaneously."
@@ -199,7 +200,7 @@ def _handle_security_options(options):
             options["tlsdisableocspendpointcheck"] = True
 
     # Handle co-occurence of CRL and OCSP-related options.
-    tlscrlfile = _getopt("tlscrlfile")
+    tlscrlfile = options.get("tlscrlfile")
     if tlscrlfile is not None:
         for opt in ("tlsinsecure", "tlsallowinvalidcertificates", "tlsdisableocspendpointcheck"):
             if options.get(opt) is True:
@@ -264,18 +265,18 @@ def _handle_option_deprecations(options):
 
 def _normalize_options(options):
     """Normalizes option names in the options dictionary by converting them to
-    their internally-used names. Also handles use of the tlsInsecure option.
+    their internally-used names.
 
     :Parameters:
         - `options`: Instance of _CaseInsensitiveDictionary containing
           MongoDB URI options.
     """
+    # Expand the tlsInsecure option.
     tlsinsecure = options.get("tlsinsecure")
     if tlsinsecure is not None:
         for opt in _IMPLICIT_TLSINSECURE_OPTS:
-            intname = INTERNAL_URI_OPTION_NAME_MAP[opt]
-            # Internal options are logical inverse of public options.
-            options[intname] = not tlsinsecure
+            # Implicit options are logically the same as tlsInsecure.
+            options[opt] = tlsinsecure
 
     for optname in list(options):
         intname = INTERNAL_URI_OPTION_NAME_MAP.get(optname, None)
@@ -334,14 +335,14 @@ def split_options(opts, validate=True, warn=False, normalize=True):
 
     options = _handle_option_deprecations(options)
 
+    if normalize:
+        options = _normalize_options(options)
+
     if validate:
         options = validate_options(options, warn)
         if options.get("authsource") == "":
             raise InvalidURI("the authSource database cannot be an empty string")
 
-    if normalize:
-        options = _normalize_options(options)
-
     return options
 
 
@@ -394,7 +395,14 @@ def _check_options(nodes, options):
 
 
 def parse_uri(
-    uri, default_port=DEFAULT_PORT, validate=True, warn=False, normalize=True, connect_timeout=None
+    uri,
+    default_port=DEFAULT_PORT,
+    validate=True,
+    warn=False,
+    normalize=True,
+    connect_timeout=None,
+    srv_service_name=None,
+    srv_max_hosts=None,
 ):
     """Parse and validate a MongoDB URI.
 
@@ -427,6 +435,11 @@ def parse_uri(
           to their internally-used names. Default: ``True``.
         - `connect_timeout` (optional): The maximum time in milliseconds to
           wait for a response from the DNS server.
+        - 'srv_service_name` (optional): A custom SRV service name
+
+    .. versionchanged:: 4.0
+       To better follow RFC 3986, unquoted percent signs ("%") are no longer
+       supported.
 
     .. versionchanged:: 3.9
         Added the ``normalize`` parameter.
@@ -490,7 +503,8 @@ def parse_uri(
 
         if opts:
             options.update(split_options(opts, validate, warn, normalize))
-
+    if srv_service_name is None:
+        srv_service_name = options.get("srvServiceName", SRV_SERVICE_NAME)
     if "@" in host_part:
         userinfo, _, hosts = host_part.rpartition("@")
         user, passwd = parse_userinfo(userinfo)
@@ -504,7 +518,7 @@ def parse_uri(
 
     hosts = unquote_plus(hosts)
     fqdn = None
-
+    srv_max_hosts = srv_max_hosts or options.get("srvMaxHosts")
     if is_srv:
         if options.get("directConnection"):
             raise ConfigurationError(
@@ -520,7 +534,7 @@ def parse_uri(
         # Use the connection timeout. connectTimeoutMS passed as a keyword
         # argument overrides the same option passed in the connection string.
         connect_timeout = connect_timeout or options.get("connectTimeoutMS")
-        dns_resolver = _SrvResolver(fqdn, connect_timeout=connect_timeout)
+        dns_resolver = _SrvResolver(fqdn, connect_timeout, srv_service_name, srv_max_hosts)
         nodes = dns_resolver.get_hosts()
         dns_options = dns_resolver.get_options()
         if dns_options:
@@ -532,8 +546,20 @@ def parse_uri(
             for opt, val in parsed_dns_options.items():
                 if opt not in options:
                     options[opt] = val
-        if "ssl" not in options:
-            options["ssl"] = True if validate else "true"
+        if options.get("loadBalanced") and srv_max_hosts:
+            raise InvalidURI("You cannot specify loadBalanced with srvMaxHosts")
+        if options.get("replicaSet") and srv_max_hosts:
+            raise InvalidURI("You cannot specify replicaSet with srvMaxHosts")
+        if "tls" not in options and "ssl" not in options:
+            options["tls"] = True if validate else "true"
+    elif not is_srv and options.get("srvServiceName") is not None:
+        raise ConfigurationError(
+            "The srvServiceName option is only allowed " "with 'mongodb+srv://' URIs"
+        )
+    elif not is_srv and srv_max_hosts:
+        raise ConfigurationError(
+            "The srvMaxHosts option is only allowed " "with 'mongodb+srv://' URIs"
+        )
     else:
         nodes = split_hosts(hosts, default_port=default_port)
 
@@ -550,8 +576,43 @@ def parse_uri(
     }
 
 
+def _parse_kms_tls_options(kms_tls_options):
+    """Parse KMS TLS connection options."""
+    if not kms_tls_options:
+        return {}
+    if not isinstance(kms_tls_options, dict):
+        raise TypeError("kms_tls_options must be a dict")
+    contexts = {}
+    for provider, opts in kms_tls_options.items():
+        if not isinstance(opts, dict):
+            raise TypeError(f'kms_tls_options["{provider}"] must be a dict')
+        opts.setdefault("tls", True)
+        opts = _CaseInsensitiveDictionary(opts)
+        opts = _handle_security_options(opts)
+        opts = _normalize_options(opts)
+        opts = validate_options(opts)
+        ssl_context, allow_invalid_hostnames = _parse_ssl_options(opts)
+        if ssl_context is None:
+            raise ConfigurationError("TLS is required for KMS providers")
+        if allow_invalid_hostnames:
+            raise ConfigurationError("Insecure TLS options prohibited")
+
+        for n in [
+            "tlsInsecure",
+            "tlsAllowInvalidCertificates",
+            "tlsAllowInvalidHostnames",
+            "tlsDisableOCSPEndpointCheck",
+            "tlsDisableCertificateRevocationCheck",
+        ]:
+            if n in opts:
+                raise ConfigurationError(f"Insecure TLS options prohibited: {n}")
+            contexts[provider] = ssl_context
+    return contexts
+
+
 if __name__ == "__main__":
     import pprint
+    import sys
 
     try:
         pprint.pprint(parse_uri(sys.argv[1]))
