@@ -50,7 +50,7 @@ import pymongo.errors
 from bson.son import SON
 from pymongo import common, message
 from pymongo.common import partition_node
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import HelloCompat
 from pymongo.server_api import ServerApi
 from pymongo.ssl_support import HAVE_SSL, _ssl
 from pymongo.uri_parser import parse_uri
@@ -154,7 +154,7 @@ class client_knobs(object):
         self.old_min_heartbeat_interval = None
         self.old_kill_cursor_frequency = None
         self.old_events_queue_frequency = None
-        self._enabled = True
+        self._enabled = False
         self._stack = None
 
     def enable(self):
@@ -191,6 +191,17 @@ class client_knobs(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.disable()
 
+    def __call__(self, func):
+        def make_wrapper(f):
+            @wraps(f)
+            def wrap(*args, **kwargs):
+                with self:
+                    return f(*args, **kwargs)
+
+            return wrap
+
+        return make_wrapper(func)
+
     def __del__(self):
         if self._enabled:
             msg = (
@@ -229,12 +240,13 @@ class ClientContext(object):
         self.auth_enabled = False
         self.test_commands_enabled = False
         self.server_parameters = {}
+        self._hello = None
         self.is_mongos = False
         self.mongoses = []
         self.is_rs = False
         self.has_ipv6 = False
         self.tls = False
-        self.ssl_certfile = False
+        self.tlsCertificateKeyFile = False
         self.server_is_resolvable = is_server_resolvable()
         self.default_client_options = {}
         self.sessions_enabled = False
@@ -251,9 +263,22 @@ class ClientContext(object):
             server_api = ServerApi(MONGODB_API_VERSION)
             self.default_client_options["server_api"] = server_api
 
+    @property
+    def client_options(self):
+        """Return the MongoClient options for creating a duplicate client."""
+        opts = client_context.default_client_options.copy()
+        if client_context.auth_enabled:
+            opts["username"] = db_user
+            opts["password"] = db_pwd
+        if self.replica_set_name:
+            opts["replicaSet"] = self.replica_set_name
+        return opts
+
     @property
     def hello(self):
-        return self.client.admin.command(HelloCompat.LEGACY_CMD)
+        if not self._hello:
+            self._hello = self.client.admin.command(HelloCompat.LEGACY_CMD)
+        return self._hello
 
     def _connect(self, host, port, **kwargs):
         # Jython takes a long time to connect.
@@ -265,11 +290,11 @@ class ClientContext(object):
         client = pymongo.MongoClient(host, port, serverSelectionTimeoutMS=timeout_ms, **kwargs)
         try:
             try:
-                client.admin.command("ping")  # Can we connect?
+                client.admin.command(HelloCompat.LEGACY_CMD)  # Can we connect?
             except pymongo.errors.OperationFailure as exc:
                 # SERVER-32063
                 self.connection_attempts.append(
-                    "connected client %r, but hello failed: %s" % (client, exc)
+                    "connected client %r, but legacy hello failed: %s" % (client, exc)
                 )
             else:
                 self.connection_attempts.append("successfully connected client %r" % (client,))
@@ -301,7 +326,7 @@ class ClientContext(object):
             if self.client:
                 self.tls = True
                 self.default_client_options.update(TLS_OPTIONS)
-                self.ssl_certfile = True
+                self.tlsCertificateKeyFile = True
 
         if self.client:
             self.connected = True
@@ -369,6 +394,7 @@ class ClientContext(object):
                     )
 
                 # Get the authoritative hello result from the primary.
+                self._hello = None
                 hello = self.hello
                 nodes = [partition_node(node.lower()) for node in hello.get("hosts", [])]
                 nodes.extend([partition_node(node.lower()) for node in hello.get("passives", [])])
@@ -587,10 +613,8 @@ class ClientContext(object):
 
     def require_auth(self, func):
         """Run a test only if the server is running with auth enabled."""
-        return self.check_auth_with_sharding(
-            self._require(
-                lambda: self.auth_enabled, "Authentication is not enabled on the server", func=func
-            )
+        return self._require(
+            lambda: self.auth_enabled, "Authentication is not enabled on the server", func=func
         )
 
     def require_no_auth(self, func):
@@ -689,11 +713,6 @@ class ClientContext(object):
             lambda: not self.load_balancer, "Must not be connected to a load balancer", func=func
         )
 
-    def check_auth_with_sharding(self, func):
-        """Skip a test when connected to mongos < 2.0 and running with auth."""
-        condition = lambda: not (self.auth_enabled and self.is_mongos and self.version < (2,))
-        return self._require(condition, "Auth with sharding requires MongoDB >= 2.0.0", func=func)
-
     def is_topology_type(self, topologies):
         unknown = set(topologies) - {
             "single",
@@ -760,6 +779,20 @@ class ClientContext(object):
             func=func,
         )
 
+    def require_failCommand_blockConnection(self, func):
+        """Run a test only if the server supports failCommand blockConnection."""
+        return self._require(
+            lambda: (
+                self.test_commands_enabled
+                and (
+                    (not self.is_mongos and self.version >= (4, 2, 9))
+                    or (self.is_mongos and self.version >= (4, 4))
+                )
+            ),
+            "failCommand blockConnection is not supported",
+            func=func,
+        )
+
     def require_tls(self, func):
         """Run a test only if the client can connect over TLS."""
         return self._require(lambda: self.tls, "Must be able to connect via TLS", func=func)
@@ -768,10 +801,12 @@ class ClientContext(object):
         """Run a test only if the client can connect over TLS."""
         return self._require(lambda: not self.tls, "Must be able to connect without TLS", func=func)
 
-    def require_ssl_certfile(self, func):
-        """Run a test only if the client can connect with ssl_certfile."""
+    def require_tlsCertificateKeyFile(self, func):
+        """Run a test only if the client can connect with tlsCertificateKeyFile."""
         return self._require(
-            lambda: self.ssl_certfile, "Must be able to connect with ssl_certfile", func=func
+            lambda: self.tlsCertificateKeyFile,
+            "Must be able to connect with tlsCertificateKeyFile",
+            func=func,
         )
 
     def require_server_resolvable(self, func):
@@ -786,6 +821,21 @@ class ClientContext(object):
         """Run a test only if the deployment supports sessions."""
         return self._require(lambda: self.sessions_enabled, "Sessions not supported", func=func)
 
+    def supports_retryable_writes(self):
+        if self.storage_engine == "mmapv1":
+            return False
+        if not self.sessions_enabled:
+            return False
+        return self.is_mongos or self.is_rs
+
+    def require_retryable_writes(self, func):
+        """Run a test only if the deployment supports retryable writes."""
+        return self._require(
+            self.supports_retryable_writes,
+            "This server does not support retryable writes",
+            func=func,
+        )
+
     def supports_transactions(self):
         if self.storage_engine == "mmapv1":
             return False
@@ -818,19 +868,6 @@ class ClientContext(object):
     def mongos_seeds(self):
         return ",".join("%s:%s" % address for address in self.mongoses)
 
-    @property
-    def supports_reindex(self):
-        """Does the connected server support reindex?"""
-        return not (
-            (self.version.at_least(4, 1, 0) and self.is_mongos)
-            or (self.version.at_least(4, 5, 0) and (self.is_mongos or self.is_rs))
-        )
-
-    @property
-    def supports_getpreverror(self):
-        """Does the connected server support getpreverror?"""
-        return not (self.version.at_least(4, 1, 0) or self.is_mongos)
-
     @property
     def supports_failCommand_fail_point(self):
         """Does the server support the failCommand fail point?"""
@@ -845,6 +882,14 @@ class ClientContext(object):
         # Changed in SERVER-39567.
         return self.version.at_least(4, 1, 10)
 
+    @property
+    def max_bson_size(self):
+        return self.hello["maxBsonObjectSize"]
+
+    @property
+    def max_write_batch_size(self):
+        return self.hello["maxWriteBatchSize"]
+
 
 # Reusable client context
 client_context = ClientContext()
@@ -915,17 +960,18 @@ class IntegrationTest(PyMongoTestCase):
         else:
             cls.credentials = {}
 
+    def cleanup_colls(self, *collections):
+        """Cleanup collections faster than drop_collection."""
+        for c in collections:
+            c = self.client[c.database.name][c.name]
+            c.delete_many({})
+            c.drop_indexes()
+
     def patch_system_certs(self, ca_certs):
         patcher = SystemCertsPatcher(ca_certs)
         self.addCleanup(patcher.disable)
 
 
-# Use assertRaisesRegex if available, otherwise use Python 2.7's
-# deprecated assertRaisesRegexp, with a 'p'.
-if not hasattr(unittest.TestCase, "assertRaisesRegex"):
-    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
-
-
 class MockClientTest(unittest.TestCase):
     """Base class for TestCases that use MockClient.
 
@@ -1070,8 +1116,6 @@ def clear_warning_registry():
 
 class SystemCertsPatcher(object):
     def __init__(self, ca_certs):
-        if sys.version_info < (2, 7, 9):
-            raise SkipTest("Can't load system CA certificates.")
         if (
             ssl.OPENSSL_VERSION.lower().startswith("libressl")
             and sys.platform == "darwin"
@@ -1081,8 +1125,6 @@ class SystemCertsPatcher(object):
                 "LibreSSL on OSX doesn't support setting CA certificates "
                 "using SSL_CERT_FILE environment variable."
             )
-        if sys.platform == "win32" and _ssl.IS_PYOPENSSL:
-            raise SkipTest("SSL_CERT_FILE does not work on Windows with PyOpenSSL")
         self.original_certs = os.environ.get("SSL_CERT_FILE")
         # Tell OpenSSL where CA certificates live.
         os.environ["SSL_CERT_FILE"] = ca_certs
