@@ -18,7 +18,6 @@
 import sys
 import threading
 import time
-
 from optparse import OptionParser
 
 try:
@@ -36,31 +35,49 @@ except ImportError:
 
 
 def parse_args():
-    parser = OptionParser("""usage: %prog [options] mode url
+    parser = OptionParser(
+        """usage: %prog [options] mode url
 
-  mode:\tparallel or serial""")
+  mode:\tparallel or serial"""
+    )
 
     # Should be enough that any connection leak will exhaust available file
     # descriptors.
     parser.add_option(
-        "-n", "--nrequests", type="int",
-        dest="nrequests", default=50 * 1000,
-        help="Number of times to GET the URL, in total")
+        "-n",
+        "--nrequests",
+        type="int",
+        dest="nrequests",
+        default=50 * 1000,
+        help="Number of times to GET the URL, in total",
+    )
 
     parser.add_option(
-        "-t", "--nthreads", type="int",
-        dest="nthreads", default=100,
-        help="Number of threads with mode 'parallel'")
+        "-t",
+        "--nthreads",
+        type="int",
+        dest="nthreads",
+        default=100,
+        help="Number of threads with mode 'parallel'",
+    )
 
     parser.add_option(
-        "-q", "--quiet",
-        action="store_false", dest="verbose", default=True,
-        help="Don't print status messages to stdout")
+        "-q",
+        "--quiet",
+        action="store_false",
+        dest="verbose",
+        default=True,
+        help="Don't print status messages to stdout",
+    )
 
     parser.add_option(
-        "-c", "--continue",
-        action="store_true", dest="continue_", default=False,
-        help="Continue after HTTP errors")
+        "-c",
+        "--continue",
+        action="store_true",
+        dest="continue_",
+        default=False,
+        help="Continue after HTTP errors",
+    )
 
     try:
         options, (mode, url) = parser.parse_args()
@@ -68,7 +85,7 @@ def parse_args():
         parser.print_usage()
         sys.exit(1)
 
-    if mode not in ('parallel', 'serial'):
+    if mode not in ("parallel", "serial"):
         parser.print_usage()
         sys.exit(1)
 
@@ -117,18 +134,22 @@ class URLGetterThread(threading.Thread):
 def main(options, mode, url):
     start_time = time.time()
     errors = 0
-    if mode == 'parallel':
+    if mode == "parallel":
         nrequests_per_thread = options.nrequests // options.nthreads
 
         if options.verbose:
-            print (
-                'Getting %s %s times total in %s threads, '
-                '%s times per thread' % (
-                    url, nrequests_per_thread * options.nthreads,
-                    options.nthreads, nrequests_per_thread))
+            print(
+                "Getting %s %s times total in %s threads, "
+                "%s times per thread"
+                % (
+                    url,
+                    nrequests_per_thread * options.nthreads,
+                    options.nthreads,
+                    nrequests_per_thread,
+                )
+            )
         threads = [
-            URLGetterThread(options, url, nrequests_per_thread)
-            for _ in range(options.nthreads)
+            URLGetterThread(options, url, nrequests_per_thread) for _ in range(options.nthreads)
         ]
 
         for t in threads:
@@ -140,14 +161,11 @@ def main(options, mode, url):
         errors = sum([t.errors for t in threads])
         nthreads_with_errors = len([t for t in threads if t.errors])
         if nthreads_with_errors:
-            print('%d threads had errors! %d errors in total' % (
-                nthreads_with_errors, errors))
+            print("%d threads had errors! %d errors in total" % (nthreads_with_errors, errors))
     else:
-        assert mode == 'serial'
+        assert mode == "serial"
         if options.verbose:
-            print('Getting %s %s times in one thread' % (
-                url, options.nrequests
-            ))
+            print("Getting %s %s times in one thread" % (url, options.nrequests))
 
         for i in range(1, options.nrequests + 1):
             try:
@@ -163,16 +181,16 @@ def main(options, mode, url):
                 print(i)
 
         if errors:
-            print('%d errors!' % errors)
+            print("%d errors!" % errors)
 
     if options.verbose:
-        print('Completed in %.2f seconds' % (time.time() - start_time))
+        print("Completed in %.2f seconds" % (time.time() - start_time))
 
     if errors:
         # Failure
         sys.exit(1)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     options, mode, url = parse_args()
     main(options, mode, url)
