@@ -18,6 +18,7 @@ import multiprocessing as mp
 import os
 import sys
 import tempfile
+import time
 import warnings
 
 try:
@@ -33,7 +34,6 @@ from bson import decode, encode
 from bson.json_util import loads
 from gridfs import GridFSBucket
 from pymongo import MongoClient
-from pymongo.monotonic import time
 
 NUM_ITERATIONS = 100
 MAX_ITERATION_TIME = 300
@@ -59,11 +59,11 @@ def tearDownModule():
 
 class Timer(object):
     def __enter__(self):
-        self.start = time()
+        self.start = time.monotonic()
         return self
 
     def __exit__(self, *args):
-        self.end = time()
+        self.end = time.monotonic()
         self.interval = self.end - self.start
 
 
@@ -98,10 +98,10 @@ class PerformanceTest(object):
 
     def runTest(self):
         results = []
-        start = time()
+        start = time.monotonic()
         self.max_iterations = NUM_ITERATIONS
         for i in range(NUM_ITERATIONS):
-            if time() - start > MAX_ITERATION_TIME:
+            if time.monotonic() - start > MAX_ITERATION_TIME:
                 warnings.warn("Test timed out, completed %s iterations." % i)
                 break
             self.before()
@@ -391,7 +391,7 @@ def insert_json_file_with_file_id(filename):
 
 def read_json_file(filename):
     coll = proc_client.perftest.corpus
-    temp = tempfile.TemporaryFile()
+    temp = tempfile.TemporaryFile(mode="w")
     try:
         temp.writelines(
             [json.dumps(doc) + "\n" for doc in coll.find({"file": filename}, {"_id": False})]
