@@ -17,12 +17,7 @@
 import os
 import sys
 import threading
-
-try:
-    from urllib.parse import quote_plus
-except ImportError:
-    # Python 2
-    from urllib import quote_plus
+from urllib.parse import quote_plus
 
 sys.path[0:0] = [""]
 
@@ -40,7 +35,7 @@ from test.utils import (
 from pymongo import MongoClient, monitoring
 from pymongo.auth import HAVE_KERBEROS, _build_credentials_tuple
 from pymongo.errors import OperationFailure
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import HelloCompat
 from pymongo.read_preferences import ReadPreference
 from pymongo.saslprep import HAVE_STRINGPREP
 
@@ -81,30 +76,6 @@ class AutoAuthenticateThread(threading.Thread):
         self.success = True
 
 
-class DBAuthenticateThread(threading.Thread):
-    """Used in testing threaded authentication.
-
-    This does db.test.find_one() with a 1-second delay to ensure it must
-    check out and authenticate multiple sockets from the pool concurrently.
-
-    :Parameters:
-      `db`: An auth-protected db with a 'test' collection containing one
-      document.
-    """
-
-    def __init__(self, db, username, password):
-        super(DBAuthenticateThread, self).__init__()
-        self.db = db
-        self.username = username
-        self.password = password
-        self.success = False
-
-    def run(self):
-        self.db.authenticate(self.username, self.password)
-        assert self.db.test.find_one({"$where": delay(1)}) is not None
-        self.success = True
-
-
 class TestGSSAPI(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
@@ -339,24 +310,6 @@ class TestSASLPlain(unittest.TestCase):
             client.ldap.test.find_one()
 
     def test_sasl_plain_bad_credentials(self):
-
-        with ignore_deprecations():
-            client = MongoClient(SASL_HOST, SASL_PORT)
-
-            # Bad username
-            self.assertRaises(
-                OperationFailure, client.ldap.authenticate, "not-user", SASL_PASS, SASL_DB, "PLAIN"
-            )
-            self.assertRaises(OperationFailure, client.ldap.test.find_one)
-            self.assertRaises(OperationFailure, client.ldap.test.insert_one, {"failed": True})
-
-            # Bad password
-            self.assertRaises(
-                OperationFailure, client.ldap.authenticate, SASL_USER, "not-pwd", SASL_DB, "PLAIN"
-            )
-            self.assertRaises(OperationFailure, client.ldap.test.find_one)
-            self.assertRaises(OperationFailure, client.ldap.test.insert_one, {"failed": True})
-
         def auth_string(user, password):
             uri = "mongodb://%s:%s@%s:%d/?authMechanism=PLAIN;" "authSource=%s" % (
                 quote_plus(user),
@@ -370,23 +323,14 @@ class TestSASLPlain(unittest.TestCase):
         bad_user = MongoClient(auth_string("not-user", SASL_PASS))
         bad_pwd = MongoClient(auth_string(SASL_USER, "not-pwd"))
         # OperationFailure raised upon connecting.
-        self.assertRaises(OperationFailure, bad_user.admin.command, HelloCompat.LEGACY_CMD)
-        self.assertRaises(OperationFailure, bad_pwd.admin.command, HelloCompat.LEGACY_CMD)
+        self.assertRaises(OperationFailure, bad_user.admin.command, "ping")
+        self.assertRaises(OperationFailure, bad_pwd.admin.command, "ping")
 
 
 class TestSCRAMSHA1(IntegrationTest):
     @client_context.require_auth
-    @client_context.require_version_min(2, 7, 2)
     def setUp(self):
         super(TestSCRAMSHA1, self).setUp()
-        # Before 2.7.7, SCRAM-SHA-1 had to be enabled from the command line.
-        if client_context.version < Version(2, 7, 7):
-            cmd_line = client_context.cmd_line
-            if "SCRAM-SHA-1" not in cmd_line.get("parsed", {}).get("setParameter", {}).get(
-                "authenticationMechanisms", ""
-            ):
-                raise SkipTest("SCRAM-SHA-1 mechanism not enabled")
-
         client_context.create_user("pymongo_test", "user", "pass", roles=["userAdmin", "readWrite"])
 
     def tearDown(self):
@@ -396,13 +340,6 @@ class TestSCRAMSHA1(IntegrationTest):
     def test_scram_sha1(self):
         host, port = client_context.host, client_context.port
 
-        with ignore_deprecations():
-            client = rs_or_single_client_noauth()
-            self.assertTrue(
-                client.pymongo_test.authenticate("user", "pass", mechanism="SCRAM-SHA-1")
-            )
-            client.pymongo_test.command("dbstats")
-
         client = rs_or_single_client_noauth(
             "mongodb://user:pass@%s:%d/pymongo_test?authMechanism=SCRAM-SHA-1" % (host, port)
         )
@@ -420,6 +357,7 @@ class TestSCRAMSHA1(IntegrationTest):
             db.command("dbstats")
 
 
+# https://github.com/mongodb/specifications/blob/master/source/auth/auth.rst#scram-sha-256-and-mechanism-negotiation
 class TestSCRAM(IntegrationTest):
     @client_context.require_auth
     @client_context.require_version_min(3, 7, 2)
@@ -460,18 +398,14 @@ class TestSCRAM(IntegrationTest):
         else:
             self.assertEqual(started, ["saslStart", "saslContinue", "saslContinue"])
 
-    @ignore_deprecations
     def test_scram(self):
-        host, port = client_context.host, client_context.port
-
+        # Step 1: create users
         client_context.create_user(
             "testscram", "sha1", "pwd", roles=["dbOwner"], mechanisms=["SCRAM-SHA-1"]
         )
-
         client_context.create_user(
             "testscram", "sha256", "pwd", roles=["dbOwner"], mechanisms=["SCRAM-SHA-256"]
         )
-
         client_context.create_user(
             "testscram",
             "both",
@@ -480,106 +414,38 @@ class TestSCRAM(IntegrationTest):
             mechanisms=["SCRAM-SHA-1", "SCRAM-SHA-256"],
         )
 
-        client = rs_or_single_client_noauth(event_listeners=[self.listener])
-        self.assertTrue(client.testscram.authenticate("sha1", "pwd"))
+        # Step 2: verify auth success cases
+        client = rs_or_single_client_noauth(username="sha1", password="pwd", authSource="testscram")
         client.testscram.command("dbstats")
-        client.testscram.logout()
-        self.assertTrue(client.testscram.authenticate("sha1", "pwd", mechanism="SCRAM-SHA-1"))
-        client.testscram.command("dbstats")
-        client.testscram.logout()
-        self.assertRaises(
-            OperationFailure,
-            client.testscram.authenticate,
-            "sha1",
-            "pwd",
-            mechanism="SCRAM-SHA-256",
+
+        client = rs_or_single_client_noauth(
+            username="sha1", password="pwd", authSource="testscram", authMechanism="SCRAM-SHA-1"
         )
+        client.testscram.command("dbstats")
 
-        self.assertTrue(client.testscram.authenticate("sha256", "pwd"))
-        client.testscram.command("dbstats")
-        client.testscram.logout()
-        self.assertTrue(client.testscram.authenticate("sha256", "pwd", mechanism="SCRAM-SHA-256"))
-        client.testscram.command("dbstats")
-        client.testscram.logout()
-        self.assertRaises(
-            OperationFailure,
-            client.testscram.authenticate,
-            "sha256",
-            "pwd",
-            mechanism="SCRAM-SHA-1",
+        client = rs_or_single_client_noauth(
+            username="sha256", password="pwd", authSource="testscram"
         )
-
-        self.listener.results.clear()
-        self.assertTrue(client.testscram.authenticate("both", "pwd"))
-        started = self.listener.results["started"][0]
-        self.assertEqual(started.command.get("mechanism"), "SCRAM-SHA-256")
         client.testscram.command("dbstats")
-        client.testscram.logout()
-        self.assertTrue(client.testscram.authenticate("both", "pwd", mechanism="SCRAM-SHA-256"))
+
+        client = rs_or_single_client_noauth(
+            username="sha256", password="pwd", authSource="testscram", authMechanism="SCRAM-SHA-256"
+        )
         client.testscram.command("dbstats")
-        client.testscram.logout()
-        self.assertTrue(client.testscram.authenticate("both", "pwd", mechanism="SCRAM-SHA-1"))
+
+        # Step 2: SCRAM-SHA-1 and SCRAM-SHA-256
+        client = rs_or_single_client_noauth(
+            username="both", password="pwd", authSource="testscram", authMechanism="SCRAM-SHA-1"
+        )
+        client.testscram.command("dbstats")
+        client = rs_or_single_client_noauth(
+            username="both", password="pwd", authSource="testscram", authMechanism="SCRAM-SHA-256"
+        )
         client.testscram.command("dbstats")
-        client.testscram.logout()
-
-        self.assertRaises(OperationFailure, client.testscram.authenticate, "not-a-user", "pwd")
-
-        if HAVE_STRINGPREP:
-            # Test the use of SASLprep on passwords. For example,
-            # saslprep(u'\u2136') becomes u'IV' and saslprep(u'I\u00ADX')
-            # becomes u'IX'. SASLprep is only supported when the standard
-            # library provides stringprep.
-            client_context.create_user(
-                "testscram", u"\u2168", u"\u2163", roles=["dbOwner"], mechanisms=["SCRAM-SHA-256"]
-            )
-
-            client_context.create_user(
-                "testscram", u"IX", u"IX", roles=["dbOwner"], mechanisms=["SCRAM-SHA-256"]
-            )
-
-            self.assertTrue(client.testscram.authenticate(u"\u2168", u"\u2163"))
-            client.testscram.command("dbstats")
-            client.testscram.logout()
-            self.assertTrue(
-                client.testscram.authenticate(u"\u2168", u"\u2163", mechanism="SCRAM-SHA-256")
-            )
-            client.testscram.command("dbstats")
-            client.testscram.logout()
-            self.assertTrue(client.testscram.authenticate(u"\u2168", u"IV"))
-            client.testscram.command("dbstats")
-            client.testscram.logout()
-
-            self.assertTrue(client.testscram.authenticate(u"IX", u"I\u00ADX"))
-            client.testscram.command("dbstats")
-            client.testscram.logout()
-            self.assertTrue(
-                client.testscram.authenticate(u"IX", u"I\u00ADX", mechanism="SCRAM-SHA-256")
-            )
-            client.testscram.command("dbstats")
-            client.testscram.logout()
-            self.assertTrue(client.testscram.authenticate(u"IX", u"IX"))
-            client.testscram.command("dbstats")
-            client.testscram.logout()
-
-            client = rs_or_single_client_noauth(
-                u"mongodb://\u2168:\u2163@%s:%d/testscram" % (host, port)
-            )
-            client.testscram.command("dbstats")
-            client = rs_or_single_client_noauth(
-                u"mongodb://\u2168:IV@%s:%d/testscram" % (host, port)
-            )
-            client.testscram.command("dbstats")
-
-            client = rs_or_single_client_noauth(
-                u"mongodb://IX:I\u00ADX@%s:%d/testscram" % (host, port)
-            )
-            client.testscram.command("dbstats")
-            client = rs_or_single_client_noauth(u"mongodb://IX:IX@%s:%d/testscram" % (host, port))
-            client.testscram.command("dbstats")
 
         self.listener.results.clear()
         client = rs_or_single_client_noauth(
-            "mongodb://both:pwd@%s:%d/testscram" % (host, port), event_listeners=[self.listener]
+            username="both", password="pwd", authSource="testscram", event_listeners=[self.listener]
         )
         client.testscram.command("dbstats")
         if client_context.version.at_least(4, 4, -1):
@@ -590,17 +456,27 @@ class TestSCRAM(IntegrationTest):
             started = self.listener.results["started"][0]
             self.assertEqual(started.command.get("mechanism"), "SCRAM-SHA-256")
 
+        # Step 3: verify auth failure conditions
         client = rs_or_single_client_noauth(
-            "mongodb://both:pwd@%s:%d/testscram?authMechanism=SCRAM-SHA-1" % (host, port)
+            username="sha1", password="pwd", authSource="testscram", authMechanism="SCRAM-SHA-256"
         )
-        client.testscram.command("dbstats")
+        with self.assertRaises(OperationFailure):
+            client.testscram.command("dbstats")
 
         client = rs_or_single_client_noauth(
-            "mongodb://both:pwd@%s:%d/testscram?authMechanism=SCRAM-SHA-256" % (host, port)
+            username="sha256", password="pwd", authSource="testscram", authMechanism="SCRAM-SHA-1"
         )
-        client.testscram.command("dbstats")
+        with self.assertRaises(OperationFailure):
+            client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth(
+            username="not-a-user", password="pwd", authSource="testscram"
+        )
+        with self.assertRaises(OperationFailure):
+            client.testscram.command("dbstats")
 
         if client_context.is_rs:
+            host, port = client_context.host, client_context.port
             uri = "mongodb://both:pwd@%s:%d/testscram" "?replicaSet=%s" % (
                 host,
                 port,
@@ -611,6 +487,69 @@ class TestSCRAM(IntegrationTest):
             db = client.get_database("testscram", read_preference=ReadPreference.SECONDARY)
             db.command("dbstats")
 
+    @unittest.skipUnless(HAVE_STRINGPREP, "Cannot test without stringprep")
+    def test_scram_saslprep(self):
+        # Step 4: test SASLprep
+        host, port = client_context.host, client_context.port
+        # Test the use of SASLprep on passwords. For example,
+        # saslprep('\u2136') becomes 'IV' and saslprep('I\u00ADX')
+        # becomes 'IX'. SASLprep is only supported when the standard
+        # library provides stringprep.
+        client_context.create_user(
+            "testscram", "\u2168", "\u2163", roles=["dbOwner"], mechanisms=["SCRAM-SHA-256"]
+        )
+        client_context.create_user(
+            "testscram", "IX", "IX", roles=["dbOwner"], mechanisms=["SCRAM-SHA-256"]
+        )
+
+        client = rs_or_single_client_noauth(
+            username="\u2168", password="\u2163", authSource="testscram"
+        )
+        client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth(
+            username="\u2168",
+            password="\u2163",
+            authSource="testscram",
+            authMechanism="SCRAM-SHA-256",
+        )
+        client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth(
+            username="\u2168", password="IV", authSource="testscram"
+        )
+        client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth(
+            username="IX", password="I\u00ADX", authSource="testscram"
+        )
+        client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth(
+            username="IX",
+            password="I\u00ADX",
+            authSource="testscram",
+            authMechanism="SCRAM-SHA-256",
+        )
+        client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth(
+            username="IX", password="IX", authSource="testscram", authMechanism="SCRAM-SHA-256"
+        )
+        client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth(
+            "mongodb://\u2168:\u2163@%s:%d/testscram" % (host, port)
+        )
+        client.testscram.command("dbstats")
+        client = rs_or_single_client_noauth("mongodb://\u2168:IV@%s:%d/testscram" % (host, port))
+        client.testscram.command("dbstats")
+
+        client = rs_or_single_client_noauth("mongodb://IX:I\u00ADX@%s:%d/testscram" % (host, port))
+        client.testscram.command("dbstats")
+        client = rs_or_single_client_noauth("mongodb://IX:IX@%s:%d/testscram" % (host, port))
+        client.testscram.command("dbstats")
+
     def test_cache(self):
         client = single_client()
         # Force authentication.
@@ -659,34 +598,6 @@ class TestSCRAM(IntegrationTest):
             self.assertTrue(thread.success)
 
 
-class TestThreadedAuth(IntegrationTest):
-    @client_context.require_auth
-    def test_db_authenticate_threaded(self):
-
-        db = client_context.client.db
-        coll = db.test
-        coll.drop()
-        coll.insert_one({"_id": 1})
-
-        client_context.create_user("db", "user", "pass", roles=["dbOwner"])
-        self.addCleanup(db.command, "dropUser", "user")
-
-        db = rs_or_single_client_noauth().db
-        db.authenticate("user", "pass")
-        # No error.
-        db.authenticate("user", "pass")
-
-        db = rs_or_single_client_noauth().db
-        threads = []
-        for _ in range(4):
-            threads.append(DBAuthenticateThread(db, "user", "pass"))
-        for thread in threads:
-            thread.start()
-        for thread in threads:
-            thread.join()
-            self.assertTrue(thread.success)
-
-
 class TestAuthURIOptions(IntegrationTest):
     @client_context.require_auth
     def setUp(self):
