@@ -16,17 +16,22 @@
 
 """Test the bson module."""
 
+import array
 import collections
 import datetime
+import mmap
 import os
+import pickle
 import re
 import sys
 import tempfile
 import uuid
+from collections import OrderedDict, abc
+from io import BytesIO
 
 sys.path[0:0] = [""]
 
-from test import SkipTest, qcheck, unittest
+from test import qcheck, unittest
 from test.utils import ExceptionCatchingThread
 
 import bson
@@ -41,23 +46,19 @@ from bson import (
     encode,
     is_valid,
 )
-from bson.binary import Binary, UUIDLegacy, UuidRepresentation
+from bson.binary import Binary, UuidRepresentation
 from bson.code import Code
 from bson.codec_options import CodecOptions
 from bson.dbref import DBRef
-from bson.errors import InvalidBSON, InvalidDocument, InvalidStringData
+from bson.errors import InvalidBSON, InvalidDocument
 from bson.int64 import Int64
 from bson.max_key import MaxKey
 from bson.min_key import MinKey
 from bson.objectid import ObjectId
-from bson.py3compat import PY3, StringIO, abc, iteritems, text_type
 from bson.son import SON
 from bson.timestamp import Timestamp
 from bson.tz_util import FixedOffset, utc
 
-if PY3:
-    long = int
-
 
 class NotADict(abc.MutableMapping):
     """Non-dict type that implements the mapping protocol."""
@@ -129,22 +130,22 @@ class TestBSON(unittest.TestCase):
             self.assertEqual(doc, decoder(encoder(doc)))
 
         helper({})
-        helper({"test": u"hello"})
-        self.assertTrue(isinstance(decoder(encoder({"hello": "world"}))["hello"], text_type))
+        helper({"test": "hello"})
+        self.assertTrue(isinstance(decoder(encoder({"hello": "world"}))["hello"], str))
         helper({"mike": -10120})
         helper({"long": Int64(10)})
         helper({"really big long": 2147483648})
-        helper({u"hello": 0.0013109})
+        helper({"hello": 0.0013109})
         helper({"something": True})
         helper({"false": False})
-        helper({"an array": [1, True, 3.8, u"world"]})
-        helper({"an object": doc_class({"test": u"something"})})
+        helper({"an array": [1, True, 3.8, "world"]})
+        helper({"an object": doc_class({"test": "something"})})
         helper({"a binary": Binary(b"test", 100)})
         helper({"a binary": Binary(b"test", 128)})
         helper({"a binary": Binary(b"test", 254)})
         helper({"another binary": Binary(b"test", 2)})
-        helper(SON([(u"test dst", datetime.datetime(1993, 4, 4, 2))]))
-        helper(SON([(u"test negative dst", datetime.datetime(1, 1, 1, 1, 1, 1))]))
+        helper(SON([("test dst", datetime.datetime(1993, 4, 4, 2))]))
+        helper(SON([("test negative dst", datetime.datetime(1, 1, 1, 1, 1, 1))]))
         helper({"big float": float(10000000000)})
         helper({"ref": DBRef("coll", 5)})
         helper({"ref": DBRef("coll", 5, foo="bar", bar=4)})
@@ -186,7 +187,7 @@ class TestBSON(unittest.TestCase):
 
     def test_basic_validation(self):
         self.assertRaises(TypeError, is_valid, 100)
-        self.assertRaises(TypeError, is_valid, u"test")
+        self.assertRaises(TypeError, is_valid, "test")
         self.assertRaises(TypeError, is_valid, 10.4)
 
         self.assertInvalid(b"test")
@@ -268,7 +269,7 @@ class TestBSON(unittest.TestCase):
 
     def test_basic_decode(self):
         self.assertEqual(
-            {"test": u"hello world"},
+            {"test": "hello world"},
             decode(
                 b"\x1B\x00\x00\x00\x0E\x74\x65\x73\x74\x00\x0C"
                 b"\x00\x00\x00\x68\x65\x6C\x6C\x6F\x20\x77\x6F"
@@ -276,7 +277,7 @@ class TestBSON(unittest.TestCase):
             ),
         )
         self.assertEqual(
-            [{"test": u"hello world"}, {}],
+            [{"test": "hello world"}, {}],
             decode_all(
                 b"\x1B\x00\x00\x00\x0E\x74\x65\x73\x74"
                 b"\x00\x0C\x00\x00\x00\x68\x65\x6C\x6C"
@@ -285,7 +286,7 @@ class TestBSON(unittest.TestCase):
             ),
         )
         self.assertEqual(
-            [{"test": u"hello world"}, {}],
+            [{"test": "hello world"}, {}],
             list(
                 decode_iter(
                     b"\x1B\x00\x00\x00\x0E\x74\x65\x73\x74"
@@ -296,10 +297,10 @@ class TestBSON(unittest.TestCase):
             ),
         )
         self.assertEqual(
-            [{"test": u"hello world"}, {}],
+            [{"test": "hello world"}, {}],
             list(
                 decode_file_iter(
-                    StringIO(
+                    BytesIO(
                         b"\x1B\x00\x00\x00\x0E\x74\x65\x73\x74"
                         b"\x00\x0C\x00\x00\x00\x68\x65\x6C\x6C"
                         b"\x6f\x20\x77\x6F\x72\x6C\x64\x00\x00"
@@ -315,15 +316,11 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(docs, decode_all(bytearray(bs)))
         self.assertEqual(docs, decode_all(memoryview(bs)))
         self.assertEqual(docs, decode_all(memoryview(b"1" + bs + b"1")[1:-1]))
-        if PY3:
-            import array
-            import mmap
-
-            self.assertEqual(docs, decode_all(array.array("B", bs)))
-            with mmap.mmap(-1, len(bs)) as mm:
-                mm.write(bs)
-                mm.seek(0)
-                self.assertEqual(docs, decode_all(mm))
+        self.assertEqual(docs, decode_all(array.array("B", bs)))
+        with mmap.mmap(-1, len(bs)) as mm:
+            mm.write(bs)
+            mm.seek(0)
+            self.assertEqual(docs, decode_all(mm))
 
     def test_decode_buffer_protocol(self):
         doc = {"foo": "bar"}
@@ -332,21 +329,17 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(doc, decode(bytearray(bs)))
         self.assertEqual(doc, decode(memoryview(bs)))
         self.assertEqual(doc, decode(memoryview(b"1" + bs + b"1")[1:-1]))
-        if PY3:
-            import array
-            import mmap
-
-            self.assertEqual(doc, decode(array.array("B", bs)))
-            with mmap.mmap(-1, len(bs)) as mm:
-                mm.write(bs)
-                mm.seek(0)
-                self.assertEqual(doc, decode(mm))
+        self.assertEqual(doc, decode(array.array("B", bs)))
+        with mmap.mmap(-1, len(bs)) as mm:
+            mm.write(bs)
+            mm.seek(0)
+            self.assertEqual(doc, decode(mm))
 
     def test_invalid_decodes(self):
         # Invalid object size (not enough bytes in document for even
         # an object size of first object.
         # NOTE: decode_all and decode_iter don't care, not sure if they should?
-        self.assertRaises(InvalidBSON, list, decode_file_iter(StringIO(b"\x1B")))
+        self.assertRaises(InvalidBSON, list, decode_file_iter(BytesIO(b"\x1B")))
 
         bad_bsons = [
             # An object size that's too small to even include the object size,
@@ -383,7 +376,7 @@ class TestBSON(unittest.TestCase):
             with self.assertRaises(InvalidBSON, msg=msg):
                 list(decode_iter(data))
             with self.assertRaises(InvalidBSON, msg=msg):
-                list(decode_file_iter(StringIO(data)))
+                list(decode_file_iter(BytesIO(data)))
             with tempfile.TemporaryFile() as scratch:
                 scratch.write(data)
                 scratch.seek(0, os.SEEK_SET)
@@ -414,13 +407,13 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(encode({}), BSON(b"\x05\x00\x00\x00\x00"))
         self.assertEqual(encode({}), b"\x05\x00\x00\x00\x00")
         self.assertEqual(
-            encode({"test": u"hello world"}),
+            encode({"test": "hello world"}),
             b"\x1B\x00\x00\x00\x02\x74\x65\x73\x74\x00\x0C\x00"
             b"\x00\x00\x68\x65\x6C\x6C\x6F\x20\x77\x6F\x72\x6C"
             b"\x64\x00\x00",
         )
         self.assertEqual(
-            encode({u"mike": 100}),
+            encode({"mike": 100}),
             b"\x0F\x00\x00\x00\x10\x6D\x69\x6B\x65\x00\x64\x00" b"\x00\x00\x00",
         )
         self.assertEqual(
@@ -477,7 +470,7 @@ class TestBSON(unittest.TestCase):
             b"\x00\x00return function(){ return x; }\x00\t\x00"
             b"\x00\x00\x08x\x00\x00\x00\x00",
         )
-        unicode_empty_scope = Code(u"function(){ return 'héllo';}", {})
+        unicode_empty_scope = Code("function(){ return 'héllo';}", {})
         self.assertEqual(
             encode({"$field": unicode_empty_scope}),
             b"8\x00\x00\x00\x0f$field\x00+\x00\x00\x00\x1e\x00"
@@ -533,7 +526,7 @@ class TestBSON(unittest.TestCase):
         ref_only = {"ref": {"$ref": "collection"}}
         id_only = {"ref": {"$id": ObjectId()}}
 
-        self.assertEqual(DBRef("collection", id=None), decode(encode(ref_only))["ref"])
+        self.assertEqual(ref_only, decode(encode(ref_only)))
         self.assertEqual(id_only, decode(encode(id_only)))
 
     def test_bytes_as_keys(self):
@@ -541,10 +534,7 @@ class TestBSON(unittest.TestCase):
         # Since `bytes` are stored as Binary you can't use them
         # as keys in python 3.x. Using binary data as a key makes
         # no sense in BSON anyway and little sense in python.
-        if PY3:
-            self.assertRaises(InvalidDocument, encode, doc)
-        else:
-            self.assertTrue(encode(doc))
+        self.assertRaises(InvalidDocument, encode, doc)
 
     def test_datetime_encode_decode(self):
         # Negative timestamps
@@ -636,31 +626,21 @@ class TestBSON(unittest.TestCase):
         d = {"x": datetime.datetime(1993, 4, 4, 2)}
         self.assertEqual(d, decode(encode(d)))
 
+    @unittest.skip("Disabled due to http://bugs.python.org/issue25222")
     def test_bad_encode(self):
-        if not PY3:
-            # Python3 treats this as a unicode string which won't raise
-            # an exception. If we passed the string as bytes instead we
-            # still wouldn't get an error since we store bytes as BSON
-            # binary subtype 0.
-            self.assertRaises(
-                InvalidStringData, encode, {"lalala": "\xf4\xe0\xf0\xe1\xc0 Color Touch"}
-            )
-        # Work around what seems like a regression in python 3.5.0.
-        # See http://bugs.python.org/issue25222
-        if sys.version_info[:2] < (3, 5):
-            evil_list = {"a": []}
-            evil_list["a"].append(evil_list)
-            evil_dict = {}
-            evil_dict["a"] = evil_dict
-            for evil_data in [evil_dict, evil_list]:
-                self.assertRaises(Exception, encode, evil_data)
+        evil_list = {"a": []}
+        evil_list["a"].append(evil_list)
+        evil_dict = {}
+        evil_dict["a"] = evil_dict
+        for evil_data in [evil_dict, evil_list]:
+            self.assertRaises(Exception, encode, evil_data)
 
     def test_overflow(self):
-        self.assertTrue(encode({"x": long(9223372036854775807)}))
-        self.assertRaises(OverflowError, encode, {"x": long(9223372036854775808)})
+        self.assertTrue(encode({"x": 9223372036854775807}))
+        self.assertRaises(OverflowError, encode, {"x": 9223372036854775808})
 
-        self.assertTrue(encode({"x": long(-9223372036854775808)}))
-        self.assertRaises(OverflowError, encode, {"x": long(-9223372036854775809)})
+        self.assertTrue(encode({"x": -9223372036854775808}))
+        self.assertRaises(OverflowError, encode, {"x": -9223372036854775809})
 
     def test_small_long_encode_decode(self):
         encoded1 = encode({"x": 256})
@@ -681,73 +661,58 @@ class TestBSON(unittest.TestCase):
 
     def test_uuid(self):
         id = uuid.uuid4()
-        transformed_id = decode(encode({"id": id}))["id"]
+        # The default uuid_representation is UNSPECIFIED
+        with self.assertRaisesRegex(ValueError, "cannot encode native uuid"):
+            bson.decode_all(encode({"uuid": id}))
 
+        opts = CodecOptions(uuid_representation=UuidRepresentation.STANDARD)
+        transformed_id = decode(encode({"id": id}, codec_options=opts), codec_options=opts)["id"]
         self.assertTrue(isinstance(transformed_id, uuid.UUID))
         self.assertEqual(id, transformed_id)
         self.assertNotEqual(uuid.uuid4(), transformed_id)
 
     def test_uuid_legacy(self):
-
         id = uuid.uuid4()
-        legacy = UUIDLegacy(id)
+        legacy = Binary.from_uuid(id, UuidRepresentation.PYTHON_LEGACY)
         self.assertEqual(3, legacy.subtype)
-        transformed = decode(encode({"uuid": legacy}))["uuid"]
-        self.assertTrue(isinstance(transformed, uuid.UUID))
+        bin = decode(encode({"uuid": legacy}))["uuid"]
+        self.assertTrue(isinstance(bin, Binary))
+        transformed = bin.as_uuid(UuidRepresentation.PYTHON_LEGACY)
         self.assertEqual(id, transformed)
-        self.assertNotEqual(UUIDLegacy(uuid.uuid4()), UUIDLegacy(transformed))
 
     # The C extension was segfaulting on unicode RegExs, so we have this test
     # that doesn't really test anything but the lack of a segfault.
     def test_unicode_regex(self):
-        regex = re.compile(u"revisi\xf3n")
+        regex = re.compile("revisi\xf3n")
         decode(encode({"regex": regex}))
 
     def test_non_string_keys(self):
         self.assertRaises(InvalidDocument, encode, {8.9: "test"})
 
     def test_utf8(self):
-        w = {u"aéあ": u"aéあ"}
+        w = {"aéあ": "aéあ"}
         self.assertEqual(w, decode(encode(w)))
 
-        # b'a\xe9' == u"aé".encode("iso-8859-1")
+        # b'a\xe9' == "aé".encode("iso-8859-1")
         iso8859_bytes = b"a\xe9"
         y = {"hello": iso8859_bytes}
-        if PY3:
-            # Stored as BSON binary subtype 0.
-            out = decode(encode(y))
-            self.assertTrue(isinstance(out["hello"], bytes))
-            self.assertEqual(out["hello"], iso8859_bytes)
-        else:
-            # Python 2.
-            try:
-                encode(y)
-            except InvalidStringData as e:
-                self.assertTrue(repr(iso8859_bytes) in str(e))
-
-            # The next two tests only make sense in python 2.x since
-            # you can't use `bytes` type as document keys in python 3.x.
-            x = {u"aéあ".encode("utf-8"): u"aéあ".encode("utf-8")}
-            self.assertEqual(w, decode(encode(x)))
-
-            z = {iso8859_bytes: "hello"}
-            self.assertRaises(InvalidStringData, encode, z)
+        # Stored as BSON binary subtype 0.
+        out = decode(encode(y))
+        self.assertTrue(isinstance(out["hello"], bytes))
+        self.assertEqual(out["hello"], iso8859_bytes)
 
     def test_null_character(self):
         doc = {"a": "\x00"}
         self.assertEqual(doc, decode(encode(doc)))
 
-        # This test doesn't make much sense in Python2
-        # since {'a': '\x00'} == {'a': u'\x00'}.
-        # Decoding here actually returns {'a': '\x00'}
-        doc = {"a": u"\x00"}
+        doc = {"a": "\x00"}
         self.assertEqual(doc, decode(encode(doc)))
 
         self.assertRaises(InvalidDocument, encode, {b"\x00": "a"})
-        self.assertRaises(InvalidDocument, encode, {u"\x00": "a"})
+        self.assertRaises(InvalidDocument, encode, {"\x00": "a"})
 
         self.assertRaises(InvalidDocument, encode, {"a": re.compile(b"ab\x00c")})
-        self.assertRaises(InvalidDocument, encode, {"a": re.compile(u"ab\x00c")})
+        self.assertRaises(InvalidDocument, encode, {"a": re.compile("ab\x00c")})
 
     def test_move_id(self):
         self.assertEqual(
@@ -794,22 +759,18 @@ class TestBSON(unittest.TestCase):
         class _myfloat(float):
             pass
 
-        class _myunicode(text_type):
+        class _myunicode(str):
             pass
 
         d = {"a": _myint(42), "b": _myfloat(63.9), "c": _myunicode("hello world")}
         d2 = decode(encode(d))
-        for key, value in iteritems(d2):
+        for key, value in d2.items():
             orig_value = d[key]
             orig_type = orig_value.__class__.__bases__[0]
             self.assertEqual(type(value), orig_type)
             self.assertEqual(value, orig_type(value))
 
     def test_ordered_dict(self):
-        try:
-            from collections import OrderedDict
-        except ImportError:
-            raise SkipTest("No OrderedDict")
         d = OrderedDict([("one", 1), ("two", 2), ("three", 3), ("four", 4)])
         self.assertEqual(d, decode(encode(d), CodecOptions(document_class=OrderedDict)))
 
@@ -828,8 +789,8 @@ class TestBSON(unittest.TestCase):
         self.assertEqual(doc1, decode(doc1_bson))
 
         # Valid Python regex, with flags.
-        re2 = re.compile(u".*", re.I | re.M | re.S | re.U | re.X)
-        bson_re2 = Regex(u".*", re.I | re.M | re.S | re.U | re.X)
+        re2 = re.compile(".*", re.I | re.M | re.S | re.U | re.X)
+        bson_re2 = Regex(".*", re.I | re.M | re.S | re.U | re.X)
 
         doc2_with_re = {"r": re2}
         doc2_with_bson_re = {"r": bson_re2}
@@ -1017,7 +978,7 @@ class TestCodecOptions(unittest.TestCase):
     def test_codec_options_repr(self):
         r = (
             "CodecOptions(document_class=dict, tz_aware=False, "
-            "uuid_representation=UuidRepresentation.PYTHON_LEGACY, "
+            "uuid_representation=UuidRepresentation.UNSPECIFIED, "
             "unicode_decode_error_handler='strict', "
             "tzinfo=None, type_registry=TypeRegistry(type_codecs=[], "
             "fallback_encoder=None))"
@@ -1026,76 +987,42 @@ class TestCodecOptions(unittest.TestCase):
 
     def test_decode_all_defaults(self):
         # Test decode_all()'s default document_class is dict and tz_aware is
-        # False. The default uuid_representation is PYTHON_LEGACY but this
-        # decodes same as STANDARD, so all this test proves about UUID decoding
-        # is that it's not CSHARP_LEGACY or JAVA_LEGACY.
-        doc = {"sub_document": {}, "uuid": uuid.uuid4(), "dt": datetime.datetime.utcnow()}
+        # False.
+        doc = {"sub_document": {}, "dt": datetime.datetime.utcnow()}
 
         decoded = bson.decode_all(bson.encode(doc))[0]
         self.assertIsInstance(decoded["sub_document"], dict)
-        self.assertEqual(decoded["uuid"], doc["uuid"])
         self.assertIsNone(decoded["dt"].tzinfo)
-
-    def test_decode_all_kwarg(self):
-        doc = {"a": uuid.uuid4()}
-        opts = CodecOptions(uuid_representation=UuidRepresentation.STANDARD)
-        encoded = encode(doc, codec_options=opts)
-        # Positional codec_options
-        self.assertEqual([doc], decode_all(encoded, opts))
-        # Keyword codec_options
-        self.assertEqual([doc], decode_all(encoded, codec_options=opts))
+        # The default uuid_representation is UNSPECIFIED
+        with self.assertRaisesRegex(ValueError, "cannot encode native uuid"):
+            bson.decode_all(bson.encode({"uuid": uuid.uuid4()}))
 
     def test_unicode_decode_error_handler(self):
         enc = encode({"keystr": "foobar"})
 
-        # Test handling of bad key value.
+        # Test handling of bad key value, bad string value, and both.
         invalid_key = enc[:7] + b"\xe9" + enc[8:]
-        replaced_key = b"ke\xe9str".decode("utf-8", "replace")
-        ignored_key = b"ke\xe9str".decode("utf-8", "ignore")
-
-        dec = decode(invalid_key, CodecOptions(unicode_decode_error_handler="replace"))
-        self.assertEqual(dec, {replaced_key: u"foobar"})
-
-        dec = decode(invalid_key, CodecOptions(unicode_decode_error_handler="ignore"))
-        self.assertEqual(dec, {ignored_key: u"foobar"})
-
-        self.assertRaises(
-            InvalidBSON, decode, invalid_key, CodecOptions(unicode_decode_error_handler="strict")
-        )
-        self.assertRaises(InvalidBSON, decode, invalid_key, CodecOptions())
-        self.assertRaises(InvalidBSON, decode, invalid_key)
-
-        # Test handing of bad string value.
-        invalid_val = BSON(enc[:18] + b"\xe9" + enc[19:])
-        replaced_val = b"fo\xe9bar".decode("utf-8", "replace")
-        ignored_val = b"fo\xe9bar".decode("utf-8", "ignore")
-
-        dec = decode(invalid_val, CodecOptions(unicode_decode_error_handler="replace"))
-        self.assertEqual(dec, {u"keystr": replaced_val})
-
-        dec = decode(invalid_val, CodecOptions(unicode_decode_error_handler="ignore"))
-        self.assertEqual(dec, {u"keystr": ignored_val})
-
-        self.assertRaises(
-            InvalidBSON, decode, invalid_val, CodecOptions(unicode_decode_error_handler="strict")
-        )
-        self.assertRaises(InvalidBSON, decode, invalid_val, CodecOptions())
-        self.assertRaises(InvalidBSON, decode, invalid_val)
-
-        # Test handing bad key + bad value.
+        invalid_val = enc[:18] + b"\xe9" + enc[19:]
         invalid_both = enc[:7] + b"\xe9" + enc[8:18] + b"\xe9" + enc[19:]
 
-        dec = decode(invalid_both, CodecOptions(unicode_decode_error_handler="replace"))
-        self.assertEqual(dec, {replaced_key: replaced_val})
+        # Ensure that strict mode raises an error.
+        for invalid in [invalid_key, invalid_val, invalid_both]:
+            self.assertRaises(
+                InvalidBSON, decode, invalid, CodecOptions(unicode_decode_error_handler="strict")
+            )
+            self.assertRaises(InvalidBSON, decode, invalid, CodecOptions())
+            self.assertRaises(InvalidBSON, decode, invalid)
 
-        dec = decode(invalid_both, CodecOptions(unicode_decode_error_handler="ignore"))
-        self.assertEqual(dec, {ignored_key: ignored_val})
-
-        self.assertRaises(
-            InvalidBSON, decode, invalid_both, CodecOptions(unicode_decode_error_handler="strict")
-        )
-        self.assertRaises(InvalidBSON, decode, invalid_both, CodecOptions())
-        self.assertRaises(InvalidBSON, decode, invalid_both)
+        # Test all other error handlers.
+        for handler in ["replace", "backslashreplace", "surrogateescape", "ignore"]:
+            expected_key = b"ke\xe9str".decode("utf-8", handler)
+            expected_val = b"fo\xe9bar".decode("utf-8", handler)
+            doc = decode(invalid_key, CodecOptions(unicode_decode_error_handler=handler))
+            self.assertEqual(doc, {expected_key: "foobar"})
+            doc = decode(invalid_val, CodecOptions(unicode_decode_error_handler=handler))
+            self.assertEqual(doc, {"keystr": expected_val})
+            doc = decode(invalid_both, CodecOptions(unicode_decode_error_handler=handler))
+            self.assertEqual(doc, {expected_key: expected_val})
 
         # Test handling bad error mode.
         dec = decode(enc, CodecOptions(unicode_decode_error_handler="junk"))
@@ -1105,6 +1032,87 @@ class TestCodecOptions(unittest.TestCase):
             InvalidBSON, decode, invalid_both, CodecOptions(unicode_decode_error_handler="junk")
         )
 
+    def round_trip_pickle(self, obj, pickled_with_older):
+        pickled_with_older_obj = pickle.loads(pickled_with_older)
+        for protocol in range(pickle.HIGHEST_PROTOCOL + 1):
+            pkl = pickle.dumps(obj, protocol=protocol)
+            obj2 = pickle.loads(pkl)
+            self.assertEqual(obj, obj2)
+            self.assertEqual(pickled_with_older_obj, obj2)
+
+    def test_regex_pickling(self):
+        reg = Regex(".?")
+        pickled_with_3 = (
+            b"\x80\x04\x959\x00\x00\x00\x00\x00\x00\x00\x8c\n"
+            b"bson.regex\x94\x8c\x05Regex\x94\x93\x94)\x81\x94}"
+            b"\x94(\x8c\x07pattern\x94\x8c\x02.?\x94\x8c\x05flag"
+            b"s\x94K\x00ub."
+        )
+        self.round_trip_pickle(reg, pickled_with_3)
+
+    def test_timestamp_pickling(self):
+        ts = Timestamp(0, 1)
+        pickled_with_3 = (
+            b"\x80\x04\x95Q\x00\x00\x00\x00\x00\x00\x00\x8c"
+            b"\x0ebson.timestamp\x94\x8c\tTimestamp\x94\x93\x94)"
+            b"\x81\x94}\x94("
+            b"\x8c\x10_Timestamp__time\x94K\x00\x8c"
+            b"\x0f_Timestamp__inc\x94K\x01ub."
+        )
+        self.round_trip_pickle(ts, pickled_with_3)
+
+    def test_dbref_pickling(self):
+        dbr = DBRef("foo", 5)
+        pickled_with_3 = (
+            b"\x80\x04\x95q\x00\x00\x00\x00\x00\x00\x00\x8c\n"
+            b"bson.dbref\x94\x8c\x05DBRef\x94\x93\x94)\x81\x94}"
+            b"\x94(\x8c\x12_DBRef__collection\x94\x8c\x03foo\x94"
+            b"\x8c\n_DBRef__id\x94K\x05\x8c\x10_DBRef__database"
+            b"\x94N\x8c\x0e_DBRef__kwargs\x94}\x94ub."
+        )
+        self.round_trip_pickle(dbr, pickled_with_3)
+
+        dbr = DBRef("foo", 5, database="db", kwargs1=None)
+        pickled_with_3 = (
+            b"\x80\x04\x95\x81\x00\x00\x00\x00\x00\x00\x00\x8c"
+            b"\nbson.dbref\x94\x8c\x05DBRef\x94\x93\x94)\x81\x94}"
+            b"\x94(\x8c\x12_DBRef__collection\x94\x8c\x03foo\x94"
+            b"\x8c\n_DBRef__id\x94K\x05\x8c\x10_DBRef__database"
+            b"\x94\x8c\x02db\x94\x8c\x0e_DBRef__kwargs\x94}\x94"
+            b"\x8c\x07kwargs1\x94Nsub."
+        )
+
+        self.round_trip_pickle(dbr, pickled_with_3)
+
+    def test_minkey_pickling(self):
+        mink = MinKey()
+        pickled_with_3 = (
+            b"\x80\x04\x95\x1e\x00\x00\x00\x00\x00\x00\x00\x8c"
+            b"\x0cbson.min_key\x94\x8c\x06MinKey\x94\x93\x94)"
+            b"\x81\x94."
+        )
+
+        self.round_trip_pickle(mink, pickled_with_3)
+
+    def test_maxkey_pickling(self):
+        maxk = MaxKey()
+        pickled_with_3 = (
+            b"\x80\x04\x95\x1e\x00\x00\x00\x00\x00\x00\x00\x8c"
+            b"\x0cbson.max_key\x94\x8c\x06MaxKey\x94\x93\x94)"
+            b"\x81\x94."
+        )
+
+        self.round_trip_pickle(maxk, pickled_with_3)
+
+    def test_int64_pickling(self):
+        i64 = Int64(9)
+        pickled_with_3 = (
+            b"\x80\x04\x95\x1e\x00\x00\x00\x00\x00\x00\x00\x8c\n"
+            b"bson.int64\x94\x8c\x05Int64\x94\x93\x94K\t\x85\x94"
+            b"\x81\x94."
+        )
+        self.round_trip_pickle(i64, pickled_with_3)
+
 
 if __name__ == "__main__":
     unittest.main()
