@@ -22,7 +22,6 @@ import sys
 import threading
 import time
 import uuid
-from contextlib import contextmanager
 from itertools import product
 
 sys.path[0:0] = [""]
@@ -38,7 +37,6 @@ from test.utils import (
 
 from bson import SON, ObjectId, Timestamp, encode, json_util
 from bson.binary import ALL_UUID_REPRESENTATIONS, PYTHON_LEGACY, STANDARD, Binary
-from bson.py3compat import iteritems
 from bson.raw_bson import DEFAULT_RAW_BSON_OPTIONS, RawBSONDocument
 from pymongo import MongoClient
 from pymongo.command_cursor import CommandCursor
@@ -863,7 +861,9 @@ class TestDatabaseChangeStream(TestChangeStreamBase, APITestsMixin):
         collnames = self.generate_unique_collnames(3)
         with self.change_stream() as change_stream:
             for collname in collnames:
-                self._insert_and_check(change_stream, collname, {"_id": uuid.uuid4()})
+                self._insert_and_check(
+                    change_stream, collname, {"_id": Binary.from_uuid(uuid.uuid4())}
+                )
 
     def test_isolation(self):
         # Ensure inserts to other dbs don't show up in our ChangeStream.
@@ -871,8 +871,8 @@ class TestDatabaseChangeStream(TestChangeStreamBase, APITestsMixin):
         self.assertNotEqual(other_db, self.db, msg="Isolation must be tested on separate DBs")
         collname = self.id()
         with self.change_stream() as change_stream:
-            other_db[collname].insert_one({"_id": uuid.uuid4()})
-            self._insert_and_check(change_stream, collname, {"_id": uuid.uuid4()})
+            other_db[collname].insert_one({"_id": Binary.from_uuid(uuid.uuid4())})
+            self._insert_and_check(change_stream, collname, {"_id": Binary.from_uuid(uuid.uuid4())})
         self.client.drop_database(other_db)
 
 
@@ -1070,7 +1070,7 @@ class TestAllLegacyScenarios(IntegrationTest):
     def assert_dict_is_subset(self, superdict, subdict):
         """Check that subdict is a subset of superdict."""
         exempt_fields = ["documentKey", "_id", "getMore"]
-        for key, value in iteritems(subdict):
+        for key, value in subdict.items():
             if key not in superdict:
                 self.fail("Key %s not found in %s" % (key, superdict))
             if isinstance(value, dict):
@@ -1088,7 +1088,7 @@ class TestAllLegacyScenarios(IntegrationTest):
     def check_event(self, event, expectation_dict):
         if event is None:
             self.fail()
-        for key, value in iteritems(expectation_dict):
+        for key, value in expectation_dict.items():
             if isinstance(value, dict):
                 self.assert_dict_is_subset(getattr(event, key), value)
             else:
@@ -1124,7 +1124,7 @@ def get_change_stream(client, scenario_def, test):
     cs_pipeline = test["changeStreamPipeline"]
     options = test["changeStreamOptions"]
     cs_options = {}
-    for key, value in iteritems(options):
+    for key, value in options.items():
         cs_options[camel_to_snake(key)] = value
 
     # Create and return change stream
@@ -1181,7 +1181,7 @@ def create_test(scenario_def, test):
             # Note: expectations may be missing, null, or a list of events.
             # Extra events emitted by the test are intentionally ignored.
             for idx, expectation in enumerate(test.get("expectations") or []):
-                for event_type, event_desc in iteritems(expectation):
+                for event_type, event_desc in expectation.items():
                     results_key = event_type.split("_")[1]
                     event = results[results_key][idx] if len(results[results_key]) > idx else None
                     self.check_event(event, event_desc)
