@@ -14,6 +14,7 @@
 
 """Test the mongo_client module."""
 
+import _thread as thread
 import contextlib
 import copy
 import datetime
@@ -50,7 +51,6 @@ from test.utils import (
     delay,
     get_pool,
     gevent_monkey_patched,
-    ignore_deprecations,
     is_greenthread_patched,
     lazy_client_trial,
     one,
@@ -65,10 +65,10 @@ from test.utils import (
 import pymongo
 from bson import encode
 from bson.codec_options import CodecOptions, TypeEncoder, TypeRegistry
-from bson.py3compat import thread
 from bson.son import SON
 from bson.tz_util import utc
-from pymongo import auth, message, monitoring
+from pymongo import event_loggers, message, monitoring
+from pymongo.client_options import ClientOptions
 from pymongo.command_cursor import CommandCursor
 from pymongo.common import _UUID_REPRESENTATIONS, CONNECT_TIMEOUT
 from pymongo.compression_support import _HAVE_SNAPPY, _HAVE_ZSTD
@@ -80,17 +80,17 @@ from pymongo.errors import (
     ConfigurationError,
     ConnectionFailure,
     InvalidName,
+    InvalidOperation,
     InvalidURI,
     NetworkTimeout,
     OperationFailure,
     ServerSelectionTimeoutError,
     WriteConcernError,
 )
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import HelloCompat
 from pymongo.mongo_client import MongoClient
 from pymongo.monitoring import ServerHeartbeatListener, ServerHeartbeatStartedEvent
-from pymongo.monotonic import time as monotonic_time
-from pymongo.pool import _METADATA, SocketInfo, PoolOptions
+from pymongo.pool import _METADATA, PoolOptions, SocketInfo
 from pymongo.read_preferences import ReadPreference
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import readable_server_selector, writable_server_selector
@@ -98,7 +98,10 @@ from pymongo.server_type import SERVER_TYPE
 from pymongo.settings import TOPOLOGY_TYPE
 from pymongo.srv_resolver import _HAVE_DNSPYTHON
 from pymongo.topology import _ErrorContext
-from pymongo.topology_description import TopologyDescription
+from pymongo.topology_description import (
+    TopologyDescription,
+    _updated_topology_description_srv_polling,
+)
 from pymongo.write_concern import WriteConcern
 
 
@@ -119,14 +122,12 @@ class ClientUnitTest(unittest.TestCase):
             socketTimeoutMS=None,
             connectTimeoutMS=20000,
             waitQueueTimeoutMS=None,
-            waitQueueMultiple=None,
             replicaSet=None,
             read_preference=ReadPreference.PRIMARY,
             ssl=False,
-            ssl_keyfile=None,
-            ssl_certfile=None,
-            ssl_cert_reqs=0,  # ssl.CERT_NONE
-            ssl_ca_certs=None,
+            tlsCertificateKeyFile=None,
+            tlsAllowInvalidCertificates=True,
+            tlsCAFile=None,
             connect=False,
             serverSelectionTimeoutMS=12000,
         )
@@ -137,8 +138,6 @@ class ClientUnitTest(unittest.TestCase):
         # socket.Socket.settimeout takes a float in seconds
         self.assertEqual(20.0, pool_opts.connect_timeout)
         self.assertEqual(None, pool_opts.wait_queue_timeout)
-        self.assertEqual(None, pool_opts.wait_queue_multiple)
-        self.assertTrue(pool_opts.socket_keepalive)
         self.assertEqual(None, pool_opts._ssl_context)
         self.assertEqual(None, options.replica_set_name)
         self.assertEqual(ReadPreference.PRIMARY, client.read_preference)
@@ -170,8 +169,7 @@ class ClientUnitTest(unittest.TestCase):
         self.assertRaises(ConfigurationError, MongoClient, [])
 
     def test_max_pool_size_zero(self):
-        with self.assertRaises(ValueError):
-            MongoClient(maxPoolSize=0)
+        MongoClient(maxPoolSize=0)
 
     def test_uri_detection(self):
         self.assertRaises(ConfigurationError, MongoClient, "/foo")
@@ -286,10 +284,6 @@ class ClientUnitTest(unittest.TestCase):
         )
         self.assertEqual(c.read_preference, ReadPreference.NEAREST)
 
-    @unittest.skipIf(
-        sys.version_info[0] == 3 and sys.version_info[1] == 4,
-        "PYTHON-2442: workaround namedtuple._asdict() bug on Python 3.4",
-    )
     def test_metadata(self):
         metadata = copy.deepcopy(_METADATA)
         metadata["application"] = {"name": "foobar"}
@@ -304,6 +298,8 @@ class ClientUnitTest(unittest.TestCase):
         self.assertRaises(ValueError, MongoClient, appname="x" * 129)
         # Bad "driver" options.
         self.assertRaises(TypeError, DriverInfo, "Foo", 1, "a")
+        self.assertRaises(TypeError, DriverInfo, version="1", platform="a")
+        self.assertRaises(TypeError, DriverInfo)
         self.assertRaises(TypeError, MongoClient, driver=1)
         self.assertRaises(TypeError, MongoClient, driver="abc")
         self.assertRaises(TypeError, MongoClient, driver=("Foo", "1", "a"))
@@ -400,7 +396,7 @@ class ClientUnitTest(unittest.TestCase):
         clopts = c._MongoClient__options
         opts = clopts._options
 
-        self.assertEqual(opts["ssl"], False)
+        self.assertEqual(opts["tls"], False)
         self.assertEqual(clopts.replica_set_name, "newname")
         self.assertEqual(clopts.read_preference, ReadPreference.SECONDARY_PREFERRED)
 
@@ -450,7 +446,7 @@ class ClientUnitTest(unittest.TestCase):
 
         # Matching SSL and TLS options should not cause errors.
         c = MongoClient("mongodb://localhost/?ssl=false", tls=False, connect=False)
-        self.assertEqual(c._MongoClient__options._options["ssl"], False)
+        self.assertEqual(c._MongoClient__options._options["tls"], False)
 
         # Conflicting tlsInsecure options should raise an error.
         with self.assertRaises(InvalidURI):
@@ -462,14 +458,50 @@ class ClientUnitTest(unittest.TestCase):
 
         # Conflicting legacy tlsInsecure options should also raise an error.
         with self.assertRaises(InvalidURI):
-            MongoClient("mongodb://localhost/?tlsInsecure=true", connect=False, ssl_cert_reqs=True)
+            MongoClient(
+                "mongodb://localhost/?tlsInsecure=true",
+                connect=False,
+                tlsAllowInvalidCertificates=False,
+            )
 
         # Conflicting kwargs should raise InvalidURI
         with self.assertRaises(InvalidURI):
             MongoClient(ssl=True, tls=False)
 
+    def test_event_listeners(self):
+        c = MongoClient(event_listeners=[], connect=False)
+        self.assertEqual(c.options.event_listeners, [])
+        listeners = [
+            event_loggers.CommandLogger(),
+            event_loggers.HeartbeatLogger(),
+            event_loggers.ServerLogger(),
+            event_loggers.TopologyLogger(),
+            event_loggers.ConnectionPoolLogger(),
+        ]
+        c = MongoClient(event_listeners=listeners, connect=False)
+        self.assertEqual(c.options.event_listeners, listeners)
+
+    def test_client_options(self):
+        c = MongoClient(connect=False)
+        self.assertIsInstance(c.options, ClientOptions)
+        self.assertIsInstance(c.options.pool_options, PoolOptions)
+        self.assertEqual(c.options.server_selection_timeout, 30)
+        self.assertEqual(c.options.pool_options.max_idle_time_seconds, None)
+        self.assertIsInstance(c.options.retry_writes, bool)
+        self.assertIsInstance(c.options.retry_reads, bool)
+
 
 class TestClient(IntegrationTest):
+    def test_multiple_uris(self):
+        with self.assertRaises(ConfigurationError):
+            MongoClient(
+                host=[
+                    "mongodb+srv://cluster-a.abc12.mongodb.net",
+                    "mongodb+srv://cluster-b.abc12.mongodb.net",
+                    "mongodb+srv://cluster-c.abc12.mongodb.net",
+                ]
+            )
+
     def test_max_idle_time_reaper_default(self):
         with client_knobs(kill_cursor_frequency=0.1):
             # Assert reaper doesn't remove sockets when maxIdleTimeMS not set
@@ -609,20 +641,19 @@ class TestClient(IntegrationTest):
         c = rs_or_single_client(connect=False)
         self.assertIsInstance(c.is_mongos, bool)
         c = rs_or_single_client(connect=False)
-        self.assertIsInstance(c.max_pool_size, int)
+        self.assertIsInstance(c.options.pool_options.max_pool_size, int)
         self.assertIsInstance(c.nodes, frozenset)
 
         c = rs_or_single_client(connect=False)
         self.assertEqual(c.codec_options, CodecOptions())
-        self.assertIsInstance(c.max_bson_size, int)
         c = rs_or_single_client(connect=False)
         self.assertFalse(c.primary)
         self.assertFalse(c.secondaries)
         c = rs_or_single_client(connect=False)
-        self.assertIsInstance(c.max_write_batch_size, int)
         self.assertIsInstance(c.topology_description, TopologyDescription)
         self.assertEqual(c.topology_description, c._topology._description)
-
+        self.assertIsNone(c.address)  # PYTHON-2981
+        c.admin.command("ping")  # connect
         if client_context.is_rs:
             # The primary's host and port are from the replica set config.
             self.assertIsNotNone(c.address)
@@ -639,15 +670,32 @@ class TestClient(IntegrationTest):
         self.assertRaises(ConnectionFailure, c.pymongo_test.test.find_one)
 
     def test_equality(self):
-        c = connected(rs_or_single_client())
+        seed = "%s:%s" % list(self.client._topology_settings.seeds)[0]
+        c = rs_or_single_client(seed, connect=False)
+        self.addCleanup(c.close)
         self.assertEqual(client_context.client, c)
-
         # Explicitly test inequality
         self.assertFalse(client_context.client != c)
 
+        c = rs_or_single_client("invalid.com", connect=False)
+        self.addCleanup(c.close)
+        self.assertNotEqual(client_context.client, c)
+        self.assertTrue(client_context.client != c)
+        # Seeds differ:
+        self.assertNotEqual(MongoClient("a", connect=False), MongoClient("b", connect=False))
+        # Same seeds but out of order still compares equal:
+        self.assertEqual(
+            MongoClient(["a", "b", "c"], connect=False), MongoClient(["c", "a", "b"], connect=False)
+        )
+
     def test_hashable(self):
-        c = connected(rs_or_single_client())
+        seed = "%s:%s" % list(self.client._topology_settings.seeds)[0]
+        c = rs_or_single_client(seed, connect=False)
+        self.addCleanup(c.close)
         self.assertIn(c, {client_context.client})
+        c = rs_or_single_client("invalid.com", connect=False)
+        self.addCleanup(c.close)
+        self.assertNotIn(c, {client_context.client})
 
     def test_host_w_port(self):
         with self.assertRaises(ValueError):
@@ -718,82 +766,65 @@ class TestClient(IntegrationTest):
         for doc in client.list_databases():
             self.assertIs(type(doc), dict)
 
-        if client_context.version.at_least(3, 4, 2):
-            self.client.pymongo_test.test.insert_one({})
-            cursor = self.client.list_databases(filter={"name": "admin"})
-            docs = list(cursor)
-            self.assertEqual(1, len(docs))
-            self.assertEqual(docs[0]["name"], "admin")
+        self.client.pymongo_test.test.insert_one({})
+        cursor = self.client.list_databases(filter={"name": "admin"})
+        docs = list(cursor)
+        self.assertEqual(1, len(docs))
+        self.assertEqual(docs[0]["name"], "admin")
 
-        if client_context.version.at_least(3, 4, 3):
-            cursor = self.client.list_databases(nameOnly=True)
-            for doc in cursor:
-                self.assertEqual(["name"], list(doc))
+        cursor = self.client.list_databases(nameOnly=True)
+        for doc in cursor:
+            self.assertEqual(["name"], list(doc))
 
-    def _test_list_names(self, meth):
-        self.client.pymongo_test.test.insert_one({"dummy": u"object"})
-        self.client.pymongo_test_mike.test.insert_one({"dummy": u"object"})
+    def test_list_database_names(self):
+        self.client.pymongo_test.test.insert_one({"dummy": "object"})
+        self.client.pymongo_test_mike.test.insert_one({"dummy": "object"})
         cmd_docs = self.client.admin.command("listDatabases")["databases"]
         cmd_names = [doc["name"] for doc in cmd_docs]
 
-        db_names = meth()
+        db_names = self.client.list_database_names()
         self.assertTrue("pymongo_test" in db_names)
         self.assertTrue("pymongo_test_mike" in db_names)
         self.assertEqual(db_names, cmd_names)
 
-    def test_list_database_names(self):
-        self._test_list_names(self.client.list_database_names)
-
-    def test_database_names(self):
-        self._test_list_names(self.client.database_names)
-
     def test_drop_database(self):
         self.assertRaises(TypeError, self.client.drop_database, 5)
         self.assertRaises(TypeError, self.client.drop_database, None)
 
-        self.client.pymongo_test.test.insert_one({"dummy": u"object"})
-        self.client.pymongo_test2.test.insert_one({"dummy": u"object"})
+        self.client.pymongo_test.test.insert_one({"dummy": "object"})
+        self.client.pymongo_test2.test.insert_one({"dummy": "object"})
         dbs = self.client.list_database_names()
         self.assertIn("pymongo_test", dbs)
         self.assertIn("pymongo_test2", dbs)
         self.client.drop_database("pymongo_test")
 
-        if client_context.version.at_least(3, 3, 9) and client_context.is_rs:
+        if client_context.is_rs:
             wc_client = rs_or_single_client(w=len(client_context.nodes) + 1)
             with self.assertRaises(WriteConcernError):
                 wc_client.drop_database("pymongo_test2")
 
         self.client.drop_database(self.client.pymongo_test2)
-
-        raise SkipTest("This test often fails due to SERVER-2329")
-
         dbs = self.client.list_database_names()
         self.assertNotIn("pymongo_test", dbs)
         self.assertNotIn("pymongo_test2", dbs)
 
     def test_close(self):
-        coll = self.client.pymongo_test.bar
-
-        self.client.close()
-        self.client.close()
-
-        coll.count_documents({})
-
-        self.client.close()
-        self.client.close()
-
-        coll.count_documents({})
+        test_client = rs_or_single_client()
+        coll = test_client.pymongo_test.bar
+        test_client.close()
+        self.assertRaises(InvalidOperation, coll.count_documents, {})
 
     def test_close_kills_cursors(self):
         if sys.platform.startswith("java"):
             # We can't figure out how to make this test reliable with Jython.
             raise SkipTest("Can't test with Jython")
+        test_client = rs_or_single_client()
         # Kill any cursors possibly queued up by previous tests.
         gc.collect()
-        self.client._process_periodic_tasks()
+        test_client._process_periodic_tasks()
 
         # Add some test data.
-        coll = self.client.pymongo_test.test_close_kills_cursors
+        coll = test_client.pymongo_test.test_close_kills_cursors
         docs_inserted = 1000
         coll.insert_many([{"i": i} for i in range(docs_inserted)])
 
@@ -811,13 +842,13 @@ class TestClient(IntegrationTest):
         gc.collect()
 
         # Close the client and ensure the topology is closed.
-        self.assertTrue(self.client._topology._opened)
-        self.client.close()
-        self.assertFalse(self.client._topology._opened)
-
+        self.assertTrue(test_client._topology._opened)
+        test_client.close()
+        self.assertFalse(test_client._topology._opened)
+        test_client = rs_or_single_client()
         # The killCursors task should not need to re-open the topology.
-        self.client._process_periodic_tasks()
-        self.assertFalse(self.client._topology._opened)
+        test_client._process_periodic_tasks()
+        self.assertTrue(test_client._topology._opened)
 
     def test_close_stops_kill_cursors_thread(self):
         client = rs_client()
@@ -828,12 +859,9 @@ class TestClient(IntegrationTest):
         client.close()
         self.assertTrue(client._kill_cursors_executor._stopped)
 
-        # Reusing the closed client should restart the thread.
-        client.admin.command("ping")
-        self.assertFalse(client._kill_cursors_executor._stopped)
-
-        # Again, closing the client should stop the thread.
-        client.close()
+        # Reusing the closed client should raise an InvalidOperation error.
+        self.assertRaises(InvalidOperation, client.admin.command, "ping")
+        # Thread is still stopped.
         self.assertTrue(client._kill_cursors_executor._stopped)
 
     def test_uri_connect_option(self):
@@ -934,35 +962,6 @@ class TestClient(IntegrationTest):
         with self.assertRaises(OperationFailure):
             rs_or_single_client_noauth(username="ad min", password="foo").server_info()
 
-    @client_context.require_auth
-    @ignore_deprecations
-    def test_multiple_logins(self):
-        client_context.create_user("pymongo_test", "user1", "pass", roles=["readWrite"])
-        client_context.create_user("pymongo_test", "user2", "pass", roles=["readWrite"])
-        self.addCleanup(remove_all_users, self.client.pymongo_test)
-
-        client = rs_or_single_client_noauth(
-            "mongodb://user1:pass@%s:%d/pymongo_test" % (client_context.host, client_context.port)
-        )
-
-        client.pymongo_test.test.find_one()
-        with self.assertRaises(OperationFailure):
-            # Can't log in to the same database with multiple users.
-            client.pymongo_test.authenticate("user2", "pass")
-
-        client.pymongo_test.test.find_one()
-        client.pymongo_test.logout()
-        with self.assertRaises(OperationFailure):
-            client.pymongo_test.test.find_one()
-
-        client.pymongo_test.authenticate("user2", "pass")
-        client.pymongo_test.test.find_one()
-
-        with self.assertRaises(OperationFailure):
-            client.pymongo_test.authenticate("user1", "pass")
-
-        client.pymongo_test.test.find_one()
-
     @client_context.require_auth
     def test_lazy_auth_raises_operation_failure(self):
         lazy_client = rs_or_single_client_noauth(
@@ -1022,8 +1021,8 @@ class TestClient(IntegrationTest):
         self.assertEqual(10.5, get_pool(client).opts.connect_timeout)
         self.assertEqual(10.5, get_pool(client).opts.socket_timeout)
         self.assertEqual(10.5, get_pool(client).opts.max_idle_time_seconds)
-        self.assertEqual(10500, client.max_idle_time_ms)
-        self.assertEqual(10.5, client.server_selection_timeout)
+        self.assertEqual(10.5, client.options.pool_options.max_idle_time_seconds)
+        self.assertEqual(10.5, client.options.server_selection_timeout)
 
     def test_socket_timeout_ms_validation(self):
         c = rs_or_single_client(socketTimeoutMS=10 * 1000)
@@ -1061,10 +1060,10 @@ class TestClient(IntegrationTest):
 
     def test_server_selection_timeout(self):
         client = MongoClient(serverSelectionTimeoutMS=100, connect=False)
-        self.assertAlmostEqual(0.1, client.server_selection_timeout)
+        self.assertAlmostEqual(0.1, client.options.server_selection_timeout)
 
         client = MongoClient(serverSelectionTimeoutMS=0, connect=False)
-        self.assertAlmostEqual(0, client.server_selection_timeout)
+        self.assertAlmostEqual(0, client.options.server_selection_timeout)
 
         self.assertRaises(ValueError, MongoClient, serverSelectionTimeoutMS="foo", connect=False)
         self.assertRaises(ValueError, MongoClient, serverSelectionTimeoutMS=-1, connect=False)
@@ -1073,41 +1072,27 @@ class TestClient(IntegrationTest):
         )
 
         client = MongoClient("mongodb://localhost/?serverSelectionTimeoutMS=100", connect=False)
-        self.assertAlmostEqual(0.1, client.server_selection_timeout)
+        self.assertAlmostEqual(0.1, client.options.server_selection_timeout)
 
         client = MongoClient("mongodb://localhost/?serverSelectionTimeoutMS=0", connect=False)
-        self.assertAlmostEqual(0, client.server_selection_timeout)
+        self.assertAlmostEqual(0, client.options.server_selection_timeout)
 
         # Test invalid timeout in URI ignored and set to default.
         client = MongoClient("mongodb://localhost/?serverSelectionTimeoutMS=-1", connect=False)
-        self.assertAlmostEqual(30, client.server_selection_timeout)
+        self.assertAlmostEqual(30, client.options.server_selection_timeout)
 
         client = MongoClient("mongodb://localhost/?serverSelectionTimeoutMS=", connect=False)
-        self.assertAlmostEqual(30, client.server_selection_timeout)
+        self.assertAlmostEqual(30, client.options.server_selection_timeout)
 
     def test_waitQueueTimeoutMS(self):
         client = rs_or_single_client(waitQueueTimeoutMS=2000)
         self.assertEqual(get_pool(client).opts.wait_queue_timeout, 2)
 
-    def test_waitQueueMultiple(self):
-        client = rs_or_single_client(maxPoolSize=3, waitQueueMultiple=2)
-        pool = get_pool(client)
-        self.assertEqual(pool.opts.wait_queue_multiple, 2)
-        self.assertEqual(pool._socket_semaphore.waiter_semaphore.counter, 6)
-
     def test_socketKeepAlive(self):
-        for socketKeepAlive in [True, False]:
-            with warnings.catch_warnings(record=True) as ctx:
-                warnings.simplefilter("always")
-                client = rs_or_single_client(socketKeepAlive=socketKeepAlive)
-                self.assertTrue(
-                    any("The socketKeepAlive option is deprecated" in str(k) for k in ctx)
-                )
-                pool = get_pool(client)
-                self.assertEqual(socketKeepAlive, pool.opts.socket_keepalive)
-                with pool.get_socket({}) as sock_info:
-                    keepalive = sock_info.sock.getsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE)
-                    self.assertEqual(socketKeepAlive, bool(keepalive))
+        pool = get_pool(self.client)
+        with pool.get_socket({}) as sock_info:
+            keepalive = sock_info.sock.getsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE)
+            self.assertTrue(keepalive)
 
     def test_tz_aware(self):
         self.assertRaises(ValueError, MongoClient, tz_aware="foo")
@@ -1142,45 +1127,13 @@ class TestClient(IntegrationTest):
             uri += "/?replicaSet=" + client_context.replica_set_name
 
         client = rs_or_single_client_noauth(uri)
-        client.pymongo_test.test.insert_one({"dummy": u"object"})
-        client.pymongo_test_bernie.test.insert_one({"dummy": u"object"})
+        client.pymongo_test.test.insert_one({"dummy": "object"})
+        client.pymongo_test_bernie.test.insert_one({"dummy": "object"})
 
         dbs = client.list_database_names()
         self.assertTrue("pymongo_test" in dbs)
         self.assertTrue("pymongo_test_bernie" in dbs)
 
-    @ignore_deprecations
-    @client_context.require_no_mongos
-    def test_fsync_lock_unlock(self):
-        self.assertFalse(self.client.is_locked)
-        # async flushing not supported on windows...
-        if sys.platform not in ("cygwin", "win32"):
-            # Work around async becoming a reserved keyword in Python 3.7
-            opts = {"async": True}
-            self.client.fsync(**opts)
-            self.assertFalse(self.client.is_locked)
-        self.client.fsync(lock=True)
-        self.assertTrue(self.client.is_locked)
-        locked = True
-        self.client.unlock()
-        for _ in range(5):
-            locked = self.client.is_locked
-            if not locked:
-                break
-            time.sleep(1)
-        self.assertFalse(locked)
-
-    def test_deprecated_methods(self):
-        with warnings.catch_warnings():
-            warnings.simplefilter("error", DeprecationWarning)
-            with self.assertRaisesRegex(DeprecationWarning, "is_locked is deprecated"):
-                _ = self.client.is_locked
-            if not client_context.is_mongos:
-                with self.assertRaisesRegex(DeprecationWarning, "fsync is deprecated"):
-                    self.client.fsync(lock=True)
-                with self.assertRaisesRegex(DeprecationWarning, "unlock is deprecated"):
-                    self.client.unlock()
-
     def test_contextlib(self):
         client = rs_or_single_client()
         client.pymongo_test.drop_collection("test")
@@ -1192,12 +1145,13 @@ class TestClient(IntegrationTest):
 
         with contextlib.closing(client):
             self.assertEqual("bar", client.pymongo_test.test.find_one()["foo"])
-            self.assertEqual(1, len(get_pool(client).sockets))
-        self.assertEqual(0, len(get_pool(client).sockets))
-
+        with self.assertRaises(InvalidOperation):
+            client.pymongo_test.test.find_one()
+        client = rs_or_single_client()
         with client as client:
             self.assertEqual("bar", client.pymongo_test.test.find_one()["foo"])
-        self.assertEqual(0, len(get_pool(client).sockets))
+        with self.assertRaises(InvalidOperation):
+            client.pymongo_test.test.find_one()
 
     def test_interrupt_signal(self):
         if sys.platform.startswith("java"):
@@ -1333,7 +1287,7 @@ class TestClient(IntegrationTest):
         self.assertTrue(sock_info.closed)
 
         # The semaphore was decremented despite the error.
-        self.assertTrue(pool._socket_semaphore.acquire(blocking=False))
+        self.assertEqual(0, pool.requests)
 
     @client_context.require_auth
     def test_auth_network_error(self):
@@ -1343,11 +1297,6 @@ class TestClient(IntegrationTest):
         # Get a client with one socket so we detect if it's leaked.
         c = connected(rs_or_single_client(maxPoolSize=1, waitQueueTimeoutMS=1, retryReads=False))
 
-        # Simulate an authenticate() call on a different socket.
-        credentials = auth._build_credentials_tuple("DEFAULT", "admin", db_user, db_pwd, {}, None)
-
-        c._cache_credentials("test", credentials, connect=False)
-
         # Cause a network error on the actual socket.
         pool = get_pool(c)
         socket_info = one(pool.sockets)
@@ -1592,9 +1541,9 @@ class TestClient(IntegrationTest):
 
         # Assert that application operations do not block.
         for _ in range(10):
-            start = monotonic_time()
+            start = time.monotonic()
             client.admin.command("ping")
-            total = monotonic_time() - start
+            total = time.monotonic() - start
             # Each ping command should not take more than 2 seconds
             self.assertLess(total, 2)
 
@@ -1637,7 +1586,7 @@ class TestClient(IntegrationTest):
         gc.collect()
         with client_knobs(min_heartbeat_interval=0.003):
             client = MongoClient(
-                "invalid:27017", heartbeatFrequencyMS=3, serverSelectionTimeoutMS=150
+                "invalid:27017", heartbeatFrequencyMS=3, serverSelectionTimeoutMS=100
             )
             initial_count = server_description_count()
             self.addCleanup(client.close)
@@ -1647,14 +1596,72 @@ class TestClient(IntegrationTest):
             final_count = server_description_count()
             # If a bug like PYTHON-2433 is reintroduced then too many
             # ServerDescriptions will be kept alive and this test will fail:
-            # AssertionError: 19 != 46 within 15 delta (27 difference)
-            # On Python 3.11 we seem to get more of a delta.
-            self.assertAlmostEqual(initial_count, final_count, delta=20)
+            # AssertionError: 4 != 22 within 5 delta (18 difference)
+            self.assertAlmostEqual(initial_count, final_count, delta=10)
 
-    @unittest.skipIf(_HAVE_DNSPYTHON, "dnspython must not be installed")
-    def test_srv_no_dnspython_error(self):
-        with self.assertRaisesRegex(ConfigurationError, 'The "dnspython" module must be'):
-            MongoClient("mongodb+srv://test1.test.build.10gen.cc/")
+    @client_context.require_failCommand_fail_point
+    def test_network_error_message(self):
+        client = single_client(retryReads=False)
+        self.addCleanup(client.close)
+        client.admin.command("ping")  # connect
+        with self.fail_point(
+            {"mode": {"times": 1}, "data": {"closeConnection": True, "failCommands": ["find"]}}
+        ):
+            expected = "%s:%s: " % client.address
+            with self.assertRaisesRegex(AutoReconnect, expected):
+                client.pymongo_test.test.find_one({})
+
+    @unittest.skipIf("PyPy" in sys.version, "PYTHON-2938 could fail on PyPy")
+    def test_process_periodic_tasks(self):
+        client = rs_or_single_client()
+        coll = client.db.collection
+        coll.insert_many([{} for _ in range(5)])
+        cursor = coll.find(batch_size=2)
+        cursor.next()
+        c_id = cursor.cursor_id
+        self.assertIsNotNone(c_id)
+        client.close()
+        # Add cursor to kill cursors queue
+        del cursor
+        wait_until(
+            lambda: client._MongoClient__kill_cursors_queue,
+            "waited for cursor to be added to queue",
+        )
+        client._process_periodic_tasks()  # This must not raise or print any exceptions
+        with self.assertRaises(InvalidOperation):
+            coll.insert_many([{} for _ in range(5)])
+
+    @unittest.skipUnless(_HAVE_DNSPYTHON, "DNS-related tests need dnspython to be installed")
+    def test_service_name_from_kwargs(self):
+        client = MongoClient(
+            "mongodb+srv://user:password@test22.test.build.10gen.cc",
+            srvServiceName="customname",
+            connect=False,
+        )
+        self.assertEqual(client._topology_settings.srv_service_name, "customname")
+        client = MongoClient(
+            "mongodb+srv://user:password@test22.test.build.10gen.cc"
+            "/?srvServiceName=shouldbeoverriden",
+            srvServiceName="customname",
+            connect=False,
+        )
+        self.assertEqual(client._topology_settings.srv_service_name, "customname")
+        client = MongoClient(
+            "mongodb+srv://user:password@test22.test.build.10gen.cc" "/?srvServiceName=customname",
+            connect=False,
+        )
+        self.assertEqual(client._topology_settings.srv_service_name, "customname")
+
+    @unittest.skipUnless(_HAVE_DNSPYTHON, "DNS-related tests need dnspython to be installed")
+    def test_srv_max_hosts_kwarg(self):
+        client = MongoClient("mongodb+srv://test1.test.build.10gen.cc/")
+        self.assertGreater(len(client.topology_description.server_descriptions()), 1)
+        client = MongoClient("mongodb+srv://test1.test.build.10gen.cc/", srvmaxhosts=1)
+        self.assertEqual(len(client.topology_description.server_descriptions()), 1)
+        client = MongoClient(
+            "mongodb+srv://test1.test.build.10gen.cc/?srvMaxHosts=1", srvmaxhosts=2
+        )
+        self.assertEqual(len(client.topology_description.server_descriptions()), 2)
 
 
 class TestExhaustCursor(IntegrationTest):
@@ -1685,7 +1692,7 @@ class TestExhaustCursor(IntegrationTest):
 
         # The socket was checked in and the semaphore was decremented.
         self.assertIn(sock_info, pool.sockets)
-        self.assertTrue(pool._socket_semaphore.acquire(blocking=False))
+        self.assertEqual(0, pool.requests)
 
     def test_exhaust_getmore_server_error(self):
         # When doing a getmore on an exhaust cursor, the socket stays checked
@@ -1743,7 +1750,7 @@ class TestExhaustCursor(IntegrationTest):
 
         # The socket was closed and the semaphore was decremented.
         self.assertNotIn(sock_info, pool.sockets)
-        self.assertTrue(pool._socket_semaphore.acquire(blocking=False))
+        self.assertEqual(0, pool.requests)
 
     def test_exhaust_getmore_network_error(self):
         # When doing a getmore on an exhaust cursor, the socket stays checked
@@ -1770,7 +1777,7 @@ class TestExhaustCursor(IntegrationTest):
 
         # The socket was closed and the semaphore was decremented.
         self.assertNotIn(sock_info, pool.sockets)
-        self.assertTrue(pool._socket_semaphore.acquire(blocking=False))
+        self.assertEqual(0, pool.requests)
 
 
 class TestClientLazyConnect(IntegrationTest):
@@ -1834,48 +1841,28 @@ class TestClientLazyConnect(IntegrationTest):
 
         lazy_client_trial(reset, find_one, test, self._get_client)
 
-    def test_max_bson_size(self):
-        c = self._get_client()
-
-        # max_bson_size will cause the client to connect.
-        hello = c.db.command(HelloCompat.LEGACY_CMD)
-        self.assertEqual(hello["maxBsonObjectSize"], c.max_bson_size)
-        if "maxMessageSizeBytes" in hello:
-            self.assertEqual(hello["maxMessageSizeBytes"], c.max_message_size)
-
 
 class TestMongoClientFailover(MockClientTest):
     def test_discover_primary(self):
-        # Disable background refresh.
-        with client_knobs(heartbeat_frequency=999999):
-            c = MockClient(
-                standalones=[],
-                members=["a:1", "b:2", "c:3"],
-                mongoses=[],
-                host="b:2",  # Pass a secondary.
-                replicaSet="rs",
-            )
-            self.addCleanup(c.close)
+        c = MockClient(
+            standalones=[],
+            members=["a:1", "b:2", "c:3"],
+            mongoses=[],
+            host="b:2",  # Pass a secondary.
+            replicaSet="rs",
+            heartbeatFrequencyMS=500,
+        )
+        self.addCleanup(c.close)
 
-            wait_until(lambda: len(c.nodes) == 3, "connect")
-            self.assertEqual(c.address, ("a", 1))
+        wait_until(lambda: len(c.nodes) == 3, "connect")
 
-            # Fail over.
-            c.kill_host("a:1")
-            c.mock_primary = "b:2"
-
-            c.close()
-            self.assertEqual(0, len(c.nodes))
-
-            t = c._get_topology()
-            t.select_servers(writable_server_selector)  # Reconnect.
-            self.assertEqual(c.address, ("b", 2))
-
-            # a:1 not longer in nodes.
-            self.assertLess(len(c.nodes), 3)
-
-            # c:3 is rediscovered.
-            t.select_server_by_address(("c", 3))
+        self.assertEqual(c.address, ("a", 1))
+        # Fail over.
+        c.kill_host("a:1")
+        c.mock_primary = "b:2"
+        wait_until(lambda: c.address == ("b", 2), "wait for server " "address to be " "updated")
+        # a:1 not longer in nodes.
+        self.assertLess(len(c.nodes), 3)
 
     def test_reconnect(self):
         # Verify the node list isn't forgotten during a network failure.
@@ -2082,6 +2069,8 @@ class TestClientPool(MockClientTest):
         # Assert that we do not create connections to unknown servers.
         arbiter = c._topology.get_server_by_address(("d", 4))
         self.assertFalse(arbiter.pool.sockets)
+        # Arbiter pool is not marked ready.
+        self.assertEqual(listener.event_count(monitoring.PoolReadyEvent), 2)
 
     @client_context.require_connection
     def test_direct_client_maintains_pool_to_arbiter(self):
@@ -2105,6 +2094,8 @@ class TestClientPool(MockClientTest):
         self.assertEqual(listener.event_count(monitoring.ConnectionCreatedEvent), 1)
         arbiter = c._topology.get_server_by_address(("c", 3))
         self.assertEqual(len(arbiter.pool.sockets), 1)
+        # Arbiter pool is marked ready.
+        self.assertEqual(listener.event_count(monitoring.PoolReadyEvent), 1)
 
 
 if __name__ == "__main__":
