@@ -17,7 +17,7 @@
 import functools
 import warnings
 from test import IntegrationTest, client_context, unittest
-from test.utils import EventListener, ignore_deprecations, rs_or_single_client
+from test.utils import EventListener, rs_or_single_client
 
 from pymongo.collation import (
     Collation,
@@ -87,18 +87,6 @@ class TestCollationObject(unittest.TestCase):
         )
 
 
-def raisesConfigurationErrorForOldMongoDB(func):
-    @functools.wraps(func)
-    def wrapper(self, *args, **kwargs):
-        if client_context.version.at_least(3, 3, 9):
-            return func(self, *args, **kwargs)
-        else:
-            with self.assertRaises(ConfigurationError):
-                return func(self, *args, **kwargs)
-
-    return wrapper
-
-
 class TestCollation(IntegrationTest):
     @classmethod
     @client_context.require_connection
@@ -129,7 +117,6 @@ class TestCollation(IntegrationTest):
     def assertCollationInLastCommand(self):
         self.assertEqual(self.collation.document, self.last_command_started()["collation"])
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_create_collection(self):
         self.db.test.drop()
         self.db.create_collection("test", collation=self.collation)
@@ -145,39 +132,19 @@ class TestCollation(IntegrationTest):
         model = IndexModel([("a", 1), ("b", -1)], collation=self.collation)
         self.assertEqual(self.collation.document, model.document["collation"])
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_create_index(self):
         self.db.test.create_index("foo", collation=self.collation)
         ci_cmd = self.listener.results["started"][0].command
         self.assertEqual(self.collation.document, ci_cmd["indexes"][0]["collation"])
 
-    @raisesConfigurationErrorForOldMongoDB
-    def test_ensure_index(self):
-        self.db.test.ensure_index("foo", collation=self.collation)
-        ci_cmd = self.listener.results["started"][0].command
-        self.assertEqual(self.collation.document, ci_cmd["indexes"][0]["collation"])
-
-    @raisesConfigurationErrorForOldMongoDB
     def test_aggregate(self):
         self.db.test.aggregate([{"$group": {"_id": 42}}], collation=self.collation)
         self.assertCollationInLastCommand()
 
-    @raisesConfigurationErrorForOldMongoDB
-    @ignore_deprecations
-    def test_count(self):
-        self.db.test.count(collation=self.collation)
-        self.assertCollationInLastCommand()
-
-        self.listener.results.clear()
-        self.db.test.find(collation=self.collation).count()
-        self.assertCollationInLastCommand()
-
-    @raisesConfigurationErrorForOldMongoDB
     def test_count_documents(self):
         self.db.test.count_documents({}, collation=self.collation)
         self.assertCollationInLastCommand()
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_distinct(self):
         self.db.test.distinct("foo", collation=self.collation)
         self.assertCollationInLastCommand()
@@ -186,14 +153,12 @@ class TestCollation(IntegrationTest):
         self.db.test.find(collation=self.collation).distinct("foo")
         self.assertCollationInLastCommand()
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_find_command(self):
         self.db.test.insert_one({"is this thing on?": True})
         self.listener.results.clear()
         next(self.db.test.find(collation=self.collation))
         self.assertCollationInLastCommand()
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_explain_command(self):
         self.listener.results.clear()
         self.db.test.find(collation=self.collation).explain()
@@ -202,26 +167,6 @@ class TestCollation(IntegrationTest):
             self.collation.document, self.last_command_started()["explain"]["collation"]
         )
 
-    @raisesConfigurationErrorForOldMongoDB
-    @client_context.require_version_max(4, 1, 0, -1)
-    def test_group(self):
-        self.db.test.group(
-            "foo",
-            {"foo": {"$gt": 42}},
-            {},
-            "function(a, b) { return a; }",
-            collation=self.collation,
-        )
-        self.assertCollationInLastCommand()
-
-    @raisesConfigurationErrorForOldMongoDB
-    def test_map_reduce(self):
-        self.db.test.map_reduce(
-            "function() {}", "function() {}", "output", collation=self.collation
-        )
-        self.assertCollationInLastCommand()
-
-    @raisesConfigurationErrorForOldMongoDB
     def test_delete(self):
         self.db.test.delete_one({"foo": 42}, collation=self.collation)
         command = self.listener.results["started"][0].command
@@ -232,23 +177,7 @@ class TestCollation(IntegrationTest):
         command = self.listener.results["started"][0].command
         self.assertEqual(self.collation.document, command["deletes"][0]["collation"])
 
-        self.listener.results.clear()
-        self.db.test.remove({"foo": 42}, collation=self.collation)
-        command = self.listener.results["started"][0].command
-        self.assertEqual(self.collation.document, command["deletes"][0]["collation"])
-
-    @raisesConfigurationErrorForOldMongoDB
     def test_update(self):
-        self.db.test.update({"foo": 42}, {"$set": {"foo": "bar"}}, collation=self.collation)
-        command = self.listener.results["started"][0].command
-        self.assertEqual(self.collation.document, command["updates"][0]["collation"])
-
-        self.listener.results.clear()
-        self.db.test.save({"_id": 12345}, collation=self.collation)
-        command = self.listener.results["started"][0].command
-        self.assertEqual(self.collation.document, command["updates"][0]["collation"])
-
-        self.listener.results.clear()
         self.db.test.replace_one({"foo": 42}, {"foo": 43}, collation=self.collation)
         command = self.listener.results["started"][0].command
         self.assertEqual(self.collation.document, command["updates"][0]["collation"])
@@ -263,12 +192,7 @@ class TestCollation(IntegrationTest):
         command = self.listener.results["started"][0].command
         self.assertEqual(self.collation.document, command["updates"][0]["collation"])
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_find_and(self):
-        self.db.test.find_and_modify({"foo": 42}, {"$set": {"foo": 43}}, collation=self.collation)
-        self.assertCollationInLastCommand()
-
-        self.listener.results.clear()
         self.db.test.find_one_and_delete({"foo": 42}, collation=self.collation)
         self.assertCollationInLastCommand()
 
@@ -282,7 +206,6 @@ class TestCollation(IntegrationTest):
         self.db.test.find_one_and_replace({"foo": 42}, {"foo": 43}, collation=self.collation)
         self.assertCollationInLastCommand()
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_bulk_write(self):
         self.db.test.collection.bulk_write(
             [
@@ -314,44 +237,6 @@ class TestCollation(IntegrationTest):
         check_ops(delete_cmd["deletes"])
         check_ops(update_cmd["updates"])
 
-    @raisesConfigurationErrorForOldMongoDB
-    def test_bulk(self):
-        bulk = self.db.test.initialize_ordered_bulk_op()
-        bulk.find({"noCollation": 42}).remove_one()
-        bulk.find({"noCollation": 42}).remove()
-        bulk.find({"foo": 42}, collation=self.collation).remove_one()
-        bulk.find({"foo": 42}, collation=self.collation).remove()
-        bulk.find({"noCollation": 24}).replace_one({"bar": 42})
-        bulk.find({"noCollation": 84}).upsert().update_one({"$set": {"foo": 10}})
-        bulk.find({"noCollation": 45}).update({"$set": {"bar": 42}})
-        bulk.find({"foo": 24}, collation=self.collation).replace_one({"foo": 42})
-        bulk.find({"foo": 84}, collation=self.collation).upsert().update_one({"$set": {"foo": 10}})
-        bulk.find({"foo": 45}, collation=self.collation).update({"$set": {"foo": 42}})
-        bulk.execute()
-
-        delete_cmd = self.listener.results["started"][0].command
-        update_cmd = self.listener.results["started"][1].command
-
-        def check_ops(ops):
-            for op in ops:
-                if "noCollation" in op["q"]:
-                    self.assertNotIn("collation", op)
-                else:
-                    self.assertEqual(self.collation.document, op["collation"])
-
-        check_ops(delete_cmd["deletes"])
-        check_ops(update_cmd["updates"])
-
-    @client_context.require_version_max(3, 3, 8)
-    def test_mixed_bulk_collation(self):
-        bulk = self.db.test.initialize_unordered_bulk_op()
-        bulk.find({"foo": 42}).upsert().update_one({"$set": {"bar": 10}})
-        bulk.find({"foo": 43}, collation=self.collation).remove_one()
-        with self.assertRaises(ConfigurationError):
-            bulk.execute()
-        self.assertIsNone(self.db.test.find_one({"foo": 42}))
-
-    @raisesConfigurationErrorForOldMongoDB
     def test_indexes_same_keys_different_collations(self):
         self.db.test.drop()
         usa_collation = Collation("en_US")
@@ -384,19 +269,12 @@ class TestCollation(IntegrationTest):
             collection.update_one(
                 {"hello": "world"}, {"$set": {"hello": "moon"}}, collation=self.collation
             )
-        bulk = collection.initialize_ordered_bulk_op()
-        bulk.find({"hello": "world"}, collation=self.collation).update_one(
-            {"$set": {"hello": "moon"}}
-        )
-        with self.assertRaises(ConfigurationError):
-            bulk.execute()
         update_one = UpdateOne(
             {"hello": "world"}, {"$set": {"hello": "moon"}}, collation=self.collation
         )
         with self.assertRaises(ConfigurationError):
             collection.bulk_write([update_one])
 
-    @raisesConfigurationErrorForOldMongoDB
     def test_cursor_collation(self):
         self.db.test.insert_one({"hello": "world"})
         next(self.db.test.find().collation(self.collation))
