@@ -19,7 +19,6 @@
 import contextlib
 import re
 import sys
-import threading
 from codecs import utf_8_decode
 from collections import defaultdict
 
@@ -31,7 +30,6 @@ from test.utils import (
     IMPOSSIBLE_WRITE_CONCERN,
     EventListener,
     get_pool,
-    ignore_deprecations,
     is_mongos,
     rs_or_single_client,
     single_client,
@@ -39,14 +37,12 @@ from test.utils import (
 )
 
 from bson import encode
-from bson.code import Code
 from bson.codec_options import CodecOptions
 from bson.objectid import ObjectId
-from bson.py3compat import itervalues
 from bson.raw_bson import RawBSONDocument
 from bson.regex import Regex
 from bson.son import SON
-from pymongo import ASCENDING, DESCENDING, GEO2D, GEOHAYSTACK, GEOSPHERE, HASHED, TEXT
+from pymongo import ASCENDING, DESCENDING, GEO2D, GEOSPHERE, HASHED, TEXT
 from pymongo.bulk import BulkWriteError
 from pymongo.collection import Collection, ReturnDocument
 from pymongo.command_cursor import CommandCursor
@@ -144,7 +140,7 @@ class TestCollection(IntegrationTest):
 
     @contextlib.contextmanager
     def write_concern_collection(self):
-        if client_context.version.at_least(3, 3, 9) and client_context.is_rs:
+        if client_context.is_rs:
             with self.assertRaises(WriteConcernError):
                 # Unsatisfiable write concern.
                 yield Collection(
@@ -163,7 +159,6 @@ class TestCollection(IntegrationTest):
     def test_hashable(self):
         self.assertIn(self.db.test.mike, {self.db["test.mike"]})
 
-    @client_context.require_version_min(3, 3, 9)
     def test_create(self):
         # No Exception.
         db = client_context.client.pymongo_test
@@ -327,8 +322,6 @@ class TestCollection(IntegrationTest):
     @client_context.require_no_mongos
     @client_context.require_test_commands
     def test_index_management_max_time_ms(self):
-        if client_context.version[:2] == (3, 4) and client_context.version[2] < 4:
-            raise unittest.SkipTest("SERVER-27711")
         coll = self.db.test
         self.client.admin.command("configureFailPoint", "maxTimeAlwaysTimeOut", mode="alwaysOn")
         try:
@@ -338,42 +331,9 @@ class TestCollection(IntegrationTest):
             )
             self.assertRaises(ExecutionTimeout, coll.drop_index, "foo", maxTimeMS=1)
             self.assertRaises(ExecutionTimeout, coll.drop_indexes, maxTimeMS=1)
-            self.assertRaises(ExecutionTimeout, coll.reindex, maxTimeMS=1)
         finally:
             self.client.admin.command("configureFailPoint", "maxTimeAlwaysTimeOut", mode="off")
 
-    def test_reindex(self):
-        if not client_context.supports_reindex:
-            raise unittest.SkipTest("reindex is no longer supported by mongos 4.1+")
-        db = self.db
-        db.drop_collection("test")
-        db.test.insert_one({"foo": "bar", "who": "what", "when": "how"})
-        db.test.create_index("foo")
-        db.test.create_index("who")
-        db.test.create_index("when")
-        info = db.test.index_information()
-
-        def check_result(result):
-            self.assertEqual(4, result["nIndexes"])
-            indexes = result["indexes"]
-            names = [idx["name"] for idx in indexes]
-            for name in names:
-                self.assertTrue(name in info)
-            for key in info:
-                self.assertTrue(key in names)
-
-        reindexed = db.test.reindex()
-        if "raw" in reindexed:
-            # mongos
-            for result in itervalues(reindexed["raw"]):
-                check_result(result)
-        else:
-            check_result(reindexed)
-
-        coll = Collection(self.db, "test", write_concern=WriteConcern(w=100))
-        # No error since writeConcern is not sent.
-        coll.reindex()
-
     def test_list_indexes(self):
         db = self.db
         db.test.drop()
@@ -446,7 +406,7 @@ class TestCollection(IntegrationTest):
         ).inserted_id
         db.test.insert_one({"pos": {"long": 34.2, "lat": 37.3}, "type": "restaurant"})
         db.test.insert_one({"pos": {"long": 59.1, "lat": 87.2}, "type": "office"})
-        db.test.create_index([("pos", GEOHAYSTACK), ("type", ASCENDING)], bucketSize=1)
+        db.test.create_index([("pos", "geoHaystack"), ("type", ASCENDING)], bucketSize=1)
 
         results = db.command(
             SON(
@@ -543,21 +503,6 @@ class TestCollection(IntegrationTest):
         db.test.insert_one({"i": 2})  # duplicate
         db.test.insert_one({"i": 3})
 
-    @client_context.require_version_max(2, 6)
-    def test_index_drop_dups(self):
-        # Try dropping duplicates
-        db = self.db
-        self._drop_dups_setup(db)
-
-        # No error, just drop the duplicate
-        db.test.create_index([("i", ASCENDING)], unique=True, dropDups=True)
-
-        # Duplicate was dropped
-        self.assertEqual(3, db.test.count_documents({}))
-
-        # Index was created, plus the index on _id
-        self.assertEqual(2, len(db.test.index_information()))
-
     def test_index_dont_drop_dups(self):
         # Try *not* dropping duplicates
         db = self.db
@@ -596,7 +541,6 @@ class TestCollection(IntegrationTest):
                     return stage
         return {}
 
-    @client_context.require_version_min(3, 1, 9, -1)
     def test_index_filter(self):
         db = self.db
         db.drop_collection("test")
@@ -791,6 +735,20 @@ class TestCollection(IntegrationTest):
         self.assertFalse(result.acknowledged)
         self.assertEqual(20, db.test.count_documents({}))
 
+    def test_insert_many_generator(self):
+        coll = self.db.test
+        coll.delete_many({})
+
+        def gen():
+            yield {"a": 1, "b": 1}
+            yield {"a": 1, "b": 2}
+            yield {"a": 2, "b": 3}
+            yield {"a": 3, "b": 5}
+            yield {"a": 5, "b": 8}
+
+        result = coll.insert_many(gen())
+        self.assertEqual(5, len(result.inserted_ids))
+
     def test_insert_many_invalid(self):
         db = self.db
 
@@ -854,14 +812,35 @@ class TestCollection(IntegrationTest):
         wait_until(lambda: 0 == db.test.count_documents({}), "delete 2 documents")
 
     def test_command_document_too_large(self):
-        large = "*" * (self.client.max_bson_size + _COMMAND_OVERHEAD)
+        large = "*" * (client_context.max_bson_size + _COMMAND_OVERHEAD)
         coll = self.db.test
         self.assertRaises(DocumentTooLarge, coll.insert_one, {"data": large})
         # update_one and update_many are the same
         self.assertRaises(DocumentTooLarge, coll.replace_one, {}, {"data": large})
         self.assertRaises(DocumentTooLarge, coll.delete_one, {"data": large})
 
-    @client_context.require_version_min(3, 1, 9, -1)
+    def test_write_large_document(self):
+        max_size = client_context.max_bson_size
+        half_size = int(max_size / 2)
+        max_str = "x" * max_size
+        half_str = "x" * half_size
+        self.assertEqual(max_size, 16777216)
+
+        self.assertRaises(OperationFailure, self.db.test.insert_one, {"foo": max_str})
+        self.assertRaises(
+            OperationFailure, self.db.test.replace_one, {}, {"foo": max_str}, upsert=True
+        )
+        self.assertRaises(OperationFailure, self.db.test.insert_many, [{"x": 1}, {"foo": max_str}])
+        self.db.test.insert_many([{"foo": half_str}, {"foo": half_str}])
+
+        self.db.test.insert_one({"bar": "x"})
+        # Use w=0 here to test legacy doc size checking in all server versions
+        unack_coll = self.db.test.with_options(write_concern=WriteConcern(w=0))
+        self.assertRaises(
+            DocumentTooLarge, unack_coll.replace_one, {"bar": "x"}, {"bar": "x" * (max_size - 14)}
+        )
+        self.db.test.replace_one({"bar": "x"}, {"bar": "x" * (max_size - 32)})
+
     def test_insert_bypass_document_validation(self):
         db = self.db
         db.test.drop()
@@ -877,14 +856,8 @@ class TestCollection(IntegrationTest):
         self.assertTrue(isinstance(result, InsertOneResult))
         self.assertEqual(2, result.inserted_id)
 
-        if client_context.version < (3, 6):
-            # Uses OP_INSERT which does not support bypass_document_validation.
-            self.assertRaises(
-                OperationFailure, db_w0.test.insert_one, {"y": 1}, bypass_document_validation=True
-            )
-        else:
-            db_w0.test.insert_one({"y": 1}, bypass_document_validation=True)
-            wait_until(lambda: db_w0.test.find_one({"y": 1}), "find w:0 inserted document")
+        db_w0.test.insert_one({"y": 1}, bypass_document_validation=True)
+        wait_until(lambda: db_w0.test.find_one({"y": 1}), "find w:0 inserted document")
 
         # Test insert_many
         docs = [{"_id": i, "x": 100 - i} for i in range(3, 100)]
@@ -916,7 +889,6 @@ class TestCollection(IntegrationTest):
             bypass_document_validation=True,
         )
 
-    @client_context.require_version_min(3, 1, 9, -1)
     def test_replace_bypass_document_validation(self):
         db = self.db
         db.test.drop()
@@ -948,20 +920,9 @@ class TestCollection(IntegrationTest):
         self.assertEqual(1, db.test.count_documents({"a": 103}))
 
         db.test.insert_one({"y": 1}, bypass_document_validation=True)
-        if client_context.version < (3, 6):
-            # Uses OP_UPDATE which does not support bypass_document_validation.
-            self.assertRaises(
-                OperationFailure,
-                db_w0.test.replace_one,
-                {"y": 1},
-                {"x": 1},
-                bypass_document_validation=True,
-            )
-        else:
-            db_w0.test.replace_one({"y": 1}, {"x": 1}, bypass_document_validation=True)
-            wait_until(lambda: db_w0.test.find_one({"x": 1}), "find w:0 replaced document")
+        db_w0.test.replace_one({"y": 1}, {"x": 1}, bypass_document_validation=True)
+        wait_until(lambda: db_w0.test.find_one({"x": 1}), "find w:0 replaced document")
 
-    @client_context.require_version_min(3, 1, 9, -1)
     def test_update_bypass_document_validation(self):
         db = self.db
         db.test.drop()
@@ -992,18 +953,8 @@ class TestCollection(IntegrationTest):
         self.assertEqual(1, db.test.count_documents({"z": 0}))
 
         db.test.insert_one({"y": 1, "x": 0}, bypass_document_validation=True)
-        if client_context.version < (3, 6):
-            # Uses OP_UPDATE which does not support bypass_document_validation.
-            self.assertRaises(
-                OperationFailure,
-                db_w0.test.update_one,
-                {"y": 1},
-                {"$inc": {"x": 1}},
-                bypass_document_validation=True,
-            )
-        else:
-            db_w0.test.update_one({"y": 1}, {"$inc": {"x": 1}}, bypass_document_validation=True)
-            wait_until(lambda: db_w0.test.find_one({"y": 1, "x": 1}), "find w:0 updated document")
+        db_w0.test.update_one({"y": 1}, {"$inc": {"x": 1}}, bypass_document_validation=True)
+        wait_until(lambda: db_w0.test.find_one({"y": 1, "x": 1}), "find w:0 updated document")
 
         # Test update_many
         db.test.insert_many([{"z": i} for i in range(3, 101)])
@@ -1040,23 +991,11 @@ class TestCollection(IntegrationTest):
 
         db.test.insert_one({"m": 1, "x": 0}, bypass_document_validation=True)
         db.test.insert_one({"m": 1, "x": 0}, bypass_document_validation=True)
-        if client_context.version < (3, 6):
-            # Uses OP_UPDATE which does not support bypass_document_validation.
-            self.assertRaises(
-                OperationFailure,
-                db_w0.test.update_many,
-                {"m": 1},
-                {"$inc": {"x": 1}},
-                bypass_document_validation=True,
-            )
-        else:
-            db_w0.test.update_many({"m": 1}, {"$inc": {"x": 1}}, bypass_document_validation=True)
-            wait_until(
-                lambda: db_w0.test.count_documents({"m": 1, "x": 1}) == 2,
-                "find w:0 updated documents",
-            )
+        db_w0.test.update_many({"m": 1}, {"$inc": {"x": 1}}, bypass_document_validation=True)
+        wait_until(
+            lambda: db_w0.test.count_documents({"m": 1, "x": 1}) == 2, "find w:0 updated documents"
+        )
 
-    @client_context.require_version_min(3, 1, 9, -1)
     def test_bypass_document_validation_bulk_write(self):
         db = self.db
         db.test.drop()
@@ -1129,7 +1068,6 @@ class TestCollection(IntegrationTest):
         self.assertTrue("x" not in db.test.find_one(projection={"x": 0}))
         self.assertTrue("mike" in db.test.find_one(projection={"x": 0}))
 
-    @client_context.require_version_max(4, 9, -1)  # PYTHON-2721
     def test_find_w_regex(self):
         db = self.db
         db.test.delete_many({})
@@ -1162,7 +1100,7 @@ class TestCollection(IntegrationTest):
         self.assertEqual(obj["_id"], numeric)
 
         for x in db.test.find():
-            self.assertEqual(x["hello"], u"world")
+            self.assertEqual(x["hello"], "world")
             self.assertTrue("_id" in x)
 
     def test_unique_index(self):
@@ -1244,18 +1182,10 @@ class TestCollection(IntegrationTest):
         # Should raise DuplicateKeyError, not InvalidBSON
         self.assertRaises(DuplicateKeyError, db.test.insert_one, {"text": text})
 
-        self.assertRaises(DuplicateKeyError, db.test.insert, {"text": text})
-
-        self.assertRaises(DuplicateKeyError, db.test.insert, [{"text": text}])
-
         self.assertRaises(
             DuplicateKeyError, db.test.replace_one, {"_id": ObjectId()}, {"text": text}, upsert=True
         )
 
-        self.assertRaises(
-            DuplicateKeyError, db.test.update, {"_id": ObjectId()}, {"text": text}, upsert=True
-        )
-
         # Should raise BulkWriteError, not InvalidBSON
         self.assertRaises(BulkWriteError, db.test.insert_many, [{"text": text}])
 
@@ -1264,17 +1194,13 @@ class TestCollection(IntegrationTest):
         self.addCleanup(coll.drop)
 
         coll.create_index("a", unique=True)
-        coll.insert_one({"a": u"unicode \U0001f40d"})
+        coll.insert_one({"a": "unicode \U0001f40d"})
         with self.assertRaisesRegex(DuplicateKeyError, "E11000 duplicate key error") as ctx:
-            coll.insert_one({"a": u"unicode \U0001f40d"})
+            coll.insert_one({"a": "unicode \U0001f40d"})
 
         # Once more for good measure.
         self.assertIn("E11000 duplicate key error", str(ctx.exception))
 
-        if sys.version_info[0] == 2:
-            # Test unicode(error) conversion.
-            self.assertIn("E11000 duplicate key error", unicode(ctx.exception))
-
     def test_wtimeout(self):
         # Ensure setting wtimeout doesn't disable write concern altogether.
         # See SERVER-12596.
@@ -1439,24 +1365,6 @@ class TestCollection(IntegrationTest):
         self.assertRaises(InvalidOperation, lambda: result.upserted_id)
         self.assertFalse(result.acknowledged)
 
-    # MongoDB >= 3.5.8 allows dotted fields in updates
-    @client_context.require_version_max(3, 5, 7)
-    def test_update_with_invalid_keys(self):
-        self.db.drop_collection("test")
-        self.assertTrue(self.db.test.insert_one({"hello": "world"}))
-        doc = self.db.test.find_one()
-        doc["a.b"] = "c"
-
-        # Replace
-        self.assertRaises(OperationFailure, self.db.test.replace_one, {"hello": "world"}, doc)
-        # Upsert
-        self.assertRaises(
-            OperationFailure, self.db.test.replace_one, {"foo": "bar"}, doc, upsert=True
-        )
-
-        # Check that the last two ops didn't actually modify anything
-        self.assertTrue("a.b" not in self.db.test.find_one())
-
     def test_update_check_keys(self):
         self.db.drop_collection("test")
         self.assertTrue(self.db.test.insert_one({"hello": "world"}))
@@ -1488,7 +1396,16 @@ class TestCollection(IntegrationTest):
     def test_acknowledged_delete(self):
         db = self.db
         db.drop_collection("test")
-        db.test.insert_many([{"x": 1}, {"x": 1}])
+        db.create_collection("test", capped=True, size=1000)
+
+        db.test.insert_one({"x": 1})
+        self.assertEqual(1, db.test.count_documents({}))
+
+        # Can't remove from capped collection.
+        self.assertRaises(OperationFailure, db.test.delete_one, {"x": 1})
+        db.drop_collection("test")
+        db.test.insert_one({"x": 1})
+        db.test.insert_one({"x": 1})
         self.assertEqual(2, db.test.delete_many({}).deleted_count)
         self.assertEqual(0, db.test.delete_many({}).deleted_count)
 
@@ -1498,20 +1415,6 @@ class TestCollection(IntegrationTest):
         coll.insert_one({"x": 1})
         self.db.command("getlasterror", w=1, wtimeout=1)
 
-    @ignore_deprecations
-    def test_count(self):
-        db = self.db
-        db.drop_collection("test")
-
-        self.assertEqual(db.test.count(), 0)
-        db.test.insert_many([{}, {}])
-        self.assertEqual(db.test.count(), 2)
-        db.test.insert_many([{"foo": "bar"}, {"foo": "baz"}])
-        self.assertEqual(db.test.find({"foo": "bar"}).count(), 1)
-        self.assertEqual(db.test.count({"foo": "bar"}), 1)
-        self.assertEqual(db.test.find({"foo": re.compile(r"ba.*")}).count(), 2)
-        self.assertEqual(db.test.count({"foo": re.compile(r"ba.*")}), 2)
-
     def test_count_documents(self):
         db = self.db
         db.drop_collection("test")
@@ -1545,12 +1448,7 @@ class TestCollection(IntegrationTest):
         self.assertRaises(TypeError, db.test.aggregate, "wow")
 
         pipeline = {"$project": {"_id": False, "foo": True}}
-        # MongoDB 3.5.1+ requires either the 'cursor' or 'explain' options.
-        if client_context.version.at_least(3, 5, 1):
-            result = db.test.aggregate([pipeline])
-        else:
-            result = db.test.aggregate([pipeline], useCursor=False)
-
+        result = db.test.aggregate([pipeline])
         self.assertTrue(isinstance(result, CommandCursor))
         self.assertEqual([{"foo": [1, 2]}], list(result))
 
@@ -1567,11 +1465,7 @@ class TestCollection(IntegrationTest):
 
         pipeline = {"$project": {"_id": False, "foo": True}}
         coll = db.get_collection("test", codec_options=CodecOptions(document_class=RawBSONDocument))
-        # MongoDB 3.5.1+ requires either the 'cursor' or 'explain' options.
-        if client_context.version.at_least(3, 5, 1):
-            result = coll.aggregate([pipeline])
-        else:
-            result = coll.aggregate([pipeline], useCursor=False)
+        result = coll.aggregate([pipeline])
         self.assertTrue(isinstance(result, CommandCursor))
         first_result = next(result)
         self.assertIsInstance(first_result, RawBSONDocument)
@@ -1583,9 +1477,6 @@ class TestCollection(IntegrationTest):
         cursor = db.test.aggregate([projection], cursor={})
         self.assertTrue(isinstance(cursor, CommandCursor))
 
-        cursor = db.test.aggregate([projection], useCursor=True)
-        self.assertTrue(isinstance(cursor, CommandCursor))
-
     def test_aggregation_cursor(self):
         db = self.db
         if client_context.has_secondaries:
@@ -1632,45 +1523,6 @@ class TestCollection(IntegrationTest):
 
             self.assertTrue(cursor.alive)
 
-    @client_context.require_no_mongos
-    @client_context.require_version_max(4, 1, 0)
-    @ignore_deprecations
-    def test_parallel_scan(self):
-        db = self.db
-        db.drop_collection("test")
-        if client_context.has_secondaries:
-            # Test that getMore messages are sent to the right server.
-            db = self.client.get_database(
-                db.name,
-                read_preference=ReadPreference.SECONDARY,
-                write_concern=WriteConcern(w=self.w),
-            )
-
-        coll = db.test
-        coll.insert_many([{"_id": i} for i in range(8000)])
-        docs = []
-        threads = [
-            threading.Thread(target=docs.extend, args=(cursor,)) for cursor in coll.parallel_scan(3)
-        ]
-        for t in threads:
-            t.start()
-        for t in threads:
-            t.join()
-
-        self.assertEqual(set(range(8000)), set(doc["_id"] for doc in docs))
-
-    @client_context.require_no_mongos
-    @client_context.require_version_min(3, 3, 10)
-    @client_context.require_version_max(4, 1, 0)
-    @client_context.require_test_commands
-    @ignore_deprecations
-    def test_parallel_scan_max_time_ms(self):
-        self.client.admin.command("configureFailPoint", "maxTimeAlwaysTimeOut", mode="alwaysOn")
-        try:
-            self.assertRaises(ExecutionTimeout, self.db.test.parallel_scan, 3, maxTimeMS=1)
-        finally:
-            self.client.admin.command("configureFailPoint", "maxTimeAlwaysTimeOut", mode="off")
-
     def test_large_limit(self):
         db = self.db
         db.drop_collection("test_large_limit")
@@ -1761,7 +1613,9 @@ class TestCollection(IntegrationTest):
         self.assertTrue("hello" in db.test.find_one(projection=frozenset(["hello"])))
         self.assertTrue("hello" not in db.test.find_one(projection=frozenset(["foo"])))
 
-        self.assertEqual(["_id"], list(db.test.find_one(projection=[])))
+        self.assertEqual(["_id"], list(db.test.find_one(projection={"_id": True})))
+        self.assertTrue("hello" in list(db.test.find_one(projection={})))
+        self.assertTrue("hello" in list(db.test.find_one(projection=[])))
 
         self.assertEqual(None, db.test.find_one({"hello": "foo"}))
         self.assertEqual(None, db.test.find_one(ObjectId()))
@@ -1913,115 +1767,102 @@ class TestCollection(IntegrationTest):
         self.db.test.insert_many([{"x": 1}, {"x": 2}])
         self.db.test.create_index("x")
 
-        cursor = self.db.test.find({"$min": {"x": 2}, "$query": {}})
-        if client_context.requires_hint_with_min_max_queries:
-            cursor = cursor.hint("x_1")
+        cursor = self.db.test.find({"$min": {"x": 2}, "$query": {}}, hint="x_1")
 
         docs = list(cursor)
         self.assertEqual(1, len(docs))
         self.assertEqual(2, docs[0]["x"])
 
     def test_numerous_inserts(self):
-        # Ensure we don't exceed server's 1000-document batch size limit.
+        # Ensure we don't exceed server's maxWriteBatchSize size limit.
         self.db.test.drop()
-        n_docs = 2100
+        n_docs = client_context.max_write_batch_size + 100
         self.db.test.insert_many([{} for _ in range(n_docs)])
         self.assertEqual(n_docs, self.db.test.count_documents({}))
         self.db.test.drop()
 
-    def test_map_reduce(self):
-        db = self.db
-        db.drop_collection("test")
+    def test_insert_many_large_batch(self):
+        # Tests legacy insert.
+        db = self.client.test_insert_large_batch
+        self.addCleanup(self.client.drop_database, "test_insert_large_batch")
+        max_bson_size = client_context.max_bson_size
+        # Write commands are limited to 16MB + 16k per batch
+        big_string = "x" * int(max_bson_size / 2)
 
-        db.test.insert_one({"id": 1, "tags": ["dog", "cat"]})
-        db.test.insert_one({"id": 2, "tags": ["cat"]})
-        db.test.insert_one({"id": 3, "tags": ["mouse", "cat", "dog"]})
-        db.test.insert_one({"id": 4, "tags": []})
+        # Batch insert that requires 2 batches.
+        successful_insert = [
+            {"x": big_string},
+            {"x": big_string},
+            {"x": big_string},
+            {"x": big_string},
+        ]
+        db.collection_0.insert_many(successful_insert)
+        self.assertEqual(4, db.collection_0.count_documents({}))
 
-        map = Code(
-            "function () {" "  this.tags.forEach(function(z) {" "    emit(z, 1);" "  });" "}"
+        db.collection_0.drop()
+
+        # Test that inserts fail after first error.
+        insert_second_fails = [
+            {"_id": "id0", "x": big_string},
+            {"_id": "id0", "x": big_string},
+            {"_id": "id1", "x": big_string},
+            {"_id": "id2", "x": big_string},
+        ]
+
+        with self.assertRaises(BulkWriteError):
+            db.collection_1.insert_many(insert_second_fails)
+
+        self.assertEqual(1, db.collection_1.count_documents({}))
+
+        db.collection_1.drop()
+
+        # 2 batches, 2nd insert fails, unacknowledged, ordered.
+        unack_coll = db.collection_2.with_options(write_concern=WriteConcern(w=0))
+        unack_coll.insert_many(insert_second_fails)
+        wait_until(
+            lambda: 1 == db.collection_2.count_documents({}), "insert 1 document", timeout=60
         )
-        reduce = Code(
-            "function (key, values) {"
-            "  var total = 0;"
-            "  for (var i = 0; i < values.length; i++) {"
-            "    total += values[i];"
-            "  }"
-            "  return total;"
-            "}"
+
+        db.collection_2.drop()
+
+        # 2 batches, ids of docs 0 and 1 are dupes, ids of docs 2 and 3 are
+        # dupes. Acknowledged, unordered.
+        insert_two_failures = [
+            {"_id": "id0", "x": big_string},
+            {"_id": "id0", "x": big_string},
+            {"_id": "id1", "x": big_string},
+            {"_id": "id1", "x": big_string},
+        ]
+
+        with self.assertRaises(OperationFailure) as context:
+            db.collection_3.insert_many(insert_two_failures, ordered=False)
+
+        self.assertIn("id1", str(context.exception))
+
+        # Only the first and third documents should be inserted.
+        self.assertEqual(2, db.collection_3.count_documents({}))
+
+        db.collection_3.drop()
+
+        # 2 batches, 2 errors, unacknowledged, unordered.
+        unack_coll = db.collection_4.with_options(write_concern=WriteConcern(w=0))
+        unack_coll.insert_many(insert_two_failures, ordered=False)
+
+        # Only the first and third documents are inserted.
+        wait_until(
+            lambda: 2 == db.collection_4.count_documents({}), "insert 2 documents", timeout=60
         )
-        result = db.test.map_reduce(map, reduce, out="mrunittests")
-        self.assertEqual(3, result.find_one({"_id": "cat"})["value"])
-        self.assertEqual(2, result.find_one({"_id": "dog"})["value"])
-        self.assertEqual(1, result.find_one({"_id": "mouse"})["value"])
 
-        db.test.insert_one({"id": 5, "tags": ["hampster"]})
-        result = db.test.map_reduce(map, reduce, out="mrunittests")
-        self.assertEqual(1, result.find_one({"_id": "hampster"})["value"])
-        db.test.delete_one({"id": 5})
-
-        result = db.test.map_reduce(map, reduce, out={"merge": "mrunittests"})
-        self.assertEqual(3, result.find_one({"_id": "cat"})["value"])
-        self.assertEqual(1, result.find_one({"_id": "hampster"})["value"])
-
-        result = db.test.map_reduce(map, reduce, out={"reduce": "mrunittests"})
-
-        self.assertEqual(6, result.find_one({"_id": "cat"})["value"])
-        self.assertEqual(4, result.find_one({"_id": "dog"})["value"])
-        self.assertEqual(2, result.find_one({"_id": "mouse"})["value"])
-        self.assertEqual(1, result.find_one({"_id": "hampster"})["value"])
-
-        result = db.test.map_reduce(map, reduce, out={"replace": "mrunittests"})
-        self.assertEqual(3, result.find_one({"_id": "cat"})["value"])
-        self.assertEqual(2, result.find_one({"_id": "dog"})["value"])
-        self.assertEqual(1, result.find_one({"_id": "mouse"})["value"])
-
-        # Create the output database.
-        db.client.mrtestdb.mrunittests.insert_one({})
-        result = db.test.map_reduce(
-            map, reduce, out=SON([("replace", "mrunittests"), ("db", "mrtestdb")])
-        )
-        self.assertEqual(3, result.find_one({"_id": "cat"})["value"])
-        self.assertEqual(2, result.find_one({"_id": "dog"})["value"])
-        self.assertEqual(1, result.find_one({"_id": "mouse"})["value"])
-        self.client.drop_database("mrtestdb")
-
-        full_result = db.test.map_reduce(map, reduce, out="mrunittests", full_response=True)
-        self.assertEqual("mrunittests", full_result["result"])
-        if client_context.version < (4, 3):
-            self.assertEqual(6, full_result["counts"]["emit"])
-
-        result = db.test.map_reduce(map, reduce, out="mrunittests", limit=2)
-        self.assertEqual(2, result.find_one({"_id": "cat"})["value"])
-        self.assertEqual(1, result.find_one({"_id": "dog"})["value"])
-        self.assertEqual(None, result.find_one({"_id": "mouse"}))
-
-        result = db.test.map_reduce(map, reduce, out={"inline": 1})
-        self.assertTrue(isinstance(result, dict))
-        self.assertTrue("results" in result)
-        self.assertTrue(result["results"][1]["_id"] in ("cat", "dog", "mouse"))
-
-        result = db.test.inline_map_reduce(map, reduce)
-        self.assertTrue(isinstance(result, list))
-        self.assertEqual(3, len(result))
-        self.assertTrue(result[1]["_id"] in ("cat", "dog", "mouse"))
-
-        full_result = db.test.inline_map_reduce(map, reduce, full_response=True)
-        self.assertEqual(3, len(full_result["results"]))
-        if client_context.version < (4, 3):
-            self.assertEqual(6, full_result["counts"]["emit"])
-
-        with self.write_concern_collection() as coll:
-            coll.map_reduce(map, reduce, "output")
+        db.collection_4.drop()
 
     def test_messages_with_unicode_collection_names(self):
         db = self.db
 
-        db[u"Employés"].insert_one({"x": 1})
-        db[u"Employés"].replace_one({"x": 1}, {"x": 2})
-        db[u"Employés"].delete_many({})
-        db[u"Employés"].find_one()
-        list(db[u"Employés"].find())
+        db["Employés"].insert_one({"x": 1})
+        db["Employés"].replace_one({"x": 1}, {"x": 2})
+        db["Employés"].delete_many({})
+        db["Employés"].find_one()
+        list(db["Employés"].find())
 
     def test_drop_indexes_non_existent(self):
         self.db.drop_collection("test")
@@ -2042,16 +1883,6 @@ class TestCollection(IntegrationTest):
         c.insert_one({"bad": bad})
         self.assertEqual("bar", c.find_one()["bad"]["foo"])
 
-    @client_context.require_version_max(3, 5, 5)
-    def test_array_filters_unsupported(self):
-        c = self.db.test
-        with self.assertRaises(ConfigurationError):
-            c.update_one({}, {"$set": {"y.$[i].b": 5}}, array_filters=[{"i.b": 1}])
-        with self.assertRaises(ConfigurationError):
-            c.update_many({}, {"$set": {"y.$[i].b": 5}}, array_filters=[{"i.b": 1}])
-        with self.assertRaises(ConfigurationError):
-            c.find_one_and_update({}, {"$set": {"y.$[i].b": 5}}, array_filters=[{"i.b": 1}])
-
     def test_array_filters_validation(self):
         # array_filters must be a list.
         c = self.db.test
@@ -2135,74 +1966,43 @@ class TestCollection(IntegrationTest):
         # Authenticate the client and throw out auth commands from the listener.
         db.command("ping")
         results.clear()
-        if client_context.version.at_least(3, 1, 9, -1):
-            c_w0.find_and_modify({"_id": 1}, {"$set": {"foo": "bar"}})
-            self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
-            results.clear()
-
-            c_w0.find_one_and_update({"_id": 1}, {"$set": {"foo": "bar"}})
-            self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
-            results.clear()
-
-            c_w0.find_one_and_replace({"_id": 1}, {"foo": "bar"})
-            self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
-            results.clear()
-
-            c_w0.find_one_and_delete({"_id": 1})
-            self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
-            results.clear()
-
-            # Test write concern errors.
-            if client_context.is_rs:
-                c_wc_error = db.get_collection(
-                    "test", write_concern=WriteConcern(w=len(client_context.nodes) + 1)
-                )
-                self.assertRaises(
-                    WriteConcernError,
-                    c_wc_error.find_and_modify,
-                    {"_id": 1},
-                    {"$set": {"foo": "bar"}},
-                )
-                self.assertRaises(
-                    WriteConcernError,
-                    c_wc_error.find_one_and_update,
-                    {"_id": 1},
-                    {"$set": {"foo": "bar"}},
-                )
-                self.assertRaises(
-                    WriteConcernError,
-                    c_wc_error.find_one_and_replace,
-                    {"w": 0},
-                    results["started"][0].command["writeConcern"],
-                )
-                self.assertRaises(
-                    WriteConcernError,
-                    c_wc_error.find_one_and_delete,
-                    {"w": 0},
-                    results["started"][0].command["writeConcern"],
-                )
-                results.clear()
-        else:
-            c_w0.find_and_modify({"_id": 1}, {"$set": {"foo": "bar"}})
-            self.assertNotIn("writeConcern", results["started"][0].command)
-            results.clear()
-
-            c_w0.find_one_and_update({"_id": 1}, {"$set": {"foo": "bar"}})
-            self.assertNotIn("writeConcern", results["started"][0].command)
-            results.clear()
-
-            c_w0.find_one_and_replace({"_id": 1}, {"foo": "bar"})
-            self.assertNotIn("writeConcern", results["started"][0].command)
-            results.clear()
-
-            c_w0.find_one_and_delete({"_id": 1})
-            self.assertNotIn("writeConcern", results["started"][0].command)
-            results.clear()
-
-        c_default.find_and_modify({"_id": 1}, {"$set": {"foo": "bar"}})
-        self.assertNotIn("writeConcern", results["started"][0].command)
+        c_w0.find_one_and_update({"_id": 1}, {"$set": {"foo": "bar"}})
+        self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
         results.clear()
 
+        c_w0.find_one_and_replace({"_id": 1}, {"foo": "bar"})
+        self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
+        results.clear()
+
+        c_w0.find_one_and_delete({"_id": 1})
+        self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
+        results.clear()
+
+        # Test write concern errors.
+        if client_context.is_rs:
+            c_wc_error = db.get_collection(
+                "test", write_concern=WriteConcern(w=len(client_context.nodes) + 1)
+            )
+            self.assertRaises(
+                WriteConcernError,
+                c_wc_error.find_one_and_update,
+                {"_id": 1},
+                {"$set": {"foo": "bar"}},
+            )
+            self.assertRaises(
+                WriteConcernError,
+                c_wc_error.find_one_and_replace,
+                {"w": 0},
+                results["started"][0].command["writeConcern"],
+            )
+            self.assertRaises(
+                WriteConcernError,
+                c_wc_error.find_one_and_delete,
+                {"w": 0},
+                results["started"][0].command["writeConcern"],
+            )
+            results.clear()
+
         c_default.find_one_and_update({"_id": 1}, {"$set": {"foo": "bar"}})
         self.assertNotIn("writeConcern", results["started"][0].command)
         results.clear()
@@ -2299,6 +2099,10 @@ class TestCollection(IntegrationTest):
             cmd.to_dict(), SON([("find", "coll"), ("$dumb", 2), ("filter", {"foo": 1})]).to_dict()
         )
 
+    def test_bool(self):
+        with self.assertRaises(NotImplementedError):
+            bool(Collection(self.db, "test"))
+
 
 if __name__ == "__main__":
     unittest.main()
