@@ -22,8 +22,7 @@ sys.path[0:0] = [""]
 from test import IntegrationTest, client_context, unittest
 from test.utils import connected, rs_or_single_client, single_client
 
-from bson.binary import PYTHON_LEGACY, STANDARD, UUIDLegacy
-from bson.code import Code
+from bson.binary import PYTHON_LEGACY, STANDARD, Binary, UuidRepresentation
 from bson.codec_options import CodecOptions
 from bson.objectid import ObjectId
 from pymongo.errors import OperationFailure
@@ -41,17 +40,20 @@ class TestCommon(IntegrationTest):
         coll.drop()
 
         # Test property
-        self.assertEqual(PYTHON_LEGACY, coll.codec_options.uuid_representation)
+        self.assertEqual(UuidRepresentation.UNSPECIFIED, coll.codec_options.uuid_representation)
 
         # Test basic query
         uu = uuid.uuid4()
         # Insert as binary subtype 3
+        coll = self.db.get_collection("uuid", CodecOptions(uuid_representation=PYTHON_LEGACY))
+        legacy_opts = coll.codec_options
         coll.insert_one({"uu": uu})
         self.assertEqual(uu, coll.find_one({"uu": uu})["uu"])
         coll = self.db.get_collection("uuid", CodecOptions(uuid_representation=STANDARD))
         self.assertEqual(STANDARD, coll.codec_options.uuid_representation)
         self.assertEqual(None, coll.find_one({"uu": uu}))
-        self.assertEqual(uu, coll.find_one({"uu": UUIDLegacy(uu)})["uu"])
+        uul = Binary.from_uuid(uu, PYTHON_LEGACY)
+        self.assertEqual(uul, coll.find_one({"uu": uul})["uu"])
 
         # Test count_documents
         self.assertEqual(0, coll.count_documents({"uu": uu}))
@@ -89,60 +91,29 @@ class TestCommon(IntegrationTest):
         # Test command
         self.assertEqual(
             5,
-            self.db.command("findAndModify", "uuid", update={"$set": {"i": 6}}, query={"_id": uu})[
-                "value"
-            ]["i"],
+            self.db.command(
+                "findAndModify",
+                "uuid",
+                update={"$set": {"i": 6}},
+                query={"_id": uu},
+                codec_options=legacy_opts,
+            )["value"]["i"],
         )
         self.assertEqual(
             6,
             self.db.command(
-                "findAndModify", "uuid", update={"$set": {"i": 7}}, query={"_id": UUIDLegacy(uu)}
+                "findAndModify",
+                "uuid",
+                update={"$set": {"i": 7}},
+                query={"_id": Binary.from_uuid(uu, PYTHON_LEGACY)},
             )["value"]["i"],
         )
 
-        # Test (inline)_map_reduce
-        coll.drop()
-        coll.insert_one({"_id": uu, "x": 1, "tags": ["dog", "cat"]})
-        coll.insert_one({"_id": uuid.uuid4(), "x": 3, "tags": ["mouse", "cat", "dog"]})
-
-        map = Code(
-            "function () {" "  this.tags.forEach(function(z) {" "    emit(z, 1);" "  });" "}"
-        )
-
-        reduce = Code(
-            "function (key, values) {"
-            "  var total = 0;"
-            "  for (var i = 0; i < values.length; i++) {"
-            "    total += values[i];"
-            "  }"
-            "  return total;"
-            "}"
-        )
-
-        coll = self.db.get_collection("uuid", CodecOptions(uuid_representation=STANDARD))
-        q = {"_id": uu}
-        result = coll.inline_map_reduce(map, reduce, query=q)
-        self.assertEqual([], result)
-
-        result = coll.map_reduce(map, reduce, "results", query=q)
-        self.assertEqual(0, self.db.results.count_documents({}))
-
-        coll = self.db.get_collection("uuid", CodecOptions(uuid_representation=PYTHON_LEGACY))
-        q = {"_id": uu}
-        result = coll.inline_map_reduce(map, reduce, query=q)
-        self.assertEqual(2, len(result))
-
-        result = coll.map_reduce(map, reduce, "results", query=q)
-        self.assertEqual(2, self.db.results.count_documents({}))
-
-        self.db.drop_collection("result")
-        coll.drop()
-
     def test_write_concern(self):
         c = rs_or_single_client(connect=False)
         self.assertEqual(WriteConcern(), c.write_concern)
 
-        c = rs_or_single_client(connect=False, w=2, wtimeout=1000)
+        c = rs_or_single_client(connect=False, w=2, wTimeoutMS=1000)
         wc = WriteConcern(w=2, wtimeout=1000)
         self.assertEqual(wc, c.write_concern)
 
