@@ -21,7 +21,6 @@ import re
 import sys
 import threading
 import time
-import warnings
 
 sys.path[0:0] = [""]
 
@@ -36,9 +35,8 @@ from test.utils import (
 
 from bson import decode_all
 from bson.code import Code
-from bson.py3compat import PY3
 from bson.son import SON
-from pymongo import ALL, ASCENDING, DESCENDING, OFF
+from pymongo import ASCENDING, DESCENDING
 from pymongo.collation import Collation
 from pymongo.cursor import Cursor, CursorType
 from pymongo.errors import (
@@ -48,12 +46,8 @@ from pymongo.errors import (
     OperationFailure,
 )
 from pymongo.read_concern import ReadConcern
-from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 
-if PY3:
-    long = int
-
 
 class TestCursor(IntegrationTest):
     def test_deepcopy_cursor_littered_with_regexes(self):
@@ -162,7 +156,7 @@ class TestCursor(IntegrationTest):
         coll.insert_one({"amalia": 2})
 
         coll.find().max_time_ms(None)
-        coll.find().max_time_ms(long(1))
+        coll.find().max_time_ms(1)
 
         cursor = coll.find().max_time_ms(999)
         self.assertEqual(999, cursor._Cursor__max_time_ms)
@@ -193,7 +187,6 @@ class TestCursor(IntegrationTest):
             finally:
                 client.admin.command("configureFailPoint", "maxTimeAlwaysTimeOut", mode="off")
 
-    @client_context.require_version_min(3, 1, 9, -1)
     def test_max_await_time_ms(self):
         db = self.db
         db.pymongo_test.drop()
@@ -204,7 +197,7 @@ class TestCursor(IntegrationTest):
         coll.insert_one({"amalia": 2})
 
         coll.find().max_await_time_ms(None)
-        coll.find().max_await_time_ms(long(1))
+        coll.find().max_await_time_ms(1)
 
         # When cursor is not tailable_await
         cursor = coll.find()
@@ -397,7 +390,7 @@ class TestCursor(IntegrationTest):
         self.assertRaises(TypeError, db.test.find().limit, None)
         self.assertRaises(TypeError, db.test.find().limit, "hello")
         self.assertRaises(TypeError, db.test.find().limit, 5.5)
-        self.assertTrue(db.test.find().limit(long(5)))
+        self.assertTrue(db.test.find().limit(5))
 
         db.test.drop()
         db.test.insert_many([{"x": i} for i in range(100)])
@@ -438,7 +431,6 @@ class TestCursor(IntegrationTest):
             break
         self.assertRaises(InvalidOperation, a.limit, 5)
 
-    @ignore_deprecations  # Ignore max without hint.
     def test_max(self):
         db = self.db
         db.test.drop()
@@ -448,10 +440,7 @@ class TestCursor(IntegrationTest):
         db.test.insert_many([{"j": j, "k": j} for j in range(10)])
 
         def find(max_spec, expected_index):
-            cursor = db.test.find().max(max_spec)
-            if client_context.requires_hint_with_min_max_queries:
-                cursor = cursor.hint(expected_index)
-            return cursor
+            return db.test.find().max(max_spec).hint(expected_index)
 
         cursor = find([("j", 3)], j_index)
         self.assertEqual(len(list(cursor)), 3)
@@ -477,7 +466,6 @@ class TestCursor(IntegrationTest):
         self.assertRaises(TypeError, db.test.find().max, 10)
         self.assertRaises(TypeError, db.test.find().max, {"j": 10})
 
-    @ignore_deprecations  # Ignore min without hint.
     def test_min(self):
         db = self.db
         db.test.drop()
@@ -487,10 +475,7 @@ class TestCursor(IntegrationTest):
         db.test.insert_many([{"j": j, "k": j} for j in range(10)])
 
         def find(min_spec, expected_index):
-            cursor = db.test.find().min(min_spec)
-            if client_context.requires_hint_with_min_max_queries:
-                cursor = cursor.hint(expected_index)
-            return cursor
+            return db.test.find().min(min_spec).hint(expected_index)
 
         cursor = find([("j", 3)], j_index)
         self.assertEqual(len(list(cursor)), 7)
@@ -516,23 +501,15 @@ class TestCursor(IntegrationTest):
         self.assertRaises(TypeError, db.test.find().min, 10)
         self.assertRaises(TypeError, db.test.find().min, {"j": 10})
 
-    @client_context.require_version_max(4, 1, -1)
     def test_min_max_without_hint(self):
         coll = self.db.test
         j_index = [("j", ASCENDING)]
         coll.create_index(j_index)
 
-        with warnings.catch_warnings(record=True) as warns:
-            warnings.simplefilter("default", DeprecationWarning)
+        with self.assertRaises(InvalidOperation):
             list(coll.find().min([("j", 3)]))
-            self.assertIn("using a min/max query operator", str(warns[0]))
-            # Ensure the warning is raised with the proper stack level.
-            del warns[:]
-            list(coll.find().min([("j", 3)]))
-            self.assertIn("using a min/max query operator", str(warns[0]))
-            del warns[:]
+        with self.assertRaises(InvalidOperation):
             list(coll.find().max([("j", 3)]))
-            self.assertIn("using a min/max query operator", str(warns[0]))
 
     def test_batch_size(self):
         db = self.db
@@ -543,7 +520,7 @@ class TestCursor(IntegrationTest):
         self.assertRaises(TypeError, db.test.find().batch_size, "hello")
         self.assertRaises(TypeError, db.test.find().batch_size, 5.5)
         self.assertRaises(ValueError, db.test.find().batch_size, -1)
-        self.assertTrue(db.test.find().batch_size(long(5)))
+        self.assertTrue(db.test.find().batch_size(5))
         a = db.test.find()
         for _ in a:
             break
@@ -578,12 +555,8 @@ class TestCursor(IntegrationTest):
 
         cur = db.test.find().batch_size(1)
         next(cur)
-        if client_context.version.at_least(3, 1, 9):
-            # find command batchSize should be 1
-            self.assertEqual(0, len(cur._Cursor__data))
-        else:
-            # OP_QUERY ntoreturn should be 2
-            self.assertEqual(1, len(cur._Cursor__data))
+        # find command batchSize should be 1
+        self.assertEqual(0, len(cur._Cursor__data))
         next(cur)
         self.assertEqual(0, len(cur._Cursor__data))
         next(cur)
@@ -667,7 +640,7 @@ class TestCursor(IntegrationTest):
         self.assertRaises(TypeError, db.test.find().skip, "hello")
         self.assertRaises(TypeError, db.test.find().skip, 5.5)
         self.assertRaises(ValueError, db.test.find().skip, -5)
-        self.assertTrue(db.test.find().skip(long(5)))
+        self.assertTrue(db.test.find().skip(5))
 
         db.drop_collection("test")
 
@@ -755,60 +728,6 @@ class TestCursor(IntegrationTest):
             break
         self.assertRaises(InvalidOperation, a.sort, "x", ASCENDING)
 
-    @ignore_deprecations
-    def test_count(self):
-        db = self.db
-        db.test.drop()
-
-        self.assertEqual(0, db.test.find().count())
-
-        db.test.insert_many([{"x": i} for i in range(10)])
-
-        self.assertEqual(10, db.test.find().count())
-        self.assertTrue(isinstance(db.test.find().count(), int))
-        self.assertEqual(10, db.test.find().limit(5).count())
-        self.assertEqual(10, db.test.find().skip(5).count())
-
-        self.assertEqual(1, db.test.find({"x": 1}).count())
-        self.assertEqual(5, db.test.find({"x": {"$lt": 5}}).count())
-
-        a = db.test.find()
-        b = a.count()
-        for _ in a:
-            break
-        self.assertEqual(b, a.count())
-
-        self.assertEqual(0, db.test.acollectionthatdoesntexist.find().count())
-
-    @ignore_deprecations
-    def test_count_with_hint(self):
-        collection = self.db.test
-        collection.drop()
-
-        collection.insert_many([{"i": 1}, {"i": 2}])
-        self.assertEqual(2, collection.find().count())
-
-        collection.create_index([("i", 1)])
-
-        self.assertEqual(1, collection.find({"i": 1}).hint("_id_").count())
-        self.assertEqual(2, collection.find().hint("_id_").count())
-
-        self.assertRaises(OperationFailure, collection.find({"i": 1}).hint("BAD HINT").count)
-
-        # Create a sparse index which should have no entries.
-        collection.create_index([("x", 1)], sparse=True)
-
-        self.assertEqual(0, collection.find({"i": 1}).hint("x_1").count())
-        self.assertEqual(0, collection.find({"i": 1}).hint([("x", 1)]).count())
-
-        if client_context.version.at_least(3, 3, 2):
-            self.assertEqual(0, collection.find().hint("x_1").count())
-            self.assertEqual(0, collection.find().hint([("x", 1)]).count())
-        else:
-            self.assertEqual(2, collection.find().hint("x_1").count())
-            self.assertEqual(2, collection.find().hint([("x", 1)]).count())
-
-    @ignore_deprecations
     def test_where(self):
         db = self.db
         db.test.drop()
@@ -836,10 +755,6 @@ class TestCursor(IntegrationTest):
             self.assertEqual(3, len(list(db.test.find().where(code_with_scope))))
 
         self.assertEqual(10, len(list(db.test.find())))
-
-        self.assertEqual(3, db.test.find().where("this.x < 3").count())
-        self.assertEqual(10, db.test.find().count())
-        self.assertEqual(3, db.test.find().where(u"this.x < 3").count())
         self.assertEqual([0, 1, 2], [a["x"] for a in db.test.find().where("this.x < 3")])
         self.assertEqual([], [a["x"] for a in db.test.find({"x": 5}).where("this.x < 3")])
         self.assertEqual([5], [a["x"] for a in db.test.find({"x": 5}).where("this.x > 3")])
@@ -885,7 +800,7 @@ class TestCursor(IntegrationTest):
 
         self.assertEqual(cursor, cursor.rewind())
 
-    # manipulate, oplog_reply, and snapshot are all deprecated.
+    # oplog_reply, and snapshot are all deprecated.
     @ignore_deprecations
     def test_clone(self):
         self.db.test.insert_many([{"x": i} for i in range(1, 4)])
@@ -934,7 +849,6 @@ class TestCursor(IntegrationTest):
             allow_partial_results=True,
             oplog_replay=True,
             batch_size=123,
-            manipulate=False,
             collation={"locale": "en_US"},
             hint=[("_id", 1)],
             max_scan=100,
@@ -991,12 +905,6 @@ class TestCursor(IntegrationTest):
         self.assertRaises(StopIteration, cursor.next)
         self.assertRaises(StopIteration, cursor2.next)
 
-    @ignore_deprecations
-    def test_count_with_fields(self):
-        self.db.test.drop()
-        self.db.test.insert_one({"x": 1})
-        self.assertEqual(1, self.db.test.find({}, ["a"]).count())
-
     def test_bad_getitem(self):
         self.assertRaises(TypeError, lambda x: self.db.test.find()[x], "hello")
         self.assertRaises(TypeError, lambda x: self.db.test.find()[x], 5.5)
@@ -1029,7 +937,7 @@ class TestCursor(IntegrationTest):
             self.fail()
 
         self.assertEqual(5, len(list(self.db.test.find()[20:25])))
-        self.assertEqual(5, len(list(self.db.test.find()[long(20) : long(25)])))
+        self.assertEqual(5, len(list(self.db.test.find()[20:25])))
         for a, b in zip(count(20), self.db.test.find()[20:25]):
             self.assertEqual(a, b["i"])
 
@@ -1064,37 +972,13 @@ class TestCursor(IntegrationTest):
         self.assertEqual(50, self.db.test.find()[50]["i"])
         self.assertEqual(50, self.db.test.find().skip(50)[0]["i"])
         self.assertEqual(50, self.db.test.find().skip(49)[1]["i"])
-        self.assertEqual(50, self.db.test.find()[long(50)]["i"])
+        self.assertEqual(50, self.db.test.find()[50]["i"])
         self.assertEqual(99, self.db.test.find()[99]["i"])
 
         self.assertRaises(IndexError, lambda x: self.db.test.find()[x], -1)
         self.assertRaises(IndexError, lambda x: self.db.test.find()[x], 100)
         self.assertRaises(IndexError, lambda x: self.db.test.find().skip(50)[x], 50)
 
-    @ignore_deprecations
-    def test_count_with_limit_and_skip(self):
-        self.assertRaises(TypeError, self.db.test.find().count, "foo")
-
-        def check_len(cursor, length):
-            self.assertEqual(len(list(cursor)), cursor.count(True))
-            self.assertEqual(length, cursor.count(True))
-
-        self.db.drop_collection("test")
-        self.db.test.insert_many([{"i": i} for i in range(100)])
-
-        check_len(self.db.test.find(), 100)
-
-        check_len(self.db.test.find().limit(10), 10)
-        check_len(self.db.test.find().limit(110), 100)
-
-        check_len(self.db.test.find().skip(10), 90)
-        check_len(self.db.test.find().skip(110), 0)
-
-        check_len(self.db.test.find().limit(10).skip(10), 10)
-        check_len(self.db.test.find()[10:20], 10)
-        check_len(self.db.test.find().limit(10).skip(95), 5)
-        check_len(self.db.test.find()[95:105], 5)
-
     def test_len(self):
         self.assertRaises(TypeError, len, self.db.test.find())
 
@@ -1240,40 +1124,28 @@ class TestCursor(IntegrationTest):
         self.assertTrue(c1.alive)
 
     @client_context.require_no_mongos
-    @ignore_deprecations
     def test_comment(self):
-        # MongoDB 3.1.5 changed the ns for commands.
-        regex = {"$regex": r"pymongo_test.(\$cmd|test)"}
-
-        if client_context.version.at_least(3, 5, 8, -1):
-            query_key = "command.comment"
-        elif client_context.version.at_least(3, 1, 8, -1):
-            query_key = "query.comment"
-        else:
-            query_key = "query.$comment"
-
         self.client.drop_database(self.db)
-        self.db.set_profiling_level(ALL)
+        self.db.command("profile", 2)  # Profile ALL commands.
         try:
             list(self.db.test.find().comment("foo"))
-            op = self.db.system.profile.find(
-                {"ns": "pymongo_test.test", "op": "query", query_key: "foo"}
+            count = self.db.system.profile.count_documents(
+                {"ns": "pymongo_test.test", "op": "query", "command.comment": "foo"}
             )
-            self.assertEqual(op.count(), 1)
-
-            self.db.test.find().comment("foo").count()
-            op = self.db.system.profile.find(
-                {"ns": regex, "op": "command", "command.count": "test", "command.comment": "foo"}
-            )
-            self.assertEqual(op.count(), 1)
+            self.assertEqual(count, 1)
 
             self.db.test.find().comment("foo").distinct("type")
-            op = self.db.system.profile.find(
-                {"ns": regex, "op": "command", "command.distinct": "test", "command.comment": "foo"}
+            count = self.db.system.profile.count_documents(
+                {
+                    "ns": "pymongo_test.test",
+                    "op": "command",
+                    "command.distinct": "test",
+                    "command.comment": "foo",
+                }
             )
-            self.assertEqual(op.count(), 1)
+            self.assertEqual(count, 1)
         finally:
-            self.db.set_profiling_level(OFF)
+            self.db.command("profile", 0)  # Turn off profiling.
             self.db.system.profile.drop()
 
         self.db.test.insert_many([{}, {}])
@@ -1281,57 +1153,6 @@ class TestCursor(IntegrationTest):
         next(cursor)
         self.assertRaises(InvalidOperation, cursor.comment, "hello")
 
-    def test_modifiers(self):
-        c = self.db.test
-
-        # "modifiers" is deprecated.
-        with ignore_deprecations():
-            cur = c.find()
-            self.assertTrue("$query" not in cur._Cursor__query_spec())
-            cur = c.find().comment("testing").max_time_ms(500)
-            self.assertTrue("$query" in cur._Cursor__query_spec())
-            self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
-            self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
-            cur = c.find(modifiers={"$maxTimeMS": 500, "$comment": "testing"})
-            self.assertTrue("$query" in cur._Cursor__query_spec())
-            self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
-            self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
-
-            # Keyword arg overwrites modifier.
-            # If we remove the "modifiers" arg, delete this test after checking
-            # that TestCommandMonitoring.test_find_options covers all cases.
-            cur = c.find(comment="hi", modifiers={"$comment": "bye"})
-            self.assertEqual(cur._Cursor__query_spec()["$comment"], "hi")
-
-            cur = c.find(max_scan=1, modifiers={"$maxScan": 2})
-            self.assertEqual(cur._Cursor__query_spec()["$maxScan"], 1)
-
-            cur = c.find(max_time_ms=1, modifiers={"$maxTimeMS": 2})
-            self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 1)
-
-            cur = c.find(min=1, modifiers={"$min": 2})
-            self.assertEqual(cur._Cursor__query_spec()["$min"], 1)
-
-            cur = c.find(max=1, modifiers={"$max": 2})
-            self.assertEqual(cur._Cursor__query_spec()["$max"], 1)
-
-            cur = c.find(return_key=True, modifiers={"$returnKey": False})
-            self.assertEqual(cur._Cursor__query_spec()["$returnKey"], True)
-
-            cur = c.find(hint=[("a", 1)], modifiers={"$hint": {"b": "1"}})
-            self.assertEqual(cur._Cursor__query_spec()["$hint"], {"a": 1})
-
-            # The arg is named show_record_id after the "find" command arg, the
-            # modifier is named $showDiskLoc for the OP_QUERY modifier. It's
-            # stored as $showDiskLoc then upgraded to showRecordId if we send a
-            # "find" command.
-            cur = c.find(show_record_id=True, modifiers={"$showDiskLoc": False})
-            self.assertEqual(cur._Cursor__query_spec()["$showDiskLoc"], True)
-
-            if not client_context.version.at_least(3, 7, 3):
-                cur = c.find(snapshot=True, modifiers={"$snapshot": False})
-                self.assertEqual(cur._Cursor__query_spec()["$snapshot"], True)
-
     def test_alive(self):
         self.db.test.delete_many({})
         self.db.test.insert_many([{} for _ in range(3)])
@@ -1396,7 +1217,6 @@ class TestCursor(IntegrationTest):
         cursor = Cursor.__new__(Cursor)  # Skip calling __init__
         cursor.__del__()  # no error
 
-    @client_context.require_version_min(3, 6)
     def test_getMore_does_not_send_readPreference(self):
         listener = AllowListEventListener("find", "getMore")
         client = rs_or_single_client(event_listeners=[listener])
@@ -1426,11 +1246,6 @@ class TestRawBatchCursor(IntegrationTest):
         self.assertEqual(1, len(batches))
         self.assertEqual(docs, decode_all(batches[0]))
 
-    def test_manipulate(self):
-        c = self.db.test
-        with self.assertRaises(InvalidOperation):
-            c.find_raw_batches(manipulate=True)
-
     @client_context.require_transactions
     def test_find_raw_transaction(self):
         c = self.db.test
@@ -1540,28 +1355,15 @@ class TestRawBatchCursor(IntegrationTest):
         with self.assertRaises(InvalidOperation):
             self.db.test.find_raw_batches()[0]
 
-    @client_context.require_version_min(3, 4)
     def test_collation(self):
         next(self.db.test.find_raw_batches(collation=Collation("en_US")))
 
-    @client_context.require_version_max(3, 2)
-    def test_collation_error(self):
-        with self.assertRaises(ConfigurationError):
-            next(self.db.test.find_raw_batches(collation=Collation("en_US")))
-
-    @client_context.require_version_min(3, 2)
     @client_context.require_no_mmap  # MMAPv1 does not support read concern
     def test_read_concern(self):
         self.db.get_collection("test", write_concern=WriteConcern(w="majority")).insert_one({})
         c = self.db.get_collection("test", read_concern=ReadConcern("majority"))
         next(c.find_raw_batches())
 
-    @client_context.require_version_max(3, 1)
-    def test_read_concern_error(self):
-        c = self.db.get_collection("test", read_concern=ReadConcern("majority"))
-        with self.assertRaises(ConfigurationError):
-            next(c.find_raw_batches())
-
     def test_monitoring(self):
         listener = EventListener()
         client = rs_or_single_client(event_listeners=[listener])
@@ -1723,15 +1525,9 @@ class TestRawBatchCommandCursor(IntegrationTest):
         with self.assertRaises(InvalidOperation):
             self.db.test.aggregate_raw_batches([])[0]
 
-    @client_context.require_version_min(3, 4)
     def test_collation(self):
         next(self.db.test.aggregate_raw_batches([], collation=Collation("en_US")))
 
-    @client_context.require_version_max(3, 2)
-    def test_collation_error(self):
-        with self.assertRaises(ConfigurationError):
-            next(self.db.test.aggregate_raw_batches([], collation=Collation("en_US")))
-
     def test_monitoring(self):
         listener = EventListener()
         client = rs_or_single_client(event_listeners=[listener])
