@@ -25,7 +25,7 @@ sys.path[0:0] = [""]
 
 from test import client_context, unittest
 from test.test_client import IntegrationTest
-from test.utils import ignore_deprecations, rs_client
+from test.utils import rs_client
 
 from bson import (
     _BUILT_IN_TYPES,
@@ -39,7 +39,6 @@ from bson import (
     decode_iter,
     encode,
 )
-from bson.code import Code
 from bson.codec_options import (
     CodecOptions,
     TypeCodec,
@@ -49,7 +48,6 @@ from bson.codec_options import (
 )
 from bson.errors import InvalidDocument
 from bson.int64 import Int64
-from bson.py3compat import text_type
 from bson.raw_bson import RawBSONDocument
 from gridfs import GridIn, GridOut
 from pymongo.collection import ReturnDocument
@@ -122,7 +120,7 @@ UNINT_CODECOPTS = CodecOptions(
 
 
 class UppercaseTextDecoder(TypeDecoder):
-    bson_type = text_type
+    bson_type = str
 
     def transform_bson(self, value):
         return value.upper()
@@ -646,25 +644,6 @@ class TestCollectionWCustomType(IntegrationTest):
         for doc_cls in [RawBSONDocument, OrderedDict]:
             run_test(doc_cls)
 
-    @client_context.require_version_max(4, 1, 0, -1)
-    def test_group_w_custom_type(self):
-        db = self.db
-        test = db.get_collection("test", codec_options=UNINT_CODECOPTS)
-        test.insert_many(
-            [
-                {"sku": "a", "qty": UndecipherableInt64Type(2)},
-                {"sku": "b", "qty": UndecipherableInt64Type(5)},
-                {"sku": "a", "qty": UndecipherableInt64Type(1)},
-            ]
-        )
-
-        self.assertEqual(
-            [
-                {"sku": "b", "qty": UndecipherableInt64Type(5)},
-            ],
-            test.group(["sku", "qty"], {"sku": "b"}, {}, "function (obj, prev) { }"),
-        )
-
     def test_aggregate_w_custom_type_decoder(self):
         db = self.db
         db.test.insert_many(
@@ -703,36 +682,6 @@ class TestCollectionWCustomType(IntegrationTest):
 
         self.assertEqual(values, test.distinct("a"))
 
-    def test_map_reduce_w_custom_type(self):
-        test = self.db.get_collection("test", codec_options=UPPERSTR_DECODER_CODECOPTS)
-
-        test.insert_many(
-            [
-                {"_id": 1, "sku": "abcd", "qty": 1},
-                {"_id": 2, "sku": "abcd", "qty": 2},
-                {"_id": 3, "sku": "abcd", "qty": 3},
-            ]
-        )
-
-        map = Code("function () {" "  emit(this.sku, this.qty);" "}")
-        reduce = Code("function (key, values) {" "  return Array.sum(values);" "}")
-
-        result = test.map_reduce(map, reduce, out={"inline": 1})
-        self.assertTrue(isinstance(result, dict))
-        self.assertTrue("results" in result)
-        self.assertEqual(result["results"][0], {"_id": "ABCD", "value": 6})
-
-        result = test.inline_map_reduce(map, reduce)
-        self.assertTrue(isinstance(result, list))
-        self.assertEqual(1, len(result))
-        self.assertEqual(result[0]["_id"], "ABCD")
-
-        full_result = test.inline_map_reduce(map, reduce, full_response=True)
-        result = full_result["results"]
-        self.assertTrue(isinstance(result, list))
-        self.assertEqual(1, len(result))
-        self.assertEqual(result[0]["_id"], "ABCD")
-
     def test_find_one_and__w_custom_type_decoder(self):
         db = self.db
         c = db.get_collection("test", codec_options=UNINT_DECODER_CODECOPTS)
@@ -759,22 +708,6 @@ class TestCollectionWCustomType(IntegrationTest):
         self.assertEqual(doc["x"].value, 3)
         self.assertIsNone(c.find_one())
 
-    @ignore_deprecations
-    def test_find_and_modify_w_custom_type_decoder(self):
-        db = self.db
-        c = db.get_collection("test", codec_options=UNINT_DECODER_CODECOPTS)
-        c.insert_one({"_id": 1, "x": Int64(1)})
-
-        doc = c.find_and_modify({"_id": 1}, {"$inc": {"x": Int64(10)}})
-        self.assertEqual(doc["_id"], 1)
-        self.assertIsInstance(doc["x"], UndecipherableInt64Type)
-        self.assertEqual(doc["x"].value, 1)
-
-        doc = c.find_one()
-        self.assertEqual(doc["_id"], 1)
-        self.assertIsInstance(doc["x"], UndecipherableInt64Type)
-        self.assertEqual(doc["x"].value, 11)
-
 
 class TestGridFileCustomType(IntegrationTest):
     def setUp(self):
@@ -809,7 +742,7 @@ class TestGridFileCustomType(IntegrationTest):
         self.assertEqual(["foo"], two.aliases)
         self.assertEqual({"foo": "red", "bar": "blue"}, two.metadata)
         self.assertEqual(3, two.bar)
-        self.assertEqual("5eb63bbbe01eeed093cb22bb8f5acdc3", two.md5)
+        self.assertEqual(None, two.md5)
 
         for attr in [
             "_id",
@@ -938,11 +871,11 @@ class ChangeStreamsWCustomTypesTestMixin(object):
 
 class TestCollectionChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCustomTypesTestMixin):
     @classmethod
-    @client_context.require_version_min(3, 6, 0)
     @client_context.require_no_mmap
     @client_context.require_no_standalone
     def setUpClass(cls):
         super(TestCollectionChangeStreamsWCustomTypes, cls).setUpClass()
+        cls.db.test.delete_many({})
 
     def tearDown(self):
         self.input_target.drop()
@@ -962,6 +895,7 @@ class TestDatabaseChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCusto
     @client_context.require_no_standalone
     def setUpClass(cls):
         super(TestDatabaseChangeStreamsWCustomTypes, cls).setUpClass()
+        cls.db.test.delete_many({})
 
     def tearDown(self):
         self.input_target.drop()
@@ -981,6 +915,7 @@ class TestClusterChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCustom
     @client_context.require_no_standalone
     def setUpClass(cls):
         super(TestClusterChangeStreamsWCustomTypes, cls).setUpClass()
+        cls.db.test.delete_many({})
 
     def tearDown(self):
         self.input_target.drop()
