@@ -66,7 +66,7 @@ class TestSampleShellCommands(IntegrationTest):
         cursor = db.inventory.find({"item": "canvas"})
         # End Example 2
 
-        self.assertEqual(cursor.count(), 1)
+        self.assertEqual(len(list(cursor)), 1)
 
         # Start Example 3
         db.inventory.insert_many(
@@ -740,7 +740,6 @@ class TestSampleShellCommands(IntegrationTest):
 
         self.assertEqual(db.inventory.count_documents({}), 0)
 
-    @client_context.require_version_min(3, 5, 11)
     @client_context.require_replica_set
     @client_context.require_no_mmap
     def test_change_streams(self):
@@ -834,38 +833,33 @@ class TestSampleShellCommands(IntegrationTest):
         )
         # End Aggregation Example 3
 
-        # $lookup was new in 3.2. The let and pipeline options
-        # were added in 3.6.
-        if client_context.version.at_least(3, 6, 0):
-            # Start Aggregation Example 4
-            db.air_alliances.aggregate(
-                [
-                    {
-                        "$lookup": {
-                            "from": "air_airlines",
-                            "let": {"constituents": "$airlines"},
-                            "pipeline": [
-                                {"$match": {"$expr": {"$in": ["$name", "$$constituents"]}}}
-                            ],
-                            "as": "airlines",
-                        }
-                    },
-                    {
-                        "$project": {
-                            "_id": 0,
-                            "name": 1,
-                            "airlines": {
-                                "$filter": {
-                                    "input": "$airlines",
-                                    "as": "airline",
-                                    "cond": {"$eq": ["$$airline.country", "Canada"]},
-                                }
-                            },
-                        }
-                    },
-                ]
-            )
-            # End Aggregation Example 4
+        # Start Aggregation Example 4
+        db.air_alliances.aggregate(
+            [
+                {
+                    "$lookup": {
+                        "from": "air_airlines",
+                        "let": {"constituents": "$airlines"},
+                        "pipeline": [{"$match": {"$expr": {"$in": ["$name", "$$constituents"]}}}],
+                        "as": "airlines",
+                    }
+                },
+                {
+                    "$project": {
+                        "_id": 0,
+                        "name": 1,
+                        "airlines": {
+                            "$filter": {
+                                "input": "$airlines",
+                                "as": "airline",
+                                "cond": {"$eq": ["$$airline.country", "Canada"]},
+                            }
+                        },
+                    }
+                },
+            ]
+        )
+        # End Aggregation Example 4
 
     def test_commands(self):
         db = self.db
@@ -893,7 +887,6 @@ class TestSampleShellCommands(IntegrationTest):
         )
         # End Index Example 1
 
-    @client_context.require_version_min(3, 6, 0)
     @client_context.require_replica_set
     def test_misc(self):
         # Marketing examples
@@ -1135,7 +1128,6 @@ class TestTransactionExamples(IntegrationTest):
 
 
 class TestCausalConsistencyExamples(IntegrationTest):
-    @client_context.require_version_min(3, 6, 0)
     @client_context.require_secondaries_count(1)
     @client_context.require_no_mmap
     def test_causal_consistency(self):
@@ -1204,7 +1196,6 @@ class TestVersionedApiExamples(IntegrationTest):
         client = MongoClient(uri, server_api=ServerApi("1", deprecation_errors=True))
         # End Versioned API Example 4
 
-    @unittest.skip("PYTHON-3167 count has been added to API version 1")
     @client_context.require_version_min(4, 7)
     def test_versioned_api_migration(self):
         # SERVER-58785
