@@ -18,8 +18,10 @@
 """
 
 import datetime
+import io
 import sys
 import zipfile
+from io import BytesIO
 
 sys.path[0:0] = [""]
 
@@ -27,7 +29,6 @@ from test import IntegrationTest, qcheck, unittest
 from test.utils import EventListener, rs_or_single_client
 
 from bson.objectid import ObjectId
-from bson.py3compat import StringIO
 from gridfs import GridFS
 from gridfs.errors import NoFile
 from gridfs.grid_file import (
@@ -84,8 +85,7 @@ class TestGridFileNoConnect(unittest.TestCase):
 
 class TestGridFile(IntegrationTest):
     def setUp(self):
-        self.db.drop_collection("fs.files")
-        self.db.drop_collection("fs.chunks")
+        self.cleanup_colls(self.db.fs.files, self.db.fs.chunks)
 
     def test_basic(self):
         f = GridIn(self.db.fs, filename="test")
@@ -116,7 +116,7 @@ class TestGridFile(IntegrationTest):
         f = GridIn(self.db.fs)
         f.write(b"hello world\n")
         f.close()
-        self.assertEqual("6f5902ac237024bdd0c176cb93063dc4", f.md5)
+        self.assertEqual(None, f.md5)
 
     def test_alternate_collection(self):
         self.db.alt.files.delete_many({})
@@ -132,9 +132,6 @@ class TestGridFile(IntegrationTest):
         g = GridOut(self.db.alt, f._id)
         self.assertEqual(b"hello world", g.read())
 
-        # test that md5 still works...
-        self.assertEqual("5eb63bbbe01eeed093cb22bb8f5acdc3", g.md5)
-
     def test_grid_in_default_opts(self):
         self.assertRaises(TypeError, GridIn, "foo")
 
@@ -198,7 +195,7 @@ class TestGridFile(IntegrationTest):
 
         self.assertEqual({"foo": 1}, a.metadata)
 
-        self.assertEqual("d41d8cd98f00b204e9800998ecf8427e", a.md5)
+        self.assertEqual(None, a.md5)
         self.assertRaises(AttributeError, setattr, a, "md5", 5)
 
         # Make sure custom attributes that were set both before and after
@@ -229,7 +226,7 @@ class TestGridFile(IntegrationTest):
         self.assertTrue(isinstance(b.upload_date, datetime.datetime))
         self.assertEqual(None, b.aliases)
         self.assertEqual(None, b.metadata)
-        self.assertEqual("d41d8cd98f00b204e9800998ecf8427e", b.md5)
+        self.assertEqual(None, b.md5)
 
         for attr in [
             "_id",
@@ -288,7 +285,7 @@ class TestGridFile(IntegrationTest):
         self.assertEqual(["foo"], two.aliases)
         self.assertEqual({"foo": 1, "bar": 2}, two.metadata)
         self.assertEqual(3, two.bar)
-        self.assertEqual("5eb63bbbe01eeed093cb22bb8f5acdc3", two.md5)
+        self.assertEqual(None, two.md5)
 
         for attr in [
             "_id",
@@ -334,7 +331,7 @@ class TestGridFile(IntegrationTest):
 
         five = GridIn(self.db.fs, chunk_size=2)
         five.write(b"hello")
-        buffer = StringIO(b" world")
+        buffer = BytesIO(b" world")
         five.write(buffer)
         five.write(b" and mongodb")
         five.close()
@@ -353,6 +350,20 @@ class TestGridFile(IntegrationTest):
         self.assertRaises(ValueError, f.write, "test")
         f.close()
 
+    def test_closed(self):
+        f = GridIn(self.db.fs, chunkSize=5)
+        f.write(b"Hello world.\nHow are you?")
+        f.close()
+
+        g = GridOut(self.db.fs, f._id)
+        self.assertFalse(g.closed)
+        g.read(1)
+        self.assertFalse(g.closed)
+        g.read(100)
+        self.assertFalse(g.closed)
+        g.close()
+        self.assertTrue(g.closed)
+
     def test_multi_chunk_file(self):
         random_string = b"a" * (DEFAULT_CHUNK_SIZE + 1000)
 
@@ -480,6 +491,62 @@ Bye"""
         self.assertEqual(b"e", g.readline(1))
         self.assertEqual(b"llo world,\n", g.readline())
 
+    def test_readlines(self):
+        f = GridIn(self.db.fs, chunkSize=5)
+        f.write(
+            (
+                b"""Hello world,
+How are you?
+Hope all is well.
+Bye"""
+            )
+        )
+        f.close()
+
+        # Try read(), then readlines().
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual(b"He", g.read(2))
+        self.assertEqual([b"llo world,\n", b"How are you?\n"], g.readlines(11))
+        self.assertEqual([b"Hope all is well.\n", b"Bye"], g.readlines())
+        self.assertEqual([], g.readlines())
+
+        # Try readline(), then readlines().
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual(b"Hello world,\n", g.readline())
+        self.assertEqual([b"How are you?\n", b"Hope all is well.\n"], g.readlines(13))
+        self.assertEqual(b"Bye", g.readline())
+        self.assertEqual([], g.readlines())
+
+        # Only readlines().
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual(
+            [b"Hello world,\n", b"How are you?\n", b"Hope all is well.\n", b"Bye"], g.readlines()
+        )
+
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual(
+            [b"Hello world,\n", b"How are you?\n", b"Hope all is well.\n", b"Bye"], g.readlines(0)
+        )
+
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual([b"Hello world,\n"], g.readlines(1))
+        self.assertEqual([b"How are you?\n"], g.readlines(12))
+        self.assertEqual([b"Hope all is well.\n", b"Bye"], g.readlines(18))
+
+        # Try readlines() first, then read().
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual([b"Hello world,\n"], g.readlines(1))
+        self.assertEqual(b"H", g.read(1))
+        self.assertEqual([b"ow are you?\n", b"Hope all is well.\n"], g.readlines(29))
+        self.assertEqual([b"Bye"], g.readlines(1))
+
+        # Try readlines() first, then readline().
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual([b"Hello world,\n"], g.readlines(1))
+        self.assertEqual(b"How are you?\n", g.readline())
+        self.assertEqual([b"Hope all is well.\n"], g.readlines(17))
+        self.assertEqual(b"Bye", g.readline())
+
     def test_iterator(self):
         f = GridIn(self.db.fs)
         f.close()
@@ -487,19 +554,26 @@ Bye"""
         self.assertEqual([], list(g))
 
         f = GridIn(self.db.fs)
-        f.write(b"hello world")
+        f.write(b"hello world\nhere are\nsome lines.")
         f.close()
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual([b"hello world"], list(g))
-        self.assertEqual(b"hello", g.read(5))
-        self.assertEqual([b"hello world"], list(g))
-        self.assertEqual(b" worl", g.read(5))
+        self.assertEqual([b"hello world\n", b"here are\n", b"some lines."], list(g))
+        self.assertEqual(b"", g.read(5))
+        self.assertEqual([], list(g))
+
+        g = GridOut(self.db.fs, f._id)
+        self.assertEqual(b"hello world\n", next(iter(g)))
+        self.assertEqual(b"here", g.read(4))
+        self.assertEqual(b" are\n", next(iter(g)))
+        self.assertEqual(b"some lines", g.read(10))
+        self.assertEqual(b".", next(iter(g)))
+        self.assertRaises(StopIteration, iter(g).__next__)
 
         f = GridIn(self.db.fs, chunk_size=2)
         f.write(b"hello world")
         f.close()
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual([b"he", b"ll", b"o ", b"wo", b"rl", b"d"], list(g))
+        self.assertEqual([b"hello world"], list(g))
 
     def test_read_unaligned_buffer_size(self):
         in_data = b"This is a text that doesn't " b"quite fit in a single 16-byte chunk."
@@ -537,21 +611,21 @@ Bye"""
 
     def test_write_unicode(self):
         f = GridIn(self.db.fs)
-        self.assertRaises(TypeError, f.write, u"foo")
+        self.assertRaises(TypeError, f.write, "foo")
 
         f = GridIn(self.db.fs, encoding="utf-8")
-        f.write(u"foo")
+        f.write("foo")
         f.close()
 
         g = GridOut(self.db.fs, f._id)
         self.assertEqual(b"foo", g.read())
 
         f = GridIn(self.db.fs, encoding="iso-8859-1")
-        f.write(u"aé")
+        f.write("aé")
         f.close()
 
         g = GridOut(self.db.fs, f._id)
-        self.assertEqual(u"aé".encode("iso-8859-1"), g.read())
+        self.assertEqual("aé".encode("iso-8859-1"), g.read())
 
     def test_set_after_close(self):
         f = GridIn(self.db.fs, _id="foo", bar="baz")
@@ -599,7 +673,7 @@ Bye"""
 
     def test_prechunked_string(self):
         def write_me(s, chunk_size):
-            buf = StringIO(s)
+            buf = BytesIO(s)
             infile = GridIn(self.db.fs)
             while True:
                 to_write = buf.read(chunk_size)
@@ -678,7 +752,7 @@ Bye"""
         self.assertIn("getMore", listener.started_command_names())
 
     def test_zip(self):
-        zf = StringIO()
+        zf = BytesIO()
         z = zipfile.ZipFile(zf, "w")
         z.writestr("test.txt", b"hello world")
         z.close()
@@ -695,6 +769,21 @@ Bye"""
         self.assertSequenceEqual(z.namelist(), ["test.txt"])
         self.assertEqual(z.read("test.txt"), b"hello world")
 
+    def test_grid_out_unsupported_operations(self):
+        f = GridIn(self.db.fs, chunkSize=3)
+        f.write(b"hello world")
+        f.close()
+
+        g = GridOut(self.db.fs, f._id)
+
+        self.assertRaises(io.UnsupportedOperation, g.writelines, [b"some", b"lines"])
+        self.assertRaises(io.UnsupportedOperation, g.write, b"some text")
+        self.assertRaises(io.UnsupportedOperation, g.fileno)
+        self.assertRaises(io.UnsupportedOperation, g.truncate)
+
+        self.assertFalse(g.writable())
+        self.assertFalse(g.isatty())
+
 
 if __name__ == "__main__":
     unittest.main()
