@@ -16,32 +16,25 @@
 
 """Tests for the gridfs package.
 """
+
+import datetime
 import sys
+import threading
+import time
+from io import BytesIO
 
 sys.path[0:0] = [""]
 
-import datetime
-import threading
-import time
 from test import IntegrationTest, client_context, unittest
-from test.test_replica_set_client import TestReplicaSetClientBase
-from test.utils import (
-    ignore_deprecations,
-    joinall,
-    one,
-    rs_client,
-    rs_or_single_client,
-    single_client,
-)
+from test.utils import joinall, one, rs_client, rs_or_single_client, single_client
 
 import gridfs
 from bson.binary import Binary
-from bson.py3compat import StringIO, string_type
 from gridfs.errors import CorruptGridFile, FileExists, NoFile
 from gridfs.grid_file import GridOutCursor
 from pymongo.errors import (
     ConfigurationError,
-    ConnectionFailure,
+    NotPrimaryError,
     ServerSelectionTimeoutError,
 )
 from pymongo.mongo_client import MongoClient
@@ -96,10 +89,9 @@ class TestGridfs(IntegrationTest):
         cls.alt = gridfs.GridFS(cls.db, "alt")
 
     def setUp(self):
-        self.db.drop_collection("fs.files")
-        self.db.drop_collection("fs.chunks")
-        self.db.drop_collection("alt.files")
-        self.db.drop_collection("alt.chunks")
+        self.cleanup_colls(
+            self.db.fs.files, self.db.fs.chunks, self.db.alt.files, self.db.alt.chunks
+        )
 
     def test_basic(self):
         oid = self.fs.put(b"hello world")
@@ -156,7 +148,7 @@ class TestGridfs(IntegrationTest):
         self.assertEqual(oid, raw["_id"])
         self.assertTrue(isinstance(raw["uploadDate"], datetime.datetime))
         self.assertEqual(255 * 1024, raw["chunkSize"])
-        self.assertTrue(isinstance(raw["md5"], string_type))
+        self.assertNotIn("md5", raw)
 
     def test_corrupt_chunk(self):
         files_id = self.fs.put(b"foobar")
@@ -171,12 +163,11 @@ class TestGridfs(IntegrationTest):
             self.fs.delete(files_id)
 
     def test_put_ensures_index(self):
-        # setUp has dropped collections.
-        names = self.db.list_collection_names()
-        self.assertFalse([name for name in names if name.startswith("fs")])
-
         chunks = self.db.fs.chunks
         files = self.db.fs.files
+        # Ensure the collections are removed.
+        chunks.drop()
+        files.drop()
         self.fs.put(b"junk")
 
         self.assertTrue(
@@ -335,7 +326,7 @@ class TestGridfs(IntegrationTest):
         self.fs.delete(three)
 
     def test_put_filelike(self):
-        oid = self.fs.put(StringIO(b"hello world"), chunk_size=1)
+        oid = self.fs.put(BytesIO(b"hello world"), chunk_size=1)
         self.assertEqual(11, self.db.fs.chunks.count_documents({}))
         self.assertEqual(b"hello world", self.fs.get(oid).read())
 
@@ -376,14 +367,14 @@ class TestGridfs(IntegrationTest):
         self.assertFalse(self.fs.exists({"foo": {"$gt": 12}}))
 
     def test_put_unicode(self):
-        self.assertRaises(TypeError, self.fs.put, u"hello")
+        self.assertRaises(TypeError, self.fs.put, "hello")
 
-        oid = self.fs.put(u"hello", encoding="utf-8")
+        oid = self.fs.put("hello", encoding="utf-8")
         self.assertEqual(b"hello", self.fs.get(oid).read())
         self.assertEqual("utf-8", self.fs.get(oid).encoding)
 
-        oid = self.fs.put(u"aé", encoding="iso-8859-1")
-        self.assertEqual(u"aé".encode("iso-8859-1"), self.fs.get(oid).read())
+        oid = self.fs.put("aé", encoding="iso-8859-1")
+        self.assertEqual("aé".encode("iso-8859-1"), self.fs.get(oid).read())
         self.assertEqual("iso-8859-1", self.fs.get(oid).encoding)
 
     def test_missing_length_iter(self):
@@ -411,7 +402,6 @@ class TestGridfs(IntegrationTest):
         f = fs.new_file()
         self.assertRaises(ServerSelectionTimeoutError, f.close)
 
-    @ignore_deprecations
     def test_gridfs_find(self):
         self.fs.put(b"test2", filename="two")
         time.sleep(0.01)
@@ -420,8 +410,9 @@ class TestGridfs(IntegrationTest):
         self.fs.put(b"test1", filename="one")
         time.sleep(0.01)
         self.fs.put(b"test2++", filename="two")
-        self.assertEqual(3, self.fs.find({"filename": "two"}).count())
-        self.assertEqual(4, self.fs.find().count())
+        files = self.db.fs.files
+        self.assertEqual(3, files.count_documents({"filename": "two"}))
+        self.assertEqual(4, files.count_documents({}))
         cursor = self.fs.find(no_cursor_timeout=False).sort("uploadDate", -1).skip(1).limit(2)
         gout = next(cursor)
         self.assertEqual(b"test1", gout.read())
@@ -471,21 +462,6 @@ class TestGridfs(IntegrationTest):
 
     def test_md5(self):
         gin = self.fs.new_file()
-        gin.write(b"includes md5 sum")
-        gin.close()
-        self.assertIsNotNone(gin.md5)
-        md5sum = gin.md5
-
-        gout = self.fs.get(gin._id)
-        self.assertIsNotNone(gout.md5)
-        self.assertEqual(md5sum, gout.md5)
-
-        _id = self.fs.put(b"also includes md5 sum")
-        gout = self.fs.get(_id)
-        self.assertIsNotNone(gout.md5)
-
-        fs = gridfs.GridFS(self.db, disable_md5=True)
-        gin = fs.new_file()
         gin.write(b"no md5 sum")
         gin.close()
         self.assertIsNone(gin.md5)
@@ -493,12 +469,12 @@ class TestGridfs(IntegrationTest):
         gout = self.fs.get(gin._id)
         self.assertIsNone(gout.md5)
 
-        _id = fs.put(b"still no md5 sum")
+        _id = self.fs.put(b"still no md5 sum")
         gout = self.fs.get(_id)
         self.assertIsNone(gout.md5)
 
 
-class TestGridfsReplicaSet(TestReplicaSetClientBase):
+class TestGridfsReplicaSet(IntegrationTest):
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
@@ -509,7 +485,7 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
         client_context.client.drop_database("gfsreplica")
 
     def test_gridfs_replica_set(self):
-        rsc = rs_client(w=self.w, read_preference=ReadPreference.SECONDARY)
+        rsc = rs_client(w=client_context.w, read_preference=ReadPreference.SECONDARY)
 
         fs = gridfs.GridFS(rsc.gfsreplica, "gfsreplicatest")
 
@@ -521,10 +497,7 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
         self.assertEqual(b"foo", content)
 
     def test_gridfs_secondary(self):
-        primary_host, primary_port = self.primary
-        primary_connection = single_client(primary_host, primary_port)
-
-        secondary_host, secondary_port = one(self.secondaries)
+        secondary_host, secondary_port = one(self.client.secondaries)
         secondary_connection = single_client(
             secondary_host, secondary_port, read_preference=ReadPreference.SECONDARY
         )
@@ -534,12 +507,12 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
         fs = gridfs.GridFS(secondary_connection.gfsreplica, "gfssecondarytest")
 
         # This won't detect secondary, raises error
-        self.assertRaises(ConnectionFailure, fs.put, b"foo")
+        self.assertRaises(NotPrimaryError, fs.put, b"foo")
 
     def test_gridfs_secondary_lazy(self):
         # Should detect it's connected to secondary and not attempt to
         # create index.
-        secondary_host, secondary_port = one(self.secondaries)
+        secondary_host, secondary_port = one(self.client.secondaries)
         client = single_client(
             secondary_host, secondary_port, read_preference=ReadPreference.SECONDARY, connect=False
         )
@@ -549,7 +522,7 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
 
         # Connects, doesn't create index.
         self.assertRaises(NoFile, fs.get_last_version)
-        self.assertRaises(ConnectionFailure, fs.put, "data")
+        self.assertRaises(NotPrimaryError, fs.put, "data")
 
 
 if __name__ == "__main__":
