@@ -13,6 +13,7 @@
 # limitations under the License.
 
 """Test GridFSBucket class."""
+
 import copy
 import datetime
 import os
@@ -20,16 +21,14 @@ import re
 import sys
 from json import loads
 
-import gridfs
-
 sys.path[0:0] = [""]
 
 from test import IntegrationTest, unittest
 
+import gridfs
 from bson import Binary
 from bson.int64 import Int64
 from bson.json_util import object_hook
-from bson.py3compat import bytes_from_hex
 from gridfs.errors import CorruptGridFile, NoFile
 
 # Commands.
@@ -64,10 +63,9 @@ class TestAllScenarios(IntegrationTest):
         }
 
     def init_db(self, data, test):
-        self.db.drop_collection("fs.files")
-        self.db.drop_collection("fs.chunks")
-        self.db.drop_collection("expected.files")
-        self.db.drop_collection("expected.chunks")
+        self.cleanup_colls(
+            self.db.fs.files, self.db.fs.chunks, self.db.expected.files, self.db.expected.chunks
+        )
 
         # Read in data.
         if data["files"]:
@@ -209,7 +207,7 @@ def create_tests():
                 for key, val in jsn.items():
                     if key in ("data", "source", "result"):
                         if "$hex" in val:
-                            jsn[key] = Binary(bytes_from_hex(val["$hex"]))
+                            jsn[key] = Binary(bytes.fromhex(val["$hex"]))
                     if isinstance(jsn[key], dict):
                         str2hex(jsn[key])
                     if isinstance(jsn[key], list):
