@@ -23,8 +23,7 @@ from test import IntegrationTest, client_knobs, unittest
 from test.utils import HeartbeatEventListener, MockPool, single_client, wait_until
 
 from pymongo.errors import ConnectionFailure
-from pymongo.hello import Hello
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import Hello, HelloCompat
 from pymongo.monitor import Monitor
 
 
