@@ -24,7 +24,7 @@ sys.path[0:0] = [""]
 
 from test import IntegrationTest, unittest
 
-from bson import EPOCH_AWARE, SON, json_util
+from bson import EPOCH_AWARE, EPOCH_NAIVE, SON, json_util
 from bson.binary import (
     ALL_UUID_REPRESENTATIONS,
     MD5_SUBTYPE,
@@ -36,7 +36,12 @@ from bson.binary import (
 from bson.code import Code
 from bson.dbref import DBRef
 from bson.int64 import Int64
-from bson.json_util import STRICT_JSON_OPTIONS, DatetimeRepresentation
+from bson.json_util import (
+    LEGACY_JSON_OPTIONS,
+    DatetimeRepresentation,
+    JSONMode,
+    JSONOptions,
+)
 from bson.max_key import MaxKey
 from bson.min_key import MinKey
 from bson.objectid import ObjectId
@@ -44,7 +49,12 @@ from bson.regex import Regex
 from bson.timestamp import Timestamp
 from bson.tz_util import FixedOffset, utc
 
-PY3 = sys.version_info[0] == 3
+STRICT_JSON_OPTIONS = JSONOptions(
+    strict_number_long=True,
+    datetime_representation=DatetimeRepresentation.ISO8601,
+    strict_uuid=True,
+    json_mode=JSONMode.LEGACY,
+)
 
 
 class TestJsonUtil(unittest.TestCase):
@@ -58,12 +68,16 @@ class TestJsonUtil(unittest.TestCase):
         self.round_trip({"hello": "world"})
 
     def test_json_options_with_options(self):
-        opts = json_util.JSONOptions(datetime_representation=DatetimeRepresentation.NUMBERLONG)
+        opts = JSONOptions(
+            datetime_representation=DatetimeRepresentation.NUMBERLONG, json_mode=JSONMode.LEGACY
+        )
         self.assertEqual(opts.datetime_representation, DatetimeRepresentation.NUMBERLONG)
-        opts2 = opts.with_options(datetime_representation=DatetimeRepresentation.ISO8601)
+        opts2 = opts.with_options(
+            datetime_representation=DatetimeRepresentation.ISO8601, json_mode=JSONMode.LEGACY
+        )
         self.assertEqual(opts2.datetime_representation, DatetimeRepresentation.ISO8601)
 
-        opts = json_util.JSONOptions(strict_number_long=True)
+        opts = JSONOptions(strict_number_long=True, json_mode=JSONMode.LEGACY)
         self.assertEqual(opts.strict_number_long, True)
         opts2 = opts.with_options(strict_number_long=False)
         self.assertEqual(opts2.strict_number_long, False)
@@ -92,66 +106,63 @@ class TestJsonUtil(unittest.TestCase):
         )
 
     def test_datetime(self):
+        tz_aware_opts = json_util.DEFAULT_JSON_OPTIONS.with_options(tz_aware=True)
         # only millis, not micros
-        self.round_trip({"date": datetime.datetime(2009, 12, 9, 15, 49, 45, 191000, utc)})
+        self.round_trip(
+            {"date": datetime.datetime(2009, 12, 9, 15, 49, 45, 191000, utc)},
+            json_options=tz_aware_opts,
+        )
+        self.round_trip({"date": datetime.datetime(2009, 12, 9, 15, 49, 45, 191000)})
 
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000+0000"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000000+0000"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000+00:00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000000+00:00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000000+00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000Z"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000000Z"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00Z"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        # No explicit offset
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000000"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        # Localtime behind UTC
-        jsn = '{"dt": { "$date" : "1969-12-31T16:00:00.000-0800"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1969-12-31T16:00:00.000000-0800"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1969-12-31T16:00:00.000-08:00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1969-12-31T16:00:00.000000-08:00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1969-12-31T16:00:00.000000-08"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        # Localtime ahead of UTC
-        jsn = '{"dt": { "$date" : "1970-01-01T01:00:00.000+0100"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T01:00:00.000000+0100"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T01:00:00.000+01:00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T01:00:00.000000+01:00"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
-        jsn = '{"dt": { "$date" : "1970-01-01T01:00:00.000000+01"}}'
-        self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
+        for jsn in [
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000+0000"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000000+0000"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000+00:00"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000000+00:00"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000000+00"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000Z"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000000Z"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00Z"}}',
+            '{"dt": {"$date": "1970-01-01T00:00:00.000"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00"}}',
+            '{"dt": { "$date" : "1970-01-01T00:00:00.000000"}}',
+            '{"dt": { "$date" : "1969-12-31T16:00:00.000-0800"}}',
+            '{"dt": { "$date" : "1969-12-31T16:00:00.000000-0800"}}',
+            '{"dt": { "$date" : "1969-12-31T16:00:00.000-08:00"}}',
+            '{"dt": { "$date" : "1969-12-31T16:00:00.000000-08:00"}}',
+            '{"dt": { "$date" : "1969-12-31T16:00:00.000000-08"}}',
+            '{"dt": { "$date" : "1970-01-01T01:00:00.000+0100"}}',
+            '{"dt": { "$date" : "1970-01-01T01:00:00.000000+0100"}}',
+            '{"dt": { "$date" : "1970-01-01T01:00:00.000+01:00"}}',
+            '{"dt": { "$date" : "1970-01-01T01:00:00.000000+01:00"}}',
+            '{"dt": { "$date" : "1970-01-01T01:00:00.000000+01"}}',
+        ]:
+            self.assertEqual(EPOCH_AWARE, json_util.loads(jsn, json_options=tz_aware_opts)["dt"])
+            self.assertEqual(EPOCH_NAIVE, json_util.loads(jsn)["dt"])
 
         dtm = datetime.datetime(1, 1, 1, 1, 1, 1, 0, utc)
         jsn = '{"dt": {"$date": -62135593139000}}'
-        self.assertEqual(dtm, json_util.loads(jsn)["dt"])
+        self.assertEqual(dtm, json_util.loads(jsn, json_options=tz_aware_opts)["dt"])
         jsn = '{"dt": {"$date": {"$numberLong": "-62135593139000"}}}'
-        self.assertEqual(dtm, json_util.loads(jsn)["dt"])
+        self.assertEqual(dtm, json_util.loads(jsn, json_options=tz_aware_opts)["dt"])
 
         # Test dumps format
         pre_epoch = {"dt": datetime.datetime(1, 1, 1, 1, 1, 1, 10000, utc)}
         post_epoch = {"dt": datetime.datetime(1972, 1, 1, 1, 1, 1, 10000, utc)}
-        self.assertEqual('{"dt": {"$date": -62135593138990}}', json_util.dumps(pre_epoch))
-        self.assertEqual('{"dt": {"$date": 63075661010}}', json_util.dumps(post_epoch))
+        self.assertEqual(
+            '{"dt": {"$date": {"$numberLong": "-62135593138990"}}}', json_util.dumps(pre_epoch)
+        )
+        self.assertEqual(
+            '{"dt": {"$date": "1972-01-01T01:01:01.010Z"}}', json_util.dumps(post_epoch)
+        )
+        self.assertEqual(
+            '{"dt": {"$date": -62135593138990}}',
+            json_util.dumps(pre_epoch, json_options=LEGACY_JSON_OPTIONS),
+        )
+        self.assertEqual(
+            '{"dt": {"$date": 63075661010}}',
+            json_util.dumps(post_epoch, json_options=LEGACY_JSON_OPTIONS),
+        )
         self.assertEqual(
             '{"dt": {"$date": {"$numberLong": "-62135593138990"}}}',
             json_util.dumps(pre_epoch, json_options=STRICT_JSON_OPTIONS),
@@ -161,8 +172,8 @@ class TestJsonUtil(unittest.TestCase):
             json_util.dumps(post_epoch, json_options=STRICT_JSON_OPTIONS),
         )
 
-        number_long_options = json_util.JSONOptions(
-            datetime_representation=DatetimeRepresentation.NUMBERLONG
+        number_long_options = JSONOptions(
+            datetime_representation=DatetimeRepresentation.NUMBERLONG, json_mode=JSONMode.LEGACY
         )
         self.assertEqual(
             '{"dt": {"$date": {"$numberLong": "63075661010"}}}',
@@ -188,23 +199,25 @@ class TestJsonUtil(unittest.TestCase):
         # Test tz_aware and tzinfo options
         self.assertEqual(
             datetime.datetime(1972, 1, 1, 1, 1, 1, 10000, utc),
-            json_util.loads('{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}')["dt"],
+            json_util.loads(
+                '{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}', json_options=tz_aware_opts
+            )["dt"],
         )
         self.assertEqual(
             datetime.datetime(1972, 1, 1, 1, 1, 1, 10000, utc),
             json_util.loads(
                 '{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}',
-                json_options=json_util.JSONOptions(tz_aware=True, tzinfo=utc),
+                json_options=JSONOptions(tz_aware=True, tzinfo=utc),
             )["dt"],
         )
         self.assertEqual(
             datetime.datetime(1972, 1, 1, 1, 1, 1, 10000),
             json_util.loads(
                 '{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}',
-                json_options=json_util.JSONOptions(tz_aware=False),
+                json_options=JSONOptions(tz_aware=False),
             )["dt"],
         )
-        self.round_trip(pre_epoch_naive, json_options=json_util.JSONOptions(tz_aware=False))
+        self.round_trip(pre_epoch_naive, json_options=JSONOptions(tz_aware=False))
 
         # Test a non-utc timezone
         pacific = FixedOffset(-8 * 60, "US/Pacific")
@@ -214,12 +227,14 @@ class TestJsonUtil(unittest.TestCase):
             json_util.dumps(aware_datetime, json_options=STRICT_JSON_OPTIONS),
         )
         self.round_trip(
-            aware_datetime, json_options=json_util.JSONOptions(tz_aware=True, tzinfo=pacific)
+            aware_datetime,
+            json_options=JSONOptions(json_mode=JSONMode.LEGACY, tz_aware=True, tzinfo=pacific),
         )
         self.round_trip(
             aware_datetime,
-            json_options=json_util.JSONOptions(
+            json_options=JSONOptions(
                 datetime_representation=DatetimeRepresentation.ISO8601,
+                json_mode=JSONMode.LEGACY,
                 tz_aware=True,
                 tzinfo=pacific,
             ),
@@ -258,11 +273,18 @@ class TestJsonUtil(unittest.TestCase):
 
         # Check order.
         self.assertEqual(
-            '{"$regex": ".*", "$options": "mx"}', json_util.dumps(Regex(".*", re.M | re.X))
+            '{"$regularExpression": {"pattern": ".*", "options": "mx"}}',
+            json_util.dumps(Regex(".*", re.M | re.X)),
         )
 
         self.assertEqual(
-            '{"$regex": ".*", "$options": "mx"}', json_util.dumps(re.compile(b".*", re.M | re.X))
+            '{"$regularExpression": {"pattern": ".*", "options": "mx"}}',
+            json_util.dumps(re.compile(b".*", re.M | re.X)),
+        )
+
+        self.assertEqual(
+            '{"$regex": ".*", "$options": "mx"}',
+            json_util.dumps(Regex(".*", re.M | re.X), json_options=LEGACY_JSON_OPTIONS),
         )
 
     def test_regex_validation(self):
@@ -287,30 +309,54 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual(dct, rtdct)
         self.assertEqual('{"ts": {"$timestamp": {"t": 4, "i": 13}}}', res)
 
+    def test_uuid_default(self):
+        # Cannot directly encode native UUIDs with the default
+        # uuid_representation.
+        doc = {"uuid": uuid.UUID("f47ac10b-58cc-4372-a567-0e02b2c3d479")}
+        with self.assertRaisesRegex(ValueError, "cannot encode native uuid"):
+            json_util.dumps(doc)
+        legacy_jsn = '{"uuid": {"$uuid": "f47ac10b58cc4372a5670e02b2c3d479"}}'
+        expected = {"uuid": Binary(b"\xf4z\xc1\x0bX\xccCr\xa5g\x0e\x02\xb2\xc3\xd4y", 4)}
+        self.assertEqual(json_util.loads(legacy_jsn), expected)
+
     def test_uuid(self):
         doc = {"uuid": uuid.UUID("f47ac10b-58cc-4372-a567-0e02b2c3d479")}
-        self.round_trip(doc)
+        uuid_legacy_opts = LEGACY_JSON_OPTIONS.with_options(
+            uuid_representation=UuidRepresentation.PYTHON_LEGACY
+        )
+        self.round_trip(doc, json_options=uuid_legacy_opts)
         self.assertEqual(
-            '{"uuid": {"$uuid": "f47ac10b58cc4372a5670e02b2c3d479"}}', json_util.dumps(doc)
+            '{"uuid": {"$uuid": "f47ac10b58cc4372a5670e02b2c3d479"}}',
+            json_util.dumps(doc, json_options=LEGACY_JSON_OPTIONS),
         )
         self.assertEqual(
             '{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}',
-            json_util.dumps(doc, json_options=json_util.STRICT_JSON_OPTIONS),
+            json_util.dumps(
+                doc,
+                json_options=STRICT_JSON_OPTIONS.with_options(
+                    uuid_representation=UuidRepresentation.PYTHON_LEGACY
+                ),
+            ),
         )
         self.assertEqual(
             '{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
             json_util.dumps(
                 doc,
-                json_options=json_util.JSONOptions(strict_uuid=True, uuid_representation=STANDARD),
+                json_options=JSONOptions(
+                    strict_uuid=True, json_mode=JSONMode.LEGACY, uuid_representation=STANDARD
+                ),
             ),
         )
         self.assertEqual(
             doc,
-            json_util.loads('{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}'),
+            json_util.loads(
+                '{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}',
+                json_options=uuid_legacy_opts,
+            ),
         )
         for uuid_representation in set(ALL_UUID_REPRESENTATIONS) - {UuidRepresentation.UNSPECIFIED}:
-            options = json_util.JSONOptions(
-                strict_uuid=True, uuid_representation=uuid_representation
+            options = JSONOptions(
+                strict_uuid=True, json_mode=JSONMode.LEGACY, uuid_representation=uuid_representation
             )
             self.round_trip(doc, json_options=options)
             # Ignore UUID representation when decoding BSON binary subtype 4.
@@ -324,8 +370,10 @@ class TestJsonUtil(unittest.TestCase):
 
     def test_uuid_uuid_rep_unspecified(self):
         _uuid = uuid.uuid4()
-        options = json_util.JSONOptions(
-            strict_uuid=True, uuid_representation=UuidRepresentation.UNSPECIFIED
+        options = JSONOptions(
+            strict_uuid=True,
+            json_mode=JSONMode.LEGACY,
+            uuid_representation=UuidRepresentation.UNSPECIFIED,
         )
 
         # Cannot directly encode native UUIDs with UNSPECIFIED.
@@ -344,14 +392,11 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual(doc, json_util.loads(ext_json_str, json_options=options))
         # $uuid-encoded fields
         doc = {"uuid": Binary(_uuid.bytes, subtype=4)}
-        ext_json_str = json_util.dumps({"uuid": _uuid})
+        ext_json_str = json_util.dumps({"uuid": _uuid}, json_options=LEGACY_JSON_OPTIONS)
         self.assertEqual(doc, json_util.loads(ext_json_str, json_options=options))
 
     def test_binary(self):
-        if PY3:
-            bin_type_dict = {"bin": b"\x00\x01\x02\x03\x04"}
-        else:
-            bin_type_dict = {"bin": Binary(b"\x00\x01\x02\x03\x04")}
+        bin_type_dict = {"bin": b"\x00\x01\x02\x03\x04"}
         md5_type_dict = {
             "md5": Binary(b" n7\x18\xaf\t/\xd1\xd1/\x80\xca\xe7q\xcc\xac", MD5_SUBTYPE)
         }
@@ -363,18 +408,15 @@ class TestJsonUtil(unittest.TestCase):
 
         # Binary with subtype 0 is decoded into bytes in Python 3.
         bin = json_util.loads('{"bin": {"$binary": "AAECAwQ=", "$type": "00"}}')["bin"]
-        if PY3:
-            self.assertEqual(type(bin), bytes)
-        else:
-            self.assertEqual(type(bin), Binary)
+        self.assertEqual(type(bin), bytes)
 
         # PYTHON-443 ensure old type formats are supported
-        json_bin_dump = json_util.dumps(bin_type_dict)
-        self.assertTrue('"$type": "00"' in json_bin_dump)
+        json_bin_dump = json_util.dumps(bin_type_dict, json_options=LEGACY_JSON_OPTIONS)
+        self.assertIn('"$type": "00"', json_bin_dump)
         self.assertEqual(
             bin_type_dict, json_util.loads('{"bin": {"$type": 0, "$binary": "AAECAwQ="}}')
         )
-        json_bin_dump = json_util.dumps(md5_type_dict)
+        json_bin_dump = json_util.dumps(md5_type_dict, json_options=LEGACY_JSON_OPTIONS)
         # Check order.
         self.assertEqual(
             '{"md5": {"$binary": "IG43GK8JL9HRL4DK53HMrA==",' + ' "$type": "05"}}', json_bin_dump
@@ -385,8 +427,8 @@ class TestJsonUtil(unittest.TestCase):
             json_util.loads('{"md5": {"$type": 5, "$binary":' ' "IG43GK8JL9HRL4DK53HMrA=="}}'),
         )
 
-        json_bin_dump = json_util.dumps(custom_type_dict)
-        self.assertTrue('"$type": "80"' in json_bin_dump)
+        json_bin_dump = json_util.dumps(custom_type_dict, json_options=LEGACY_JSON_OPTIONS)
+        self.assertIn('"$type": "80"', json_bin_dump)
         self.assertEqual(
             custom_type_dict,
             json_util.loads('{"custom": {"$type": 128, "$binary":' ' "aGVsbG8="}}'),
@@ -428,22 +470,18 @@ class TestJsonUtil(unittest.TestCase):
         jsn = '{"weight": {"$numberLong": "65535"}}'
         self.assertEqual(json_util.loads(jsn)["weight"], Int64(65535))
         self.assertEqual(json_util.dumps({"weight": Int64(65535)}), '{"weight": 65535}')
-        json_options = json_util.JSONOptions(strict_number_long=True)
+        json_options = JSONOptions(strict_number_long=True, json_mode=JSONMode.LEGACY)
         self.assertEqual(json_util.dumps({"weight": Int64(65535)}, json_options=json_options), jsn)
 
     def test_loads_document_class(self):
         # document_class dict should always work
         self.assertEqual(
             {"foo": "bar"},
-            json_util.loads(
-                '{"foo": "bar"}', json_options=json_util.JSONOptions(document_class=dict)
-            ),
+            json_util.loads('{"foo": "bar"}', json_options=JSONOptions(document_class=dict)),
         )
         self.assertEqual(
             SON([("foo", "bar"), ("b", 1)]),
-            json_util.loads(
-                '{"foo": "bar", "b": 1}', json_options=json_util.JSONOptions(document_class=SON)
-            ),
+            json_util.loads('{"foo": "bar", "b": 1}', json_options=JSONOptions(document_class=SON)),
         )
 
 
