@@ -26,7 +26,6 @@ from test.utils import oid_generated_on_process
 
 from bson.errors import InvalidId
 from bson.objectid import _MAX_COUNTER_VALUE, ObjectId
-from bson.py3compat import PY3, _unicode
 from bson.tz_util import FixedOffset, utc
 
 
@@ -50,16 +49,12 @@ class TestObjectId(unittest.TestCase):
 
     def test_unicode(self):
         a = ObjectId()
-        self.assertEqual(a, ObjectId(_unicode(a)))
-        self.assertEqual(
-            ObjectId("123456789012123456789012"), ObjectId(u"123456789012123456789012")
-        )
-        self.assertRaises(InvalidId, ObjectId, u"hello")
+        self.assertEqual(a, ObjectId(a))
+        self.assertRaises(InvalidId, ObjectId, "hello")
 
     def test_from_hex(self):
         ObjectId("123456789012123456789012")
         self.assertRaises(InvalidId, ObjectId, "123456789012123456789G12")
-        self.assertRaises(InvalidId, ObjectId, u"123456789012123456789G12")
 
     def test_repr_str(self):
         self.assertEqual(
@@ -140,13 +135,9 @@ class TestObjectId(unittest.TestCase):
             b"S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np5\nb."
         )
 
-        if PY3:
-            # Have to load using 'latin-1' since these were pickled in python2.x.
-            oid_1_9 = pickle.loads(pickled_with_1_9, encoding="latin-1")
-            oid_1_10 = pickle.loads(pickled_with_1_10, encoding="latin-1")
-        else:
-            oid_1_9 = pickle.loads(pickled_with_1_9)
-            oid_1_10 = pickle.loads(pickled_with_1_10)
+        # Have to load using 'latin-1' since these were pickled in python2.x.
+        oid_1_9 = pickle.loads(pickled_with_1_9, encoding="latin-1")
+        oid_1_10 = pickle.loads(pickled_with_1_10, encoding="latin-1")
 
         self.assertEqual(oid_1_9, ObjectId("4d9a66561376c00b88000000"))
         self.assertEqual(oid_1_9, oid_1_10)
@@ -191,7 +182,7 @@ class TestObjectId(unittest.TestCase):
         for tstamp, exp_datetime_args in TEST_DATA.items():
             oid = generate_objectid_with_timestamp(tstamp)
             # 32-bit platforms may overflow in datetime.fromtimestamp.
-            if tstamp > 0x7FFFFFFF and sys.maxsize < 2 ** 32:
+            if tstamp > 0x7FFFFFFF and sys.maxsize < 2**32:
                 try:
                     oid.generation_time
                 except (OverflowError, ValueError):
