@@ -24,12 +24,7 @@ import time
 from bson.codec_options import DEFAULT_CODEC_OPTIONS
 from bson.son import SON
 from pymongo import MongoClient, message
-from pymongo.errors import (
-    AutoReconnect,
-    ConnectionFailure,
-    DuplicateKeyError,
-    ExceededMaxWaiters,
-)
+from pymongo.errors import AutoReconnect, ConnectionFailure, DuplicateKeyError
 
 sys.path[0:0] = [""]
 
@@ -104,18 +99,11 @@ class NonUnique(MongoThread):
                 raise AssertionError("Should have raised DuplicateKeyError")
 
 
-class Disconnect(MongoThread):
-    def run_mongo_thread(self):
-        for _ in range(N):
-            self.client.close()
-
-
 class SocketGetter(MongoThread):
     """Utility for TestPooling.
 
     Checks out a socket and holds it forever. Used in
-    test_no_wait_queue_timeout, test_wait_queue_multiple, and
-    test_no_wait_queue_multiple.
+    test_no_wait_queue_timeout.
     """
 
     def __init__(self, client, pool):
@@ -176,9 +164,11 @@ class _TestPoolingBase(IntegrationTest):
         # Start the pool with the correct ssl options.
         pool_options = client_context.client._topology_settings.pool_options
         kwargs["ssl_context"] = pool_options._ssl_context
-        kwargs["ssl_match_hostname"] = pool_options.ssl_match_hostname
+        kwargs["tls_allow_invalid_hostnames"] = pool_options.tls_allow_invalid_hostnames
         kwargs["server_api"] = pool_options.server_api
-        return Pool(pair, PoolOptions(*args, **kwargs))
+        pool = Pool(pair, PoolOptions(*args, **kwargs))
+        pool.ready()
+        return pool
 
 
 class TestPooling(_TestPoolingBase):
@@ -194,9 +184,6 @@ class TestPooling(_TestPoolingBase):
     def test_no_disconnect(self):
         run_cases(self.c, [NonUnique, Unique, InsertOneAndFind])
 
-    def test_disconnect(self):
-        run_cases(self.c, [InsertOneAndFind, Disconnect, Unique])
-
     def test_pool_reuses_open_socket(self):
         # Test Pool's _check_closed() method doesn't close a healthy socket.
         cx_pool = self.create_pool(max_pool_size=10)
@@ -276,7 +263,7 @@ class TestPooling(_TestPoolingBase):
         self.assertTrue(socket_checker.select(s, write=True, timeout=0))
         self.assertTrue(socket_checker.select(s, write=True, timeout=0.05))
         # Make the socket readable
-        _, msg, _ = message.query(
+        _, msg, _ = message._query(
             0, "admin.$cmd", 0, -1, SON([("ping", 1)]), None, DEFAULT_CODEC_OPTIONS
         )
         s.sendall(msg)
@@ -296,10 +283,14 @@ class TestPooling(_TestPoolingBase):
     def test_return_socket_after_reset(self):
         pool = self.create_pool()
         with pool.get_socket({}) as sock:
+            self.assertEqual(pool.active_sockets, 1)
+            self.assertEqual(pool.operation_count, 1)
             pool.reset()
 
         self.assertTrue(sock.closed)
         self.assertEqual(0, len(pool.sockets))
+        self.assertEqual(pool.active_sockets, 0)
+        self.assertEqual(pool.operation_count, 0)
 
     def test_pool_check(self):
         # Test that Pool recovers from two connection failures in a row.
@@ -362,30 +353,7 @@ class TestPooling(_TestPoolingBase):
         self.assertEqual(t.state, "sock")
         self.assertEqual(t.sock, s1)
 
-    def test_wait_queue_multiple(self):
-        wait_queue_multiple = 3
-        pool = self.create_pool(max_pool_size=2, wait_queue_multiple=wait_queue_multiple)
-
-        # Reach max_size sockets.
-        with pool.get_socket({}):
-            with pool.get_socket({}):
-
-                # Reach max_size * wait_queue_multiple waiters.
-                threads = []
-                for _ in range(6):
-                    t = SocketGetter(self.c, pool)
-                    t.start()
-                    threads.append(t)
-
-                time.sleep(1)
-                for t in threads:
-                    self.assertEqual(t.state, "get_socket")
-
-                with self.assertRaises(ExceededMaxWaiters):
-                    with pool.get_socket({}):
-                        pass
-
-    def test_no_wait_queue_multiple(self):
+    def test_checkout_more_than_max_pool_size(self):
         pool = self.create_pool(max_pool_size=2)
 
         socks = []
@@ -407,6 +375,42 @@ class TestPooling(_TestPoolingBase):
         for socket_info in socks:
             socket_info.close_socket(None)
 
+    def test_maxConnecting(self):
+        client = rs_or_single_client()
+        self.addCleanup(client.close)
+        self.client.test.test.insert_one({})
+        self.addCleanup(self.client.test.test.delete_many, {})
+        pool = get_pool(client)
+        docs = []
+
+        # Run 50 short running operations
+        def find_one():
+            docs.append(client.test.test.find_one({}))
+
+        threads = [threading.Thread(target=find_one) for _ in range(50)]
+        for thread in threads:
+            thread.start()
+        for thread in threads:
+            thread.join(10)
+
+        self.assertEqual(len(docs), 50)
+        self.assertLessEqual(len(pool.sockets), 50)
+        # TLS and auth make connection establishment more expensive than
+        # the query which leads to more threads hitting maxConnecting.
+        # The end result is fewer total connections and better latency.
+        if client_context.tls and client_context.auth_enabled:
+            self.assertLessEqual(len(pool.sockets), 30)
+        else:
+            self.assertLessEqual(len(pool.sockets), 50)
+        # MongoDB 4.4.1 with auth + ssl:
+        # maxConnecting = 2:         6 connections in ~0.231+ seconds
+        # maxConnecting = unbounded: 50 connections in ~0.642+ seconds
+        #
+        # MongoDB 4.4.1 with no-auth no-ssl Python 3.8:
+        # maxConnecting = 2:         15-22 connections in ~0.108+ seconds
+        # maxConnecting = unbounded: 30+ connections in ~0.140+ seconds
+        print(len(pool.sockets))
+
 
 class TestPoolMaxSize(_TestPoolingBase):
     def test_max_pool_size(self):
@@ -444,7 +448,7 @@ class TestPoolMaxSize(_TestPoolingBase):
         joinall(threads)
         self.assertEqual(nthreads, self.n_passed)
         self.assertTrue(len(cx_pool.sockets) > 1)
-        self.assertEqual(max_pool_size, cx_pool._socket_semaphore.counter)
+        self.assertEqual(0, cx_pool.requests)
 
     def test_max_pool_size_none(self):
         c = rs_or_single_client(maxPoolSize=None)
@@ -476,10 +480,13 @@ class TestPoolMaxSize(_TestPoolingBase):
         joinall(threads)
         self.assertEqual(nthreads, self.n_passed)
         self.assertTrue(len(cx_pool.sockets) > 1)
+        self.assertEqual(cx_pool.max_pool_size, float("inf"))
 
     def test_max_pool_size_zero(self):
-        with self.assertRaises(ValueError):
-            rs_or_single_client(maxPoolSize=0)
+        c = rs_or_single_client(maxPoolSize=0)
+        self.addCleanup(c.close)
+        pool = get_pool(c)
+        self.assertEqual(pool.max_pool_size, float("inf"))
 
     def test_max_pool_size_with_connection_failure(self):
         # The pool acquires its semaphore before attempting to connect; ensure
@@ -488,6 +495,7 @@ class TestPoolMaxSize(_TestPoolingBase):
             ("somedomainthatdoesntexist.org", 27017),
             PoolOptions(max_pool_size=1, connect_timeout=1, socket_timeout=1, wait_queue_timeout=1),
         )
+        test_pool.ready()
 
         # First call to get_socket fails; if pool doesn't release its semaphore
         # then the second call raises "ConnectionFailure: Timed out waiting for
