@@ -18,7 +18,7 @@ from test import IntegrationTest, client_context
 from test.utils import OvertCommandListener, rs_or_single_client, single_client
 
 from bson.son import SON
-from pymongo.errors import ConfigurationError
+from pymongo.errors import OperationFailure
 from pymongo.read_concern import ReadConcern
 
 
@@ -62,16 +62,12 @@ class TestReadConcern(IntegrationTest):
         client = rs_or_single_client(uri, connect=False)
         self.assertEqual(ReadConcern("majority"), client.read_concern)
 
-    @client_context.require_version_max(3, 1)
     def test_invalid_read_concern(self):
-        coll = self.db.get_collection("coll", read_concern=ReadConcern("majority"))
-        self.assertRaisesRegexp(
-            ConfigurationError,
-            "read concern level of majority is not valid " "with a max wire version of [0-3]",
-            coll.count,
-        )
+        coll = self.db.get_collection("coll", read_concern=ReadConcern("unknown"))
+        # We rely on the server to validate read concern.
+        with self.assertRaises(OperationFailure):
+            coll.find_one()
 
-    @client_context.require_version_min(3, 1, 9, -1)
     def test_find_command(self):
         # readConcern not sent in command if not specified.
         coll = self.db.coll
@@ -94,7 +90,6 @@ class TestReadConcern(IntegrationTest):
             self.listener.results["started"][0].command,
         )
 
-    @client_context.require_version_min(3, 1, 9, -1)
     def test_command_cursor(self):
         # readConcern not sent in command if not specified.
         coll = self.db.coll
@@ -119,35 +114,3 @@ class TestReadConcern(IntegrationTest):
             self.assertIn("readConcern", self.listener.results["started"][0].command)
         else:
             self.assertNotIn("readConcern", self.listener.results["started"][0].command)
-
-    def test_map_reduce_out(self):
-        coll = self.db.get_collection("coll", read_concern=ReadConcern("local"))
-        coll.map_reduce(
-            "function() { emit(this._id, this.value); }",
-            "function(key, values) { return 42; }",
-            out="output_collection",
-        )
-        self.assertNotIn("readConcern", self.listener.results["started"][0].command)
-
-        if client_context.version.at_least(3, 1, 9, -1):
-            self.listener.results.clear()
-            coll.map_reduce(
-                "function() { emit(this._id, this.value); }",
-                "function(key, values) { return 42; }",
-                out={"inline": 1},
-            )
-            self.assertEqual(
-                {"level": "local"}, self.listener.results["started"][0].command["readConcern"]
-            )
-
-    @client_context.require_version_min(3, 1, 9, -1)
-    def test_inline_map_reduce(self):
-        coll = self.db.get_collection("coll", read_concern=ReadConcern("local"))
-        tuple(
-            coll.inline_map_reduce(
-                "function() { emit(this._id, this.value); }", "function(key, values) { return 42; }"
-            )
-        )
-        self.assertEqual(
-            {"level": "local"}, self.listener.results["started"][0].command["readConcern"]
-        )
