@@ -19,16 +19,13 @@ import copy
 import pickle
 import random
 import sys
-import warnings
 
 sys.path[0:0] = [""]
 
-from test import IntegrationTest, SkipTest, client_context, db_pwd, db_user, unittest
-from test.test_replica_set_client import TestReplicaSetClientBase
+from test import IntegrationTest, SkipTest, client_context, unittest
 from test.utils import (
     OvertCommandListener,
     connected,
-    ignore_deprecations,
     one,
     rs_client,
     single_client,
@@ -36,7 +33,6 @@ from test.utils import (
 )
 from test.version import Version
 
-from bson.py3compat import MAXSIZE
 from bson.son import SON
 from pymongo.errors import ConfigurationError, OperationFailure
 from pymongo.message import _maybe_add_read_preference
@@ -90,7 +86,7 @@ class TestReadPreferenceObjects(unittest.TestCase):
             self.assertEqual(pref, copy.deepcopy(pref))
 
 
-class TestReadPreferencesBase(TestReplicaSetClientBase):
+class TestReadPreferencesBase(IntegrationTest):
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
@@ -101,7 +97,7 @@ class TestReadPreferencesBase(TestReplicaSetClientBase):
         # Insert some data so we can use cursors in read_from_which_host
         self.client.pymongo_test.test.drop()
         self.client.get_database(
-            "pymongo_test", write_concern=WriteConcern(w=self.w)
+            "pymongo_test", write_concern=WriteConcern(w=client_context.w)
         ).test.insert_many([{"_id": i} for i in range(10)])
 
         self.addCleanup(self.client.pymongo_test.test.drop)
@@ -129,7 +125,7 @@ class TestReadPreferencesBase(TestReplicaSetClientBase):
 
     def assertReadsFrom(self, expected, **kwargs):
         c = rs_client(**kwargs)
-        wait_until(lambda: len(c.nodes - c.arbiters) == self.w, "discovered all nodes")
+        wait_until(lambda: len(c.nodes - c.arbiters) == client_context.w, "discovered all nodes")
 
         used = self.read_from_which_kind(c)
         self.assertEqual(expected, used, "Cursor used %s, expected %s" % (used, expected))
@@ -156,7 +152,6 @@ class TestSingleSecondaryOk(TestReadPreferencesBase):
         self.assertEqual(10, len(list(coll.find())))
 
         # Test some database helpers.
-        self.assertIsNotNone(db.collection_names())
         self.assertIsNotNone(db.list_collection_names())
         self.assertIsNotNone(db.validate_collection("test"))
         self.assertIsNotNone(db.command("ping"))
@@ -167,9 +162,6 @@ class TestSingleSecondaryOk(TestReadPreferencesBase):
         self.assertIsNotNone(coll.aggregate([]))
         self.assertIsNotNone(coll.index_information())
 
-        # Test some "magic" namespace helpers.
-        self.assertIsNotNone(db.current_op())
-
 
 class TestReadPreferences(TestReadPreferencesBase):
     def test_mode_validation(self):
@@ -204,13 +196,19 @@ class TestReadPreferences(TestReadPreferencesBase):
         self.assertRaises(TypeError, Secondary, tag_sets=["foo"])
 
     def test_threshold_validation(self):
-        self.assertEqual(17, rs_client(localThresholdMS=17).options.local_threshold_ms)
+        self.assertEqual(
+            17, rs_client(localThresholdMS=17, connect=False).options.local_threshold_ms
+        )
 
-        self.assertEqual(42, rs_client(localThresholdMS=42).options.local_threshold_ms)
+        self.assertEqual(
+            42, rs_client(localThresholdMS=42, connect=False).options.local_threshold_ms
+        )
 
-        self.assertEqual(666, rs_client(localthresholdms=666).options.local_threshold_ms)
+        self.assertEqual(
+            666, rs_client(localThresholdMS=666, connect=False).options.local_threshold_ms
+        )
 
-        self.assertEqual(0, rs_client(localthresholdms=0).options.local_threshold_ms)
+        self.assertEqual(0, rs_client(localThresholdMS=0, connect=False).options.local_threshold_ms)
 
         self.assertRaises(ValueError, rs_client, localthresholdms=-1)
 
@@ -251,7 +249,7 @@ class TestReadPreferences(TestReadPreferencesBase):
         # member
         c = rs_client(read_preference=ReadPreference.NEAREST, localThresholdMS=10000)  # 10 seconds
 
-        data_members = set(self.hosts).difference(set(self.arbiters))
+        data_members = {self.client.primary} | self.client.secondaries
 
         # This is a probabilistic test; track which members we've read from so
         # far, and keep reading until we've used all the members or give up.
@@ -280,7 +278,7 @@ class TestReadPreferences(TestReadPreferencesBase):
 class ReadPrefTester(MongoClient):
     def __init__(self, *args, **kwargs):
         self.has_read_from = set()
-        client_options = client_context.default_client_options.copy()
+        client_options = client_context.client_options
         client_options.update(kwargs)
         super(ReadPrefTester, self).__init__(*args, **client_options)
 
@@ -314,22 +312,21 @@ _PREF_MAP = [
 ]
 
 
-class TestCommandAndReadPreference(TestReplicaSetClientBase):
+class TestCommandAndReadPreference(IntegrationTest):
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
         super(TestCommandAndReadPreference, cls).setUpClass()
         cls.c = ReadPrefTester(
             client_context.pair,
-            replicaSet=cls.name,
             # Ignore round trip times, to test ReadPreference modes only.
             localThresholdMS=1000 * 1000,
         )
-        if client_context.auth_enabled:
-            cls.c.admin.authenticate(db_user, db_pwd)
         cls.client_version = Version.from_client(cls.c)
-        # mapReduce and group fail with no collection
-        coll = cls.c.pymongo_test.get_collection("test", write_concern=WriteConcern(w=cls.w))
+        # mapReduce fails if the collection does not exist.
+        coll = cls.c.pymongo_test.get_collection(
+            "test", write_concern=WriteConcern(w=client_context.w)
+        )
         coll.insert_one({})
 
     @classmethod
@@ -392,37 +389,10 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
         # the collection already exists.
         self._test_primary_helper(
             lambda: self.c.pymongo_test.create_collection(
-                "some_collection%s" % random.randint(0, MAXSIZE)
+                "some_collection%s" % random.randint(0, sys.maxsize)
             )
         )
 
-    @client_context.require_version_max(4, 1, 0, -1)
-    def test_group(self):
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            self._test_coll_helper(
-                True, self.c.pymongo_test.test, "group", {"a": 1}, {}, {}, "function() { }"
-            )
-
-    def test_map_reduce(self):
-        self._test_coll_helper(
-            False,
-            self.c.pymongo_test.test,
-            "map_reduce",
-            "function() { }",
-            "function() { }",
-            {"inline": 1},
-        )
-
-    def test_inline_map_reduce(self):
-        self._test_coll_helper(
-            True, self.c.pymongo_test.test, "inline_map_reduce", "function() { }", "function() { }"
-        )
-
-    @ignore_deprecations
-    def test_count(self):
-        self._test_coll_helper(True, self.c.pymongo_test.test, "count")
-
     def test_count_documents(self):
         self._test_coll_helper(True, self.c.pymongo_test.test, "count_documents", {})
 
@@ -549,8 +519,6 @@ class TestMongosAndReadPreference(IntegrationTest):
             out = _maybe_add_read_preference({}, pref)
             self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
-    # Require OP_MSG so that $readPreference is visible in the command event.
-    @client_context.require_version_min(3, 6)
     def test_send_hedge(self):
         cases = {
             "primaryPreferred": PrimaryPreferred,
@@ -665,7 +633,6 @@ class TestMongosAndReadPreference(IntegrationTest):
             self.assertEqual(last_id, results[0]["_id"])
 
     @client_context.require_mongos
-    @client_context.require_version_min(3, 3, 12)
     def test_mongos_max_staleness(self):
         # Sanity check that we're sending maxStalenessSeconds
         coll = client_context.client.pymongo_test.get_collection(
