@@ -16,12 +16,15 @@
 
 import copy
 import os
+import pprint
 import sys
+import threading
 
 sys.path[0:0] = [""]
 
 from test import IntegrationTest, SkipTest, client_context, client_knobs, unittest
 from test.utils import (
+    CMAPListener,
     DeprecationFilter,
     OvertCommandListener,
     TestCreator,
@@ -32,7 +35,6 @@ from test.version import Version
 
 from bson.codec_options import DEFAULT_CODEC_OPTIONS
 from bson.int64 import Int64
-from bson.objectid import ObjectId
 from bson.raw_bson import RawBSONDocument
 from bson.son import SON
 from pymongo.errors import (
@@ -42,6 +44,12 @@ from pymongo.errors import (
     WriteConcernError,
 )
 from pymongo.mongo_client import MongoClient
+from pymongo.monitoring import (
+    ConnectionCheckedOutEvent,
+    ConnectionCheckOutFailedEvent,
+    ConnectionCheckOutFailedReason,
+    PoolClearedEvent,
+)
 from pymongo.operations import (
     DeleteMany,
     DeleteOne,
@@ -94,7 +102,7 @@ test_creator = TestCreator(create_test, TestAllScenarios, _TEST_PATH)
 test_creator.create_tests()
 
 
-def _retryable_single_statement_ops(coll):
+def retryable_single_statement_ops(coll):
     return [
         (coll.bulk_write, [[InsertOne({}), InsertOne({})]], {}),
         (coll.bulk_write, [[InsertOne({}), InsertOne({})]], {"ordered": False}),
@@ -118,29 +126,6 @@ def _retryable_single_statement_ops(coll):
     ]
 
 
-def retryable_single_statement_ops(coll):
-    return _retryable_single_statement_ops(coll) + [
-        # Deprecated methods.
-        # Insert with single or multiple documents.
-        (coll.insert, [{}], {}),
-        (coll.insert, [[{}]], {}),
-        (coll.insert, [[{}, {}]], {}),
-        # Save with and without an _id.
-        (coll.save, [{}], {}),
-        (coll.save, [{"_id": ObjectId()}], {}),
-        # Non-multi update.
-        (coll.update, [{}, {"$set": {"a": 1}}], {}),
-        # Non-multi remove.
-        (coll.remove, [{}], {"multi": False}),
-        # Replace.
-        (coll.find_and_modify, [{}, {"a": 3}], {}),
-        # Update.
-        (coll.find_and_modify, [{}, {"$set": {"a": 1}}], {}),
-        # Delete.
-        (coll.find_and_modify, [{}, {}], {"remove": True}),
-    ]
-
-
 def non_retryable_single_statement_ops(coll):
     return [
         (
@@ -151,23 +136,6 @@ def non_retryable_single_statement_ops(coll):
         (coll.bulk_write, [[DeleteOne({}), DeleteMany({})]], {}),
         (coll.update_many, [{}, {"$set": {"a": 1}}], {}),
         (coll.delete_many, [{}], {}),
-        # Deprecated methods.
-        # Multi remove.
-        (coll.remove, [{}], {}),
-        # Multi update.
-        (coll.update, [{}, {"$set": {"a": 1}}], {"multi": True}),
-        # Unacknowledged deprecated methods.
-        (coll.insert, [{}], {"w": 0}),
-        # Unacknowledged Non-multi update.
-        (coll.update, [{}, {"$set": {"a": 1}}], {"w": 0}),
-        # Unacknowledged Non-multi remove.
-        (coll.remove, [{}], {"multi": False, "w": 0}),
-        # Unacknowledged Replace.
-        (coll.find_and_modify, [{}, {"a": 3}], {"writeConcern": {"w": 0}}),
-        # Unacknowledged Update.
-        (coll.find_and_modify, [{}, {"$set": {"a": 1}}], {"writeConcern": {"w": 0}}),
-        # Unacknowledged Delete.
-        (coll.find_and_modify, [{}, {}], {"remove": True, "writeConcern": {"w": 0}}),
     ]
 
 
@@ -200,8 +168,8 @@ class TestRetryableWritesMMAPv1(IgnoreDeprecationsTest):
     def tearDownClass(cls):
         cls.knobs.disable()
         cls.client.close()
+        super(TestRetryableWritesMMAPv1, cls).tearDownClass()
 
-    @client_context.require_version_min(3, 5)
     @client_context.require_no_standalone
     def test_actionable_error_message(self):
         if client_context.storage_engine != "mmapv1":
@@ -236,21 +204,13 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
         super(TestRetryableWrites, cls).tearDownClass()
 
     def setUp(self):
-        if (
-            client_context.version.at_least(3, 5)
-            and client_context.is_rs
-            and client_context.test_commands_enabled
-        ):
+        if client_context.is_rs and client_context.test_commands_enabled:
             self.client.admin.command(
                 SON([("configureFailPoint", "onPrimaryTransactionalWrite"), ("mode", "alwaysOn")])
             )
 
     def tearDown(self):
-        if (
-            client_context.version.at_least(3, 5)
-            and client_context.is_rs
-            and client_context.test_commands_enabled
-        ):
+        if client_context.is_rs and client_context.test_commands_enabled:
             self.client.admin.command(
                 SON([("configureFailPoint", "onPrimaryTransactionalWrite"), ("mode", "off")])
             )
@@ -270,7 +230,6 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
                     "%s sent txnNumber with %s" % (msg, event.command_name),
                 )
 
-    @client_context.require_version_min(3, 5)
     @client_context.require_no_standalone
     def test_supported_single_statement_supported_cluster(self):
         for method, args, kwargs in retryable_single_statement_ops(self.db.retryable_write_test):
@@ -313,9 +272,7 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
             self.assertEqual(retry_attempt.command["txnNumber"], initial_transaction_id, msg)
 
     def test_supported_single_statement_unsupported_cluster(self):
-        if client_context.version.at_least(3, 5) and (
-            client_context.is_rs or client_context.is_mongos
-        ):
+        if client_context.is_rs or client_context.is_mongos:
             raise SkipTest("This cluster supports retryable writes")
 
         for method, args, kwargs in retryable_single_statement_ops(self.db.retryable_write_test):
@@ -366,7 +323,6 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
                 method(*args, **kwargs)
             self.assertEqual(len(listener.results["started"]), 0, msg)
 
-    @client_context.require_version_min(3, 5)
     @client_context.require_replica_set
     @client_context.require_test_commands
     def test_retry_timeout_raises_original_error(self):
@@ -397,7 +353,6 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
                 method(*args, **kwargs)
             self.assertEqual(len(listener.results["started"]), 1, msg)
 
-    @client_context.require_version_min(3, 5)
     @client_context.require_replica_set
     @client_context.require_test_commands
     def test_batch_splitting(self):
@@ -435,7 +390,6 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
         }
         self.assertEqual(bulk_result.bulk_api_result, expected_result)
 
-    @client_context.require_version_min(3, 5)
     @client_context.require_replica_set
     @client_context.require_test_commands
     def test_batch_splitting_retry_fails(self):
@@ -546,9 +500,84 @@ class TestWriteConcernError(IntegrationTest):
         self.assertIn("RetryableWriteError", result["errorLabels"])
 
 
+class InsertThread(threading.Thread):
+    def __init__(self, collection):
+        super().__init__()
+        self.daemon = True
+        self.collection = collection
+        self.passed = False
+
+    def run(self):
+        self.collection.insert_one({})
+        self.passed = True
+
+
+class TestPoolPausedError(IntegrationTest):
+    # Pools don't get paused in load balanced mode.
+    RUN_ON_LOAD_BALANCER = False
+    RUN_ON_SERVERLESS = False
+
+    @client_context.require_failCommand_blockConnection
+    @client_context.require_retryable_writes
+    @client_knobs(heartbeat_frequency=0.05, min_heartbeat_interval=0.05)
+    def test_pool_paused_error_is_retryable(self):
+        cmap_listener = CMAPListener()
+        cmd_listener = OvertCommandListener()
+        client = rs_or_single_client(maxPoolSize=1, event_listeners=[cmap_listener, cmd_listener])
+        self.addCleanup(client.close)
+        for _ in range(10):
+            cmap_listener.reset()
+            cmd_listener.reset()
+            threads = [InsertThread(client.pymongo_test.test) for _ in range(2)]
+            fail_command = {
+                "mode": {"times": 1},
+                "data": {
+                    "failCommands": ["insert"],
+                    "blockConnection": True,
+                    "blockTimeMS": 1000,
+                    "errorCode": 91,
+                    "errorLabels": ["RetryableWriteError"],
+                },
+            }
+            with self.fail_point(fail_command):
+                for thread in threads:
+                    thread.start()
+                for thread in threads:
+                    thread.join()
+                for thread in threads:
+                    self.assertTrue(thread.passed)
+            # It's possible that SDAM can rediscover the server and mark the
+            # pool ready before the thread in the wait queue has a chance
+            # to run. Repeat the test until the thread actually encounters
+            # a PoolClearedError.
+            if cmap_listener.event_count(ConnectionCheckOutFailedEvent):
+                break
+
+        # Via CMAP monitoring, assert that the first check out succeeds.
+        cmap_events = cmap_listener.events_by_type(
+            (ConnectionCheckedOutEvent, ConnectionCheckOutFailedEvent, PoolClearedEvent)
+        )
+        msg = pprint.pformat(cmap_listener.events)
+        self.assertIsInstance(cmap_events[0], ConnectionCheckedOutEvent, msg)
+        self.assertIsInstance(cmap_events[1], PoolClearedEvent, msg)
+        self.assertIsInstance(cmap_events[2], ConnectionCheckOutFailedEvent, msg)
+        self.assertEqual(cmap_events[2].reason, ConnectionCheckOutFailedReason.CONN_ERROR, msg)
+        self.assertIsInstance(cmap_events[3], ConnectionCheckedOutEvent, msg)
+
+        # Connection check out failures are not reflected in command
+        # monitoring because we only publish command events _after_ checking
+        # out a connection.
+        started = cmd_listener.results["started"]
+        msg = pprint.pformat(cmd_listener.results)
+        self.assertEqual(3, len(started), msg)
+        succeeded = cmd_listener.results["succeeded"]
+        self.assertEqual(2, len(succeeded), msg)
+        failed = cmd_listener.results["failed"]
+        self.assertEqual(1, len(failed), msg)
+
+
 # TODO: Make this a real integration test where we stepdown the primary.
 class TestRetryableWritesTxnNumber(IgnoreDeprecationsTest):
-    @client_context.require_version_min(3, 6)
     @client_context.require_replica_set
     @client_context.require_no_mmap
     def test_increment_transaction_id_without_sending_command(self):
@@ -567,7 +596,7 @@ class TestRetryableWritesTxnNumber(IgnoreDeprecationsTest):
             topology.select_server = select_server
             raise ConnectionFailure("Connection refused")
 
-        for method, args, kwargs in _retryable_single_statement_ops(client.db.retryable_write_test):
+        for method, args, kwargs in retryable_single_statement_ops(client.db.retryable_write_test):
             listener.results.clear()
             topology.select_server = raise_connection_err_select_server
             with client.start_session() as session:
