@@ -17,26 +17,20 @@
 import copy
 import os
 import sys
+import time
+from io import BytesIO
 
 sys.path[0:0] = [""]
 
-from test import IntegrationTest, SkipTest, client_context, db_pwd, db_user, unittest
-from test.utils import (
-    EventListener,
-    TestCreator,
-    ignore_deprecations,
-    rs_or_single_client,
-    wait_until,
-)
+from test import IntegrationTest, SkipTest, client_context, unittest
+from test.utils import EventListener, TestCreator, rs_or_single_client, wait_until
 from test.utils_spec_runner import SpecRunner
 
 from bson import DBRef
-from bson.py3compat import StringIO
 from gridfs import GridFS, GridFSBucket
-from pymongo import ASCENDING, OFF, IndexModel, InsertOne, monitoring
+from pymongo import ASCENDING, IndexModel, InsertOne, monitoring
 from pymongo.common import _MAX_END_SESSIONS
 from pymongo.errors import ConfigurationError, InvalidOperation, OperationFailure
-from pymongo.monotonic import time as _time
 from pymongo.read_concern import ReadConcern
 
 
@@ -110,7 +104,7 @@ class TestSession(IntegrationTest):
         for f, args, kw in ops:
             with client.start_session() as s:
                 last_use = s._server_session.last_use
-                start = _time()
+                start = time.monotonic()
                 self.assertLessEqual(last_use, start)
                 listener.results.clear()
                 # In case "f" modifies its inputs.
@@ -227,33 +221,14 @@ class TestSession(IntegrationTest):
         client.close()
         self.assertEqual(len(listener.results["started"]), 0)
 
-    @ignore_deprecations  # fsync and unlock
     def test_client(self):
         client = self.client
-
-        # Make sure if the test fails we unlock the server.
-        def unlock():
-            try:
-                client.unlock()
-            except OperationFailure:
-                pass
-
-        self.addCleanup(unlock)
-
         ops = [
             (client.server_info, [], {}),
-            (client.database_names, [], {}),
+            (client.list_database_names, [], {}),
             (client.drop_database, ["pymongo_test"], {}),
         ]
 
-        if not client_context.is_mongos:
-            ops.extend(
-                [
-                    (client.fsync, [], {"lock": True}),
-                    (client.unlock, [], {}),
-                ]
-            )
-
         self._test_ops(client, *ops)
 
     def test_database(self):
@@ -262,35 +237,13 @@ class TestSession(IntegrationTest):
         ops = [
             (db.command, ["ping"], {}),
             (db.create_collection, ["collection"], {}),
-            (db.collection_names, [], {}),
             (db.list_collection_names, [], {}),
             (db.validate_collection, ["collection"], {}),
             (db.drop_collection, ["collection"], {}),
-            (db.current_op, [], {}),
-            (db.profiling_info, [], {}),
             (db.dereference, [DBRef("collection", 1)], {}),
         ]
-
-        if not client_context.is_mongos:
-            ops.append((db.set_profiling_level, [OFF], {}))
-            ops.append((db.profiling_level, [], {}))
-
         self._test_ops(client, *ops)
 
-    @client_context.require_auth
-    @ignore_deprecations
-    def test_user_admin(self):
-        client = self.client
-        db = client.pymongo_test
-
-        self._test_ops(
-            client,
-            (db.add_user, ["session-test", "pass"], {"roles": ["read"]}),
-            # Do it again to test updateUser command.
-            (db.add_user, ["session-test", "pass"], {"roles": ["read"]}),
-            (db.remove_user, ["session-test"], {}),
-        )
-
     @staticmethod
     def collection_write_ops(coll):
         """Generate database write ops for tests."""
@@ -304,7 +257,6 @@ class TestSession(IntegrationTest):
             (coll.update_many, [{}, {"$set": {"a": 1}}], {}),
             (coll.delete_one, [{}], {}),
             (coll.delete_many, [{}], {}),
-            (coll.map_reduce, ["function() {}", "function() {}", "output"], {}),
             (coll.find_one_and_replace, [{}, {}], {}),
             (coll.find_one_and_update, [{}, {"$set": {"a": 1}}], {}),
             (coll.find_one_and_delete, [{}, {}], {}),
@@ -328,9 +280,7 @@ class TestSession(IntegrationTest):
             [
                 (coll.distinct, ["a"], {}),
                 (coll.find_one, [], {}),
-                (coll.count, [], {}),
                 (coll.count_documents, [{}], {}),
-                (coll.inline_map_reduce, ["function() {}", "function() {}"], {}),
                 (coll.list_indexes, [], {}),
                 (coll.index_information, [], {}),
                 (coll.options, [], {}),
@@ -338,44 +288,8 @@ class TestSession(IntegrationTest):
             ]
         )
 
-        if client_context.supports_reindex:
-            ops.append((coll.reindex, [], {}))
-
         self._test_ops(client, *ops)
 
-    @client_context.require_no_mongos
-    @client_context.require_version_max(4, 1, 0)
-    @ignore_deprecations
-    def test_parallel_collection_scan(self):
-        listener = self.listener
-        client = self.client
-        coll = client.pymongo_test.collection
-        coll.insert_many([{"_id": i} for i in range(1000)])
-
-        listener.results.clear()
-
-        def scan(session=None):
-            cursors = coll.parallel_scan(4, session=session)
-            for c in cursors:
-                c.batch_size(2)
-                list(c)
-
-        listener.results.clear()
-        with client.start_session() as session:
-            scan(session)
-        cursor_lsids = {}
-        for event in listener.results["started"]:
-            self.assertIn(
-                "lsid", event.command, "parallel_scan sent no lsid with %s" % (event.command_name,)
-            )
-
-            if event.command_name == "getMore":
-                cursor_id = event.command["getMore"]
-                if cursor_id in cursor_lsids:
-                    self.assertEqual(cursor_lsids[cursor_id], event.command["lsid"])
-                else:
-                    cursor_lsids[cursor_id] = event.command["lsid"]
-
     def test_cursor_clone(self):
         coll = self.client.pymongo_test.collection
         # Ensure some batches.
@@ -412,7 +326,6 @@ class TestSession(IntegrationTest):
         ops = [
             ("find", lambda session: list(coll.find(session=session))),
             ("getitem", lambda session: coll.find(session=session)[0]),
-            ("count", lambda session: coll.find(session=session).count()),
             ("distinct", lambda session: coll.find(session=session).distinct("a")),
             ("explain", lambda session: coll.find(session=session).explain()),
         ]
@@ -516,7 +429,7 @@ class TestSession(IntegrationTest):
             for f in files:
                 f.read()
 
-        sio = StringIO()
+        sio = BytesIO()
 
         self._test_ops(
             client,
@@ -790,6 +703,11 @@ class TestSession(IntegrationTest):
             with self.client.start_session(causal_consistency=True, snapshot=True):
                 pass
 
+    def test_session_not_copyable(self):
+        client = self.client
+        with client.start_session() as s:
+            self.assertRaises(TypeError, lambda: copy.copy(s))
+
 
 class TestCausalConsistency(unittest.TestCase):
     @classmethod
@@ -877,7 +795,6 @@ class TestCausalConsistency(unittest.TestCase):
         self._test_reads(lambda coll, session: list(coll.aggregate([], session=session)))
         self._test_reads(lambda coll, session: list(coll.find({}, session=session)))
         self._test_reads(lambda coll, session: coll.find_one({}, session=session))
-        self._test_reads(lambda coll, session: coll.count(session=session))
         self._test_reads(lambda coll, session: coll.count_documents({}, session=session))
         self._test_reads(lambda coll, session: coll.distinct("foo", session=session))
         self._test_reads(
@@ -885,34 +802,6 @@ class TestCausalConsistency(unittest.TestCase):
         )
         self._test_reads(lambda coll, session: list(coll.find_raw_batches({}, session=session)))
 
-        # SERVER-40938 removed support for casually consistent mapReduce.
-        map_reduce_exc = None
-        if client_context.version.at_least(4, 1, 12):
-            map_reduce_exc = OperationFailure
-        # SERVER-44635 The mapReduce in aggregation project added back
-        # support for casually consistent mapReduce.
-        if client_context.version < (4, 3):
-            self._test_reads(
-                lambda coll, session: coll.map_reduce(
-                    "function() {}", "function() {}", "inline", session=session
-                ),
-                exception=map_reduce_exc,
-            )
-            self._test_reads(
-                lambda coll, session: coll.inline_map_reduce(
-                    "function() {}", "function() {}", session=session
-                ),
-                exception=map_reduce_exc,
-            )
-        if not client_context.is_mongos and not client_context.version.at_least(4, 1, 0):
-
-            def scan(coll, session):
-                cursors = coll.parallel_scan(1, session=session)
-                for cur in cursors:
-                    list(cur)
-
-            self._test_reads(lambda coll, session: scan(coll, session=session))
-
         self.assertRaises(
             ConfigurationError,
             self._test_reads,
@@ -968,9 +857,6 @@ class TestCausalConsistency(unittest.TestCase):
         self._test_writes(lambda coll, session: coll.drop_index("foo_1", session=session))
         self._test_writes(lambda coll, session: coll.drop_indexes(session=session))
 
-        if client_context.supports_reindex:
-            self._test_writes(lambda coll, session: coll.reindex(session=session))
-
     def _test_no_read_concern(self, op):
         coll = self.client.pymongo_test.test
         with self.client.start_session() as sess:
@@ -1023,24 +909,6 @@ class TestCausalConsistency(unittest.TestCase):
         # Not a write, but explain also doesn't support readConcern.
         self._test_no_read_concern(lambda coll, session: coll.find({}, session=session).explain())
 
-    @client_context.require_no_standalone
-    @unittest.skipIf(client_context.serverless, "Serverless does not support currentOp")
-    def test_writes_do_not_include_read_concern_current_op(self):
-        # Not a write, but currentOp also doesn't support readConcern.
-        self._test_no_read_concern(lambda coll, session: coll.database.current_op(session=session))
-
-    @client_context.require_no_standalone
-    @unittest.skipIf(client_context.serverless, "Serverless does not support mapReduce")
-    def test_writes_do_not_include_read_concern_map_reduce(self):
-        self._test_no_read_concern(
-            lambda coll, session: coll.map_reduce(
-                "function() {}", "function() {}", "mrout", session=session
-            )
-        )
-
-        if client_context.supports_reindex:
-            self._test_no_read_concern(lambda coll, session: coll.reindex(session=session))
-
     @client_context.require_no_standalone
     @client_context.require_version_max(4, 1, 0)
     def test_aggregate_out_does_not_include_read_concern(self):
@@ -1127,130 +995,12 @@ class TestCausalConsistency(unittest.TestCase):
         self.assertIsNone(after_cluster_time)
 
 
-class TestSessionsMultiAuth(IntegrationTest):
-    @client_context.require_auth
-    @client_context.require_sessions
-    def setUp(self):
-        super(TestSessionsMultiAuth, self).setUp()
-
-        client_context.create_user("pymongo_test", "second-user", "pass", roles=["readWrite"])
-        self.addCleanup(client_context.drop_user, "pymongo_test", "second-user")
-
-    @ignore_deprecations
-    @client_context.require_version_max(5, 99, 99)
-    def test_session_authenticate_multiple(self):
-        listener = SessionTestListener()
-        # Logged in as root.
-        client = rs_or_single_client(event_listeners=[listener])
-        db = client.pymongo_test
-        db.authenticate("second-user", "pass")
-
-        with self.assertRaises(InvalidOperation):
-            client.start_session()
-
-        # No implicit sessions.
-        listener.results.clear()
-        db.collection.find_one()
-        event = listener.first_command_started()
-        self.assertNotIn(
-            "lsid",
-            event.command,
-            "find_one with multi-auth shouldn't have sent lsid with %s" % (event.command_name),
-        )
-
-    @ignore_deprecations
-    def test_explicit_session_logout(self):
-        listener = SessionTestListener()
-
-        # Changing auth invalidates the session. Start as root.
-        client = rs_or_single_client(event_listeners=[listener])
-        db = client.pymongo_test
-        db.collection.insert_many([{} for _ in range(10)])
-        self.addCleanup(db.collection.drop)
-
-        with client.start_session() as s:
-            listener.results.clear()
-            cursor = db.collection.find(session=s).batch_size(2)
-            next(cursor)
-            event = listener.first_command_started()
-            self.assertEqual(event.command_name, "find")
-            self.assertEqual(
-                s.session_id,
-                event.command.get("lsid"),
-                "find() sent wrong lsid with %s cmd" % (event.command_name,),
-            )
-
-            client.admin.logout()
-            db.authenticate("second-user", "pass")
-
-            err = "Cannot use session after authenticating with different" " credentials"
-
-            with self.assertRaisesRegex(InvalidOperation, err):
-                # Auth has changed between find and getMore.
-                list(cursor)
-
-            with self.assertRaisesRegex(InvalidOperation, err):
-                db.collection.bulk_write([InsertOne({})], session=s)
-
-            with self.assertRaisesRegex(InvalidOperation, err):
-                db.collection_names(session=s)
-
-            with self.assertRaisesRegex(InvalidOperation, err):
-                db.collection.find_one(session=s)
-
-            with self.assertRaisesRegex(InvalidOperation, err):
-                list(db.collection.aggregate([], session=s))
-
-    @ignore_deprecations
-    def test_implicit_session_logout(self):
-        listener = SessionTestListener()
-
-        # Changing auth doesn't invalidate the session. Start as root.
-        client = rs_or_single_client(event_listeners=[listener])
-        db = client.pymongo_test
-
-        for name, f in [
-            ("bulk_write", lambda: db.collection.bulk_write([InsertOne({})])),
-            ("collection_names", db.collection_names),
-            ("find_one", db.collection.find_one),
-            ("aggregate", lambda: list(db.collection.aggregate([]))),
-        ]:
-
-            def sub_test():
-                listener.results.clear()
-                f()
-                for event in listener.results["started"]:
-                    self.assertIn(
-                        "lsid",
-                        event.command,
-                        "%s sent no lsid with %s" % (name, event.command_name),
-                    )
-
-            # We switch auth without clearing the pool of session ids. The
-            # server considers these to be new sessions since it's a new user.
-            # The old sessions time out on the server after 30 minutes.
-            client.admin.logout()
-            db.authenticate("second-user", "pass")
-            sub_test()
-            db.logout()
-            client.admin.authenticate(db_user, db_pwd)
-            sub_test()
-
-
-class TestSessionsNotSupported(IntegrationTest):
-    @client_context.require_version_max(3, 5, 10)
-    def test_sessions_not_supported(self):
-        with self.assertRaisesRegex(ConfigurationError, "Sessions are not supported"):
-            self.client.start_session()
-
-
 class TestClusterTime(IntegrationTest):
     def setUp(self):
         super(TestClusterTime, self).setUp()
         if "$clusterTime" not in client_context.hello:
             raise SkipTest("$clusterTime not supported")
 
-    @ignore_deprecations
     def test_cluster_time(self):
         listener = SessionTestListener()
         # Prevent heartbeats from updating $clusterTime between operations.
@@ -1262,14 +1012,6 @@ class TestClusterTime(IntegrationTest):
         self.addCleanup(collection.drop)
         self.addCleanup(client.pymongo_test.collection2.drop)
 
-        def bulk_insert(ordered):
-            if ordered:
-                bulk = collection.initialize_ordered_bulk_op()
-            else:
-                bulk = collection.initialize_unordered_bulk_op()
-            bulk.insert({})
-            bulk.execute()
-
         def rename_and_drop():
             # Ensure collection exists.
             collection.insert_one({})
@@ -1308,8 +1050,6 @@ class TestClusterTime(IntegrationTest):
             ("delete_one", lambda: collection.delete_one({})),
             ("delete_many", lambda: collection.delete_many({})),
             ("bulk_write", lambda: collection.bulk_write([InsertOne({})])),
-            ("ordered bulk", lambda: bulk_insert(True)),
-            ("unordered bulk", lambda: bulk_insert(False)),
             ("rename_and_drop", rename_and_drop),
         ]
 
