@@ -21,9 +21,9 @@ import sys
 
 sys.path[0:0] = [""]
 
-from test import SkipTest, unittest
+from collections import OrderedDict
+from test import unittest
 
-from bson.py3compat import b
 from bson.son import SON
 
 
@@ -96,11 +96,11 @@ class TestSON(unittest.TestCase):
     def test_pickle_backwards_compatability(self):
         # This string was generated by pickling a SON object in pymongo
         # version 2.1.1
-        pickled_with_2_1_1 = b(
+        pickled_with_2_1_1 = (
             "ccopy_reg\n_reconstructor\np0\n(cbson.son\nSON\np1\n"
             "c__builtin__\ndict\np2\n(dp3\ntp4\nRp5\n(dp6\n"
             "S'_SON__keys'\np7\n(lp8\nsb."
-        )
+        ).encode("utf8")
         son_2_1_1 = pickle.loads(pickled_with_2_1_1)
         self.assertEqual(son_2_1_1, SON([]))
 
@@ -179,6 +179,24 @@ class TestSON(unittest.TestCase):
         test_son.popitem()
         self.assertEqual(2, len(test_son))
 
+    def test_keys(self):
+        # Test to make sure that set operations do not throw an error
+        d = SON().keys()
+        for i in [OrderedDict, dict]:
+            try:
+                d - i().keys()
+            except TypeError:
+                self.fail(
+                    "SON().keys() is not returning an object compatible "
+                    "with %s objects" % (str(i))
+                )
+        # Test to verify correctness
+        d = SON({"k": "v"}).keys()
+        for i in [OrderedDict, dict]:
+            self.assertEqual(d | i({"k1": 0}).keys(), {"k", "k1"})
+        for i in [OrderedDict, dict]:
+            self.assertEqual(d - i({"k": 0}).keys(), set())
+
 
 if __name__ == "__main__":
     unittest.main()
