@@ -36,12 +36,12 @@ class SrvPollingKnobs(object):
         self,
         ttl_time=None,
         min_srv_rescan_interval=None,
-        dns_resolver_nodelist_response=None,
+        nodelist_callback=None,
         count_resolver_calls=False,
     ):
         self.ttl_time = ttl_time
         self.min_srv_rescan_interval = min_srv_rescan_interval
-        self.dns_resolver_nodelist_response = dns_resolver_nodelist_response
+        self.nodelist_callback = nodelist_callback
         self.count_resolver_calls = count_resolver_calls
 
         self.old_min_srv_rescan_interval = None
@@ -56,8 +56,8 @@ class SrvPollingKnobs(object):
 
         def mock_get_hosts_and_min_ttl(resolver, *args):
             nodes, ttl = self.old_dns_resolver_response(resolver)
-            if self.dns_resolver_nodelist_response is not None:
-                nodes = self.dns_resolver_nodelist_response()
+            if self.nodelist_callback is not None:
+                nodes = self.nodelist_callback()
             if self.ttl_time is not None:
                 ttl = self.ttl_time
             return nodes, ttl
@@ -125,14 +125,7 @@ class TestSrvPolling(unittest.TestCase):
         (WAIT_TIME * 10) seconds. Also check that the resolver is called at
         least once.
         """
-
-        def predicate():
-            if set(expected_nodelist) == set(self.get_nodelist(client)):
-                return pymongo.srv_resolver._SrvResolver.get_hosts_and_min_ttl.call_count >= 1
-            return False
-
-        wait_until(predicate, "Node list equals expected nodelist", timeout=100 * WAIT_TIME)
-
+        sleep(WAIT_TIME * 10)
         nodelist = self.get_nodelist(client)
         if set(expected_nodelist) != set(nodelist):
             msg = "Client nodelist %s changed unexpectedly (expected %s)"
@@ -167,8 +160,7 @@ class TestSrvPolling(unittest.TestCase):
             self.assert_nodelist_change(self.BASE_SRV_RESPONSE, client)
             # Patch list of hosts returned by DNS query.
             with SrvPollingKnobs(
-                dns_resolver_nodelist_response=dns_resolver_response,
-                count_resolver_calls=count_resolver_calls,
+                nodelist_callback=dns_resolver_response, count_resolver_calls=count_resolver_calls
             ):
                 assertion_method(expected_response, client)
 
@@ -224,7 +216,7 @@ class TestSrvPolling(unittest.TestCase):
         with SrvPollingKnobs(
             ttl_time=WAIT_TIME,
             min_srv_rescan_interval=WAIT_TIME,
-            dns_resolver_nodelist_response=initial_callback,
+            nodelist_callback=initial_callback,
             count_resolver_calls=True,
         ):
             # Client uses unpatched method to get initial nodelist
@@ -233,9 +225,7 @@ class TestSrvPolling(unittest.TestCase):
             self.assert_nodelist_nochange(self.BASE_SRV_RESPONSE, client)
 
         with SrvPollingKnobs(
-            ttl_time=WAIT_TIME,
-            min_srv_rescan_interval=WAIT_TIME,
-            dns_resolver_nodelist_response=final_callback,
+            ttl_time=WAIT_TIME, min_srv_rescan_interval=WAIT_TIME, nodelist_callback=final_callback
         ):
             # Nodelist should reflect new valid DNS resolver response.
             self.assert_nodelist_change(response_final, client)
@@ -252,6 +242,82 @@ class TestSrvPolling(unittest.TestCase):
 
         self._test_recover_from_initial(erroring_seedlist)
 
+    def test_10_all_dns_selected(self):
+        response = [
+            ("localhost.test.build.10gen.cc", 27017),
+            ("localhost.test.build.10gen.cc", 27019),
+            ("localhost.test.build.10gen.cc", 27020),
+        ]
+
+        def nodelist_callback():
+            return response
+
+        with SrvPollingKnobs(ttl_time=WAIT_TIME, min_srv_rescan_interval=WAIT_TIME):
+            client = MongoClient(self.CONNECTION_STRING, srvMaxHosts=0)
+            self.addCleanup(client.close)
+            with SrvPollingKnobs(nodelist_callback=nodelist_callback):
+                self.assert_nodelist_change(response, client)
+
+    def test_11_all_dns_selected(self):
+        response = [
+            ("localhost.test.build.10gen.cc", 27019),
+            ("localhost.test.build.10gen.cc", 27020),
+        ]
+
+        def nodelist_callback():
+            return response
+
+        with SrvPollingKnobs(ttl_time=WAIT_TIME, min_srv_rescan_interval=WAIT_TIME):
+            client = MongoClient(self.CONNECTION_STRING, srvMaxHosts=2)
+            self.addCleanup(client.close)
+            with SrvPollingKnobs(nodelist_callback=nodelist_callback):
+                self.assert_nodelist_change(response, client)
+
+    def test_12_new_dns_randomly_selected(self):
+        response = [
+            ("localhost.test.build.10gen.cc", 27020),
+            ("localhost.test.build.10gen.cc", 27019),
+            ("localhost.test.build.10gen.cc", 27017),
+        ]
+
+        def nodelist_callback():
+            return response
+
+        with SrvPollingKnobs(ttl_time=WAIT_TIME, min_srv_rescan_interval=WAIT_TIME):
+            client = MongoClient(self.CONNECTION_STRING, srvMaxHosts=2)
+            self.addCleanup(client.close)
+            with SrvPollingKnobs(nodelist_callback=nodelist_callback):
+                sleep(2 * common.MIN_SRV_RESCAN_INTERVAL)
+                final_topology = set(client.topology_description.server_descriptions())
+                self.assertIn(("localhost.test.build.10gen.cc", 27017), final_topology)
+                self.assertEqual(len(final_topology), 2)
+
+    def test_does_not_flipflop(self):
+        with SrvPollingKnobs(ttl_time=WAIT_TIME, min_srv_rescan_interval=WAIT_TIME):
+            client = MongoClient(self.CONNECTION_STRING, srvMaxHosts=1)
+            self.addCleanup(client.close)
+            old = set(client.topology_description.server_descriptions())
+            sleep(4 * WAIT_TIME)
+            new = set(client.topology_description.server_descriptions())
+            self.assertSetEqual(old, new)
+
+    def test_srv_service_name(self):
+        # Construct a valid final response callback distinct from base.
+        response = [
+            ("localhost.test.build.10gen.cc.", 27019),
+            ("localhost.test.build.10gen.cc.", 27020),
+        ]
+
+        def nodelist_callback():
+            return response
+
+        with SrvPollingKnobs(ttl_time=WAIT_TIME, min_srv_rescan_interval=WAIT_TIME):
+            client = MongoClient(
+                "mongodb+srv://test22.test.build.10gen.cc/?srvServiceName" "=customname"
+            )
+            with SrvPollingKnobs(nodelist_callback=nodelist_callback):
+                self.assert_nodelist_change(response, client)
+
 
 if __name__ == "__main__":
     unittest.main()
