@@ -20,21 +20,7 @@ import sys
 
 sys.path[0:0] = [""]
 
-try:
-    from urllib.parse import quote_plus
-except ImportError:
-    # Python 2
-    from urllib import quote_plus
-
-from test import (
-    HAVE_IPADDRESS,
-    IntegrationTest,
-    SkipTest,
-    client_context,
-    db_pwd,
-    db_user,
-    unittest,
-)
+from test import HAVE_IPADDRESS, IntegrationTest, SkipTest, client_context, unittest
 from test.utils import (
     EventListener,
     cat_files,
@@ -42,11 +28,12 @@ from test.utils import (
     ignore_deprecations,
     remove_all_users,
 )
+from urllib.parse import quote_plus
 
 from pymongo import MongoClient, ssl_support
 from pymongo.errors import ConfigurationError, ConnectionFailure, OperationFailure
-from pymongo.hello_compat import HelloCompat
-from pymongo.ssl_support import HAVE_SSL, _ssl, get_ssl_context, validate_cert_reqs
+from pymongo.hello import HelloCompat
+from pymongo.ssl_support import HAVE_SSL, _ssl, get_ssl_context
 from pymongo.write_concern import WriteConcern
 
 _HAVE_PYOPENSSL = False
@@ -63,7 +50,7 @@ try:
 
     _HAVE_PYOPENSSL = True
 except ImportError:
-    pass
+    _load_trusted_ca_certs = None
 
 
 if HAVE_SSL:
@@ -99,48 +86,39 @@ class TestClientSSL(unittest.TestCase):
         self.assertRaises(ConfigurationError, MongoClient, ssl=True)
 
         # Implied
-        self.assertRaises(ConfigurationError, MongoClient, ssl_certfile=CLIENT_PEM)
+        self.assertRaises(ConfigurationError, MongoClient, tlsCertificateKeyFile=CLIENT_PEM)
 
     @unittest.skipUnless(HAVE_SSL, "The ssl module is not available.")
     @ignore_deprecations
     def test_config_ssl(self):
         # Tests various ssl configurations
         self.assertRaises(ValueError, MongoClient, ssl="foo")
-        self.assertRaises(ConfigurationError, MongoClient, ssl=False, ssl_certfile=CLIENT_PEM)
+        self.assertRaises(
+            ConfigurationError, MongoClient, tls=False, tlsCertificateKeyFile=CLIENT_PEM
+        )
         self.assertRaises(TypeError, MongoClient, ssl=0)
         self.assertRaises(TypeError, MongoClient, ssl=5.5)
         self.assertRaises(TypeError, MongoClient, ssl=[])
 
-        self.assertRaises(IOError, MongoClient, ssl_certfile="NoSuchFile")
-        self.assertRaises(TypeError, MongoClient, ssl_certfile=True)
-        self.assertRaises(TypeError, MongoClient, ssl_certfile=[])
-        self.assertRaises(IOError, MongoClient, ssl_keyfile="NoSuchFile")
-        self.assertRaises(TypeError, MongoClient, ssl_keyfile=True)
-        self.assertRaises(TypeError, MongoClient, ssl_keyfile=[])
+        self.assertRaises(IOError, MongoClient, tlsCertificateKeyFile="NoSuchFile")
+        self.assertRaises(TypeError, MongoClient, tlsCertificateKeyFile=True)
+        self.assertRaises(TypeError, MongoClient, tlsCertificateKeyFile=[])
 
         # Test invalid combinations
-        self.assertRaises(ConfigurationError, MongoClient, ssl=False, ssl_keyfile=CLIENT_PEM)
-        self.assertRaises(ConfigurationError, MongoClient, ssl=False, ssl_certfile=CLIENT_PEM)
         self.assertRaises(
-            ConfigurationError,
-            MongoClient,
-            ssl=False,
-            ssl_keyfile=CLIENT_PEM,
-            ssl_certfile=CLIENT_PEM,
+            ConfigurationError, MongoClient, tls=False, tlsCertificateKeyFile=CLIENT_PEM
+        )
+        self.assertRaises(ConfigurationError, MongoClient, tls=False, tlsCAFile=CA_PEM)
+        self.assertRaises(ConfigurationError, MongoClient, tls=False, tlsCRLFile=CRL_PEM)
+        self.assertRaises(
+            ConfigurationError, MongoClient, tls=False, tlsAllowInvalidCertificates=False
+        )
+        self.assertRaises(
+            ConfigurationError, MongoClient, tls=False, tlsAllowInvalidHostnames=False
+        )
+        self.assertRaises(
+            ConfigurationError, MongoClient, tls=False, tlsDisableOCSPEndpointCheck=False
         )
-
-        self.assertRaises(ValueError, validate_cert_reqs, "ssl_cert_reqs", 3)
-        self.assertRaises(ValueError, validate_cert_reqs, "ssl_cert_reqs", -1)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", None), None)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", ssl.CERT_NONE), ssl.CERT_NONE)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", ssl.CERT_OPTIONAL), ssl.CERT_OPTIONAL)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", ssl.CERT_REQUIRED), ssl.CERT_REQUIRED)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", 0), ssl.CERT_NONE)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", 1), ssl.CERT_OPTIONAL)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", 2), ssl.CERT_REQUIRED)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", "CERT_NONE"), ssl.CERT_NONE)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", "CERT_OPTIONAL"), ssl.CERT_OPTIONAL)
-        self.assertEqual(validate_cert_reqs("ssl_cert_reqs", "CERT_REQUIRED"), ssl.CERT_REQUIRED)
 
     @unittest.skipUnless(_HAVE_PYOPENSSL, "PyOpenSSL is not available.")
     def test_use_pyopenssl_when_available(self):
@@ -181,9 +159,9 @@ class TestSSL(IntegrationTest):
         # no --sslPEMKeyFile or with --sslWeakCertificateValidation
         self.assertClientWorks(self.client)
 
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @ignore_deprecations
-    def test_ssl_pem_passphrase(self):
+    def test_tlsCertificateKeyFilePassword(self):
         # Expects the server to be running with server.pem and ca.pem
         #
         #   --sslPEMKeyFile=/path/to/pymongo/test/certificates/server.pem
@@ -194,9 +172,9 @@ class TestSSL(IntegrationTest):
                 MongoClient,
                 "localhost",
                 ssl=True,
-                ssl_certfile=CLIENT_ENCRYPTED_PEM,
-                ssl_pem_passphrase="qwerty",
-                ssl_ca_certs=CA_PEM,
+                tlsCertificateKeyFile=CLIENT_ENCRYPTED_PEM,
+                tlsCertificateKeyFilePassword="qwerty",
+                tlsCAFile=CA_PEM,
                 serverSelectionTimeoutMS=100,
             )
         else:
@@ -204,9 +182,9 @@ class TestSSL(IntegrationTest):
                 MongoClient(
                     "localhost",
                     ssl=True,
-                    ssl_certfile=CLIENT_ENCRYPTED_PEM,
-                    ssl_pem_passphrase="qwerty",
-                    ssl_ca_certs=CA_PEM,
+                    tlsCertificateKeyFile=CLIENT_ENCRYPTED_PEM,
+                    tlsCertificateKeyFilePassword="qwerty",
+                    tlsCAFile=CA_PEM,
                     serverSelectionTimeoutMS=5000,
                     **self.credentials
                 )
@@ -214,12 +192,12 @@ class TestSSL(IntegrationTest):
 
             uri_fmt = (
                 "mongodb://localhost/?ssl=true"
-                "&ssl_certfile=%s&ssl_pem_passphrase=qwerty"
-                "&ssl_ca_certs=%s&serverSelectionTimeoutMS=5000"
+                "&tlsCertificateKeyFile=%s&tlsCertificateKeyFilePassword=qwerty"
+                "&tlsCAFile=%s&serverSelectionTimeoutMS=5000"
             )
             connected(MongoClient(uri_fmt % (CLIENT_ENCRYPTED_PEM, CA_PEM), **self.credentials))
 
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @client_context.require_no_auth
     @ignore_deprecations
     def test_cert_ssl_implicitly_set(self):
@@ -229,12 +207,12 @@ class TestSSL(IntegrationTest):
         #   --sslCAFile=/path/to/pymongo/test/certificates/ca.pem
         #
 
-        # test that setting ssl_certfile causes ssl to be set to True
+        # test that setting tlsCertificateKeyFile causes ssl to be set to True
         client = MongoClient(
             client_context.host,
             client_context.port,
-            ssl_cert_reqs=ssl.CERT_NONE,
-            ssl_certfile=CLIENT_PEM,
+            tlsAllowInvalidCertificates=True,
+            tlsCertificateKeyFile=CLIENT_PEM,
         )
         response = client.admin.command(HelloCompat.LEGACY_CMD)
         if "setName" in response:
@@ -242,13 +220,13 @@ class TestSSL(IntegrationTest):
                 client_context.pair,
                 replicaSet=response["setName"],
                 w=len(response["hosts"]),
-                ssl_cert_reqs=ssl.CERT_NONE,
-                ssl_certfile=CLIENT_PEM,
+                tlsAllowInvalidCertificates=True,
+                tlsCertificateKeyFile=CLIENT_PEM,
             )
 
         self.assertClientWorks(client)
 
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @client_context.require_no_auth
     @ignore_deprecations
     def test_cert_ssl_validation(self):
@@ -260,9 +238,9 @@ class TestSSL(IntegrationTest):
         client = MongoClient(
             "localhost",
             ssl=True,
-            ssl_certfile=CLIENT_PEM,
-            ssl_cert_reqs=ssl.CERT_REQUIRED,
-            ssl_ca_certs=CA_PEM,
+            tlsCertificateKeyFile=CLIENT_PEM,
+            tlsAllowInvalidCertificates=False,
+            tlsCAFile=CA_PEM,
         )
         response = client.admin.command(HelloCompat.LEGACY_CMD)
         if "setName" in response:
@@ -277,9 +255,9 @@ class TestSSL(IntegrationTest):
                 replicaSet=response["setName"],
                 w=len(response["hosts"]),
                 ssl=True,
-                ssl_certfile=CLIENT_PEM,
-                ssl_cert_reqs=ssl.CERT_REQUIRED,
-                ssl_ca_certs=CA_PEM,
+                tlsCertificateKeyFile=CLIENT_PEM,
+                tlsAllowInvalidCertificates=False,
+                tlsCAFile=CA_PEM,
             )
 
         self.assertClientWorks(client)
@@ -288,13 +266,13 @@ class TestSSL(IntegrationTest):
             client = MongoClient(
                 "127.0.0.1",
                 ssl=True,
-                ssl_certfile=CLIENT_PEM,
-                ssl_cert_reqs=ssl.CERT_REQUIRED,
-                ssl_ca_certs=CA_PEM,
+                tlsCertificateKeyFile=CLIENT_PEM,
+                tlsAllowInvalidCertificates=False,
+                tlsCAFile=CA_PEM,
             )
             self.assertClientWorks(client)
 
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @client_context.require_no_auth
     @ignore_deprecations
     def test_cert_ssl_uri_support(self):
@@ -304,50 +282,13 @@ class TestSSL(IntegrationTest):
         #   --sslCAFile=/path/to/pymongo/test/certificates/ca.pem
         #
         uri_fmt = (
-            "mongodb://localhost/?ssl=true&ssl_certfile=%s&ssl_cert_reqs"
-            "=%s&ssl_ca_certs=%s&ssl_match_hostname=true"
+            "mongodb://localhost/?ssl=true&tlsCertificateKeyFile=%s&tlsAllowInvalidCertificates"
+            "=%s&tlsCAFile=%s&tlsAllowInvalidHostnames=false"
         )
-        client = MongoClient(uri_fmt % (CLIENT_PEM, "CERT_REQUIRED", CA_PEM))
+        client = MongoClient(uri_fmt % (CLIENT_PEM, "true", CA_PEM))
         self.assertClientWorks(client)
 
-    @client_context.require_ssl_certfile
-    @client_context.require_no_auth
-    @ignore_deprecations
-    def test_cert_ssl_validation_optional(self):
-        # Expects the server to be running with server.pem and ca.pem
-        #
-        #   --sslPEMKeyFile=/path/to/pymongo/test/certificates/server.pem
-        #   --sslCAFile=/path/to/pymongo/test/certificates/ca.pem
-        #
-        client = MongoClient(
-            "localhost",
-            ssl=True,
-            ssl_certfile=CLIENT_PEM,
-            ssl_cert_reqs=ssl.CERT_OPTIONAL,
-            ssl_ca_certs=CA_PEM,
-        )
-
-        response = client.admin.command(HelloCompat.LEGACY_CMD)
-        if "setName" in response:
-            if response["primary"].split(":")[0] != "localhost":
-                raise SkipTest(
-                    "No hosts in the replicaset for 'localhost'. "
-                    "Cannot validate hostname in the certificate"
-                )
-
-            client = MongoClient(
-                "localhost",
-                replicaSet=response["setName"],
-                w=len(response["hosts"]),
-                ssl=True,
-                ssl_certfile=CLIENT_PEM,
-                ssl_cert_reqs=ssl.CERT_OPTIONAL,
-                ssl_ca_certs=CA_PEM,
-            )
-
-        self.assertClientWorks(client)
-
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @client_context.require_server_resolvable
     @ignore_deprecations
     def test_cert_ssl_validation_hostname_matching(self):
@@ -355,21 +296,17 @@ class TestSSL(IntegrationTest):
         #
         #   --sslPEMKeyFile=/path/to/pymongo/test/certificates/server.pem
         #   --sslCAFile=/path/to/pymongo/test/certificates/ca.pem
-
-        # Python > 2.7.9. If SSLContext doesn't have load_default_certs
-        # it also doesn't have check_hostname.
-        ctx = get_ssl_context(None, None, None, None, ssl.CERT_NONE, None, False, True)
-        if hasattr(ctx, "load_default_certs"):
+        ctx = get_ssl_context(None, None, None, None, True, True, False)
+        self.assertFalse(ctx.check_hostname)
+        ctx = get_ssl_context(None, None, None, None, True, False, False)
+        self.assertFalse(ctx.check_hostname)
+        ctx = get_ssl_context(None, None, None, None, False, True, False)
+        self.assertFalse(ctx.check_hostname)
+        ctx = get_ssl_context(None, None, None, None, False, False, False)
+        if _PY37PLUS or _HAVE_PYOPENSSL:
+            self.assertTrue(ctx.check_hostname)
+        else:
             self.assertFalse(ctx.check_hostname)
-            ctx = get_ssl_context(None, None, None, None, ssl.CERT_NONE, None, True, True)
-            self.assertFalse(ctx.check_hostname)
-            ctx = get_ssl_context(None, None, None, None, ssl.CERT_REQUIRED, None, False, True)
-            self.assertFalse(ctx.check_hostname)
-            ctx = get_ssl_context(None, None, None, None, ssl.CERT_REQUIRED, None, True, True)
-            if _PY37PLUS:
-                self.assertTrue(ctx.check_hostname)
-            else:
-                self.assertFalse(ctx.check_hostname)
 
         response = self.client.admin.command(HelloCompat.LEGACY_CMD)
 
@@ -378,9 +315,9 @@ class TestSSL(IntegrationTest):
                 MongoClient(
                     "server",
                     ssl=True,
-                    ssl_certfile=CLIENT_PEM,
-                    ssl_cert_reqs=ssl.CERT_REQUIRED,
-                    ssl_ca_certs=CA_PEM,
+                    tlsCertificateKeyFile=CLIENT_PEM,
+                    tlsAllowInvalidCertificates=False,
+                    tlsCAFile=CA_PEM,
                     serverSelectionTimeoutMS=500,
                     **self.credentials
                 )
@@ -390,10 +327,10 @@ class TestSSL(IntegrationTest):
             MongoClient(
                 "server",
                 ssl=True,
-                ssl_certfile=CLIENT_PEM,
-                ssl_cert_reqs=ssl.CERT_REQUIRED,
-                ssl_ca_certs=CA_PEM,
-                ssl_match_hostname=False,
+                tlsCertificateKeyFile=CLIENT_PEM,
+                tlsAllowInvalidCertificates=False,
+                tlsCAFile=CA_PEM,
+                tlsAllowInvalidHostnames=True,
                 serverSelectionTimeoutMS=500,
                 **self.credentials
             )
@@ -406,9 +343,9 @@ class TestSSL(IntegrationTest):
                         "server",
                         replicaSet=response["setName"],
                         ssl=True,
-                        ssl_certfile=CLIENT_PEM,
-                        ssl_cert_reqs=ssl.CERT_REQUIRED,
-                        ssl_ca_certs=CA_PEM,
+                        tlsCertificateKeyFile=CLIENT_PEM,
+                        tlsAllowInvalidCertificates=False,
+                        tlsCAFile=CA_PEM,
                         serverSelectionTimeoutMS=500,
                         **self.credentials
                     )
@@ -419,26 +356,26 @@ class TestSSL(IntegrationTest):
                     "server",
                     replicaSet=response["setName"],
                     ssl=True,
-                    ssl_certfile=CLIENT_PEM,
-                    ssl_cert_reqs=ssl.CERT_REQUIRED,
-                    ssl_ca_certs=CA_PEM,
-                    ssl_match_hostname=False,
+                    tlsCertificateKeyFile=CLIENT_PEM,
+                    tlsAllowInvalidCertificates=False,
+                    tlsCAFile=CA_PEM,
+                    tlsAllowInvalidHostnames=True,
                     serverSelectionTimeoutMS=500,
                     **self.credentials
                 )
             )
 
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @ignore_deprecations
-    def test_ssl_crlfile_support(self):
+    def test_tlsCRLFile_support(self):
         if not hasattr(ssl, "VERIFY_CRL_CHECK_LEAF") or _ssl.IS_PYOPENSSL:
             self.assertRaises(
                 ConfigurationError,
                 MongoClient,
                 "localhost",
                 ssl=True,
-                ssl_ca_certs=CA_PEM,
-                ssl_crlfile=CRL_PEM,
+                tlsCAFile=CA_PEM,
+                tlsCRLFile=CRL_PEM,
                 serverSelectionTimeoutMS=100,
             )
         else:
@@ -446,7 +383,7 @@ class TestSSL(IntegrationTest):
                 MongoClient(
                     "localhost",
                     ssl=True,
-                    ssl_ca_certs=CA_PEM,
+                    tlsCAFile=CA_PEM,
                     serverSelectionTimeoutMS=100,
                     **self.credentials
                 )
@@ -457,26 +394,24 @@ class TestSSL(IntegrationTest):
                     MongoClient(
                         "localhost",
                         ssl=True,
-                        ssl_ca_certs=CA_PEM,
-                        ssl_crlfile=CRL_PEM,
+                        tlsCAFile=CA_PEM,
+                        tlsCRLFile=CRL_PEM,
                         serverSelectionTimeoutMS=100,
                         **self.credentials
                     )
                 )
 
-            uri_fmt = (
-                "mongodb://localhost/?ssl=true&" "ssl_ca_certs=%s&serverSelectionTimeoutMS=100"
-            )
+            uri_fmt = "mongodb://localhost/?ssl=true&" "tlsCAFile=%s&serverSelectionTimeoutMS=100"
             connected(MongoClient(uri_fmt % (CA_PEM,), **self.credentials))
 
             uri_fmt = (
-                "mongodb://localhost/?ssl=true&ssl_crlfile=%s"
-                "&ssl_ca_certs=%s&serverSelectionTimeoutMS=100"
+                "mongodb://localhost/?ssl=true&tlsCRLFile=%s"
+                "&tlsCAFile=%s&serverSelectionTimeoutMS=100"
             )
             with self.assertRaises(ConnectionFailure):
                 connected(MongoClient(uri_fmt % (CRL_PEM, CA_PEM), **self.credentials))
 
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @client_context.require_server_resolvable
     @ignore_deprecations
     def test_validation_with_system_ca_certs(self):
@@ -498,7 +433,7 @@ class TestSSL(IntegrationTest):
             MongoClient(
                 "server",
                 ssl=True,
-                ssl_match_hostname=False,
+                tlsAllowInvalidHostnames=True,
                 serverSelectionTimeoutMS=100,
                 **self.credentials
             )
@@ -517,7 +452,7 @@ class TestSSL(IntegrationTest):
         )
 
     def test_system_certs_config_error(self):
-        ctx = get_ssl_context(None, None, None, None, ssl.CERT_NONE, None, False, True)
+        ctx = get_ssl_context(None, None, None, None, True, True, False)
         if (sys.platform != "win32" and hasattr(ctx, "set_default_verify_paths")) or hasattr(
             ctx, "load_default_certs"
         ):
@@ -548,11 +483,11 @@ class TestSSL(IntegrationTest):
         # Force the test on Windows, regardless of environment.
         ssl_support.HAVE_WINCERTSTORE = False
         try:
-            ctx = get_ssl_context(None, None, None, CA_PEM, ssl.CERT_REQUIRED, None, True, True)
+            ctx = get_ssl_context(None, None, CA_PEM, None, False, False, False)
             ssl_sock = ctx.wrap_socket(socket.socket())
             self.assertEqual(ssl_sock.ca_certs, CA_PEM)
 
-            ctx = get_ssl_context(None, None, None, None, None, None, True, True)
+            ctx = get_ssl_context(None, None, None, None, False, False, False)
             ssl_sock = ctx.wrap_socket(socket.socket())
             self.assertEqual(ssl_sock.ca_certs, ssl_support.certifi.where())
         finally:
@@ -569,25 +504,20 @@ class TestSSL(IntegrationTest):
         if not ssl_support.HAVE_WINCERTSTORE:
             raise SkipTest("Need wincertstore to test wincertstore.")
 
-        ctx = get_ssl_context(None, None, None, CA_PEM, ssl.CERT_REQUIRED, None, True, True)
+        ctx = get_ssl_context(None, None, CA_PEM, None, False, False, False)
         ssl_sock = ctx.wrap_socket(socket.socket())
         self.assertEqual(ssl_sock.ca_certs, CA_PEM)
 
-        ctx = get_ssl_context(None, None, None, None, None, None, True, True)
+        ctx = get_ssl_context(None, None, None, None, False, False, False)
         ssl_sock = ctx.wrap_socket(socket.socket())
         self.assertEqual(ssl_sock.ca_certs, ssl_support._WINCERTS.name)
 
     @client_context.require_auth
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @ignore_deprecations
     def test_mongodb_x509_auth(self):
         host, port = client_context.host, client_context.port
-        ssl_client = MongoClient(
-            client_context.pair, ssl=True, ssl_cert_reqs=ssl.CERT_NONE, ssl_certfile=CLIENT_PEM
-        )
-        self.addCleanup(remove_all_users, ssl_client["$external"])
-
-        ssl_client.admin.authenticate(db_user, db_pwd)
+        self.addCleanup(remove_all_users, client_context.client["$external"])
 
         # Give x509 user all necessary privileges.
         client_context.create_user(
@@ -600,58 +530,55 @@ class TestSSL(IntegrationTest):
         )
 
         noauth = MongoClient(
-            client_context.pair, ssl=True, ssl_cert_reqs=ssl.CERT_NONE, ssl_certfile=CLIENT_PEM
+            client_context.pair,
+            ssl=True,
+            tlsAllowInvalidCertificates=True,
+            tlsCertificateKeyFile=CLIENT_PEM,
         )
         self.addCleanup(noauth.close)
 
-        self.assertRaises(OperationFailure, noauth.pymongo_test.test.count)
+        with self.assertRaises(OperationFailure):
+            noauth.pymongo_test.test.find_one()
 
         listener = EventListener()
         auth = MongoClient(
             client_context.pair,
             authMechanism="MONGODB-X509",
             ssl=True,
-            ssl_cert_reqs=ssl.CERT_NONE,
-            ssl_certfile=CLIENT_PEM,
+            tlsAllowInvalidCertificates=True,
+            tlsCertificateKeyFile=CLIENT_PEM,
             event_listeners=[listener],
         )
         self.addCleanup(auth.close)
 
-        if client_context.version.at_least(3, 3, 12):
-            # No error
-            auth.pymongo_test.test.find_one()
-            names = listener.started_command_names()
-            if client_context.version.at_least(4, 4, -1):
-                # Speculative auth skips the authenticate command.
-                self.assertEqual(names, ["find"])
-            else:
-                self.assertEqual(names, ["authenticate", "find"])
+        # No error
+        auth.pymongo_test.test.find_one()
+        names = listener.started_command_names()
+        if client_context.version.at_least(4, 4, -1):
+            # Speculative auth skips the authenticate command.
+            self.assertEqual(names, ["find"])
         else:
-            # Should require a username
-            with self.assertRaises(ConfigurationError):
-                auth.pymongo_test.test.find_one()
+            self.assertEqual(names, ["authenticate", "find"])
 
         uri = "mongodb://%s@%s:%d/?authMechanism=" "MONGODB-X509" % (
             quote_plus(MONGODB_X509_USERNAME),
             host,
             port,
         )
-        client = MongoClient(uri, ssl=True, ssl_cert_reqs=ssl.CERT_NONE, ssl_certfile=CLIENT_PEM)
+        client = MongoClient(
+            uri, ssl=True, tlsAllowInvalidCertificates=True, tlsCertificateKeyFile=CLIENT_PEM
+        )
         self.addCleanup(client.close)
         # No error
         client.pymongo_test.test.find_one()
 
         uri = "mongodb://%s:%d/?authMechanism=MONGODB-X509" % (host, port)
-        client = MongoClient(uri, ssl=True, ssl_cert_reqs=ssl.CERT_NONE, ssl_certfile=CLIENT_PEM)
+        client = MongoClient(
+            uri, ssl=True, tlsAllowInvalidCertificates=True, tlsCertificateKeyFile=CLIENT_PEM
+        )
         self.addCleanup(client.close)
-        if client_context.version.at_least(3, 3, 12):
-            # No error
-            client.pymongo_test.test.find_one()
-        else:
-            # Should require a username
-            with self.assertRaises(ConfigurationError):
-                client.pymongo_test.test.find_one()
-
+        # No error
+        client.pymongo_test.test.find_one()
         # Auth should fail if username and certificate do not match
         uri = "mongodb://%s@%s:%d/?authMechanism=" "MONGODB-X509" % (
             quote_plus("not the username"),
@@ -659,7 +586,9 @@ class TestSSL(IntegrationTest):
             port,
         )
 
-        bad_client = MongoClient(uri, ssl=True, ssl_cert_reqs="CERT_NONE", ssl_certfile=CLIENT_PEM)
+        bad_client = MongoClient(
+            uri, ssl=True, tlsAllowInvalidCertificates=True, tlsCertificateKeyFile=CLIENT_PEM
+        )
         self.addCleanup(bad_client.close)
 
         with self.assertRaises(OperationFailure):
@@ -670,8 +599,8 @@ class TestSSL(IntegrationTest):
             username="not the username",
             authMechanism="MONGODB-X509",
             ssl=True,
-            ssl_cert_reqs=ssl.CERT_NONE,
-            ssl_certfile=CLIENT_PEM,
+            tlsAllowInvalidCertificates=True,
+            tlsCertificateKeyFile=CLIENT_PEM,
         )
         self.addCleanup(bad_client.close)
 
@@ -689,8 +618,8 @@ class TestSSL(IntegrationTest):
                 MongoClient(
                     uri,
                     ssl=True,
-                    ssl_cert_reqs=ssl.CERT_NONE,
-                    ssl_certfile=CA_PEM,
+                    tlsAllowInvalidCertificates=True,
+                    tlsCertificateKeyFile=CA_PEM,
                     serverSelectionTimeoutMS=100,
                 )
             )
@@ -699,7 +628,7 @@ class TestSSL(IntegrationTest):
         else:
             self.fail("Invalid certificate accepted.")
 
-    @client_context.require_ssl_certfile
+    @client_context.require_tlsCertificateKeyFile
     @ignore_deprecations
     def test_connect_with_ca_bundle(self):
         def remove(path):
