@@ -15,10 +15,8 @@
 """Test that pymongo is thread safe."""
 
 import threading
-from test import IntegrationTest, client_context, db_pwd, db_user, unittest
-from test.utils import joinall, rs_or_single_client, rs_or_single_client_noauth
-
-from pymongo.errors import OperationFailure
+from test import IntegrationTest, client_context, unittest
+from test.utils import joinall, rs_or_single_client
 
 
 @client_context.require_connection
@@ -104,20 +102,6 @@ class Update(threading.Thread):
                 assert error
 
 
-class Disconnect(threading.Thread):
-    def __init__(self, client, n):
-        threading.Thread.__init__(self)
-        self.client = client
-        self.n = n
-        self.passed = False
-
-    def run(self):
-        for _ in range(self.n):
-            self.client.close()
-
-        self.passed = True
-
-
 class TestThreads(IntegrationTest):
     def setUp(self):
         self.db = self.client.pymongo_test
@@ -172,54 +156,6 @@ class TestThreads(IntegrationTest):
         error.join()
         okay.join()
 
-    def test_client_disconnect(self):
-        db = rs_or_single_client(serverSelectionTimeoutMS=30000).pymongo_test
-        db.drop_collection("test")
-        db.test.insert_many([{"x": i} for i in range(1000)])
-
-        # Start 10 threads that execute a query, and 10 threads that call
-        # client.close() 10 times in a row.
-        threads = [SaveAndFind(db.test) for _ in range(10)]
-        threads.extend(Disconnect(db.client, 10) for _ in range(10))
-
-        for t in threads:
-            t.start()
-
-        for t in threads:
-            t.join(300)
-
-        for t in threads:
-            self.assertTrue(t.passed)
-
-
-class TestThreadsAuth(IntegrationTest):
-    @classmethod
-    @client_context.require_auth
-    def setUpClass(cls):
-        super(TestThreadsAuth, cls).setUpClass()
-
-    def test_auto_auth_login(self):
-        # Create the database upfront to workaround SERVER-39167.
-        self.client.auth_test.test.insert_one({})
-        self.addCleanup(self.client.drop_database, "auth_test")
-        client = rs_or_single_client_noauth()
-        self.assertRaises(OperationFailure, client.auth_test.test.find_one)
-
-        # Admin auth
-        client.admin.authenticate(db_user, db_pwd)
-
-        nthreads = 10
-        threads = []
-        for _ in range(nthreads):
-            t = AutoAuthenticateThreads(client.auth_test.test, 10)
-            t.start()
-            threads.append(t)
-
-        joinall(threads)
-
-        for t in threads:
-            self.assertTrue(t.success)
-
 
 if __name__ == "__main__":
     unittest.main()
