@@ -17,18 +17,13 @@
 import copy
 import sys
 import warnings
-
-try:
-    from ssl import CERT_NONE
-except ImportError:
-    CERT_NONE = 0
+from urllib.parse import quote_plus
 
 sys.path[0:0] = [""]
 
 from test import unittest
 
 from bson.binary import JAVA_LEGACY
-from bson.py3compat import _unicode, string_type
 from pymongo import ReadPreference
 from pymongo.errors import ConfigurationError, InvalidURI
 from pymongo.uri_parser import parse_uri, parse_userinfo, split_hosts, split_options
@@ -82,7 +77,7 @@ class TestURI(unittest.TestCase):
         self.assertRaises(ConfigurationError, split_options, "foo=bar;foo")
         self.assertTrue(split_options("ssl=true"))
         self.assertTrue(split_options("connect=true"))
-        self.assertTrue(split_options("ssl_match_hostname=true"))
+        self.assertTrue(split_options("tlsAllowInvalidHostnames=false"))
 
         # Test Invalid URI options that should throw warnings.
         with warnings.catch_warnings():
@@ -96,7 +91,7 @@ class TestURI(unittest.TestCase):
             self.assertRaises(Warning, split_options, "connectTimeoutMS=-1e100000", warn=True)
             self.assertRaises(Warning, split_options, "ssl=foo", warn=True)
             self.assertRaises(Warning, split_options, "connect=foo", warn=True)
-            self.assertRaises(Warning, split_options, "ssl_match_hostname=foo", warn=True)
+            self.assertRaises(Warning, split_options, "tlsAllowInvalidHostnames=foo", warn=True)
             self.assertRaises(Warning, split_options, "connectTimeoutMS=inf", warn=True)
             self.assertRaises(Warning, split_options, "connectTimeoutMS=-inf", warn=True)
             self.assertRaises(Warning, split_options, "wtimeoutms=foo", warn=True)
@@ -115,7 +110,7 @@ class TestURI(unittest.TestCase):
         self.assertRaises(ValueError, split_options, "connectTimeoutMS=-1e100000")
         self.assertRaises(ValueError, split_options, "ssl=foo")
         self.assertRaises(ValueError, split_options, "connect=foo")
-        self.assertRaises(ValueError, split_options, "ssl_match_hostname=foo")
+        self.assertRaises(ValueError, split_options, "tlsAllowInvalidHostnames=foo")
         self.assertRaises(ValueError, split_options, "connectTimeoutMS=inf")
         self.assertRaises(ValueError, split_options, "connectTimeoutMS=-inf")
         self.assertRaises(ValueError, split_options, "wtimeoutms=foo")
@@ -133,7 +128,7 @@ class TestURI(unittest.TestCase):
         self.assertEqual({"connecttimeoutms": 0.0001}, split_options("connectTimeoutMS=0.1"))
         self.assertTrue(split_options("connectTimeoutMS=300"))
         self.assertTrue(isinstance(split_options("w=5")["w"], int))
-        self.assertTrue(isinstance(split_options("w=5.5")["w"], string_type))
+        self.assertTrue(isinstance(split_options("w=5.5")["w"], str))
         self.assertTrue(split_options("w=foo"))
         self.assertTrue(split_options("w=majority"))
         self.assertTrue(split_options("wtimeoutms=500"))
@@ -425,13 +420,14 @@ class TestURI(unittest.TestCase):
                 "collection": None,
                 "database": None,
                 "nodelist": [("/MongoDB.sock", None)],
-                "options": {"ssl_certfile": "/a/b"},
+                "options": {"tlsCertificateKeyFile": "/a/b"},
                 "password": "foo/bar",
                 "username": "jesse",
                 "fqdn": None,
             },
             parse_uri(
-                "mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?ssl_certfile=/a/b", validate=False
+                "mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?tlsCertificateKeyFile=/a/b",
+                validate=False,
             ),
         )
 
@@ -440,61 +436,35 @@ class TestURI(unittest.TestCase):
                 "collection": None,
                 "database": None,
                 "nodelist": [("/MongoDB.sock", None)],
-                "options": {"ssl_certfile": "a/b"},
+                "options": {"tlsCertificateKeyFile": "a/b"},
                 "password": "foo/bar",
                 "username": "jesse",
                 "fqdn": None,
             },
             parse_uri(
-                "mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?ssl_certfile=a/b", validate=False
+                "mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?tlsCertificateKeyFile=a/b",
+                validate=False,
             ),
         )
 
     def test_tlsinsecure_simple(self):
         # check that tlsInsecure is expanded correctly.
+        self.maxDiff = None
         uri = "mongodb://example.com/?tlsInsecure=true"
         res = {
-            "ssl_match_hostname": False,
-            "ssl_cert_reqs": CERT_NONE,
-            "tlsinsecure": True,
-            "ssl_check_ocsp_endpoint": False,
+            "tlsAllowInvalidHostnames": True,
+            "tlsAllowInvalidCertificates": True,
+            "tlsInsecure": True,
+            "tlsDisableOCSPEndpointCheck": True,
         }
         self.assertEqual(res, parse_uri(uri)["options"])
 
-    def test_tlsinsecure_legacy_conflict(self):
-        # must not allow use of tlsinsecure alongside legacy TLS options.
-        # same check for modern TLS options is performed in the spec-tests.
-        uri = "mongodb://srv.com/?tlsInsecure=true&ssl_match_hostname=true"
-        with self.assertRaises(InvalidURI):
-            parse_uri(uri, validate=False, warn=False, normalize=False)
-
-    def test_tlsDisableOCSPEndpointCheck(self):
-        # check that tlsDisableOCSPEndpointCheck is handled correctly.
-        uri = "mongodb://example.com/?tlsDisableOCSPEndpointCheck=true"
-        res = {"ssl_check_ocsp_endpoint": False}
-        self.assertEqual(res, parse_uri(uri)["options"])
-
-        uri = "mongodb://example.com/?tlsDisableOCSPEndpointCheck=false"
-        res = {"ssl_check_ocsp_endpoint": True}
-        self.assertEqual(res, parse_uri(uri)["options"])
-
     def test_normalize_options(self):
         # check that options are converted to their internal names correctly.
-        uri = (
-            "mongodb://example.com/?tls=true&appname=myapp&maxPoolSize=10&" "fsync=true&wtimeout=10"
-        )
-        res = {"ssl": True, "appname": "myapp", "maxpoolsize": 10, "fsync": True, "wtimeoutms": 10}
+        uri = "mongodb://example.com/?ssl=true&appname=myapp"
+        res = {"tls": True, "appname": "myapp"}
         self.assertEqual(res, parse_uri(uri)["options"])
 
-    def test_waitQueueMultiple_deprecated(self):
-        uri = "mongodb://example.com/?waitQueueMultiple=5"
-        with warnings.catch_warnings(record=True) as ctx:
-            warnings.simplefilter("always")
-            parse_uri(uri)
-
-        self.assertEqual(len(ctx), 1)
-        self.assertTrue(issubclass(ctx[0].category, DeprecationWarning))
-
     def test_unquote_after_parsing(self):
         quoted_val = "val%21%40%23%24%25%5E%26%2A%28%29_%2B%2C%3A+etc"
         unquoted_val = "val!@#$%^&*()_+,: etc"
@@ -539,6 +509,14 @@ class TestURI(unittest.TestCase):
         ):
             parse_uri(uri)
 
+    def test_special_chars(self):
+        user = "user@ /9+:?~!$&'()*+,;="
+        pwd = "pwd@ /9+:?~!$&'()*+,;="
+        uri = "mongodb://%s:%s@localhost" % (quote_plus(user), quote_plus(pwd))
+        res = parse_uri(uri)
+        self.assertEqual(user, res["username"])
+        self.assertEqual(pwd, res["password"])
+
 
 if __name__ == "__main__":
     unittest.main()
