@@ -26,20 +26,19 @@ import threading
 import time
 import unittest
 import warnings
-from collections import defaultdict
+from collections import abc, defaultdict
 from functools import partial
 from test import client_context, db_pwd, db_user
 
-from bson import json_util, py3compat
+from bson import json_util
 from bson.objectid import ObjectId
-from bson.py3compat import abc, iteritems, string_type
 from bson.son import SON
 from pymongo import MongoClient, monitoring, operations, read_preferences
 from pymongo.collection import ReturnDocument
 from pymongo.errors import ConfigurationError, OperationFailure
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import HelloCompat
 from pymongo.monitoring import _SENSITIVE_COMMANDS
-from pymongo.pool import PoolOptions, _CancellationContext, _PoolGeneration
+from pymongo.pool import _CancellationContext, _PoolGeneration
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
 from pymongo.server_selectors import any_server_selector, writable_server_selector
@@ -47,13 +46,6 @@ from pymongo.server_type import SERVER_TYPE
 from pymongo.uri_parser import parse_uri
 from pymongo.write_concern import WriteConcern
 
-if sys.version_info[0] < 3:
-    # Python 2.7, use our backport.
-    from test.barrier import Barrier
-else:
-    from threading import Barrier
-
-
 IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=50)
 
 
@@ -111,6 +103,9 @@ class CMAPListener(BaseListener, monitoring.ConnectionPoolListener):
     def pool_created(self, event):
         self.add_event(event)
 
+    def pool_ready(self, event):
+        self.add_event(event)
+
     def pool_cleared(self, event):
         self.add_event(event)
 
@@ -256,7 +251,8 @@ class MockPool(object):
     def __init__(self, address, options, handshake=True):
         self.gen = _PoolGeneration()
         self._lock = threading.Lock()
-        self.opts = PoolOptions()
+        self.opts = options
+        self.operation_count = 0
 
     def stale_generation(self, gen, service_id):
         return self.gen.stale(gen, service_id)
@@ -271,9 +267,15 @@ class MockPool(object):
         with self._lock:
             self.gen.inc(service_id)
 
+    def ready(self):
+        pass
+
     def reset(self, service_id=None):
         self._reset()
 
+    def reset_without_pause(self):
+        self._reset()
+
     def close(self):
         self._reset()
 
@@ -291,7 +293,7 @@ class ScenarioDict(dict):
         def convert(v):
             if isinstance(v, abc.Mapping):
                 return ScenarioDict(v)
-            if isinstance(v, (py3compat.string_type, bytes)):
+            if isinstance(v, (str, bytes)):
                 return v
             if isinstance(v, abc.Sequence):
                 return [convert(item) for item in v]
@@ -498,13 +500,15 @@ def _connection_string(h):
     return "mongodb://%s" % (str(h),)
 
 
-def _mongo_client(host, port, authenticate=True, directConnection=False, **kwargs):
+def _mongo_client(host, port, authenticate=True, directConnection=None, **kwargs):
     """Create a new client over SSL/TLS if necessary."""
     host = host or client_context.host
     port = port or client_context.port
     client_options = client_context.default_client_options.copy()
     if client_context.replica_set_name and not directConnection:
         client_options["replicaSet"] = client_context.replica_set_name
+    if directConnection is not None:
+        client_options["directConnection"] = directConnection
     client_options.update(kwargs)
 
     uri = _connection_string(host)
@@ -575,7 +579,7 @@ def ensure_all_connected(client):
     connected_host_list = set([hello["me"]])
     admindb = client.get_database("admin")
 
-    # Run legacy hello until we have connected to each host at least once.
+    # Run hello until we have connected to each host at least once.
     while connected_host_list != target_host_list:
         hello = admindb.command(HelloCompat.LEGACY_CMD, read_preference=ReadPreference.SECONDARY)
         connected_host_list.update([hello["me"]])
@@ -680,20 +684,6 @@ def server_started_with_auth(client):
     return "--auth" in argv or "--keyFile" in argv
 
 
-def server_started_with_nojournal(client):
-    command_line = get_command_line(client)
-
-    # MongoDB 2.6.
-    if "parsed" in command_line:
-        parsed = command_line["parsed"]
-        if "storage" in parsed:
-            storage = parsed["storage"]
-            if "journal" in storage:
-                return not storage["journal"]["enabled"]
-
-    return server_started_with_option(client, "--nojournal", "nojournal")
-
-
 def drop_collections(db):
     # Drop all non-system collections in this database.
     for coll in db.list_collection_names(filter={"name": {"$regex": r"^(?!system\.)"}}):
@@ -714,7 +704,7 @@ def joinall(threads):
 def connected(client):
     """Convenience to wait for a newly-constructed client to connect."""
     with warnings.catch_warnings():
-        # Ignore warning that "ping" is always routed to primary even
+        # Ignore warning that ping is always routed to primary even
         # if client's read preference isn't PRIMARY.
         warnings.simplefilter("ignore", UserWarning)
         client.admin.command("ping")  # Force connection.
@@ -917,18 +907,6 @@ def is_greenthread_patched():
     return gevent_monkey_patched() or eventlet_monkey_patched()
 
 
-def cdecimal_patched():
-    """Check if Python 2.7 cdecimal patching is active."""
-    try:
-        import decimal
-
-        import cdecimal
-
-        return decimal is cdecimal
-    except ImportError:
-        return False
-
-
 def disable_replication(client):
     """Disable replication on all secondaries, requires MongoDB 3.2."""
     for host, port in client.secondaries:
@@ -995,7 +973,8 @@ def assertion_context(msg):
         yield
     except AssertionError as exc:
         msg = "%s (%s)" % (exc, msg)
-        py3compat.reraise(type(exc), msg, sys.exc_info()[2])
+        exc_type, exc_val, exc_tb = sys.exc_info()
+        raise exc_type(exc_val).with_traceback(exc_tb)
 
 
 def parse_spec_options(opts):
@@ -1016,8 +995,8 @@ def parse_spec_options(opts):
 
     if "hint" in opts:
         hint = opts.pop("hint")
-        if not isinstance(hint, string_type):
-            hint = list(iteritems(hint))
+        if not isinstance(hint, str):
+            hint = list(hint.items())
         opts["hint"] = hint
 
     # Properly format 'hint' arguments for the Bulk API tests.
@@ -1029,17 +1008,17 @@ def parse_spec_options(opts):
                 args = req.pop("arguments", {})
                 if "hint" in args:
                     hint = args.pop("hint")
-                    if not isinstance(hint, string_type):
-                        hint = list(iteritems(hint))
+                    if not isinstance(hint, str):
+                        hint = list(hint.items())
                     args["hint"] = hint
                 req["arguments"] = args
             else:
                 # Unified test format
-                bulk_model, spec = next(iteritems(req))
+                bulk_model, spec = next(iter(req.items()))
                 if "hint" in spec:
                     hint = spec.pop("hint")
-                    if not isinstance(hint, string_type):
-                        hint = list(iteritems(hint))
+                    if not isinstance(hint, str):
+                        hint = list(hint.items())
                     spec["hint"] = hint
         opts["requests"] = reqs
 
@@ -1052,7 +1031,7 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map, with_txn_callbac
         # PyMongo accepts sort as list of tuples.
         if arg_name == "sort":
             sort_dict = arguments[arg_name]
-            arguments[arg_name] = list(iteritems(sort_dict))
+            arguments[arg_name] = list(sort_dict.items())
         # Named "key" instead not fieldName.
         if arg_name == "fieldName":
             arguments["key"] = arguments.pop(arg_name)
@@ -1073,7 +1052,7 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map, with_txn_callbac
                     bulk_arguments = camel_to_snake_args(request["arguments"])
                 else:
                     # Unified test format
-                    bulk_model, spec = next(iteritems(request))
+                    bulk_model, spec = next(iter(request.items()))
                     bulk_class = getattr(operations, camel_to_upper_camel(bulk_model))
                     bulk_arguments = camel_to_snake_args(spec)
                 requests.append(bulk_class(**dict(bulk_arguments)))
