@@ -14,33 +14,28 @@
 
 """Utilities for testing driver specs."""
 
-import copy
 import functools
 import threading
-from test import IntegrationTest, client_context, client_knobs, unittest
+from collections import abc
+from test import IntegrationTest, client_context, client_knobs
 from test.utils import (
     CMAPListener,
     CompareType,
-    HeartbeatEventListener,
     OvertCommandListener,
     ServerAndTopologyEventListener,
     camel_to_snake,
     camel_to_snake_args,
-    camel_to_upper_camel,
-    parse_read_preference,
     parse_spec_options,
     prepare_spec_arguments,
     rs_client,
 )
 
 from bson import decode, encode
-from bson.binary import STANDARD, Binary
-from bson.codec_options import CodecOptions
+from bson.binary import Binary
 from bson.int64 import Int64
-from bson.py3compat import abc, iteritems, string_type, text_type
 from bson.son import SON
 from gridfs import GridFSBucket
-from pymongo import client_session, helpers, operations
+from pymongo import client_session
 from pymongo.command_cursor import CommandCursor
 from pymongo.cursor import Cursor
 from pymongo.errors import BulkWriteError, OperationFailure, PyMongoError
@@ -113,11 +108,9 @@ class SpecRunner(IntegrationTest):
         client.admin.command(cmd)
 
     def set_fail_point(self, command_args):
-        cmd = SON([("configureFailPoint", "failCommand")])
-        cmd.update(command_args)
         clients = self.mongos_clients if self.mongos_clients else [self.client]
         for client in clients:
-            self._set_fail_point(client, cmd)
+            self._set_fail_point(client, command_args)
 
     def targeted_fail_point(self, session, fail_point):
         """Run the targetedFailPoint test operation.
@@ -251,6 +244,10 @@ class SpecRunner(IntegrationTest):
             name = "open_download_stream_by_name"
         elif name == "download":
             name = "open_download_stream"
+        elif name == "map_reduce":
+            self.skipTest("PyMongo does not support mapReduce")
+        elif name == "count":
+            self.skipTest("PyMongo does not support count")
 
         database = collection.database
         collection = database.get_collection(collection.name)
@@ -263,7 +260,7 @@ class SpecRunner(IntegrationTest):
         if object_name == "gridfsbucket":
             # Only create the GridFSBucket when we need it (for the gridfs
             # retryable reads tests).
-            obj = GridFSBucket(database, bucket_name=collection.name, disable_md5=True)
+            obj = GridFSBucket(database, bucket_name=collection.name)
         else:
             objects = {
                 "client": database.client,
@@ -303,9 +300,6 @@ class SpecRunner(IntegrationTest):
                     arguments["pipeline"][-1]["$out"], read_preference=ReadPreference.PRIMARY
                 )
                 return out.find()
-        if name == "map_reduce":
-            if isinstance(result, dict) and "results" in result:
-                return result["results"]
         if "download" in name:
             result = Binary(result.read())
 
@@ -572,7 +566,7 @@ def expect_any_error(op):
 
 def expect_error_message(expected_result):
     if isinstance(expected_result, dict):
-        return isinstance(expected_result["errorContains"], text_type)
+        return isinstance(expected_result["errorContains"], str)
 
     return False
 
@@ -615,13 +609,10 @@ def end_sessions(sessions):
         s.end_session()
 
 
-OPTS = CodecOptions(document_class=dict, uuid_representation=STANDARD)
-
-
 def decode_raw(val):
     """Decode RawBSONDocuments in the given container."""
     if isinstance(val, (list, abc.Mapping)):
-        return decode(encode({"v": val}, codec_options=OPTS), OPTS)["v"]
+        return decode(encode({"v": val}))["v"]
     return val
 
 
